@@ -33,7 +33,6 @@ COMMON_FLAGS=''
 TOKUDB_BACKUP_VERSION=''
 # enable asan
 ENABLE_ASAN=0
-FIPSMODE=0
 #
 # Some programs that may be overriden
 TAR=${TAR:-tar}
@@ -107,11 +106,6 @@ do
             exit 3
         fi
         ;;
-    --enable-fipsmode )
-        shift
-        FIPSMODE=1
-        WITH_SSL="OFF"
-        ;;
     --with-zenfs )
         shift
         TARBALL_SUFFIX="-zenfs"
@@ -183,19 +177,10 @@ elif [ -n "$(command -v git)" -a -d "$SOURCEDIR/.git" ];
 then
     REVISION="$(git rev-parse --short HEAD)"
 fi
-if [ x"${FIPSMODE}" == x1 ]; then
-    PRODUCT_FULL="Percona-Server-Pro-$MYSQL_VERSION-$PERCONA_SERVER_VERSION"
-else
-    PRODUCT_FULL="Percona-Server-$MYSQL_VERSION-$PERCONA_SERVER_VERSION"
-fi
+PRODUCT_FULL="Percona-Server-$MYSQL_VERSION-$PERCONA_SERVER_VERSION"
 PRODUCT_FULL="$PRODUCT_FULL-$TAG$(uname -s)${DIST_NAME:-}.$TARGET${GLIBC_VER:-}${TARBALL_SUFFIX:-}"
-if [ x"${FIPSMODE}" == x1 ]; then
-    COMMENT="Percona Server Pro (GPL), Release ${MYSQL_VERSION_EXTRA#-}"
-    COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
-else
-    COMMENT="Percona Server (GPL), Release ${MYSQL_VERSION_EXTRA#-}"
-    COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
-fi
+COMMENT="Percona Server (GPL), Release ${MYSQL_VERSION_EXTRA#-}"
+COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}"
 
 # Compilation flags
 export CC=${CC:-gcc}
@@ -298,6 +283,7 @@ fi
         -DFORCE_INSOURCE_BUILD=1 \
         -DWITH_LIBEVENT=bundled \
         -DWITH_ZSTD=bundled \
+        -DWITH_PERCONA_TELEMETRY=ON \
         -DWITH_BOOST="$WORKDIR_ABS/libboost" \
         $WITH_MECAB_OPTION $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
 
@@ -326,12 +312,7 @@ fi
 )
 
 (
-if [ x"${FIPSMODE}" == x1 ]; then
-    LIBLIST="libgssapi.so libldap_r-2.4.so.2 libldap.so.2 liblber-2.4.so.2 liblber.so.2 libreadline.so libtinfo.so libbrotlidec.so libbrotlicommon.so librtmp.so libgssapi_krb5.so libkrb5.so libssl3.so libsmime3.so libnss3.so libnssutil3.so libplc4.so libnspr4.so libplds4.so libncurses.so.5 libtinfo.so.5 component_encryption_udf.so component_keyring_kms.so component_masking_functions.so"
-else
-    LIBLIST="libgssapi.so libldap_r-2.4.so.2 libldap.so.2 liblber-2.4.so.2 liblber.so.2 libcrypto.so libssl.so libreadline.so libtinfo.so libsasl2.so libbrotlidec.so libbrotlicommon.so librtmp.so libgssapi_krb5.so libkrb5.so libk5crypto.so libssl3.so libsmime3.so libnss3.so libnssutil3.so libplc4.so libnspr4.so libssl3.so libplds4.so libncurses.so.5 libtinfo.so.5 component_encryption_udf.so component_keyring_kms.so component_masking_functions.so"
-fi
-
+    LIBLIST="libaio.so.1 libnuma.so.1 libgssapi.so libldap_r-2.4.so.2 libldap.so.2 liblber-2.4.so.2 liblber.so.2 libreadline.so libtinfo.so libbrotlidec.so libbrotlicommon.so librtmp.so libgssapi_krb5.so libkrb5.so libk5crypto.so libsmime3.so libnss3.so libnssutil3.so libplc4.so libnspr4.so libplds4.so libncurses.so.5 libtinfo.so.5 component_encryption_udf.so component_uuid_vx_udf.so component_keyring_kms.so component_masking_functions.so"
     DIRLIST="bin lib lib/private lib/plugin lib/mysqlrouter/plugin lib/mysqlrouter/private"
 
     LIBPATH=""
