@@ -1,7 +1,7 @@
 #!/bin/sh
 
 shell_quote_string() {
-  echo "$1" | sed -e 's,\([^a-zA-Z0-9/_.=-]\),\\\1,g'
+    echo "$1" | sed -e 's,\([^a-zA-Z0-9/_.=-]\),\\\1,g'
 }
 
 usage () {
@@ -38,7 +38,7 @@ EOF
 }
 
 append_arg_to_args () {
-  args="$args "$(shell_quote_string "$1")
+    args="$args "$(shell_quote_string "$1")
 }
 
 parse_arguments() {
@@ -75,7 +75,6 @@ parse_arguments() {
             --rpm_release=*) RPM_RELEASE="$val" ;;
             --deb_release=*) DEB_RELEASE="$val" ;;
             --debug=*) DEBUG="$val" ;;
-            --enable_fipsmode=*) FIPSMODE="$val" ;;
             --help) usage ;;
             *)
               if test -n "$pick_args"
@@ -112,8 +111,8 @@ add_percona_yum_repo(){
 }
 
 switch_to_vault_repo() {
-    sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-Linux-*
-    sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-Linux-*
+    sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-*
+    sed -i 's|#\s*baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-*
     sed -i 's/enabled=0/enabled=1/g' /etc/yum.repos.d/CentOS-Linux-PowerTools.repo
 }
 
@@ -433,27 +432,30 @@ install_deps() {
     if [ "x$OS" = "xrpm" ]; then
         RHEL=$(rpm --eval %rhel)
         ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
-        if [ "x${RHEL}" = "x8" ]; then
+        if [ "x${RHEL}" = "x8" -o "x${RHEL}" = "x7" ]; then
             switch_to_vault_repo
         fi
         if [ x"$ARCH" = "xx86_64" ]; then
-            if [ "${RHEL}" -lt 9 ]; then
+            if [ "${RHEL}" -lt 8 ]; then
                 # add_percona_yum_repo
                 yum install -y https://repo.percona.com/yum/percona-release-latest.noarch.rpm
                 percona-release enable tools testing
-                percona-release enable tools experimental
+                # percona-release enable tools experimental
             else
                 yum -y install yum-utils
-                yum-config-manager --enable ol9_codeready_builder
-           fi
+                yum-config-manager --enable ol"${RHEL}"_codeready_builder
+            fi
         else
-            if [ "x${RHEL}" = "x9" ]; then
+            if [ "x${RHEL}" = "x9" -o "x${RHEL}" = "x8" ]; then
                 yum -y install yum-utils
                 yum-config-manager --enable ol"${RHEL}"_codeready_builder
             fi
         fi
         yum -y update
         yum -y install epel-release
+        if [ "x${RHEL}" = "x8" -o "x${RHEL}" = "x7" ]; then
+            switch_to_vault_repo
+        fi
         yum -y install git numactl-devel rpm-build gcc-c++ gperf ncurses-devel perl readline-devel openssl-devel jemalloc zstd
         yum -y install time zlib-devel libaio-devel bison cmake3 cmake pam-devel libeatmydata jemalloc-devel pkg-config
         yum -y install perl-Time-HiRes libcurl-devel openldap-devel unzip wget libcurl-devel patchelf systemd-devel
@@ -463,6 +465,7 @@ install_deps() {
                 echo "waiting"
                 sleep 1
             done
+            switch_to_vault_repo
             yum -y install gcc-c++ devtoolset-8-gcc-c++ devtoolset-8-binutils devtoolset-8-gcc devtoolset-8-gcc-c++
             yum -y install ccache devtoolset-8-libasan-devel devtoolset-8-libubsan-devel devtoolset-8-valgrind devtoolset-8-valgrind-devel
             yum -y install libasan libicu-devel libtool libzstd-devel lz4-devel make pkg-config
@@ -505,7 +508,9 @@ install_deps() {
         if [ "x$RHEL" = "x8" ]; then
             yum -y install libtirpc-devel
             yum -y install centos-release-stream
+            switch_to_vault_repo
             yum -y install gcc-toolset-12-gcc gcc-toolset-12-gcc-c++ gcc-toolset-12-binutils gcc-toolset-12-annobin-annocheck gcc-toolset-12-annobin-plugin-gcc
+            yum -y install gcc-toolset-12-libatomic-devel
             if [ x"$ARCH" = "xx86_64" ]; then
                 yum -y remove centos-release-stream
             fi
@@ -514,6 +519,7 @@ install_deps() {
             yum -y install libtirpc-devel
             yum -y install gflags-devel
             yum -y install gcc-toolset-12-gcc gcc-toolset-12-gcc-c++ gcc-toolset-12-binutils gcc-toolset-12-annobin-annocheck gcc-toolset-12-annobin-plugin-gcc
+            yum -y install gcc-toolset-12-libatomic-devel
             if [ x"$ARCH" = "xx86_64" ]; then
                 pushd /opt/rh/gcc-toolset-12/root/usr/lib/gcc/x86_64-redhat-linux/12/plugin/
                 ln -s annobin.so gcc-annobin.so
@@ -527,41 +533,37 @@ install_deps() {
             yum -y install MySQL-python
         fi
     else
-        apt-get -y install dirmngr || true
-        apt-get update
-	apt-get -y install lsb_release || true
-        apt-get -y install dirmngr || true
-        apt-get -y install lsb-release wget git curl
-        wget https://repo.percona.com/apt/percona-release_latest.$(lsb_release -sc)_all.deb && dpkg -i percona-release_latest.$(lsb_release -sc)_all.deb
-        percona-release enable tools testing
+        until apt-get update; do
+            echo "apt-get update failed. Retrying in 5 seconds..."
+            sleep 5
+        done
         export DEBIAN_FRONTEND="noninteractive"
+        apt-get -y install dirmngr || true
+        apt-get -y install lsb-release || true
+        apt-get -y install wget git curl
         export DIST="$(lsb_release -sc)"
-            until apt-get update; do
-            sleep 1
-            echo "waiting"
+        if [ x"${DIST}" != xnoble ];then
+            wget https://repo.percona.com/apt/percona-release_latest."${DIST}"_all.deb && dpkg -i percona-release_latest."${DIST}"_all.deb
+            percona-release enable tools testing
+        fi
+        until apt-get update; do
+            echo "apt-get update failed. Retrying in 5 seconds..."
+            sleep 5
         done
         apt-get -y purge eatmydata || true
-
-        apt-get update
         apt-get -y install psmisc pkg-config
         apt-get -y install libsasl2-modules:amd64 || apt-get -y install libsasl2-modules
         apt-get -y install dh-systemd || true
         apt-get -y install copyright-update
-        apt-get -y install curl bison cmake perl libssl-dev gcc g++ libaio-dev libldap2-dev libwrap0-dev gdb unzip gawk
-        apt-get -y install lsb-release libmecab-dev libncurses5-dev libreadline-dev libpam-dev zlib1g-dev libcurl4-openssl-dev
+        apt-get -y install curl bison cmake perl libssl-dev libaio-dev libldap2-dev libwrap0-dev gdb unzip gawk
+        apt-get -y install libmecab-dev libncurses5-dev libreadline-dev libpam-dev zlib1g-dev libcurl4-openssl-dev
         apt-get -y install libldap2-dev libnuma-dev libjemalloc-dev libc6-dbg valgrind libjson-perl libsasl2-dev patchelf
-        if [ x"${DIST}" = xfocal -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm ]; then
+        apt-get -y install libatomic1
+        if [ x"${DIST}" = xfocal -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm -o x"${DIST}" = xnoble ]; then
             apt-get -y install python3-mysqldb
         else
             apt-get -y install python-mysqldb
         fi
-        if [ x"${DIST}" = xbionic ]; then
-            apt-get -y install gcc-8 g++-8
-            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-7 700 --slave /usr/bin/g++ g++ /usr/bin/g++-7
-            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-8 800 --slave /usr/bin/g++ g++ /usr/bin/g++-8
-        else
-            apt-get -y install gcc g++
-        fi
         apt-get -y install libeatmydata
         apt-get -y install dh-apparmor
         apt-get -y install libmecab2 mecab mecab-ipadic
@@ -569,10 +571,20 @@ install_deps() {
         apt-get -y install build-essential devscripts doxygen doxygen-gui graphviz rsync
         apt-get -y install cmake autotools-dev autoconf automake build-essential devscripts debconf debhelper fakeroot libaio-dev
         apt-get -y install ccache libevent-dev libgsasl7 liblz4-dev libre2-dev libtool po-debconf
-        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbionic -o x"${DIST}" = xdisco -o x"${DIST}" = xbuster -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm ]; then
+        if [ x"${DIST}" = xbionic ]; then
+            apt-get -y install gcc-8 g++-8
+            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-7 700 --slave /usr/bin/g++ g++ /usr/bin/g++-7
+            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-8 800 --slave /usr/bin/g++ g++ /usr/bin/g++-8
+        elif [ x"${DIST}" = xnoble ]; then
+            apt-get -y install gcc-13 g++-13
+            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-13 100 --slave /usr/bin/g++ g++ /usr/bin/g++-13
+        else
+            apt-get -y install gcc g++
+        fi
+        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbionic -o x"${DIST}" = xdisco -o x"${DIST}" = xbuster -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm -o x"${DIST}" = xnoble ]; then
             apt-get -y install libeatmydata1
         fi
-        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbionic -o x"${DIST}" = xstretch -o x"${DIST}" = xdisco -o x"${DIST}" = xbuster -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm ]; then
+        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbionic -o x"${DIST}" = xstretch -o x"${DIST}" = xdisco -o x"${DIST}" = xbuster -o x"${DIST}" = xhirsute -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm -o x"${DIST}" = xnoble ]; then
             apt-get -y install libzstd-dev
         else
             apt-get -y install libzstd1-dev
@@ -580,9 +592,12 @@ install_deps() {
         if [ x${DIST} = xhirsute ]; then
             apt-get -y install libzbd-dev clang-12 pkg-config make libgflags-dev nvme-cli util-linux fio zbd-utils
         fi
-	if [[ ${DIST} == 'focal' ]] || [[ ${DIST} == 'hirsute' ]] || [[ ${DIST} == 'bullseye' ]] || [[ ${DIST} == 'jammy' ]] || [[ ${DIST} == 'bookworm' ]]; then
+        if [[ ${DIST} == 'focal' ]] || [[ ${DIST} == 'hirsute' ]] || [[ ${DIST} == 'bullseye' ]] || [[ ${DIST} == 'jammy' ]] || [[ ${DIST} == 'bookworm' ]] || [[ ${DIST} == 'noble' ]]; then
             apt-get -y install libgflags-dev
-	fi
+        fi
+        if [ x${DIST} = xnoble ]; then
+            apt-get -y install libtirpc-dev gsasl-common
+        fi
         apt-get install -y libsasl2-dev libsasl2-modules-gssapi-mit libkrb5-dev
     fi
     if [ ! -d /usr/local/percona-subunit2junitxml ]; then
@@ -667,37 +682,10 @@ build_srpm(){
     sed -i "/^%changelog/a - Release ${VERSION}-${RELEASE}" percona-server.spec
     sed -i "/^%changelog/a * $(date "+%a") $(date "+%b") $(date "+%d") $(date "+%Y") Percona Development Team <info@percona.com> - ${VERSION}-${RELEASE}" percona-server.spec
     #
-    if [[ "x${FIPSMODE}" == "x1" ]]; then
-        sed -i -e "s:percona-server-server$:percona-server-server-pro:g" \
-        -e "s:percona-server-server =:percona-server-server-pro =:g" \
-        -e "s:percona-server-client$:percona-server-client-pro:g" \
-        -e "s:percona-server-client =:percona-server-client-pro =:g" \
-        -e "s:percona-server-devel$:percona-server-devel-pro:g" \
-        -e "s:percona-mysql-router$:percona-mysql-router-pro:g" \
-        -e "s:percona-mysql-router :percona-mysql-router-pro :g" \
-        -e "s:percona-mysql-router-devel$:percona-mysql-router-devel-pro:g" \
-        -e "s:percona-mysql-router-devel =:percona-mysql-router-devel-pro =:g" \
-        -e "s:percona-server-rocksdb$:percona-server-rocksdb-pro:g" \
-        -e "s:percona-server-test$:percona-server-test-pro:g" \
-        -e "s:percona-server-shared$:percona-server-shared-pro:g" \
-        -e "s:percona-server-shared :percona-server-shared-pro :g" \
-        -e "s:Conflicts\:      percona-server-server-pro:Conflicts\:      percona-server-server:g" \
-        -e "s:Conflicts\:      percona-server-client-pro:Conflicts\:      percona-server-client:g" \
-        -e "s:Conflicts\:      percona-server-test-pro:Conflicts\:      percona-server-test:g" \
-        -e "s:Conflicts\:      percona-server-devel-pro:Conflicts\:      percona-server-devel:g" \
-        -e "s:Conflicts\:      percona-server-rocksdb-pro:Conflicts\:      percona-server-rocksdb:g" \
-        -e "s:Conflicts\:     percona-mysql-router-pro:Conflicts\:     percona-mysql-router:g" \
-        -e "s:Conflicts\:      percona-mysql-router-devel-pro:Conflicts\:      percona-mysql-router-devel:g" \
-        -e "s:Conflicts\:      percona-server-test-pro:Conflicts\:      percona-server-test:g" \
-        -e "s:Conflicts\:      percona-server-shared-pro:Conflicts\:      percona-server-shared:g" \
-        -e "s:Name\:           percona-server:Name\:           percona-server-pro:g" \
-        percona-server.spec
-    fi
-    #
     cd ${WORKDIR}/rpmbuild/SOURCES
     wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
-    wget https://boostorg.jfrog.io/artifactory/main/release/1.77.0/source/boost_1_77_0.tar.gz
-    #wget http://jenkins.percona.com/downloads/boost/${BOOST_PACKAGE_NAME}.tar.gz
+    #wget https://boostorg.jfrog.io/artifactory/main/release/1.77.0/source/boost_1_77_0.tar.gz
+    wget --no-check-certificate https://downloads.percona.com/downloads/packaging/boost/${BOOST_PACKAGE_NAME}.tar.gz
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/*.patch' --strip=3
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/filter-provides.sh' --strip=3
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/filter-requires.sh' --strip=3
@@ -732,7 +720,7 @@ build_mecab_lib(){
     ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
     MECAB_TARBAL="mecab-0.996.tar.gz"
     #MECAB_LINK="http://jenkins.percona.com/downloads/mecab/${MECAB_TARBAL}"
-    MECAB_LINK="https://downloads.percona.com/downloads/TESTING/issue-CUSTO83/${MECAB_TARBAL}"
+    MECAB_LINK="https://downloads.percona.com/downloads/packaging/${MECAB_TARBAL}"
     MECAB_DIR="${WORKDIR}/${MECAB_TARBAL%.tar.gz}"
     MECAB_INSTALL_DIR="${WORKDIR}/mecab-install"
     rm -f ${MECAB_TARBAL}
@@ -764,7 +752,7 @@ build_mecab_dict(){
     ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
     MECAB_IPADIC_TARBAL="mecab-ipadic-2.7.0-20070801.tar.gz"
     #MECAB_IPADIC_LINK="http://jenkins.percona.com/downloads/mecab/${MECAB_IPADIC_TARBAL}"
-    MECAB_IPADIC_LINK="https://downloads.percona.com/downloads/TESTING/issue-CUSTO83/${MECAB_IPADIC_TARBAL}"
+    MECAB_IPADIC_LINK="https://downloads.percona.com/downloads/packaging/${MECAB_IPADIC_TARBAL}"
     MECAB_IPADIC_DIR="${WORKDIR}/${MECAB_IPADIC_TARBAL%.tar.gz}"
     rm -f ${MECAB_IPADIC_TARBAL}
     rm -rf ${MECAB_IPADIC_DIR}
@@ -850,31 +838,15 @@ build_rpm(){
     fi
     if [ ${ARCH} = x86_64 ]; then
         if [[ ${WITH_ZENFS} == "1" ]]; then
-            if [[ "x${FIPSMODE}" == "x1" ]]; then
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --define "enable_fipsmode 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            else
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            fi
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
         else
-            if [[ "x${FIPSMODE}" == "x1" ]]; then
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "enable_fipsmode 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            else
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            fi
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
         fi
     else
         if [[ ${WITH_ZENFS} == "1" ]]; then
-            if [[ "x${FIPSMODE}" == "x1" ]]; then
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --define "enable_fipsmode 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            else
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            fi
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "with_zenfs 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
         else
-            if [[ "x${FIPSMODE}" == "x1" ]]; then
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --define "enable_fipsmode 1" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            else
-                rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
-            fi
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
         fi
     fi
 
@@ -918,88 +890,13 @@ build_source_deb(){
     TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
     RELEASE=${TMPREL%.tar.gz}
 
-    if [[ "x${FIPSMODE}" == "x1" ]]; then
-        NEWTAR=percona-server-pro_${VERSION}-${RELEASE}.orig.tar.gz
-    else
-        NEWTAR=percona-server_${VERSION}-${RELEASE}.orig.tar.gz
-    fi
+    NEWTAR=percona-server_${VERSION}-${RELEASE}.orig.tar.gz
     mv ${TARFILE} ${NEWTAR}
 
     tar xzf ${NEWTAR}
     ls -la
     cd percona-server-${VERSION}-${RELEASE}
     cp -ap build-ps/debian/ .
-    if [ x"${FIPSMODE}" == x1 ]; then
-        sed -i "s:FIPSMODE=0:FIPSMODE=1:g" debian/rules
-        sed -i "s:percona-server-server:percona-server-server-pro:g" debian/rules
-        sed -i "s:percona-server-dbg:percona-server-pro-dbg:g" debian/rules
-        sed -i "s:percona-server:percona-server-pro:g" debian/changelog
-        sed -i "s:Source\: percona-server:Source\: percona-server-pro:g" debian/control
-        sed -i "s:percona-server-rocksdb-pro:percona-server-rocksdb:g" debian/control
-        sed -i "s:Package\: percona-server-rocksdb:Package\: percona-server-rocksdb-pro:g" debian/control
-        sed -i "s:Depends\: percona-server-common:Depends\: percona-server-pro-common:g" debian/control
-        sed -i "s:Package\: percona-server-common:Package\: percona-server-pro-common:g" debian/control
-        sed -i "s:, percona-server-pro-common:, percona-server-common:g" debian/control
-        sed -i "s:Conflicts\: percona-server-pro-common:Conflicts\: percona-server-common:g" debian/control
-        sed -i "s:percona-server-client-pro:percona-server-client:g" debian/control
-        sed -i "s:Package\: percona-server-client:Package\: percona-server-client-pro:g" debian/control
-        sed -i "s:percona-server-client (=:percona-server-client-pro (=:g" debian/control
-        sed -i "s:percona-server-server-pro:percona-server-server:g" debian/control
-        sed -i "s:Package\: percona-server-server:Package\: percona-server-server-pro:g" debian/control
-        sed -i "s:percona-server-server (=:percona-server-server-pro (=:g" debian/control
-        sed -i "s:percona-server-test-pro:percona-server-test:g" debian/control
-        sed -i "s:Package\: percona-server-test:Package\: percona-server-test-pro:g" debian/control
-        sed -i "s:Replaces\: percona-server-server, percona-server-client:Replaces\: percona-server-server-pro, percona-server-client-pro:g" debian/control
-        sed -i "s:Package\: percona-server-source:Package\: percona-server-pro-source:g" debian/control
-        sed -i "s:percona-server-pro-dbg:percona-server-dbg:g" debian/control
-        sed -i "s:Package\: percona-server-dbg:Package\: percona-server-pro-dbg:g" debian/control
-        sed -i "s:percona-mysql-router-pro:percona-mysql-router:g" debian/control
-        sed -i "s:Package\: percona-mysql-router:Package\: percona-mysql-router-pro:g" debian/control
-        sed -i "s:libperconaserverproclient:libperconaserverclient:g" debian/control
-        sed -i "s:Package\: libperconaserverclient:Package\: libperconaserverproclient:g" debian/control
-        sed -i "s/\(libpercona\)server\(client.. (=\)/\1serverpro\2/g" debian/control
-        sed -i "s/\(libpercona\)server\(client..-dev (=\)/\1serverpro\2/g" debian/control
-        cp debian/percona-mysql-router.init debian/percona-mysql-router-pro.init
-        cp debian/percona-mysql-router.install debian/percona-mysql-router-pro.install
-        cp debian/percona-mysql-router.mysqlrouter.init debian/percona-mysql-router-pro.mysqlrouter.init
-        cp debian/percona-mysql-router.mysqlrouter.service debian/percona-mysql-router-pro.mysqlrouter.service
-        cp debian/percona-mysql-router.mysqlrouter.tmpfile debian/percona-mysql-router-pro.mysqlrouter.tmpfile
-        cp debian/percona-mysql-router.postinst debian/percona-mysql-router-pro.postinst
-        cp debian/percona-mysql-router.postrm debian/percona-mysql-router-pro.postrm
-        cp debian/percona-mysql-router.preinst debian/percona-mysql-router-pro.preinst
-        cp debian/percona-mysql-router.prerm debian/percona-mysql-router-pro.prerm
-        cp debian/percona-server-client.install debian/percona-server-client-pro.install
-        cp debian/percona-server-client.links debian/percona-server-client-pro.links
-        cp debian/percona-server-client.lintian-overrides debian/percona-server-client-pro.lintian-overrides
-        cp debian/percona-server-common.install debian/percona-server-pro-common.install
-        cp debian/percona-server-common.postinst debian/percona-server-pro-common.postinst
-        cp debian/percona-server-common.postrm debian/percona-server-pro-common.postrm
-        cp debian/percona-server-rocksdb.install debian/percona-server-rocksdb-pro.install
-        cp debian/percona-server-rocksdb.postinst debian/percona-server-rocksdb-pro.postinst
-        cp debian/percona-server-server.config debian/percona-server-server-pro.config
-        cp debian/percona-server-server.dirs debian/percona-server-server-pro.dirs
-        cp debian/percona-server-server.docs debian/percona-server-server-pro.docs
-        cp debian/percona-server-server.install debian/percona-server-server-pro.install
-        cp debian/percona-server-server.lintian-overrides debian/percona-server-server-pro.lintian-overrides
-        cp debian/percona-server-server.mysql.init debian/percona-server-server-pro.mysql.init
-        cp debian/percona-server-server.mysql.service debian/percona-server-server-pro.mysql.service
-        cp debian/percona-server-server.mysql.tmpfile debian/percona-server-server-pro.mysql.tmpfile
-        cp debian/percona-server-server.mysql@.service debian/percona-server-server-pro.mysql@.service
-        cp debian/percona-server-server.postinst debian/percona-server-server-pro.postinst
-        cp debian/percona-server-server.postrm debian/percona-server-server-pro.postrm
-        cp debian/percona-server-server.preinst debian/percona-server-server-pro.preinst
-        cp debian/percona-server-server.prerm debian/percona-server-server-pro.prerm
-        cp debian/percona-server-server.templates debian/percona-server-server-pro.templates
-        cp debian/percona-server-source.install debian/percona-server-pro-source.install
-        cp debian/percona-server-test.dirs debian/percona-server-test-pro.dirs
-        cp debian/percona-server-test.install debian/percona-server-test-pro.install
-        cp debian/percona-server-test.links debian/percona-server-test-pro.links
-        cp debian/libperconaserverclient21-dev.dirs debian/libperconaserverproclient21-dev.dirs
-        cp debian/libperconaserverclient21-dev.install debian/libperconaserverproclient21-dev.install
-        cp debian/libperconaserverclient21.install debian/libperconaserverproclient21.install
-        cp debian/libperconaserverclient21.lintian-overrides debian/libperconaserverproclient21.lintian-overrides
-        sed -i "s:percona-server_:percona-server-pro_:g" debian/percona-server-pro-source.install
-    fi
     dch -D unstable --force-distribution -v "${VERSION}-${RELEASE}-${DEB_RELEASE}" "Update to new upstream release Percona Server ${VERSION}-${RELEASE}-1"
     copyright-update -d debian/copyright
     dpkg-buildpackage -S
@@ -1042,10 +939,6 @@ build_deb(){
     DIRNAME=$(echo ${DSC%-${DEB_RELEASE}.dsc} | sed -e 's:_:-:g')
     VERSION=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $3}')
     RELEASE=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $4}')
-    if [[ "x${FIPSMODE}" == "x1" ]]; then
-        VERSION=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $4}')
-        RELEASE=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $5}')
-    fi
     ARCH=$(uname -m)
     export EXTRAVER=${MYSQL_VERSION_EXTRA#-}
     #
@@ -1065,9 +958,6 @@ build_deb(){
     dch -b -m -D "$DEBIAN_VERSION" --force-distribution -v "${VERSION}-${RELEASE}-${DEB_RELEASE}.${DEBIAN_VERSION}" 'Update distribution'
 
     postfix=""
-    if [ x"${FIPSMODE}" == x1 ]; then
-        postfix="-pro"
-    fi
     cd debian/
     wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
     sed -i 's:exit 0::' percona-server-server"${postfix}".postinst
@@ -1075,12 +965,14 @@ build_deb(){
     cat call-home.sh >> percona-server-server"${postfix}".postinst
     echo "CALLHOME" >> percona-server-server"${postfix}".postinst
     echo "bash +x /tmp/call-home.sh -f \"PRODUCT_FAMILY_PS\" -v \"${VERSION}-${RELEASE}-${DEB_RELEASE}\" -d \"PACKAGE\" &>/dev/null || :" >> percona-server-server"${postfix}".postinst
+    echo "chgrp percona-telemetry /usr/local/percona/telemetry_uuid &>/dev/null || :" >> percona-server-server"${postfix}".postinst
+    echo "chmod 664 /usr/local/percona/telemetry_uuid &>/dev/null || :" >> percona-server-server"${postfix}".postinst
     echo "rm -rf /tmp/call-home.sh" >> percona-server-server"${postfix}".postinst
     echo "exit 0" >> percona-server-server"${postfix}".postinst
     rm -f call-home.sh
     cd ../
 
-    if [ ${DEBIAN_VERSION} != trusty -a ${DEBIAN_VERSION} != xenial -a ${DEBIAN_VERSION} != jessie -a ${DEBIAN_VERSION} != stretch -a ${DEBIAN_VERSION} != artful -a ${DEBIAN_VERSION} != bionic -a ${DEBIAN_VERSION} != focal -a "${DEBIAN_VERSION}" != disco -a "${DEBIAN_VERSION}" != buster -a "${DEBIAN_VERSION}" != hirsute -a "${DEBIAN_VERSION}" != bullseye -a "${DEBIAN_VERSION}" != jammy -a "${DEBIAN_VERSION}" != bookworm ]; then
+    if [ ${DEBIAN_VERSION} != trusty -a ${DEBIAN_VERSION} != xenial -a ${DEBIAN_VERSION} != jessie -a ${DEBIAN_VERSION} != stretch -a ${DEBIAN_VERSION} != artful -a ${DEBIAN_VERSION} != bionic -a ${DEBIAN_VERSION} != focal -a "${DEBIAN_VERSION}" != disco -a "${DEBIAN_VERSION}" != buster -a "${DEBIAN_VERSION}" != hirsute -a "${DEBIAN_VERSION}" != bullseye -a "${DEBIAN_VERSION}" != jammy -a "${DEBIAN_VERSION}" != bookworm -a "${DEBIAN_VERSION}" != noble ]; then
         gcc47=$(which gcc-4.7 2>/dev/null || true)
         if [ -x "${gcc47}" ]; then
             export CC=gcc-4.7
@@ -1098,14 +990,18 @@ build_deb(){
         sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=date-time /' debian/rules
     fi
 
-    if [ ${DEBIAN_VERSION} = "stretch" -o ${DEBIAN_VERSION} = "bionic" -o ${DEBIAN_VERSION} = "focal" -o ${DEBIAN_VERSION} = "buster" -o ${DEBIAN_VERSION} = "disco"  -o ${DEBIAN_VERSION} = "bullseye" -o ${DEBIAN_VERSION} = "bookworm" ]; then
-        sed -i 's/export CFLAGS=/export CFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules
-        sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules
-    fi
     if [ ${DEBIAN_VERSION} = "cosmic" ]; then
         sed -i 's/export CFLAGS=/export CFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time -Wno-error=ignored-qualifiers -Wno-error=class-memaccess -Wno-error=shadow /' debian/rules
         sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time -Wno-error=ignored-qualifiers -Wno-error=class-memaccess -Wno-error=shadow /' debian/rules
     fi
+
+    if [ ${DEBIAN_VERSION} = "noble" -a ${ARCH} = "aarch64" ]; then
+        sed -i 's:dh_strip --dbg-package=percona-server-dbg:mv debian/percona-server-server/usr/lib/mysql/plugin/authentication_fido.so /tmp\n\tdh_strip --dbg-package=percona-server-dbg\n\tmv /tmp/authentication_fido.so debian/percona-server-server/usr/lib/mysql/plugin/authentication_fido.so:' debian/rules
+        sed -i 's:dh_strip -Xlibprotobuf-lite:dh_strip -Xlibprotobuf-lite --exclude=debian/percona-server-server/usr/lib/mysql/plugin/authentication_fido.so:' debian/rules
+    fi
+
+    cat debian/rules
+
     dpkg-buildpackage -rfakeroot -uc -us -b
 
     cd ${WORKDIR}
@@ -1181,24 +1077,20 @@ build_tarball(){
         cp -av /usr/include/openssl ${WORKDIR}/ssl/include/
     fi
 
-    if [[ "x${FIPSMODE}" == "x1" ]]; then
-        BUILD_PARAMETER="--enable-fipsmode "
-    fi
-
     cd ${TARFILE%.tar.gz}
     if [ "x$WITH_SSL" = "x1" ]; then
         CMAKE_OPTS="-DMINIMAL_RELWITHDEBINFO=OFF -DWITH_ROCKSDB=1 -DINSTALL_LAYOUT=STANDALONE -DWITH_SSL=$PWD/../ssl/ " bash -xe ./build-ps/build-binary.sh --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
         DIRNAME="yassl"
     else
         if [[ "${DEBUG}" == 1 ]]; then
-            CMAKE_OPTS="-DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --debug ${BUILD_PARAMETER}--with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
+            CMAKE_OPTS="-DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --debug --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
             DIRNAME="tarball"
         elif [[ ${WITH_ZENFS} == 1 ]]; then
             enable_zenfs tarball
-            CMAKE_OPTS="-DMINIMAL_RELWITHDEBINFO=OFF -DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh ${BUILD_PARAMETER}--with-zenfs --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
+            CMAKE_OPTS="-DMINIMAL_RELWITHDEBINFO=OFF -DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --with-zenfs --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
             DIRNAME="tarball"
         else
-            CMAKE_OPTS="-DMINIMAL_RELWITHDEBINFO=OFF -DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh ${BUILD_PARAMETER}--with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
+            CMAKE_OPTS="-DMINIMAL_RELWITHDEBINFO=OFF -DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
             DIRNAME="tarball"
         fi
     fi
