@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,13 +26,13 @@
 #ifndef OBJECT_FILTER_INCLUDED
 #define OBJECT_FILTER_INCLUDED
 
+#include <optional>
 #include <string>
 #include <vector>
 
 #include "client/base/abstract_program.h"
 #include "client/base/composite_options_provider.h"
 #include "client/dump/abstract_data_object.h"
-#include "nullable.h"
 
 namespace Mysql {
 namespace Tools {
@@ -40,7 +41,7 @@ namespace Dump {
 class Object_filter
     : public Mysql::Tools::Base::Options::Composite_options_provider {
  public:
-  Object_filter(Mysql::Tools::Base::Abstract_program *program);
+  explicit Object_filter(Mysql::Tools::Base::Abstract_program *program);
 
   void create_options() override;
 
@@ -91,7 +92,7 @@ class Object_filter
   bool m_dump_triggers;
   bool m_dump_events;
   bool m_dump_users;
-  Mysql::Nullable<std::string> m_include_tmp_string;
+  std::optional<std::string> m_include_tmp_string;
   Mysql::Tools::Base::Abstract_program *m_program;
 };
 
