@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2022, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,25 +28,24 @@
 
 #include <string>
 
-namespace Mysql{
-namespace Tools{
-namespace Dump{
-namespace Detail{
+namespace Mysql {
+namespace Tools {
+namespace Dump {
+namespace Detail {
 
-class Pattern_matcher
-{
-public:
-  static bool is_pattern_matched(
-    const std::string& to_match, const std::string& pattern, size_t i= 0,
-    size_t j= 0);
+class Pattern_matcher {
+ public:
+  static bool is_pattern_matched(const std::string &to_match,
+                                 const std::string &pattern, size_t i = 0,
+                                 size_t j = 0);
 
-private:
+ private:
   Pattern_matcher();
 };
 
-}
-}
-}
-}
+}  // namespace Detail
+}  // namespace Dump
+}  // namespace Tools
+}  // namespace Mysql
 
 #endif
