@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -246,7 +247,7 @@ enum ha_extra_function {
     Only used by MyISAM, called from MyISAM handler but never from server
     code on top of the handler.
     Sets lock_wait on/off:
-      Wait until file is avalable (def).
+      Wait until file is available (def).
       If file is locked, return quickly.
   */
   HA_EXTRA_WAIT_LOCK = 12,
@@ -292,9 +293,9 @@ enum ha_extra_function {
        table is not an old table when calling close_thread_table.
        close_thread_tables is called from many places as a general clean up
        function after completing a query.
-    3) It is called when deleting the QUICK_RANGE_SELECT object if the
-       QUICK_RANGE_SELECT object had its own handler object. It is called
-       immediatley before close of this local handler object.
+    3) It is called when deleting the IndexRangeScanIterator object if the
+       IndexRangeScanIterator object had its own handler object. It is called
+       immediately before close of this local handler object.
 
     If there is a READ CACHE it is reinit'ed. A cache is reinit'ed to
     restart reading or to change type of cache between READ CACHE and
@@ -313,7 +314,7 @@ enum ha_extra_function {
   HA_EXTRA_RESET_STATE,
   /*
     Informs the handler that we will not stop the transaction if we get
-    duplicate key errors during insert/upate.
+    duplicate key errors during insert/update.
     Always called in pair, triggered by INSERT IGNORE and other similar
     SQL constructs.
     Not used by MyISAM.
@@ -585,8 +586,8 @@ constexpr const ulong HA_INDEX_USES_SECONDARY_ENGINE_ATTRIBUTE{1UL << 21};
         are basically the same bitmap at the time of table creation and
         at the time of table opening/usage correspondingly.
   @note TABLE_SHARE::db_options_in_use is normally copy of db_create_options
-        but can be overriden by SE. E.g. MyISAM does this at handler::open()
-        and hander::info() time.
+        but can be overridden by SE. E.g. MyISAM does this at handler::open()
+        and handler::info() time.
 
   Also used for similar bitmaps in MyISAM (MYISAM_SHARE::options,
   MI_ISAMINFO::options).
@@ -820,7 +821,7 @@ is the global server default. */
 #define HA_ERR_FIRST 120
 /** Didn't find key on read or update */
 #define HA_ERR_KEY_NOT_FOUND 120
-/** Dupplicate key on write */
+/** Duplicate key on write */
 #define HA_ERR_FOUND_DUPP_KEY 121
 /** Internal error */
 #define HA_ERR_INTERNAL_ERROR 122
@@ -828,6 +829,8 @@ is the global server default. */
 #define HA_ERR_RECORD_CHANGED 123
 /** Wrong index given to function */
 #define HA_ERR_WRONG_INDEX 124
+/** Transaction has been rolled back */
+#define HA_ERR_ROLLED_BACK 125
 /** Indexfile is crashed */
 #define HA_ERR_CRASHED 126
 /** Record-file is crashed */
@@ -856,7 +859,7 @@ is the global server default. */
 #define HA_ERR_TOO_BIG_ROW 139
 /** Wrong create option */
 #define HA_WRONG_CREATE_OPTION 140
-/** Dupplicate unique on write */
+/** Duplicate unique on write */
 #define HA_ERR_FOUND_DUPP_UNIQUE 141
 /** Can't open charset */
 #define HA_ERR_UNKNOWN_CHARSET 142
@@ -918,7 +921,7 @@ in some other table. */
 #define HA_ERR_CORRUPT_EVENT 171
 /** New file format */
 #define HA_ERR_NEW_FILE 172
-/** The event could not be processed no other hanlder error happened */
+/** The event could not be processed no other handler error happened */
 #define HA_ERR_ROWS_EVENT_APPLY 173
 /** Error during initialization */
 #define HA_ERR_INITIALIZATION 174
@@ -1001,8 +1004,6 @@ Information in the data-dictionary needs to be updated. */
 /* Number of different errors */
 #define HA_ERR_ERRORS (HA_ERR_LAST - HA_ERR_FIRST + 1)
 
-#define HA_ERR_ENCRYPTION_KEY_MISSING 501
-
 /* Other constants */
 
 #define HA_NAMELEN 64          /* Max length of saved filename */
@@ -1051,7 +1052,7 @@ typedef ulong key_part_map;
 #define HA_STATE_KEY_CHANGED 128
 #define HA_STATE_WRITE_AT_END 256 /* set in _ps_find_writepos */
 #define HA_STATE_BUFF_SAVED 512   /* If current keybuff is info->buff */
-#define HA_STATE_ROW_CHANGED 1024 /* To invalide ROW cache */
+#define HA_STATE_ROW_CHANGED 1024 /* To invalidate ROW cache */
 #define HA_STATE_EXTEND_BLOCK 2048
 
 /* myisampack expects no more than 32 field types. */
@@ -1109,6 +1110,7 @@ enum key_range_flags {
   */
   GEOM_FLAG = 1 << 7,
   /* Deprecated, currently used only by NDB at row retrieval */
+  // Update: Seems to be unused, even by NDB
   SKIP_RANGE = 1 << 8,
   /*
     Used to indicate that index dives can be skipped. This can happen when:
