@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,6 +46,7 @@
   for functions.
 */
 #if defined(_MSC_VER)
+
 #if defined(MYSQL_DYNAMIC_PLUGIN)
 #ifdef __cplusplus
 #define MYSQL_PLUGIN_EXPORT extern "C" __declspec(dllexport)
@@ -58,10 +60,17 @@
 #define MYSQL_PLUGIN_EXPORT
 #endif
 #endif /*MYSQL_DYNAMIC_PLUGIN */
-#else  /*_MSC_VER */
+
+#else /*_MSC_VER */
+
+#if defined(MYSQL_DYNAMIC_PLUGIN)
+#define MYSQL_PLUGIN_EXPORT MY_ATTRIBUTE((visibility("default")))
+#else
 #define MYSQL_PLUGIN_EXPORT
 #endif
 
+#endif /*_MSC_VER */
+
 #ifdef __cplusplus
 class THD;
 class Item;
@@ -102,7 +111,7 @@ struct MYSQL_XID {
   Plugin API. Common for all plugin types.
 */
 
-#define MYSQL_PLUGIN_INTERFACE_VERSION 0x0109
+#define MYSQL_PLUGIN_INTERFACE_VERSION 0x010B
 
 /*
   The allowable types of plugins
@@ -130,6 +139,8 @@ struct MYSQL_XID {
 #define PLUGIN_LICENSE_GPL_STRING "GPL"
 #define PLUGIN_LICENSE_BSD_STRING "BSD"
 
+#define PLUGIN_AUTHOR_ORACLE "Oracle Corporation"
+
 /*
   Macros for beginning and ending plugin declarations.  Between
   mysql_declare_plugin and mysql_declare_plugin_end there should
@@ -165,48 +176,28 @@ struct MYSQL_XID {
 
 #define PLUGIN_OPT_NO_INSTALL 1UL   /* Not dynamically loadable */
 #define PLUGIN_OPT_NO_UNINSTALL 2UL /* Not dynamically unloadable */
+#define PLUGIN_OPT_ALLOW_EARLY 4UL  /* allow --early-plugin-load */
+#define PLUGIN_OPT_DEFAULT_OFF 8UL  /* Turned off by default */
+/*
+  All "extra" plugins declared together, same mysql_declare_plugin statement,
+  depends on the first "main" plugin.
+
+  This option is used to turn off the extra plugins if the main plugin is off,
+  even if extra option by default should be on or user specifies that some
+  extra plugin should be on.
+
+  Use it when it does not make sense to have the extra plugins on when the main
+  plugin is off.
+ */
+#define PLUGIN_OPT_DEPENDENT_EXTRA_PLUGINS 16UL
 
 /*
   declarations for server variables and command line options
 */
 
-#define PLUGIN_VAR_BOOL 0x0001
-#define PLUGIN_VAR_INT 0x0002
-#define PLUGIN_VAR_LONG 0x0003
-#define PLUGIN_VAR_LONGLONG 0x0004
-#define PLUGIN_VAR_STR 0x0005
-#define PLUGIN_VAR_ENUM 0x0006
-#define PLUGIN_VAR_SET 0x0007
-#define PLUGIN_VAR_DOUBLE 0x0008
-#define PLUGIN_VAR_UNSIGNED 0x0080
-#define PLUGIN_VAR_THDLOCAL 0x0100  /* Variable is per-connection */
-#define PLUGIN_VAR_READONLY 0x0200  /* Server variable is read only */
-#define PLUGIN_VAR_NOSYSVAR 0x0400  /* Configurable only by cmd-line */
-#define PLUGIN_VAR_NOCMDOPT 0x0800  /* Not a command line option */
-#define PLUGIN_VAR_NOCMDARG 0x1000  /* No argument for cmd line */
-#define PLUGIN_VAR_RQCMDARG 0x0000  /* Argument required for cmd line */
-#define PLUGIN_VAR_OPCMDARG 0x2000  /* Argument optional for cmd line */
-#define PLUGIN_VAR_NODEFAULT 0x4000 /* SET DEFAULT is prohibited */
-#define PLUGIN_VAR_MEMALLOC 0x8000  /* String needs memory allocated */
-#define PLUGIN_VAR_NOPERSIST                \
-  0x10000 /* SET PERSIST_ONLY is prohibited \
-             for read only variables */
-
-/**
-  There can be some variables which needs to be set before plugin is loaded but
-  not after plugin is loaded. ex: GR specific variables. Below flag must be set
-  for these kind of variables.
-*/
-#define PLUGIN_VAR_PERSIST_AS_READ_ONLY 0x20000
-#define PLUGIN_VAR_INVISIBLE 0x40000 /* Variable should not be shown */
-/**
-  This flag enables variables to be recognized by SET_VAR() HINT. Should
-  be used only THDVAR() variables, ie variables which have session scope.
- */
-#define PLUGIN_VAR_HINTUPDATEABLE 0x80000
+#include <mysql/components/services/bits/system_variables_bits.h>
 
 struct SYS_VAR;
-struct st_mysql_value;
 
 /*
   SYNOPSIS
@@ -254,7 +245,7 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd, SYS_VAR *var,
    PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_RQCMDARG |   \
    PLUGIN_VAR_MEMALLOC | PLUGIN_VAR_NODEFAULT | PLUGIN_VAR_NOPERSIST | \
    PLUGIN_VAR_PERSIST_AS_READ_ONLY | PLUGIN_VAR_INVISIBLE |            \
-   PLUGIN_VAR_HINTUPDATEABLE)
+   PLUGIN_VAR_SENSITIVE | PLUGIN_VAR_HINTUPDATEABLE)
 
 #define MYSQL_PLUGIN_VAR_HEADER \
   int flags;                    \
@@ -436,6 +427,20 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd, SYS_VAR *var,
       max,                                                                 \
       blk}
 
+#define MYSQL_SYSVAR_UINT64_T(name, varname, opt, comment, check, update, def, \
+                              min, max, blk)                                   \
+  DECLARE_MYSQL_SYSVAR_SIMPLE(name, uint64_t) = {                              \
+      PLUGIN_VAR_LONGLONG | PLUGIN_VAR_UNSIGNED | ((opt)&PLUGIN_VAR_MASK),     \
+      #name,                                                                   \
+      comment,                                                                 \
+      check,                                                                   \
+      update,                                                                  \
+      &varname,                                                                \
+      def,                                                                     \
+      min,                                                                     \
+      max,                                                                     \
+      blk}
+
 #define MYSQL_SYSVAR_ENUM(name, varname, opt, comment, check, update, def, \
                           typelib)                                         \
   DECLARE_MYSQL_SYSVAR_TYPELIB(name, unsigned long) = {                    \
@@ -739,28 +744,6 @@ struct Mysql_replication {
   int interface_version;
 };
 
-/*************************************************************************
-  st_mysql_value struct for reading values from mysqld.
-  Used by server variables framework to parse user-provided values.
-  Will be used for arguments when implementing UDFs.
-
-  Note that val_str() returns a string in temporary memory
-  that will be freed at the end of statement. Copy the string
-  if you need it to persist.
-*/
-
-#define MYSQL_VALUE_TYPE_STRING 0
-#define MYSQL_VALUE_TYPE_REAL 1
-#define MYSQL_VALUE_TYPE_INT 2
-
-struct st_mysql_value {
-  int (*value_type)(struct st_mysql_value *);
-  const char *(*val_str)(struct st_mysql_value *, char *buffer, int *length);
-  int (*val_real)(struct st_mysql_value *, double *realbuf);
-  int (*val_int)(struct st_mysql_value *, long long *intbuf);
-  int (*is_unsigned)(struct st_mysql_value *);
-};
-
 /*************************************************************************
   Miscellaneous functions for plugin implementors
 */
@@ -826,8 +809,7 @@ enum mysql_trx_stat_type {
   @param[in]    value   the value of statistics
 */
 void thd_report_innodb_stat(MYSQL_THD thd, unsigned long long trx_id,
-                            enum mysql_trx_stat_type type,
-                            unsigned long long value);
+                            enum mysql_trx_stat_type type, uint64_t value);
 
 unsigned long thd_log_slow_verbosity(const MYSQL_THD thd);
 
@@ -857,9 +839,8 @@ int thd_is_background_thread(const MYSQL_THD thd);
 int mysql_tmpfile(const char *prefix);
 
 /**
-  Check the killed state of a connection
+  Check the killed state of a connection.
 
-  @details
   In MySQL support for the KILL statement is cooperative. The KILL
   statement only sets a "killed" flag. This function returns the value
   of that flag.  A thread should check it often, especially inside
@@ -952,6 +933,11 @@ void thd_set_ha_data(MYSQL_THD thd, const struct handlerton *hton,
 
 void remove_ssl_err_thread_state();
 
+/**
+  Interface to get the number of VCPUs.
+*/
+unsigned int thd_get_num_vcpus();
+
 int thd_command(const MYSQL_THD thd);
 long long thd_start_time(const MYSQL_THD thd);
 void thd_kill(unsigned long id);
@@ -964,6 +950,10 @@ void thd_kill(unsigned long id);
 */
 int thd_get_ft_query_extra_word_chars(void);
 
+typedef bool (*ssl_reload_callback_t)(void *);
+bool register_ssl_reload_callback(ssl_reload_callback_t);
+bool deregister_ssl_reload_callback(ssl_reload_callback_t);
+
 #ifdef __cplusplus
 }
 #endif
