@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +30,7 @@
 
 #include <string.h>
 
-#define COPYRIGHT_NOTICE_CURRENT_YEAR "2021"
+#define COPYRIGHT_NOTICE_CURRENT_YEAR "2025"
 
 /*
   This define specifies copyright notice which is displayed by every MySQL
@@ -57,13 +58,14 @@
   "   it under the terms of the GNU General Public License, version 2.0,\n"   \
   "   as published by the Free Software Foundation.\n"                        \
   "\n"                                                                        \
-  "   This program is also distributed with certain software (including\n"    \
+  "   This program is designed to work with certain software (including\n"    \
   "   but not limited to OpenSSL) that is licensed under separate terms,\n"   \
   "   as designated in a particular file or component or in included "        \
   "license\n"                                                                 \
   "   documentation.  The authors of MySQL hereby grant you an additional\n"  \
   "   permission to link the program and your derivative works with the\n"    \
-  "   separately licensed software that they have included with MySQL.\n"     \
+  "   separately licensed software that they have either included with\n"     \
+  "   the program or referenced in the documentation.\n"                      \
   "\n"                                                                        \
   "   This program is distributed in the hope that it will be useful,\n"      \
   "   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"       \
@@ -97,13 +99,14 @@
   "   it under the terms of the GNU General Public License, version 2.0,\n"   \
   "   as published by the Free Software Foundation.\n"                        \
   "\n"                                                                        \
-  "   This program is also distributed with certain software (including\n"    \
+  "   This program is designed to work with certain software (including\n"    \
   "   but not limited to OpenSSL) that is licensed under separate terms,\n"   \
   "   as designated in a particular file or component or in included "        \
   "license\n"                                                                 \
   "   documentation.  The authors of MySQL hereby grant you an additional\n"  \
   "   permission to link the program and your derivative works with the\n"    \
-  "   separately licensed software that they have included with MySQL.\n"     \
+  "   separately licensed software that they have either included with\n"     \
+  "   the program or referenced in the documentation.\n"                      \
   "\n"                                                                        \
   "   Without limiting anything contained in the foregoing, this file,\n"     \
   "   which is part of C Driver for MySQL (Connector/C), is also subject to " \
