@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -376,6 +377,20 @@ enum Log_event_type {
   ENUM_END_EVENT /* end marker */
 };
 
+/// @brief Event type helpers, enclosed in the structure
+struct Log_event_type_helper {
+  /// @brief Helps to identify known GTID event - returns true
+  /// for GTID_LOG_EVENT and GTID_TAGGED_LOG_EVENT
+  inline static bool is_assigned_gtid_event(const Log_event_type &type) {
+    return type == GTID_LOG_EVENT;
+  }
+  /// @brief Helps to identify any GTID event - returns true
+  /// for GTID_LOG_EVENT, GTID_TAGGED_LOG_EVENT and ANONYMOUS_GTID_LOG_EVENT
+  inline static bool is_any_gtid_event(const Log_event_type &type) {
+    return is_assigned_gtid_event(type) || type == ANONYMOUS_GTID_LOG_EVENT;
+  }
+};
+
 /**
   Struct to pass basic information about a event: type, query, is it ignorable
 */
@@ -729,7 +744,14 @@ class Log_event_header {
     @return True if the event object is valid, false otherwise.
   */
 
+#if defined(__GNUC__) && (__GNUC__ >= 14)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wuninitialized"
+#endif
   bool get_is_valid() { return m_is_valid; }
+#if defined(__GNUC__) && (__GNUC__ >= 14)
+#pragma GCC diagnostic pop
+#endif
 
   /**
     Set if the event object shall be considered valid or not.
