@@ -1,16 +1,17 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
 # as published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
 # documentation.  The authors of MySQL hereby grant you an additional
 # permission to link the program and your derivative works with the
-# separately licensed software that they have included with MySQL.
+# separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -292,6 +293,15 @@ sub fix_rsa_public_key {
   return "$std_data/rsa_public_key.pem";
 }
 
+sub fix_bind_address {
+  my ($self) = @_;
+  if ($self->{ARGS}->{bind_local}) {
+    # Listen only for incoming network connections from local machine
+    return "localhost";
+  }
+  return undef;
+}
+
 # Rules to run for each mysqld in the config
 #  - some rules depend on each other and thus need to be run
 #    in the order listed here
@@ -308,9 +318,11 @@ my @mysqld_rules = (
   { 'datadir'                                      => \&fix_datadir },
   { 'port'                                         => \&fix_port },
   { 'admin-port'                                   => \&fix_admin_port },
+  { 'bind-address'                                 => \&fix_bind_address },
   { 'general_log'                                  => \&fix_general_log },
   { 'general_log_file'                             => \&fix_log },
   { 'loose-mysqlx-port'                            => \&fix_x_port },
+  { 'loose-mysqlx-bind-address'                    => \&fix_bind_address },
   { 'loose-mysqlx-socket'                          => \&fix_x_socket },
   { 'loose-mysqlx-ssl'                             => \&fix_ssl_disabled },
   { 'loose-mysqlx-ssl-ca'                          => "" },
@@ -760,7 +772,9 @@ sub new_config {
                            @ndb_mgmd_rules);
 
   $self->run_section_rules($config, 'ndb_mgmd.',
-    ({ 'cluster-config-suffix' => \&fix_cluster_config_suffix },));
+    ({ 'cluster-config-suffix' => \&fix_cluster_config_suffix },
+     { 'bind-address' => \&fix_bind_address },
+    ));
 
   $self->run_section_rules($config, 'cluster_config.ndbd', @ndbd_rules);
 
