@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -75,10 +76,6 @@ extern CHARSET_INFO my_charset_utf16_unicode_ci;
 extern CHARSET_INFO my_charset_utf32_unicode_ci;
 extern CHARSET_INFO my_charset_cp932_japanese_ci;
 
-bool my_charset_same(const CHARSET_INFO *cs1, const CHARSET_INFO *cs2) {
-  return ((cs1 == cs2) || !strcmp(cs1->csname, cs2->csname));
-}
-
 std::unordered_map<std::string, int> *coll_name_num_map = nullptr;
 std::unordered_map<std::string, int> *cs_name_pri_num_map = nullptr;
 std::unordered_map<std::string, int> *cs_name_bin_num_map = nullptr;
@@ -92,7 +89,7 @@ static void map_coll_name_to_number(const char *name, int num) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(coll_name_num_map != nullptr);
+  assert(coll_name_num_map != nullptr);
   (*coll_name_num_map)[lower_case_name] = num;
 }
 
@@ -103,7 +100,7 @@ static void map_cs_name_to_number(const char *name, int num, int state) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
+  assert(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
   if ((state & MY_CS_PRIMARY)) (*cs_name_pri_num_map)[lower_case_name] = num;
   if ((state & MY_CS_BINSORT)) (*cs_name_bin_num_map)[lower_case_name] = num;
 }
@@ -115,8 +112,11 @@ static uint get_collation_number_internal(const char *name) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(coll_name_num_map != nullptr);
-  return (*coll_name_num_map)[lower_case_name];
+  assert(coll_name_num_map != nullptr);
+  auto name_num_map_it = coll_name_num_map->find(lower_case_name);
+  if (name_num_map_it != coll_name_num_map->end())
+    return name_num_map_it->second;
+  return 0;
 }
 
 static void simple_cs_init_functions(CHARSET_INFO *cs) {
@@ -136,9 +136,9 @@ static bool cs_copy_data(CHARSET_INFO *to, CHARSET_INFO *from) {
     if (to->csname == nullptr) return true;
   }
 
-  if (from->name) {
-    to->name = my_once_strdup(from->name, MYF(MY_WME));
-    if (to->name == nullptr) return true;
+  if (from->m_coll_name) {
+    to->m_coll_name = my_once_strdup(from->m_coll_name, MYF(MY_WME));
+    if (to->m_coll_name == nullptr) return true;
   }
 
   if (from->comment) {
@@ -189,7 +189,7 @@ static bool cs_copy_data(CHARSET_INFO *to, CHARSET_INFO *from) {
 static bool simple_cs_is_full(CHARSET_INFO *cs) {
   return ((cs->csname && cs->tab_to_uni && cs->ctype && cs->to_upper &&
            cs->to_lower) &&
-          (cs->number && cs->name &&
+          (cs->number && cs->m_coll_name &&
            (cs->sort_order || (cs->state & MY_CS_BINSORT))));
 }
 
@@ -206,15 +206,29 @@ static void copy_uca_collation(CHARSET_INFO *to, CHARSET_INFO *from) {
   to->state |= MY_CS_AVAILABLE | MY_CS_LOADED | MY_CS_STRNXFRM | MY_CS_UNICODE;
 }
 
-static int add_collation(CHARSET_INFO *cs) {
-  if (cs->name &&
-      (cs->number || (cs->number = get_collation_number_internal(cs->name))) &&
+static void clear_cs_info(CHARSET_INFO *cs) {
+  cs->number = 0;
+  cs->primary_number = 0;
+  cs->binary_number = 0;
+  cs->m_coll_name = nullptr;
+  cs->state = 0;
+  cs->sort_order = nullptr;
+}
+
+int MY_CHARSET_LOADER::add_collation(CHARSET_INFO *cs) {
+  if (cs->m_coll_name &&
+      (cs->number ||
+       (cs->number = get_collation_number_internal(cs->m_coll_name))) &&
       cs->number < array_elements(all_charsets)) {
     if (!all_charsets[cs->number]) {
       if (!(all_charsets[cs->number] =
                 (CHARSET_INFO *)my_once_alloc(sizeof(CHARSET_INFO), MYF(0))))
         return MY_XML_ERROR;
       memset(all_charsets[cs->number], 0, sizeof(CHARSET_INFO));
+    } else if (all_charsets[cs->number]->state & MY_CS_COMPILED) {
+      // Disallow overwriting compiled character sets
+      clear_cs_info(cs);
+      return MY_XML_OK;  // Just ignore it.
     }
 
     if (cs->primary_number == cs->number) cs->state |= MY_CS_PRIMARY;
@@ -222,7 +236,7 @@ static int add_collation(CHARSET_INFO *cs) {
     if (cs->binary_number == cs->number) cs->state |= MY_CS_BINSORT;
 
     all_charsets[cs->number]->state |= cs->state;
-    map_coll_name_to_number(cs->name, cs->number);
+    map_coll_name_to_number(cs->m_coll_name, cs->number);
     map_cs_name_to_number(cs->csname, cs->number, cs->state);
 
     if (!(all_charsets[cs->number]->state & MY_CS_COMPILED)) {
@@ -237,8 +251,8 @@ static int add_collation(CHARSET_INFO *cs) {
         newcs->state |= MY_CS_AVAILABLE | MY_CS_LOADED | MY_CS_NONASCII;
       } else if (!strcmp(cs->csname, "utf8") ||
                  !strcmp(cs->csname, "utf8mb3")) {
-        copy_uca_collation(newcs, &my_charset_utf8_unicode_ci);
-        newcs->ctype = my_charset_utf8_unicode_ci.ctype;
+        copy_uca_collation(newcs, &my_charset_utf8mb3_unicode_ci);
+        newcs->ctype = my_charset_utf8mb3_unicode_ci.ctype;
         if (init_state_maps(newcs)) return MY_XML_ERROR;
       } else if (!strcmp(cs->csname, "utf8mb4")) {
         copy_uca_collation(newcs, &my_charset_utf8mb4_unicode_ci);
@@ -280,7 +294,7 @@ static int add_collation(CHARSET_INFO *cs) {
       }
     } else {
       /*
-        We need the below to make get_charset_name()
+        We need the below to make get_collaiton_name()
         and get_charset_number() working even if a
         character set has not been really incompiled.
         The above functions are used for example
@@ -296,17 +310,11 @@ static int add_collation(CHARSET_INFO *cs) {
       if (cs->csname)
         if (!(dst->csname = my_once_strdup(cs->csname, MYF(MY_WME))))
           return MY_XML_ERROR;
-      if (cs->name)
-        if (!(dst->name = my_once_strdup(cs->name, MYF(MY_WME))))
+      if (cs->m_coll_name)
+        if (!(dst->m_coll_name = my_once_strdup(cs->m_coll_name, MYF(MY_WME))))
           return MY_XML_ERROR;
     }
-    cs->number = 0;
-    cs->primary_number = 0;
-    cs->binary_number = 0;
-    cs->name = NULL;
-    cs->state = 0;
-    cs->sort_order = NULL;
-    cs->state = 0;
+    clear_cs_info(cs);
   }
   return MY_XML_OK;
 }
@@ -315,47 +323,37 @@ static int add_collation(CHARSET_INFO *cs) {
   Report character set initialization errors and warnings.
   Be silent by default: no warnings on the client side.
 */
-static void default_reporter(enum loglevel level MY_ATTRIBUTE((unused)),
-                             uint ecode MY_ATTRIBUTE((unused)), ...) {}
+static void default_reporter(enum loglevel level [[maybe_unused]],
+                             uint ecode [[maybe_unused]], ...) {}
 my_error_reporter my_charset_error_reporter = default_reporter;
 
-/**
-  Wrappers for memory functions my_malloc (and friends)
-  with C-compatbile API without extra "myf" argument.
-*/
-static void *my_once_alloc_c(size_t size) {
-  return my_once_alloc(size, MYF(MY_WME));
+void *MY_CHARSET_LOADER::once_alloc(size_t sz) {
+  return my_once_alloc(sz, MYF(MY_WME));
 }
 
-static void *my_malloc_c(size_t size) {
-  return my_malloc(key_memory_charset_loader, size, MYF(MY_WME));
+void *MY_CHARSET_LOADER::mem_malloc(size_t sz) {
+  return my_malloc(key_memory_charset_loader, sz, MYF(MY_WME));
 }
 
-static void *my_realloc_c(void *old, size_t size) {
-  return my_realloc(key_memory_charset_loader, old, size, MYF(MY_WME));
+void *MY_CHARSET_LOADER::mem_realloc(void *p, size_t sz) {
+  return my_realloc(key_memory_charset_loader, p, sz, MYF(MY_WME));
 }
 
-static void my_free_c(void *ptr) { my_free(ptr); }
+void MY_CHARSET_LOADER::mem_free(void *p) { my_free(p); }
 
-/**
-  Initialize character set loader to use mysys memory management functions.
-  @param loader  Loader to initialize
-*/
-void my_charset_loader_init_mysys(MY_CHARSET_LOADER *loader) {
-  loader->errcode = 0;
-  loader->errarg[0] = '\0';
-  loader->once_alloc = my_once_alloc_c;
-  loader->mem_malloc = my_malloc_c;
-  loader->mem_realloc = my_realloc_c;
-  loader->mem_free = my_free_c;
-  loader->reporter = my_charset_error_reporter;
-  loader->add_collation = add_collation;
+void MY_CHARSET_LOADER::reporter(enum loglevel ll, uint ecode,
+                                 const char *arg) {
+  my_charset_error_reporter(ll, ecode, arg);
+}
+void MY_CHARSET_LOADER::reporter(enum loglevel ll, uint ecode, int len,
+                                 const char *arg) {
+  my_charset_error_reporter(ll, ecode, len, arg);
 }
 
 #define MY_MAX_ALLOWED_BUF 1024 * 1024
 #define MY_CHARSET_INDEX "Index.xml"
 
-const char *charsets_dir = NULL;
+const char *charsets_dir = nullptr;
 
 static bool my_read_charset_file(MY_CHARSET_LOADER *loader,
                                  const char *filename, myf myflags) {
@@ -402,9 +400,9 @@ error:
 char *get_charsets_dir(char *buf) {
   const char *sharedir = SHAREDIR;
   char *res;
-  DBUG_ENTER("get_charsets_dir");
+  DBUG_TRACE;
 
-  if (charsets_dir != NULL)
+  if (charsets_dir != nullptr)
     strmake(buf, charsets_dir, FN_REFLEN - 1);
   else {
     if (test_if_hard_path(sharedir) ||
@@ -416,16 +414,16 @@ char *get_charsets_dir(char *buf) {
   }
   res = convert_dirname(buf, buf, NullS);
   DBUG_PRINT("info", ("charsets dir: '%s'", buf));
-  DBUG_RETURN(res);
+  return res;
 }
 
-CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE] = {NULL};
+CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE] = {nullptr};
 CHARSET_INFO *default_charset_info = &my_charset_latin1;
 
 void add_compiled_collation(CHARSET_INFO *cs) {
-  DBUG_ASSERT(cs->number < array_elements(all_charsets));
+  assert(cs->number < array_elements(all_charsets));
   all_charsets[cs->number] = cs;
-  map_coll_name_to_number(cs->name, cs->number);
+  map_coll_name_to_number(cs->m_coll_name, cs->number);
   map_cs_name_to_number(cs->csname, cs->number, cs->state);
   cs->state |= MY_CS_AVAILABLE;
 }
@@ -438,8 +436,8 @@ static void init_available_charsets(void) {
 
   memset(&all_charsets, 0, sizeof(all_charsets));
 
-  DBUG_ASSERT(coll_name_num_map == nullptr && cs_name_pri_num_map == nullptr &&
-              cs_name_bin_num_map == nullptr);
+  assert(coll_name_num_map == nullptr && cs_name_pri_num_map == nullptr &&
+         cs_name_bin_num_map == nullptr);
   coll_name_num_map = new std::unordered_map<std::string, int>(0);
   cs_name_pri_num_map = new std::unordered_map<std::string, int>(0);
   cs_name_bin_num_map = new std::unordered_map<std::string, int>(0);
@@ -447,7 +445,6 @@ static void init_available_charsets(void) {
 
   /* Copy compiled charsets */
 
-  my_charset_loader_init_mysys(&loader);
   my_stpcpy(get_charsets_dir(fname), MY_CHARSET_INDEX);
   my_read_charset_file(&loader, fname,
 #ifdef MYSQL_SERVER
@@ -464,12 +461,11 @@ static const char *get_collation_name_alias(const char *name, char *buf,
     snprintf(buf, bufsize, "utf8_%s", name + 8);
     return buf;
   }
-  /* CLDR's tailoring data for Norwegian is same as Danish */
-  if (!native_strncasecmp(name, "utf8mb4_no_0900_", 16)) {
-    snprintf(buf, bufsize, "utf8mb4_da_0900_%s", name + 16);
+  if (!native_strncasecmp(name, "utf8_", 5)) {
+    snprintf(buf, bufsize, "utf8mb3_%s", name + 5);
     return buf;
   }
-  return NULL;
+  return nullptr;
 }
 
 uint get_collation_number(const char *name) {
@@ -496,7 +492,7 @@ static uint get_charset_number_internal(const char *charset_name,
     This function might be called concurrently. C++ guarantees this read-only
     access to STL container is thread-safe.
   */
-  DBUG_ASSERT(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
+  assert(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
   if ((cs_flags & MY_CS_PRIMARY)) {
     auto name_num_map_it = cs_name_pri_num_map->find(lower_case_name);
     if (name_num_map_it != cs_name_pri_num_map->end()) {
@@ -514,13 +510,13 @@ static uint get_charset_number_internal(const char *charset_name,
     }
   }
 
-  DBUG_ASSERT(false);
+  assert(false);
   return 0;
 }
 
 static const char *get_charset_name_alias(const char *name) {
-  if (!my_strcasecmp(&my_charset_latin1, name, "utf8mb3")) return "utf8";
-  return NULL;
+  if (!my_strcasecmp(&my_charset_latin1, name, "utf8")) return "utf8mb3";
+  return nullptr;
 }
 
 uint get_charset_number(const char *charset_name, uint cs_flags) {
@@ -532,24 +528,25 @@ uint get_charset_number(const char *charset_name, uint cs_flags) {
   return 0;
 }
 
-const char *get_charset_name(uint charset_number) {
+const char *get_collation_name(uint charset_number) {
   std::call_once(charsets_initialized, init_available_charsets);
 
   if (charset_number < array_elements(all_charsets)) {
     CHARSET_INFO *cs = all_charsets[charset_number];
 
-    if (cs && (cs->number == charset_number) && cs->name) return cs->name;
+    if (cs && (cs->number == charset_number) && cs->m_coll_name)
+      return cs->m_coll_name;
   }
 
   return "?"; /* this mimics find_type() */
 }
 
-static CHARSET_INFO *get_internal_charset(MY_CHARSET_LOADER *loader,
+static CHARSET_INFO *get_internal_charset(MY_CHARSET_LOADER *loader_arg,
                                           uint cs_number, myf flags) {
   char buf[FN_REFLEN];
   CHARSET_INFO *cs;
 
-  DBUG_ASSERT(cs_number < array_elements(all_charsets));
+  assert(cs_number < array_elements(all_charsets));
 
   if ((cs = all_charsets[cs_number])) {
     if (cs->state & MY_CS_READY) /* if CS is already initialized */
@@ -566,20 +563,19 @@ static CHARSET_INFO *get_internal_charset(MY_CHARSET_LOADER *loader,
     {
       MY_CHARSET_LOADER loader;
       strxmov(get_charsets_dir(buf), cs->csname, ".xml", NullS);
-      my_charset_loader_init_mysys(&loader);
       my_read_charset_file(&loader, buf, flags);
     }
 
     if (cs->state & MY_CS_AVAILABLE) {
       if (!(cs->state & MY_CS_READY)) {
-        if ((cs->cset->init && cs->cset->init(cs, loader)) ||
-            (cs->coll->init && cs->coll->init(cs, loader))) {
-          cs = NULL;
+        if ((cs->cset->init && cs->cset->init(cs, loader_arg)) ||
+            (cs->coll->init && cs->coll->init(cs, loader_arg))) {
+          cs = nullptr;
         } else
           cs->state |= MY_CS_READY;
       }
     } else
-      cs = NULL;
+      cs = nullptr;
 
     mysql_mutex_unlock(&THR_LOCK_charset);
   }
@@ -594,16 +590,15 @@ CHARSET_INFO *get_charset(uint cs_number, myf flags) {
 
   std::call_once(charsets_initialized, init_available_charsets);
 
-  if (cs_number >= array_elements(all_charsets)) return NULL;
+  if (cs_number >= array_elements(all_charsets)) return nullptr;
 
-  my_charset_loader_init_mysys(&loader);
   cs = get_internal_charset(&loader, cs_number, flags);
 
   if (!cs && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)], cs_string[23];
     my_stpcpy(get_charsets_dir(index_file), MY_CHARSET_INDEX);
     cs_string[0] = '#';
-    int10_to_str(cs_number, cs_string + 1, 10);
+    longlong10_to_str(cs_number, cs_string + 1, 10);
     my_error(EE_UNKNOWN_CHARSET, MYF(0), cs_string, index_file);
   }
   return cs;
@@ -625,8 +620,7 @@ CHARSET_INFO *my_collation_get_by_name(MY_CHARSET_LOADER *loader,
   std::call_once(charsets_initialized, init_available_charsets);
 
   cs_number = get_collation_number(name);
-  my_charset_loader_init_mysys(loader);
-  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : NULL;
+  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : nullptr;
 
   if (!cs && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)];
@@ -638,7 +632,6 @@ CHARSET_INFO *my_collation_get_by_name(MY_CHARSET_LOADER *loader,
 
 CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags) {
   MY_CHARSET_LOADER loader;
-  my_charset_loader_init_mysys(&loader);
   return my_collation_get_by_name(&loader, cs_name, flags);
 }
 
@@ -656,13 +649,13 @@ CHARSET_INFO *my_charset_get_by_name(MY_CHARSET_LOADER *loader,
                                      myf flags) {
   uint cs_number;
   CHARSET_INFO *cs;
-  DBUG_ENTER("get_charset_by_csname");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("name: '%s'", cs_name));
 
   std::call_once(charsets_initialized, init_available_charsets);
 
   cs_number = get_charset_number(cs_name, cs_flags);
-  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : NULL;
+  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : nullptr;
 
   if (!cs && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)];
@@ -670,13 +663,12 @@ CHARSET_INFO *my_charset_get_by_name(MY_CHARSET_LOADER *loader,
     my_error(EE_UNKNOWN_CHARSET, MYF(0), cs_name, index_file);
   }
 
-  DBUG_RETURN(cs);
+  return cs;
 }
 
 CHARSET_INFO *get_charset_by_csname(const char *cs_name, uint cs_flags,
                                     myf flags) {
   MY_CHARSET_LOADER loader;
-  my_charset_loader_init_mysys(&loader);
   return my_charset_get_by_name(&loader, cs_name, cs_flags, flags);
 }
 
@@ -700,7 +692,7 @@ bool resolve_charset(const char *cs_name, const CHARSET_INFO *default_cs,
                      const CHARSET_INFO **cs) {
   *cs = get_charset_by_csname(cs_name, MY_CS_PRIMARY, MYF(0));
 
-  if (*cs == NULL) {
+  if (*cs == nullptr) {
     *cs = default_cs;
     return true;
   }
@@ -728,7 +720,7 @@ bool resolve_collation(const char *cl_name, const CHARSET_INFO *default_cl,
                        const CHARSET_INFO **cl) {
   *cl = get_charset_by_name(cl_name, MYF(0));
 
-  if (*cl == NULL) {
+  if (*cl == nullptr) {
     *cl = default_cl;
     return true;
   }
@@ -932,7 +924,7 @@ size_t escape_quotes_for_mysql(CHARSET_INFO *charset_info, char *to,
 
 void charset_uninit() {
   for (CHARSET_INFO *cs : all_charsets) {
-    if (cs && cs->coll->uninit) {
+    if (cs && cs->coll && cs->coll->uninit) {
       cs->coll->uninit(cs);
     }
   }
