@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -255,6 +256,24 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
   my_init_done = false;
 } /* my_end */
 
+/**
+  Pointer to function that handles abort. It is the std's abort() by default.
+*/
+static void (*my_abort_func)() = abort;
+
+[[noreturn]] void my_abort() {
+  my_abort_func();
+  /*
+    We should not reach here, it is required only because my_abort_func() is
+    not [[noreturn]].
+  */
+  abort();
+}
+
+void set_my_abort(void (*new_my_abort_func)()) {
+  my_abort_func = new_my_abort_func;
+}
+
 #ifdef _WIN32
 /*
   my_parameter_handler
@@ -265,9 +284,11 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
   e.g. iterator out-of-range, but pointing to valid memory.
 */
 
-void my_parameter_handler(const wchar_t *expression, const wchar_t *function,
-                          const wchar_t *file, unsigned int line,
-                          uintptr_t pReserved) {
+void my_parameter_handler(const wchar_t *expression [[maybe_unused]],
+                          const wchar_t *function [[maybe_unused]],
+                          const wchar_t *file [[maybe_unused]],
+                          unsigned int line [[maybe_unused]],
+                          uintptr_t pReserved [[maybe_unused]]) {
 #ifndef NDEBUG
   fprintf(stderr,
           "my_parameter_handler errno %d "
@@ -557,14 +578,14 @@ static PSI_memory_info all_mysys_memory[] = {
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
 static PSI_thread_info all_mysys_thread[] = {
-    {&key_thread_timer_notifier, "thread_timer_notifier", PSI_FLAG_SINGLETON, 0,
-     PSI_DOCUMENT_ME}};
+    {&key_thread_timer_notifier, "thread_timer_notifier", "timer_notifier",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
 #ifdef HAVE_PSI_INTERFACE
 void my_init_mysys_psi_keys() {
-  const char *category MY_ATTRIBUTE((unused)) = "mysys";
-  int count MY_ATTRIBUTE((unused));
+  const char *category [[maybe_unused]] = "mysys";
+  int count [[maybe_unused]];
 
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   count = sizeof(all_mysys_mutexes) / sizeof(all_mysys_mutexes[0]);
