@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   Without limiting anything contained in the foregoing, this file,
   which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -50,6 +51,7 @@
 #define HAVE_PSI_DATA_LOCK_INTERFACE
 #define HAVE_PSI_SYSTEM_INTERFACE
 #define HAVE_PSI_TLS_CHANNEL_INTERFACE
+#define HAVE_PSI_SERVER_TELEMETRY_TRACES_INTERFACE
 
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -91,13 +93,15 @@ static void register_thread_noop(const char *, PSI_thread_info *, int) {
   return;
 }
 
-static int spawn_thread_noop(PSI_thread_key, my_thread_handle *thread,
+static int spawn_thread_noop(PSI_thread_key, unsigned int,
+                             my_thread_handle *thread,
                              const my_thread_attr_t *attr,
                              my_start_routine start_routine, void *arg) {
   return my_thread_create(thread, attr, start_routine, arg);
 }
 
-static PSI_thread *new_thread_noop(PSI_thread_key, const void *, ulonglong) {
+static PSI_thread *new_thread_noop(PSI_thread_key, unsigned int, const void *,
+                                   ulonglong) {
   return nullptr;
 }
 
@@ -137,6 +141,8 @@ static void set_thread_rows_examined_noop(ulonglong) { return; }
 
 static void set_thread_info_noop(const char *, uint) { return; }
 
+static void set_thread_secondary_engine_noop(bool) { return; }
+
 static void set_thread_noop(PSI_thread *) { return; }
 
 static void set_thread_peer_port_noop(PSI_thread *, uint) { return; }
@@ -156,10 +162,9 @@ static void delete_current_thread_noop(void) { return; }
 
 static void delete_thread_noop(PSI_thread *) { return; }
 
-static int set_thread_connect_attrs_noop(
-    const char *buffer MY_ATTRIBUTE((unused)),
-    uint length MY_ATTRIBUTE((unused)),
-    const void *from_cs MY_ATTRIBUTE((unused))) {
+static int set_thread_connect_attrs_noop(const char *buffer [[maybe_unused]],
+                                         uint length [[maybe_unused]],
+                                         const void *from_cs [[maybe_unused]]) {
   return 0;
 }
 
@@ -195,6 +200,14 @@ static void notify_session_disconnect_noop(PSI_thread *) { return; }
 
 static void notify_session_change_user_noop(PSI_thread *) { return; }
 
+static void set_mem_cnt_THD_noop(THD *, THD **backup_thd) {
+  *backup_thd = nullptr;
+  return;
+}
+
+static void detect_telemetry_noop(PSI_thread * /*unused*/) {}
+static void abort_telemetry_noop(PSI_thread * /*unused*/) {}
+
 static PSI_thread_service_t psi_thread_noop = {
     register_thread_noop,
     spawn_thread_noop,
@@ -216,6 +229,7 @@ static PSI_thread_service_t psi_thread_noop = {
     set_thread_rows_sent_noop,
     set_thread_rows_examined_noop,
     set_thread_info_noop,
+    set_thread_secondary_engine_noop,
     set_thread_resource_group_noop,
     set_thread_resource_group_by_id_noop,
     set_thread_noop,
@@ -232,7 +246,10 @@ static PSI_thread_service_t psi_thread_noop = {
     unregister_notification_noop,
     notify_session_connect_noop,
     notify_session_disconnect_noop,
-    notify_session_change_user_noop};
+    notify_session_change_user_noop,
+    set_mem_cnt_THD_noop,
+    detect_telemetry_noop,
+    abort_telemetry_noop};
 
 struct PSI_thread_bootstrap *psi_thread_hook = nullptr;
 PSI_thread_service_t *psi_thread_service = &psi_thread_noop;
@@ -555,6 +572,8 @@ static void set_metadata_lock_status_noop(PSI_metadata_lock *,
 static void set_metadata_lock_duration_noop(PSI_metadata_lock *,
                                             opaque_mdl_duration) {}
 
+static void set_metadata_lock_type_noop(PSI_metadata_lock *, opaque_mdl_type) {}
+
 static void destroy_metadata_lock_noop(PSI_metadata_lock *) {}
 
 static PSI_metadata_locker *start_metadata_wait_noop(
@@ -566,8 +585,9 @@ static void end_metadata_wait_noop(PSI_metadata_locker *, int) {}
 
 static PSI_mdl_service_t psi_mdl_noop = {
     create_metadata_lock_noop,       set_metadata_lock_status_noop,
-    set_metadata_lock_duration_noop, destroy_metadata_lock_noop,
-    start_metadata_wait_noop,        end_metadata_wait_noop};
+    set_metadata_lock_duration_noop, set_metadata_lock_type_noop,
+    destroy_metadata_lock_noop,      start_metadata_wait_noop,
+    end_metadata_wait_noop};
 
 struct PSI_mdl_bootstrap *psi_mdl_hook = nullptr;
 PSI_mdl_service_t *psi_mdl_service = &psi_mdl_noop;
@@ -719,6 +739,10 @@ static void set_statement_no_good_index_used_noop(PSI_statement_locker *) {
   return;
 }
 
+static void set_statement_secondary_engine_noop(PSI_statement_locker *, bool) {
+  return;
+}
+
 static void end_statement_noop(PSI_statement_locker *, void *) { return; }
 
 static PSI_prepared_stmt *create_prepared_stmt_noop(void *, uint,
@@ -742,6 +766,10 @@ static void set_prepared_stmt_text_noop(PSI_prepared_stmt *, const char *,
   return;
 }
 
+static void set_prepared_stmt_secondary_engine_noop(PSI_prepared_stmt *, bool) {
+  return;
+}
+
 static struct PSI_digest_locker *digest_start_noop(PSI_statement_locker *) {
   return nullptr;
 }
@@ -768,6 +796,11 @@ static void drop_sp_noop(uint, const char *, uint, const char *, uint) {
   return;
 }
 
+static void notify_statement_query_attributes_noop(PSI_statement_locker *,
+                                                   bool) {}
+
+static void abort_statement_telemetry_noop(PSI_statement_locker * /*unused*/) {}
+
 static PSI_statement_service_t psi_statement_noop = {
     register_statement_noop,
     get_thread_statement_locker_noop,
@@ -791,19 +824,23 @@ static PSI_statement_service_t psi_statement_noop = {
     inc_statement_sort_scan_noop,
     set_statement_no_index_used_noop,
     set_statement_no_good_index_used_noop,
+    set_statement_secondary_engine_noop,
     end_statement_noop,
     create_prepared_stmt_noop,
     destroy_prepared_stmt_noop,
     reprepare_prepared_stmt_noop,
     execute_prepared_stmt_noop,
     set_prepared_stmt_text_noop,
+    set_prepared_stmt_secondary_engine_noop,
     digest_start_noop,
     digest_end_noop,
     get_sp_share_noop,
     release_sp_share_noop,
     start_sp_noop,
     end_sp_noop,
-    drop_sp_noop};
+    drop_sp_noop,
+    notify_statement_query_attributes_noop,
+    abort_statement_telemetry_noop};
 
 struct PSI_statement_bootstrap *psi_statement_hook = nullptr;
 PSI_statement_service_t *psi_statement_service = &psi_statement_noop;
@@ -982,3 +1019,5 @@ PSI_tls_channel_service_t *psi_tls_channel_service = &psi_tls_channel_noop;
 void set_psi_tls_channel_service(void *psi) {
   psi_tls_channel_service = (PSI_tls_channel_service_t *)psi;
 }
+
+// ===========================================================================
