@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,16 +24,18 @@
 #ifndef CONNECTION_DELAY_H
 #define CONNECTION_DELAY_H
 
-#include <mysql_com.h> /* USERNAME_LENGTH */
 #include <atomic>
 
-#include "lf.h" /* LF Hash */
+#include "lf.h"          /* LF Hash */
+#include "my_hostname.h" /* HOSTNAME_LENGTH */
 #include "my_inttypes.h"
+#include "mysql_com.h" /* USERNAME_LENGTH */
 #include "plugin/connection_control/connection_control_data.h" /* variables and status */
 #include "plugin/connection_control/connection_control_interfaces.h" /* Observer interface */
 #include "plugin/connection_control/connection_control_memory.h" /* Connection_control_alloc */
 #include "plugin/connection_control/connection_delay_api.h" /* Constants */
-#include "sql/table.h"                                      /* TABLE_LIST */
+#include "plugin/connection_control/log_rate_limiter.h"
+#include "sql/table.h"                                      /* Table_ref */
 
 namespace connection_control {
 /**
@@ -52,7 +55,7 @@ class Connection_event_record : public Connection_control_alloc {
   }
 
   /**
-    Retrives failed login count for given user entry
+    Retrieves failed login count for given user entry
 
     @returns Failed login count
   */
@@ -93,18 +96,18 @@ class Connection_delay_event : public Connection_event_records {
   Connection_delay_event();
 
   /** Destructor. Removes all entries from hash before destroying hash */
-  ~Connection_delay_event() {
+  ~Connection_delay_event() override {
     reset_all();
     lf_hash_destroy(&m_entries);
   }
 
-  void fill_IS_table(TABLE_LIST *tables);
+  void fill_IS_table(Table_ref *tables);
 
   /* Overridden function */
-  bool create_or_update_entry(const Sql_string &s);
-  bool remove_entry(const Sql_string &s);
-  bool match_entry(const Sql_string &s, void *value);
-  void reset_all();
+  bool create_or_update_entry(const Sql_string &s) override;
+  bool remove_entry(const Sql_string &s) override;
+  bool match_entry(const Sql_string &s, void *value) override;
+  void reset_all() override;
 
  private:
   /** Hash for storing Connection_event_record per user */
@@ -125,9 +128,9 @@ class Connection_delay_action : public Connection_event_observer,
                           size_t status_vars_size, mysql_rwlock_t *lock);
 
   /** Destructor */
-  ~Connection_delay_action() {
+  ~Connection_delay_action() override {
     deinit();
-    m_lock = 0;
+    m_lock = nullptr;
   }
 
   void init(Connection_event_coordinator_services *coordinator);
@@ -136,10 +139,6 @@ class Connection_delay_action : public Connection_event_observer,
     Set threshold value.
 
     @param threshold [in]        New threshold value
-
-    @returns whether threshold value was changed successfully or not
-      @retval true  Success
-      @retval false Failure. Invalid threshold value specified.
   */
 
   void set_threshold(int64 threshold) {
@@ -183,16 +182,16 @@ class Connection_delay_action : public Connection_event_observer,
   /** Get min value */
   int64 get_min_delay() { return m_min_delay.load(); }
 
-  void fill_IS_table(THD *thd, TABLE_LIST *tables, Item *cond);
+  void fill_IS_table(THD *thd, Table_ref *tables, Item *cond);
 
   /** Overridden functions */
   bool notify_event(MYSQL_THD thd,
                     Connection_event_coordinator_services *coordinator,
                     const mysql_event_connection *connection_event,
-                    Error_handler *error_handler);
+                    Error_handler *error_handler) override;
   bool notify_sys_var(Connection_event_coordinator_services *coordinator,
                       opt_connection_control variable, void *new_value,
-                      Error_handler *error_handler);
+                      Error_handler *error_handler) override;
 
  private:
   void deinit();
@@ -239,6 +238,8 @@ class Connection_delay_action : public Connection_event_observer,
   Connection_delay_event m_userhost_hash;
   /** RW lock */
   mysql_rwlock_t *m_lock;
+  /** Log messages rate limit handling object **/
+  LogRateLimiter m_log_rate_limiter;
 };
 }  // namespace connection_control
 #endif /* !CONNECTION_DELAY_H */
