@@ -1,15 +1,16 @@
-/* Copyright (c) 2013, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,9 +39,10 @@
 #include "plugin/group_replication/include/plugin_messages/transaction_message.h"
 #include "plugin/group_replication/include/plugin_messages/transaction_with_guarantee_message.h"
 #include "plugin/group_replication/include/plugin_observers/group_transaction_observation_manager.h"
+
+#ifndef NDEBUG
 #include "plugin/group_replication/include/sql_service/sql_command_test.h"
-#include "plugin/group_replication/include/sql_service/sql_service_command.h"
-#include "plugin/group_replication/include/sql_service/sql_service_interface.h"
+#endif
 
 /*
   Buffer to read the write_set value as a string.
@@ -67,11 +69,12 @@ int add_write_set(Transaction_context_log_event *tcle,
     uint64 const tmp_str_sz =
         base64_needed_encoded_length((uint64)BUFFER_READ_PKE);
     char *write_set_value =
-        (char *)my_malloc(PSI_NOT_INSTRUMENTED, tmp_str_sz, MYF(MY_WME));
+        (char *)my_malloc(key_write_set_encoded, tmp_str_sz, MYF(MY_WME));
     if (!write_set_value) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
                    ER_GRP_RPL_OOM_FAILED_TO_GENERATE_IDENTIFICATION_HASH);
+      my_free(write_set_value);
       return 1;
       /* purecov: end */
     }
@@ -80,6 +83,7 @@ int add_write_set(Transaction_context_log_event *tcle,
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
                    ER_GRP_RPL_WRITE_IDENT_HASH_BASE64_ENCODING_FAILED);
+      my_free(write_set_value);
       return 1;
       /* purecov: end */
     }
@@ -178,10 +182,18 @@ int group_replication_trans_before_commit(Trans_param *param) {
                   return 1;);
 
   DBUG_EXECUTE_IF("group_replication_before_commit_hook_wait", {
-    const char act[] = "now wait_for continue_commit";
+    const char act[] =
+        "now signal signal.group_replication_before_commit_hook_wait_reached "
+        "wait_for continue_commit";
     assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
+  DBUG_EXECUTE_IF("group_replication_pause_on_before_commit_hook", {
+    // DBUG_SYNC are hold by same MDL lock test is using
+    const uint sleep_time_seconds = VIEW_MODIFICATION_TIMEOUT * 1.5;
+    my_sleep(sleep_time_seconds * 1000000);
+  });
+
   /*
     If the originating id belongs to a thread in the plugin, the transaction
     was already certified. Channel operations can deadlock against
@@ -200,12 +212,14 @@ int group_replication_trans_before_commit(Trans_param *param) {
     if (!fail_to_lock) {
       const Group_member_info::Group_member_status member_status =
           local_member_info->get_recovery_status();
-      if (Group_member_info::MEMBER_ONLINE == member_status) {
+      if (Group_member_info::MEMBER_ONLINE == member_status ||
+          Group_member_info::MEMBER_IN_RECOVERY == member_status) {
         applier_module->get_pipeline_stats_member_collector()
             ->decrement_transactions_waiting_apply();
         applier_module->get_pipeline_stats_member_collector()
             ->increment_transactions_applied();
-      } else if (Group_member_info::MEMBER_IN_RECOVERY == member_status) {
+      }
+      if (Group_member_info::MEMBER_IN_RECOVERY == member_status) {
         applier_module->get_pipeline_stats_member_collector()
             ->increment_transactions_applied_during_recovery();
       }
@@ -227,7 +241,10 @@ int group_replication_trans_before_commit(Trans_param *param) {
     return 0;
   }
 
-  shared_plugin_stop_lock->grab_read_lock();
+  if (shared_plugin_stop_lock->try_grab_read_lock()) {
+    /* If plugin is stopping, rollback the transaction immediately. */
+    return 1;
+  }
 
   if (is_plugin_waiting_to_set_server_read_mode()) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CANNOT_EXECUTE_TRANS_WHILE_STOPPING);
@@ -295,7 +312,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
   /*
     Atomic DDL:s are logged through the transactional cache so they should
     be exempted from considering as DML by the plugin: not
-    everthing that is in the trans cache is actually DML.
+    everything that is in the trans cache is actually DML.
   */
   bool is_dml = !param->is_atomic_ddl;
   bool may_have_sbr_stmts = !is_dml;
@@ -303,6 +320,7 @@ int group_replication_trans_before_commit(Trans_param *param) {
   my_off_t cache_log_position = 0;
   const my_off_t trx_cache_log_position = param->trx_cache_log->length();
   const my_off_t stmt_cache_log_position = param->stmt_cache_log->length();
+  unsigned long long immediate_commit_timestamp = 0;
 
   if (trx_cache_log_position > 0 && stmt_cache_log_position == 0) {
     cache_log = param->trx_cache_log;
@@ -379,6 +397,13 @@ int group_replication_trans_before_commit(Trans_param *param) {
       may_have_sbr_stmts = true;
     }
 
+    DBUG_EXECUTE_IF("group_replication_after_add_write_set", {
+      const char act[] =
+          "now signal signal.group_replication_after_add_write_set_reached "
+          "wait_for signal.group_replication_after_add_write_set_continue";
+      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    });
+
     /*
       'CREATE TABLE ... AS SELECT' is considered a DML, though in reality it
       is DDL + DML, which write-sets do not capture all dependencies.
@@ -399,38 +424,17 @@ int group_replication_trans_before_commit(Trans_param *param) {
     }
   }
 
-  /*
-    The BEFORE consistency can be used on groups with members that
-    do not support GROUP_REPLICATION_CONSISTENCY_BEFORE. In order to
-    allow that, after the wait is done on the transaction begin on
-    the local member, we broadcast the transaction as a normal
-    transaction that all versions do understand.
-  */
-  if (consistency_level < GROUP_REPLICATION_CONSISTENCY_AFTER) {
-    transaction_msg = new Transaction_message();
-  } else {
-    transaction_msg = new Transaction_with_guarantee_message(consistency_level);
-  }
-
-  // serialize transaction context into a transaction message.
-  // There is a chance you encounter an OOM in Transaction_message::write()
-  // here, so we take care accordingly.
-  try {
-    binary_event_serialize(tcle, transaction_msg);
-  } catch (const std::bad_alloc &) {
-    LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
-    error = pre_wait_error;
-    goto err;
-  }
-
   if (*(param->original_commit_timestamp) == UNDEFINED_COMMIT_TIMESTAMP) {
     /*
      Assume that this transaction is original from this server and update status
      variable so that it won't be re-defined when this GTID is written to the
      binlog
     */
-    *(param->original_commit_timestamp) = my_micro_time();
-  }  // otherwise the transaction did not originate in this server
+    *(param->original_commit_timestamp) = immediate_commit_timestamp =
+        my_micro_time();
+  } else {  // the transaction did not originate in this server
+    immediate_commit_timestamp = my_micro_time();
+  }
 
   *(param->immediate_server_version) = do_server_version_int(::server_version);
   if (*(param->original_server_version) == UNDEFINED_SERVER_VERSION) {
@@ -447,26 +451,22 @@ int group_replication_trans_before_commit(Trans_param *param) {
   // Notice the GTID of atomic DDL is written to the trans cache as well.
   gle = new Gtid_log_event(
       param->server_id, is_dml || param->is_atomic_ddl, 0, sequence_number,
-      may_have_sbr_stmts, *(param->original_commit_timestamp), 0,
-      gtid_specification, *(param->original_server_version),
-      *(param->immediate_server_version));
+      may_have_sbr_stmts, *(param->original_commit_timestamp),
+      immediate_commit_timestamp, gtid_specification,
+      *(param->original_server_version), *(param->immediate_server_version));
   /*
     GR does not support event checksumming. If GR start to support event
     checksumming, the calculation below should take the checksum payload into
     account.
   */
   gle->set_trx_length_by_cache_size(cache_log_position);
-  // There is a chance you encounter an OOM in Transaction_message::write()
-  // here, so we take care accordingly.
-  try {
-    binary_event_serialize(gle, transaction_msg);
-  } catch (const std::bad_alloc &) {
-    LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
-    error = pre_wait_error;
-    goto err;
-  }
 
-  transaction_size = cache_log_position + transaction_msg->length();
+  /*
+    Only proceed to message serialization and send if the transaction
+    size does not exceed the limit.
+  */
+  transaction_size =
+      cache_log_position + tcle->get_event_length() + gle->get_event_length();
   if (is_dml && transaction_size_limit &&
       transaction_size > transaction_size_limit) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_TRANS_SIZE_EXCEEDS_LIMIT,
@@ -475,17 +475,44 @@ int group_replication_trans_before_commit(Trans_param *param) {
     goto err;
   }
 
-  // Copy binlog cache content to buffer.
-  // There is a chance you encounter an OOM in Transaction_message::write()
-  // here, so we take care accordingly.
+  /*
+    Serialize transaction context, Gtid and binlog cache into
+    transaction message.
+    There is a chance we encounter an OOM, thence consider it.
+  */
   try {
+    /*
+      The BEFORE consistency can be used on groups with members that
+      do not support GROUP_REPLICATION_CONSISTENCY_BEFORE. In order to
+      allow that, after the wait is done on the transaction begin on
+      the local member, we broadcast the transaction as a normal
+      transaction that all versions do understand.
+    */
+    if (consistency_level < GROUP_REPLICATION_CONSISTENCY_AFTER) {
+      transaction_msg = new Transaction_message(transaction_size);
+    } else {
+      transaction_msg = new Transaction_with_guarantee_message(
+          transaction_size, consistency_level);
+    }
+
+    if (binary_event_serialize(tcle, transaction_msg) ||
+        binary_event_serialize(gle, transaction_msg)) {
+      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WRITE_TO_TRANSACTION_MESSAGE_FAILED,
+                   param->thread_id);
+      error = pre_wait_error;
+      goto err;
+    }
+    /* Release memory as soon as possible. */
+    delete tcle;
+    tcle = nullptr;
+    delete gle;
+    gle = nullptr;
+
     if (cache_log->copy_to(transaction_msg)) {
-      /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WRITE_TO_TRANSACTION_MESSAGE_FAILED,
                    param->thread_id);
       error = pre_wait_error;
       goto err;
-      /* purecov: end */
     }
   } catch (const std::bad_alloc &) {
     LogPluginErr(ERROR_LEVEL, ER_OUT_OF_RESOURCES);
@@ -510,7 +537,9 @@ int group_replication_trans_before_commit(Trans_param *param) {
   };);
 
   DBUG_EXECUTE_IF("group_replication_before_message_broadcast", {
-    const char act[] = "now wait_for waiting";
+    const char act[] =
+        "now signal signal.group_replication_before_message_broadcast_reached "
+        "wait_for waiting";
     assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 #endif
@@ -522,7 +551,11 @@ int group_replication_trans_before_commit(Trans_param *param) {
   applier_module->get_flow_control_module()->do_wait();
 
   // Broadcast the Transaction Message
-  send_error = gcs_module->send_message(*transaction_msg);
+  send_error = gcs_module->send_transaction_message(*transaction_msg);
+
+  /* Release memory as soon as possible. */
+  delete transaction_msg;
+  transaction_msg = nullptr;
 
   if (send_error == GCS_MESSAGE_TOO_BIG) {
     /* purecov: begin inspected */
@@ -568,7 +601,9 @@ err:
   }
 
   DBUG_EXECUTE_IF("group_replication_after_before_commit_hook", {
-    const char act[] = "now wait_for signal.commit_continue";
+    const char act[] =
+        "now SIGNAL signal.group_replication_after_before_commit_hook_reached "
+        "WAIT_FOR signal.group_replication_after_before_commit_hook_continue";
     assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
   return error;
