@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,6 +32,7 @@
 #include "plugin/keyring/buffer.h"
 #include "plugin/keyring/checker/checker_factory.h"
 #include "plugin/keyring/common/i_keyring_io.h"
+#include "plugin/keyring/common/keyring.h"
 #include "plugin/keyring/common/keyring_memory.h"
 #include "plugin/keyring/common/logger.h"
 #include "plugin/keyring/digest.h"
@@ -49,7 +51,7 @@ class Buffered_file_io : public IKeyring_io {
 
   // ================= IKeyring_io implementation ================= //
 
-  bool init(std::string *keyring_filename) override;
+  bool init(const std::string *keyring_filename) override;
   bool flush_to_backup(ISerialized_object *serialized_object) override;
   bool flush_to_storage(ISerialized_object *serialized_object) override;
   ISerializer *get_serializer() override;
