@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,11 +33,9 @@
 #include "my_sharedlib.h"
 #include "plugin/keyring/common/i_keyring_io.h"
 #include "plugin/keyring/common/i_keys_container.h"
-#include "plugin/keyring/common/i_system_keys_container.h"
 #include "plugin/keyring/common/keyring_key.h"
 #include "plugin/keyring/common/keyring_memory.h"
 #include "plugin/keyring/common/logger.h"
-#include "sql/sys_vars_shared.h"  //For PolyLock, AutoWLock, AutoRLock
 
 namespace keyring {
 
@@ -47,20 +46,22 @@ class Keys_container : public IKeys_container {
 
  public:
   Keys_container(ILogger *logger);
-  bool init(IKeyring_io *keyring_io, std::string keyring_storage_url);
-  bool store_key(IKey *key);
-  IKey *fetch_key(IKey *key);
-  bool remove_key(IKey *key);
-  std::string get_keyring_storage_url();
-  void set_keyring_io(IKeyring_io *keyring_io);
-  std::vector<Key_metadata> get_keys_metadata() { return keys_metadata; }
+  bool init(IKeyring_io *keyring_io, std::string keyring_storage_url) override;
+  bool store_key(IKey *key) override;
+  IKey *fetch_key(IKey *key) override;
+  bool remove_key(IKey *key) override;
+  std::string get_keyring_storage_url() override;
+  void set_keyring_io(IKeyring_io *keyring_io) override;
+  std::vector<Key_metadata> get_keys_metadata() override {
+    return keys_metadata;
+  }
 
-  ~Keys_container();
+  ~Keys_container() override;
 
   size_t get_number_of_keys() { return keys_hash->size(); }
 
   /** Internal function, returns the key from the internal hash, no locks */
-  IKey *get_key_from_hash(IKey *key);
+  IKey *get_key_from_hash(IKey *key) override;
 
  protected:
   Keys_container(const Keys_container &);
@@ -82,7 +83,6 @@ class Keys_container : public IKeys_container {
   ILogger *logger;
   IKeyring_io *keyring_io;
   std::string keyring_storage_url;
-  std::unique_ptr<ISystem_keys_container> system_keys_container;
 };
 
 }  // namespace keyring
