@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +23,7 @@
 #ifndef SQL_USER_CACHE_INCLUDED
 #define SQL_USER_CACHE_INCLUDED
 
+#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <atomic>
@@ -45,7 +47,7 @@
 #include "my_inttypes.h"
 #include "my_sharedlib.h"
 #include "my_sys.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/mysql_lex_string.h"
 #include "mysql_com.h"   // SCRAMBLE_LENGTH
 #include "mysql_time.h"  // MYSQL_TIME
@@ -103,7 +105,6 @@ class ACL_HOST_AND_IP {
     @param[in]  ip_arg Buffer containing CIDR mask value.
     @param[out] val    Numeric IP mask value on success.
 
-    @return
     @retval false Parsing succeeded.
     @retval true  Parsing failed.
   */
@@ -115,7 +116,6 @@ class ACL_HOST_AND_IP {
     @param[in]  ip_arg Buffer containing subnet mask value.
     @param[out] val    Numeric IP mask value on success.
 
-    @return
     @retval false Parsing succeeded.
     @retval true  Parsing failed.
   */
@@ -127,7 +127,6 @@ class ACL_HOST_AND_IP {
     @param[in]  ip_arg Buffer containing IP value.
     @param[out] val    Numeric IP value on success.
 
-    @return
     @retval !nullptr Parsing succeeded. Returned value is the pointer following
     the buffer holding the IP.
     @retval nullptr  Parsing failed. The buffer does not contain valid IP value.
@@ -146,7 +145,7 @@ class ACL_HOST_AND_IP {
 
   bool has_wildcard() {
     return (strchr(get_host(), wild_many) || strchr(get_host(), wild_one) ||
-            ip_mask);
+            (ip_mask && (ip_mask != (long)UINT_MAX32)));
   }
 
   bool check_allow_all_hosts() {
@@ -163,13 +162,13 @@ class ACL_ACCESS {
   ACL_ACCESS() : host(), sort(0), access(0) {}
   ACL_HOST_AND_IP host;
   ulong sort;
-  ulong access;
+  Access_bitmask access;
 };
 
 /**
   @class ACL_compare
 
-  Class that compares ACL_ACCESS objects. Used in std::sort funciton.
+  Class that compares ACL_ACCESS objects. Used in std::sort functions.
 */
 class ACL_compare {
  public:
@@ -304,17 +303,28 @@ class ACL_USER : public ACL_ACCESS {
 
   void set_user(MEM_ROOT *mem, const char *user_arg);
   void set_host(MEM_ROOT *mem, const char *host_arg);
+  void set_mfa(MEM_ROOT *mem, I_multi_factor_auth *m);
   size_t get_username_length() const { return user ? strlen(user) : 0; }
   class Password_locked_state {
    public:
     bool is_active() const {
       return m_password_lock_time_days != 0 && m_failed_login_attempts != 0;
     }
+    bool is_default() const {
+      return (m_remaining_login_attempts == m_failed_login_attempts &&
+              m_daynr_locked == 0);
+    }
     int get_password_lock_time_days() const {
       return m_password_lock_time_days;
     }
     uint get_failed_login_attempts() const { return m_failed_login_attempts; }
-    void set_parameters(uint password_lock_time_days,
+    uint get_remaining_login_attempts() const {
+      return m_remaining_login_attempts;
+    }
+    long get_daynr_locked() const { return m_daynr_locked; }
+    void set_temporary_lock_state_parameters(uint remaining_login_attempts,
+                                             long daynr_locked);
+    void set_parameters(int password_lock_time_days,
                         uint failed_login_attempts);
     bool update(THD *thd, bool successful_login, long *ret_days_remaining);
     Password_locked_state()
@@ -325,22 +335,23 @@ class ACL_USER : public ACL_ACCESS {
 
    protected:
     /**
-      read from the user config. The number of days to keep the accont locked
+      read from the user config. The number of days to keep the account locked
     */
     int m_password_lock_time_days;
     /**
-      read from the user config. The number of failed login attemps before the
+      read from the user config. The number of failed login attempts before the
       account is locked
     */
     uint m_failed_login_attempts;
     /**
-      The remaining login tries, valid ony if @ref m_failed_login_attempts and
+      The remaining login tries, valid only if @ref m_failed_login_attempts and
       @ref m_password_lock_time_days are non-zero
     */
     uint m_remaining_login_attempts;
     /** The day the account is locked, 0 if not locked */
     long m_daynr_locked;
   } password_locked_state;
+  I_multi_factor_auth *m_mfa;
 };
 
 class ACL_DB : public ACL_ACCESS {
@@ -368,7 +379,7 @@ class ACL_PROXY_USER : public ACL_ACCESS {
   } old_acl_proxy_users;
 
  public:
-  ACL_PROXY_USER() {}
+  ACL_PROXY_USER() = default;
 
   void init(const char *host_arg, const char *user_arg,
             const char *proxied_host_arg, const char *proxied_user_arg,
@@ -406,7 +417,7 @@ class ACL_PROXY_USER : public ACL_ACCESS {
          (host.get_host() && host_arg && !strcmp(host.get_host(), host_arg))));
   }
 
-  void print_grant(String *str);
+  void print_grant(THD *thd, String *str);
 
   void set_data(ACL_PROXY_USER *grant) { with_grant = grant->with_grant; }
 
@@ -421,20 +432,26 @@ class ACL_PROXY_USER : public ACL_ACCESS {
                                const LEX_CSTRING &proxied_host,
                                const LEX_CSTRING &proxied_user, bool with_grant,
                                const char *grantor);
+
+  size_t get_user_length() const { return user ? strlen(user) : 0; }
+
+  size_t get_proxied_user_length() const {
+    return proxied_user ? strlen(proxied_user) : 0;
+  }
 };
 
 class acl_entry {
  public:
-  ulong access;
+  Access_bitmask access;
   uint16 length;
   char key[1];  // Key will be stored here
 };
 
 class GRANT_COLUMN {
  public:
-  ulong rights;
+  Access_bitmask rights;
   std::string column;
-  GRANT_COLUMN(String &c, ulong y);
+  GRANT_COLUMN(String &c, Access_bitmask y);
 };
 
 class GRANT_NAME {
@@ -443,13 +460,13 @@ class GRANT_NAME {
   char *db;
   const char *user;
   char *tname;
-  ulong privs;
+  Access_bitmask privs;
   ulong sort;
   std::string hash_key;
   GRANT_NAME(const char *h, const char *d, const char *u, const char *t,
-             ulong p, bool is_routine);
+             Access_bitmask p, bool is_routine);
   GRANT_NAME(TABLE *form, bool is_routine);
-  virtual ~GRANT_NAME() {}
+  virtual ~GRANT_NAME() = default;
   virtual bool ok() { return privs != 0; }
   void set_user_details(const char *h, const char *d, const char *u,
                         const char *t, bool is_routine);
@@ -457,13 +474,13 @@ class GRANT_NAME {
 
 class GRANT_TABLE : public GRANT_NAME {
  public:
-  ulong cols;
+  Access_bitmask cols;
   collation_unordered_multimap<std::string,
                                unique_ptr_destroy_only<GRANT_COLUMN>>
       hash_columns;
 
   GRANT_TABLE(const char *h, const char *d, const char *u, const char *t,
-              ulong p, ulong c);
+              Access_bitmask p, Access_bitmask c);
   explicit GRANT_TABLE(TABLE *form);
   bool init(TABLE *col_privs);
   ~GRANT_TABLE() override;
@@ -487,13 +504,12 @@ class Acl_cache_allocator : public Malloc_allocator<T> {
   };
 
   template <class U>
-  Acl_cache_allocator(
-      const Acl_cache_allocator<U> &other MY_ATTRIBUTE((unused)))
+  Acl_cache_allocator(const Acl_cache_allocator<U> &other [[maybe_unused]])
       : Malloc_allocator<T>(key_memory_acl_cache) {}
 
   template <class U>
-  Acl_cache_allocator &operator=(
-      const Acl_cache_allocator<U> &other MY_ATTRIBUTE((unused))) {}
+  Acl_cache_allocator &operator=(const Acl_cache_allocator<U> &other
+                                 [[maybe_unused]]) {}
 };
 typedef Acl_cache_allocator<ACL_USER *> Acl_user_ptr_allocator;
 typedef std::list<ACL_USER *, Acl_user_ptr_allocator> Acl_user_ptr_list;
@@ -561,6 +577,7 @@ inline GRANT_NAME *routine_hash_search(const char *host, const char *ip,
                                        const char *db, const char *user,
                                        const char *tname, bool proc,
                                        bool exact) {
+  assert(proc ? proc_priv_hash : func_priv_hash);
   return name_hash_search(proc ? *proc_priv_hash : *func_priv_hash, host, ip,
                           db, user, tname, exact, true);
 }
@@ -568,6 +585,7 @@ inline GRANT_NAME *routine_hash_search(const char *host, const char *ip,
 inline GRANT_TABLE *table_hash_search(const char *host, const char *ip,
                                       const char *db, const char *user,
                                       const char *tname, bool exact) {
+  assert(column_priv_hash);
   return name_hash_search(*column_priv_hash, host, ip, db, user, tname, exact,
                           false);
 }
@@ -646,7 +664,7 @@ class Acl_map {
   void increase_reference_count();
   void decrease_reference_count();
 
-  ulong global_acl();
+  Access_bitmask global_acl();
   Db_access_map *db_acls();
   Db_access_map *db_wild_acls();
   Table_access_map *table_acls();
@@ -664,7 +682,7 @@ class Acl_map {
   Db_access_map m_db_acls;
   Db_access_map m_db_wild_acls;
   Table_access_map m_table_acls;
-  ulong m_global_acl;
+  Access_bitmask m_global_acl;
   SP_access_map m_sp_acls;
   SP_access_map m_func_acls;
   Grant_acl_set m_with_admin_acls;
@@ -719,6 +737,10 @@ class Acl_cache {
     Removes all acl map objects with a references count of zero.
   */
   void flush_cache();
+  /**
+    Removes all acl map objects when shutdown_acl_cache is called.
+  */
+  void clear_acl_cache();
   /**
     Return a lower boundary to the current version count.
   */
@@ -811,6 +833,58 @@ class Acl_restrictions {
   malloc_unordered_map<std::string, Restrictions> m_restrictions_map;
 };
 
+/**
+  Enables preserving temporary account locking attributes during ACL DDL.
+  Enables restoring temporary account locking attributes after ACL reload.
+
+  This class is used to preserve the state of the accounts being altered by the
+  current ACL statement. The account locking data needs to be preserved since
+  the current state of account locking is not stored into the table and can't be
+  restored from it when the code needs to re-create the ACL caches from the
+  tables.
+
+  When an ACL DDL statement that can modify account locking data starts, a new
+  instance of this class is created and the current in-memory account locking
+  data is preserved for each user that is modified by the statement, if account
+  locking data is not default.
+
+  ACL DDL rollback results in the in-memory ACL cache being re-created during
+  ACL reload.
+
+  After ACL reload:
+  - For all users in the new ACL cache, the temporary account locking state is
+    restored from the old ACL cache, if account locking data is not default.
+  - For specific users in the new ACL cache, the temporary account locking state
+    is restored from the instances of this class that were created at the start
+    of the ACL DDL statement. This needs to be done since these accounts could
+    be dropped (mysql_drop_user), renamed (mysql_rename_user) or altered
+    (mysql_alter_user) in the old ACL cache.
+*/
+class ACL_temporary_lock_state {
+ public:
+  ACL_temporary_lock_state(const char *host, const char *user,
+                           uint remaining_login_attempts, long daynr_locked);
+
+  static bool is_modified(ACL_USER *acl_user);
+
+  static ACL_USER *preserve_user_lock_state(const char *host, const char *user,
+                                            Lock_state_list &user_list);
+
+  static void restore_user_lock_state(const char *host, const char *user,
+                                      uint remaining_login_attempts,
+                                      long daynr_locked);
+
+  static void restore_temporary_account_locking(
+      Prealloced_array<ACL_USER, ACL_PREALLOC_SIZE> *old_acl_users,
+      Lock_state_list *modified_user_lock_state_list);
+
+ private:
+  const char *m_host;
+  const char *m_user;
+  const uint m_remaining_login_attempts;
+  const long m_daynr_locked;
+};
+
 extern ACL_USER acl_utility_user;
 
 #endif /* SQL_USER_CACHE_INCLUDED */
