@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -546,8 +547,8 @@ extern bool initialized;
     privilege mask
 */
 
-ulong get_access(TABLE *form, uint fieldnr, uint *next_field) {
-  ulong access_bits = 0, bit;
+Access_bitmask get_access(TABLE *form, uint fieldnr, uint *next_field) {
+  Access_bitmask access_bits = 0, bit;
   char buff[2];
   String res(buff, sizeof(buff), &my_charset_latin1);
   Field **pos;
@@ -603,8 +604,9 @@ void acl_notify_htons(THD *thd, enum_sql_command operation,
   @retval True  - Error.
 */
 
-static bool acl_end_trans_and_close_tables(THD *thd,
-                                           bool rollback_transaction) {
+static bool acl_end_trans_and_close_tables(
+    THD *thd, bool rollback_transaction,
+    Lock_state_list *modified_user_lock_state_list) {
   bool result;
 
   /*
@@ -650,7 +652,7 @@ static bool acl_end_trans_and_close_tables(THD *thd,
     */
     DBUG_EXECUTE_IF("wl14084_acl_ddl_error_before_cache_reload_trigger_timeout",
                     sleep(2););
-    reload_acl_caches(thd, true);
+    reload_acl_caches(thd, true, true, modified_user_lock_state_list);
     close_thread_tables(thd);
   }
   thd->mdl_context.release_transactional_locks();
@@ -674,17 +676,22 @@ static bool acl_end_trans_and_close_tables(THD *thd,
   @param write_to_binlog        Skip writing to binlog.
                                 Used for routine grants while
                                 creating routine.
+  @param modified_user_lock_state_list
+                                List of users whose temporary account
+                                locking attributes are likely modified.
 
   @returns status of log and commit
     @retval 0 Successfully committed. Optionally : written to binlog.
     @retval 1 If an error is raised at any stage
 */
 
-bool log_and_commit_acl_ddl(THD *thd, bool transactional_tables,
-                            std::set<LEX_USER *> *extra_users, /* = NULL */
-                            Rewrite_params *rewrite_params,    /* = NULL */
-                            bool extra_error,                  /* = true */
-                            bool write_to_binlog) {            /* = true */
+bool log_and_commit_acl_ddl(
+    THD *thd, bool transactional_tables,
+    std::set<LEX_USER *> *extra_users,                /* = NULL */
+    Rewrite_params *rewrite_params,                   /* = NULL */
+    bool extra_error,                                 /* = false */
+    bool write_to_binlog,                             /* = true */
+    Lock_state_list *modified_user_lock_state_list) { /* = nullptr */
   bool result = false;
   DBUG_TRACE;
   assert(thd);
@@ -763,7 +770,10 @@ bool log_and_commit_acl_ddl(THD *thd, bool transactional_tables,
     mysql_rewrite_acl_query(thd, rlb, Consumer_type::TEXTLOG, rewrite_params);
   }
 
-  if (acl_end_trans_and_close_tables(thd, result)) result = true;
+  if (acl_end_trans_and_close_tables(thd, result,
+                                     modified_user_lock_state_list)) {
+    result = true;
+  }
 
   return result;
 }
@@ -813,9 +823,10 @@ void acl_print_ha_error(int handler_error) {
 */
 
 int replace_db_table(THD *thd, TABLE *table, const char *db,
-                     const LEX_USER &combo, ulong rights, bool revoke_grant) {
+                     const LEX_USER &combo, Access_bitmask rights,
+                     bool revoke_grant) {
   uint i;
-  ulong priv, store_rights;
+  Access_bitmask priv, store_rights;
   bool old_row_exists = false;
   int error;
   char what = (revoke_grant) ? 'N' : 'Y';
@@ -1110,8 +1121,8 @@ table_error:
 
 int replace_column_table(THD *thd, GRANT_TABLE *g_t, TABLE *table,
                          const LEX_USER &combo, List<LEX_COLUMN> &columns,
-                         const char *db, const char *table_name, ulong rights,
-                         bool revoke_grant) {
+                         const char *db, const char *table_name,
+                         Access_bitmask rights, bool revoke_grant) {
   int result = 0;
   int error;
   uchar key[MAX_KEY_LENGTH];
@@ -1151,7 +1162,7 @@ int replace_column_table(THD *thd, GRANT_TABLE *g_t, TABLE *table,
   }
 
   while ((column = iter++)) {
-    ulong privileges = column->rights;
+    Access_bitmask privileges = column->rights;
     bool old_row_exists = false;
     uchar user_key[MAX_KEY_LENGTH];
 
@@ -1190,7 +1201,7 @@ int replace_column_table(THD *thd, GRANT_TABLE *g_t, TABLE *table,
       table->field[4]->store(column->column.ptr(), column->column.length(),
                              system_charset_info);
     } else {
-      ulong tmp = (ulong)table->field[6]->val_int();
+      Access_bitmask tmp = (Access_bitmask)table->field[6]->val_int();
       tmp = fix_rights_for_column(tmp);
 
       if (revoke_grant)
@@ -1292,7 +1303,7 @@ int replace_column_table(THD *thd, GRANT_TABLE *g_t, TABLE *table,
 
     /* Scan through all rows with the same host,db,user and table */
     do {
-      ulong privileges = (ulong)table->field[6]->val_int();
+      Access_bitmask privileges = (Access_bitmask)table->field[6]->val_int();
       privileges = fix_rights_for_column(privileges);
       store_record(table, record[1]);
 
@@ -1389,12 +1400,12 @@ int replace_table_table(THD *thd, GRANT_TABLE *grant_table,
                         std::unique_ptr<GRANT_TABLE, Destroy_only<GRANT_TABLE>>
                             *deleted_grant_table,
                         TABLE *table, const LEX_USER &combo, const char *db,
-                        const char *table_name, ulong rights, ulong col_rights,
-                        bool revoke_grant) {
+                        const char *table_name, Access_bitmask rights,
+                        Access_bitmask col_rights, bool revoke_grant) {
   char grantor[USER_HOST_BUFF_SIZE];
   int old_row_exists = 1;
   int error = 0;
-  ulong store_table_rights, store_col_rights;
+  Access_bitmask store_table_rights, store_col_rights;
   uchar user_key[MAX_KEY_LENGTH];
   Acl_table_intact table_intact(thd);
   DBUG_TRACE;
@@ -1453,10 +1464,10 @@ int replace_table_table(THD *thd, GRANT_TABLE *grant_table,
   store_table_rights = get_rights_for_table(rights);
   store_col_rights = get_rights_for_column(col_rights);
   if (old_row_exists) {
-    ulong j, k;
+    Access_bitmask j, k;
     store_record(table, record[1]);
-    j = (ulong)table->field[6]->val_int();
-    k = (ulong)table->field[7]->val_int();
+    j = (Access_bitmask)table->field[6]->val_int();
+    k = (Access_bitmask)table->field[7]->val_int();
 
     if (revoke_grant) {
       /* column rights are already fixed in mysql_table_grant */
@@ -1555,12 +1566,12 @@ table_error:
 
 int replace_routine_table(THD *thd, GRANT_NAME *grant_name, TABLE *table,
                           const LEX_USER &combo, const char *db,
-                          const char *routine_name, bool is_proc, ulong rights,
-                          bool revoke_grant) {
+                          const char *routine_name, bool is_proc,
+                          Access_bitmask rights, bool revoke_grant) {
   char grantor[USER_HOST_BUFF_SIZE];
   int old_row_exists = 1;
   int error = 0;
-  ulong store_proc_rights;
+  Access_bitmask store_proc_rights;
   Acl_table_intact table_intact(thd);
   uchar user_key[MAX_KEY_LENGTH];
   DBUG_TRACE;
@@ -1624,9 +1635,9 @@ int replace_routine_table(THD *thd, GRANT_NAME *grant_name, TABLE *table,
 
   store_proc_rights = get_rights_for_procedure(rights);
   if (old_row_exists) {
-    ulong j;
+    Access_bitmask j;
     store_record(table, record[1]);
-    j = (ulong)table->field[6]->val_int();
+    j = (Access_bitmask)table->field[6]->val_int();
 
     if (revoke_grant) {
       /* column rights are already fixed in mysql_table_grant */
