@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,17 +25,19 @@
 
 #include "sql/conn_handler/connection_handler_manager.h"
 
+#include <assert.h>
+#include <ctime>
 #include <new>
 
-#include "my_dbug.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "mysql/components/services/mysql_cond_bits.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/psi_cond_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
-#include "mysql/psi/psi_base.h"
+#include "mysql/components/services/bits/mysql_cond_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_cond_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/components/services/log_builtins.h"
 #include "mysql/service_thd_wait.h"
 #include "mysqld_error.h"                              // ER_*
 #include "sql/conn_handler/channel_info.h"             // Channel_info
@@ -55,11 +58,12 @@ struct Connection_handler_functions;
 uint Connection_handler_manager::connection_count = 0;
 ulong Connection_handler_manager::max_used_connections = 0;
 ulong Connection_handler_manager::max_used_connections_time = 0;
-THD_event_functions *Connection_handler_manager::event_functions = NULL;
-THD_event_functions *Connection_handler_manager::saved_event_functions = NULL;
+THD_event_functions *Connection_handler_manager::event_functions = nullptr;
+THD_event_functions *Connection_handler_manager::saved_event_functions =
+    nullptr;
 mysql_mutex_t Connection_handler_manager::LOCK_connection_count;
 mysql_cond_t Connection_handler_manager::COND_connection_count;
-Connection_handler_manager *Connection_handler_manager::m_instance = NULL;
+Connection_handler_manager *Connection_handler_manager::m_instance = nullptr;
 ulong Connection_handler_manager::thread_handling =
     SCHEDULER_ONE_THREAD_PER_CONNECTION;
 uint Connection_handler_manager::max_threads = 0;
@@ -132,7 +136,7 @@ bool Connection_handler_manager::check_and_incr_conn_count(
 
     if (connection_count > max_used_connections) {
       max_used_connections = connection_count;
-      max_used_connections_time = (ulong)my_time(0);
+      max_used_connections_time = time(nullptr);
     }
   }
   mysql_mutex_unlock(&LOCK_connection_count);
@@ -161,7 +165,7 @@ bool Connection_handler_manager::init() {
   */
   Per_thread_connection_handler::init();
 
-  Connection_handler *connection_handler = NULL;
+  Connection_handler *connection_handler = nullptr;
   switch (Connection_handler_manager::thread_handling) {
     case SCHEDULER_ONE_THREAD_PER_CONNECTION:
       connection_handler = new (std::nothrow) Per_thread_connection_handler();
@@ -173,10 +177,10 @@ bool Connection_handler_manager::init() {
       connection_handler = new (std::nothrow) Thread_pool_connection_handler();
       break;
     default:
-      DBUG_ASSERT(false);
+      assert(false);
   }
 
-  if (connection_handler == NULL) {
+  if (connection_handler == nullptr) {
     // This is a static member function.
     Per_thread_connection_handler::destroy();
     return true;
@@ -185,7 +189,7 @@ bool Connection_handler_manager::init() {
   m_instance =
       new (std::nothrow) Connection_handler_manager(connection_handler);
 
-  if (m_instance == NULL) {
+  if (m_instance == nullptr) {
     delete connection_handler;
     // This is a static member function.
     Per_thread_connection_handler::destroy();
@@ -219,17 +223,22 @@ bool Connection_handler_manager::init() {
 void Connection_handler_manager::wait_till_no_connection() {
   mysql_mutex_lock(&LOCK_connection_count);
   while (connection_count > 0) {
+    LogErr(INFORMATION_LEVEL, ER_WAITING_FOR_NO_CONNECTIONS, connection_count);
     mysql_cond_wait(&COND_connection_count, &LOCK_connection_count);
   }
   mysql_mutex_unlock(&LOCK_connection_count);
 }
 
+void Connection_handler_manager::post_daemonize_init() {
+  m_connection_handler->post_daemonize_init();
+}
+
 void Connection_handler_manager::destroy_instance() {
   Per_thread_connection_handler::destroy();
 
-  if (m_instance != NULL) {
+  if (m_instance != nullptr) {
     delete m_instance;
-    m_instance = NULL;
+    m_instance = nullptr;
     mysql_mutex_destroy(&LOCK_connection_count);
     mysql_cond_destroy(&COND_connection_count);
   }
@@ -238,15 +247,14 @@ void Connection_handler_manager::destroy_instance() {
 void Connection_handler_manager::reset_max_used_connections() {
   mysql_mutex_lock(&LOCK_connection_count);
   max_used_connections = connection_count;
-  max_used_connections_time = (ulong)my_time(0);
+  max_used_connections_time = time(nullptr);
   mysql_mutex_unlock(&LOCK_connection_count);
 }
 
 void Connection_handler_manager::load_connection_handler(
     Connection_handler *conn_handler) {
   // We don't support loading more than one dynamic connection handler
-  DBUG_ASSERT(Connection_handler_manager::thread_handling !=
-              SCHEDULER_TYPES_COUNT);
+  assert(Connection_handler_manager::thread_handling != SCHEDULER_TYPES_COUNT);
   m_saved_connection_handler = m_connection_handler;
   m_saved_thread_handling = Connection_handler_manager::thread_handling;
   m_connection_handler = conn_handler;
@@ -255,12 +263,12 @@ void Connection_handler_manager::load_connection_handler(
 }
 
 bool Connection_handler_manager::unload_connection_handler() {
-  DBUG_ASSERT(m_saved_connection_handler != NULL);
-  if (m_saved_connection_handler == NULL) return true;
+  assert(m_saved_connection_handler != nullptr);
+  if (m_saved_connection_handler == nullptr) return true;
   delete m_connection_handler;
   m_connection_handler = m_saved_connection_handler;
   Connection_handler_manager::thread_handling = m_saved_thread_handling;
-  m_saved_connection_handler = NULL;
+  m_saved_connection_handler = nullptr;
   m_saved_thread_handling = 0;
   max_threads = m_connection_handler->get_max_threads();
   return false;
@@ -284,7 +292,7 @@ void Connection_handler_manager::process_new_connection(
 
 THD *create_thd(Channel_info *channel_info) {
   THD *thd = channel_info->create_thd();
-  if (thd == NULL)
+  if (thd == nullptr)
     channel_info->send_error_and_close_channel(ER_OUT_OF_RESOURCES, 0, false);
 
   return thd;
@@ -302,12 +310,12 @@ void increment_aborted_connects() {
 
 int my_connection_handler_set(Connection_handler_functions *chf,
                               THD_event_functions *tef) {
-  DBUG_ASSERT(chf != NULL && tef != NULL);
-  if (chf == NULL || tef == NULL) return 1;
+  assert(chf != nullptr && tef != nullptr);
+  if (chf == nullptr || tef == nullptr) return 1;
 
   Plugin_connection_handler *conn_handler =
       new (std::nothrow) Plugin_connection_handler(chf);
-  if (conn_handler == NULL) return 1;
+  if (conn_handler == nullptr) return 1;
 
   Connection_handler_manager::get_instance()->load_connection_handler(
       conn_handler);
