@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,18 +24,21 @@
 #ifndef DD_CACHE__DICTIONARY_CLIENT_INCLUDED
 #define DD_CACHE__DICTIONARY_CLIENT_INCLUDED
 
+#include <assert.h>
 #include <stddef.h>
+#include <functional>
 #include <memory>
 #include <string>
 #include <vector>
 
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "object_registry.h"  // Object_registry
 #include "sql/dd/object_id.h"
 #include "sql/dd/string_type.h"
 
 class THD;
+struct LEX_USER;
 
 namespace dd {
 class Schema;
@@ -49,7 +53,7 @@ class SPI_lru_cache;
 
 /**
   A smart-pointer for managing an SPI_lru_cache even when it is only
-  forward declared. Automaticlly allocated cache with new, and assigns
+  forward declared. Automatically allocated cache with new, and assigns
   m_spi_lru_cache to it, when dereferenced using non-const
   operator->(). Destructor deletes the object pointed to by
   m_spi_lru_cache.
@@ -122,7 +126,7 @@ class SPI_lru_cache_owner_ptr {
   objects, while the registers in the auto releasers are used for
   releasing objects.
 
-  The client also has a second registery of objects with uncommitted changes.
+  The client also has a second registry of objects with uncommitted changes.
   These are objects acquired by acquire_for_modification() or registered
   with register_uncommitted_object(). These objects are only present in
   the local registry and not in the shared cache. Once registered, the
@@ -188,7 +192,7 @@ class Dictionary_client {
     template <typename T>
     void auto_release(Cache_element<T> *element) {
       // Catch situations where we do not use a non-default releaser.
-      DBUG_ASSERT(m_prev != NULL);
+      assert(m_prev != nullptr);
       m_release_registry.put(element);
     }
 
@@ -295,8 +299,8 @@ class Dictionary_client {
   */
 
   template <typename K, typename T>
-  bool acquire(const K &key, const T **object, bool *local_committed,
-               bool *local_uncommitted) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire(const K &key, const T **object,
+                             bool *local_committed, bool *local_uncommitted);
 
   /**
     Get an uncommitted dictionary object that can be modified safely.
@@ -363,17 +367,21 @@ class Dictionary_client {
 
   template <typename T>
   void auto_delete(T *object) {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     // Make sure we do not sign up a shared object for auto delete.
     Cache_element<typename T::Cache_partition> *element = nullptr;
     m_registry_committed.get(
         static_cast<const typename T::Cache_partition *>(object), &element);
-    DBUG_ASSERT(element == nullptr);
+    assert(element == nullptr);
 
     // Make sure we do not sign up an uncommitted object for auto delete.
     m_registry_uncommitted.get(
         static_cast<const typename T::Cache_partition *>(object), &element);
-    DBUG_ASSERT(element == nullptr);
+    assert(element == nullptr);
+
+    // We must require a top level non-default releaser to ensure a
+    // predictable life span of the objects.
+    assert(m_current_releaser != &m_default_releaser);
 #endif
 
     m_uncached_objects.push_back(object);
@@ -388,12 +396,12 @@ class Dictionary_client {
 
   template <typename T>
   void no_auto_delete(T *object) {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
     // Make sure the object has been registered as uncommitted.
     Cache_element<typename T::Cache_partition> *element = nullptr;
     m_registry_uncommitted.get(
         static_cast<const typename T::Cache_partition *>(object), &element);
-    DBUG_ASSERT(element != nullptr);
+    assert(element != nullptr);
 #endif
     m_uncached_objects.erase(std::remove(m_uncached_objects.begin(),
                                          m_uncached_objects.end(), object),
@@ -480,8 +488,23 @@ class Dictionary_client {
   */
 
   template <typename Object_type>
-  bool fetch(Const_ptr_vec<Object_type> *coll, const Object_key *object_key)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch(Const_ptr_vec<Object_type> *coll,
+                           const Object_key *object_key);
+
+  /**
+    Auxiliary function to retrieve an object by its object id without caching
+    it.It is responsibility of the caller to delete the retrieved object.
+  */
+  template <typename T>
+  bool acquire_uncached_impl(Object_id id, T **object);
+
+  /**
+    Auxiliary function to retrieve a possibly uncommitted object by its object
+    id without caching it. It is responsibility of the caller to delete the
+    retrieved object.
+  */
+  template <typename T>
+  bool acquire_uncached_uncommitted_impl(Object_id id, T **object);
 
  public:
   // Initialize an instance with a default auto releaser.
@@ -490,6 +513,8 @@ class Dictionary_client {
   // Make sure all objects are released.
   ~Dictionary_client();
 
+  MY_COMPILER_DIAGNOSTIC_PUSH()
+  MY_COMPILER_CLANG_WORKAROUND_TPARAM_DOCBUG()
   /**
     Retrieve an object by its object id.
 
@@ -500,10 +525,10 @@ class Dictionary_client {
     @retval       false   No error.
     @retval       true    Error (from handling a cache miss).
   */
+  MY_COMPILER_DIAGNOSTIC_POP()
 
   template <typename T>
-  bool acquire(Object_id id, const T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire(Object_id id, const T **object);
 
   /**
     Retrieve an object by its object id.
@@ -519,8 +544,7 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire_for_modification(Object_id id, T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_for_modification(Object_id id, T **object);
 
   /**
     Retrieve an object by its object id without caching it.
@@ -539,8 +563,26 @@ class Dictionary_client {
    */
 
   template <typename T>
-  bool acquire_uncached(Object_id id, T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_uncached(Object_id id, T **object);
+
+  /**
+    Retrieve an object by its object id without caching it.
+
+    The object is not cached and owned by the caller through unique_ptr
+    it provides. The object may not be used as a parameter to the other
+    dictionary client methods since it is not known by the object registry.
+
+    @tparam       T           Dictionary object type.
+    @param        id          Object id to retrieve.
+    @param [out]  object_ptr  Smart pointer with dictionary object,
+                              if present or with nullptr otherwise.
+
+    @retval       false   No error.
+    @retval       true    Error (from reading the dictionary tables).
+  */
+  template <typename T>
+  [[nodiscard]] bool acquire_uncached(Object_id id,
+                                      std::unique_ptr<T> *object_ptr);
 
   /**
     Retrieve a possibly uncommitted object by its object id without caching it.
@@ -563,8 +605,30 @@ class Dictionary_client {
    */
 
   template <typename T>
-  bool acquire_uncached_uncommitted(Object_id id, T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_uncached_uncommitted(Object_id id, T **object);
+
+  /**
+    Retrieve a possibly uncommitted object by its object id without caching it.
+
+    The object is not cached and owned by the caller through unique_ptr
+    it provides. The object may not be used as a parameter to the other
+    dictionary client methods since it is not known by the object registry.
+
+    When the object is read from the persistent tables, the transaction
+    isolation level is READ UNCOMMITTED. This is necessary to be able to
+    read uncommitted data from an earlier stage of the same session.
+
+    @tparam       T           Dictionary object type.
+    @param        id          Object id to retrieve.
+    @param [out]  object_ptr  Smart pointer with dictionary object,
+                              if present or with nullptr otherwise.
+
+    @retval       false   No error.
+    @retval       true    Error (from reading the dictionary tables).
+  */
+  template <typename T>
+  [[nodiscard]] bool acquire_uncached_uncommitted(
+      Object_id id, std::unique_ptr<T> *object_ptr);
 
   /**
     Retrieve an object by its name.
@@ -578,8 +642,7 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire(const String_type &object_name, const T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire(const String_type &object_name, const T **object);
 
   /**
     Retrieve an object by its name.
@@ -595,8 +658,8 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire_for_modification(const String_type &object_name, T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_for_modification(const String_type &object_name,
+                                              T **object);
 
   /**
     Retrieve an object by its schema- and object name.
@@ -621,8 +684,8 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire(const String_type &schema_name, const String_type &object_name,
-               const T **object) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire(const String_type &schema_name,
+                             const String_type &object_name, const T **object);
 
   /**
     Retrieve an object by its schema- and object name.
@@ -649,9 +712,9 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire_for_modification(const String_type &schema_name,
-                                const String_type &object_name, T **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_for_modification(const String_type &schema_name,
+                                              const String_type &object_name,
+                                              T **object);
 
   /**
     Retrieve an object by its schema- and object name.
@@ -682,9 +745,9 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire(const String_type &schema_name, const String_type &object_name,
-               const typename T::Cache_partition **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire(const String_type &schema_name,
+                             const String_type &object_name,
+                             const typename T::Cache_partition **object);
 
   /**
     Retrieve an object by its schema- and object name.
@@ -717,17 +780,18 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool acquire_for_modification(const String_type &schema_name,
-                                const String_type &object_name,
-                                typename T::Cache_partition **object)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_for_modification(
+      const String_type &schema_name, const String_type &object_name,
+      typename T::Cache_partition **object);
 
   /**
     Retrieve a table object by its se private id.
 
-    @param       engine        Name of the engine storing the table.
-    @param       se_private_id SE private id of the table.
-    @param [out] table         Table object, if present; otherwise NULL.
+    @param       engine          Name of the engine storing the table.
+    @param       se_private_id   SE private id of the table.
+    @param [out] table           Table object, if present; otherwise NULL.
+    @param       skip_spi_cache  Optionally skip looking into the missing
+                                 SE private ID cache. Defaults to false.
 
     @note The object must be acquired uncached since we cannot acquire a
           metadata lock in advance since we do not know the table name.
@@ -739,10 +803,9 @@ class Dictionary_client {
                                  object of a wrong type was found).
   */
 
-  bool acquire_uncached_table_by_se_private_id(const String_type &engine,
-                                               Object_id se_private_id,
-                                               Table **table)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_uncached_table_by_se_private_id(
+      const String_type &engine, Object_id se_private_id, Table **table,
+      bool skip_spi_cache = false);
 
   /**
     Retrieve a table object by its partition se private id.
@@ -755,9 +818,8 @@ class Dictionary_client {
     @retval      true     Error (from handling a cache miss).
   */
 
-  bool acquire_uncached_table_by_partition_se_private_id(
-      const String_type &engine, Object_id se_partition_id, Table **table)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool acquire_uncached_table_by_partition_se_private_id(
+      const String_type &engine, Object_id se_partition_id, Table **table);
 
   /**
     Retrieve schema and table name by the se private id of the table.
@@ -773,11 +835,10 @@ class Dictionary_client {
     @retval      true     Error.
   */
 
-  bool get_table_name_by_se_private_id(const String_type &engine,
-                                       Object_id se_private_id,
-                                       String_type *schema_name,
-                                       String_type *table_name)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool get_table_name_by_se_private_id(const String_type &engine,
+                                                     Object_id se_private_id,
+                                                     String_type *schema_name,
+                                                     String_type *table_name);
 
   /**
     Retrieve schema and table name by the se private id of the partition.
@@ -793,11 +854,9 @@ class Dictionary_client {
     @retval      true     Error.
   */
 
-  bool get_table_name_by_partition_se_private_id(const String_type &engine,
-                                                 Object_id se_partition_id,
-                                                 String_type *schema_name,
-                                                 String_type *table_name)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool get_table_name_by_partition_se_private_id(
+      const String_type &engine, Object_id se_partition_id,
+      String_type *schema_name, String_type *table_name);
 
   /**
     Retrieve a table name of a given trigger name and schema.
@@ -812,10 +871,9 @@ class Dictionary_client {
     @retval      true     Error.
   */
 
-  bool get_table_name_by_trigger_name(const Schema &schema,
-                                      const String_type &trigger_name,
-                                      String_type *table_name)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool get_table_name_by_trigger_name(
+      const Schema &schema, const String_type &trigger_name,
+      String_type *table_name);
 
   /**
     Check if schema contains foreign key with specified name.
@@ -830,10 +888,8 @@ class Dictionary_client {
     @retval      true     Error.
   */
 
-  bool check_foreign_key_exists(const Schema &schema,
-                                const String_type &foreign_key_name,
-                                bool *exists)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool check_foreign_key_exists(
+      const Schema &schema, const String_type &foreign_key_name, bool *exists);
 
   /**
     Check if schema contains check constraint with specified name.
@@ -856,9 +912,6 @@ class Dictionary_client {
     Fetch the names of the components in the schema. Hidden components are
     ignored. E.g., Object with dd::Table::hidden() == true will be ignored.
 
-    @note          This is an intermediate solution which will be replaced
-                   by the implementation in WL#6599.
-
     @tparam        T              Type of object to retrieve names for.
     @param         schema         Schema for which to get component names.
     @param   [out] names          An std::vector containing all object names.
@@ -868,9 +921,39 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool fetch_schema_component_names(const Schema *schema,
-                                    std::vector<String_type> *names) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_schema_component_names(
+      const Schema *schema, std::vector<String_type> *names) const;
+
+  /**
+    Fetch the names of the tables in the schema belonging to specific
+    storage engine. E.g., Object with dd::Table::hidden() == true will be
+    ignored.
+
+    @param         schema         Schema for which to get component names.
+    @param         engine         Engine name of tables to match.
+    @param   [out] names          An std::vector containing all object names.
+
+    @return      true   Failure (error is reported).
+    @return      false  Success.
+  */
+
+  [[nodiscard]] bool fetch_schema_table_names_by_engine(
+      const Schema *schema, const String_type &engine,
+      std::vector<String_type> *names) const;
+
+  /**
+    Fetch the names of the server tables in the schema.  Ignore tables
+    hidden by SE.
+
+    @param         schema         Schema for which to get component names.
+    @param   [out] names          An std::vector containing all object names.
+
+    @return      true   Failure (error is reported).
+    @return      false  Success.
+  */
+
+  [[nodiscard]] bool fetch_schema_table_names_not_hidden_by_se(
+      const Schema *schema, std::vector<String_type> *names) const;
 
   /**
     Fetch all global component ids of the given type.
@@ -883,8 +966,8 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool fetch_global_component_ids(std::vector<Object_id> *ids) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_global_component_ids(
+      std::vector<Object_id> *ids) const;
 
   /**
     Fetch all global component names of the given type.
@@ -897,8 +980,27 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool fetch_global_component_names(std::vector<String_type> *names) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_global_component_names(
+      std::vector<String_type> *names) const;
+
+  /**
+    Execute the submitted lambda function for each entity of the given type
+    selected by the submitted key. If the lambda returns true, iteration
+    stops and the function returns.
+
+    @tparam Object_type Entity type to examine.
+    @param  object_key  Key to use for selecting entities.
+    @param  processor   Lambda to execute for each entity.
+    @return      true   Failure (error is reported unless the lambda
+                        returned true).
+    @return      false  Success.
+  */
+
+  template <typename Object_type>
+  [[nodiscard]] bool foreach (
+      const Object_key *object_key,
+      std::function<bool(std::unique_ptr<Object_type> &)> const &processor)
+      const;
 
   /**
     Fetch all components in the schema.
@@ -912,8 +1014,8 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool fetch_schema_components(const Schema *schema, Const_ptr_vec<T> *coll)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_schema_components(const Schema *schema,
+                                             Const_ptr_vec<T> *coll);
 
   /**
     Fetch all global components of the given type.
@@ -926,8 +1028,22 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool fetch_global_components(Const_ptr_vec<T> *coll)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_global_components(Const_ptr_vec<T> *coll);
+
+  /**
+     Check if a user is referenced as definer by some object of the given type.
+
+     @tparam        T              Type of dictionary objects to check.
+     @param         user           User name, including @ and host.
+     @param   [out] is_definer     True if the user is referenced as definer
+                                   by some object.
+
+     @return      true   Failure (error is reported, is_definer is undefined).
+     @return      false  Success.
+   */
+  template <typename T>
+  [[nodiscard]] bool is_user_definer(const LEX_USER &user,
+                                     bool *is_definer) const;
 
   /**
     Fetch Object ids of all the views referencing base table/ view/ stored
@@ -946,10 +1062,9 @@ class Dictionary_client {
     @return      false  Success.
   */
   template <typename T>
-  bool fetch_referencing_views_object_id(const char *schema,
-                                         const char *tbl_or_sf_name,
-                                         std::vector<Object_id> *view_ids) const
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_referencing_views_object_id(
+      const char *schema, const char *tbl_or_sf_name,
+      std::vector<Object_id> *view_ids) const;
 
   /**
     Fetch the names of tables (children) which have foreign keys
@@ -975,13 +1090,11 @@ class Dictionary_client {
           DROP TABLE or DROP FOREIGN KEY.
   */
 
-  bool fetch_fk_children_uncached(const String_type &parent_schema,
-                                  const String_type &parent_name,
-                                  const String_type &parent_engine,
-                                  bool uncommitted,
-                                  std::vector<String_type> *children_schemas,
-                                  std::vector<String_type> *children_names)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool fetch_fk_children_uncached(
+      const String_type &parent_schema, const String_type &parent_name,
+      const String_type &parent_engine, bool uncommitted,
+      std::vector<String_type> *children_schemas,
+      std::vector<String_type> *children_names);
 
   /**
      Invalidate a cache entry.
@@ -999,8 +1112,8 @@ class Dictionary_client {
                                   failing to get an MDL lock).
    */
 
-  bool invalidate(const String_type &schema_name, const String_type &table_name)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool invalidate(const String_type &schema_name,
+                                const String_type &table_name);
 
   /**
     Invalidate a cache entry.
@@ -1037,7 +1150,7 @@ class Dictionary_client {
           makes sure there is an exclusive meta data lock on the object
           name.
 
-    @note The argument to this funcion may come from acquire(), and may
+    @note The argument to this function may come from acquire(), and may
           be an instance that is present in the uncommitted registry,
           or in the committed registry. These use cases are handled by
           the implementation of the function. The ownership of the
@@ -1052,7 +1165,7 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool drop(const T *object) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool drop(const T *object);
 
   /**
     Store a new dictionary object.
@@ -1080,7 +1193,7 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool store(T *object) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool store(T *object);
 
   /**
     Update a persisted dictionary object, but keep the shared cache unchanged.
@@ -1089,7 +1202,7 @@ class Dictionary_client {
     verifying that an object with the same id already exists. The old object,
     which may be present in the shared dictionary cache, is not modified. To
     make the changes visible in the shared cache, please call
-    remove_uncommuitted_objects().
+    remove_uncommitted_objects().
 
     @note A precondition is that the object has been acquired from the
           shared cache indirectly by acquire_for_modification(). For storing
@@ -1108,7 +1221,7 @@ class Dictionary_client {
   */
 
   template <typename T>
-  bool update(T *new_object) MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool update(T *new_object);
 
   /**
     Remove the uncommitted objects from the client.
@@ -1143,9 +1256,8 @@ class Dictionary_client {
     @return true  - on failure
     @return false - on success
   */
-  bool remove_table_dynamic_statistics(const String_type &schema_name,
-                                       const String_type &table_name)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] bool remove_table_dynamic_statistics(
+      const String_type &schema_name, const String_type &table_name);
 
   /**
     Debug dump of a partition of the client and its registry to stderr.
