@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +52,8 @@
 #include "sql/dd/impl/tables/table_partitions.h"  // dd::tables::Table_partitions
 #include "sql/dd/impl/tables/tables.h"            // dd::tables::Tables
 #include "sql/dd/impl/tables/tablespaces.h"       // dd::tables::Tablespaces
-#include "sql/dd/impl/utils.h"                    // dd::tables::Tablespaces
+#include "sql/dd/impl/upgrade/server.h"
+#include "sql/dd/impl/utils.h"            // dd::tables::Tablespaces
 #include "sql/dd/info_schema/metadata.h"  // dd::info_schema::store_dynamic...
 #include "sql/dd/types/abstract_table.h"  // dd::Abstract_table::DD_table
 #include "sql/dd/types/column.h"          // dd::Column::DD_table
@@ -65,6 +67,7 @@
 #include "sql/derror.h"
 #include "sql/handler.h"
 #include "sql/mdl.h"
+#include "sql/mysqld.h"                 //next_query_id()
 #include "sql/opt_costconstantcache.h"  // init_optimizer_cost_module
 #include "sql/plugin_table.h"
 #include "sql/sql_base.h"   // close_cached_tables
@@ -75,6 +78,9 @@
 #include "sql/transaction.h"                    // trans_commit()
 #include "storage/perfschema/pfs_dd_version.h"  // PFS_DD_VERSION
 
+extern Cost_constant_cache *cost_constant_cache;  // defined in
+                                                  // opt_costconstantcache.cc
+
 ///////////////////////////////////////////////////////////////////////////
 
 namespace dd {
@@ -86,7 +92,7 @@ namespace dd {
 class Object_table;
 class Table;
 
-Dictionary_impl *Dictionary_impl::s_instance = NULL;
+Dictionary_impl *Dictionary_impl::s_instance = nullptr;
 
 Dictionary_impl *Dictionary_impl::instance() { return s_instance; }
 
@@ -99,7 +105,7 @@ const String_type Dictionary_impl::DEFAULT_CATALOG_NAME("def");
 bool Dictionary_impl::init(enum_dd_init_type dd_init) {
   if (dd_init == enum_dd_init_type::DD_INITIALIZE ||
       dd_init == enum_dd_init_type::DD_RESTART_OR_UPGRADE) {
-    DBUG_ASSERT(!Dictionary_impl::s_instance);
+    assert(!Dictionary_impl::s_instance);
 
     if (Dictionary_impl::s_instance) return false; /* purecov: inspected */
 
@@ -115,7 +121,11 @@ bool Dictionary_impl::init(enum_dd_init_type dd_init) {
     Upgrade process needs heap engine initialized, hence parameter 'true'
     is passed to the function.
   */
-  init_optimizer_cost_module(true);
+  bool cost_constant_inited = false;
+  if (cost_constant_cache == nullptr) {
+    init_optimizer_cost_module(true);
+    cost_constant_inited = true;
+  }
 
   // Disable table encryption privilege checks for system threads.
   bool saved_table_encryption_privilege_check =
@@ -131,13 +141,14 @@ bool Dictionary_impl::init(enum_dd_init_type dd_init) {
 
   // Creation of Data Dictionary through current server
   if (dd_init == enum_dd_init_type::DD_INITIALIZE)
-    result = ::bootstrap::run_bootstrap_thread(NULL, &bootstrap::initialize,
-                                               SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(
+        nullptr, nullptr, &bootstrap::initialize, SYSTEM_THREAD_DD_INITIALIZE);
 
   // Creation of INFORMATION_SCHEMA system views.
   else if (dd_init == enum_dd_init_type::DD_INITIALIZE_SYSTEM_VIEWS)
-    result = ::bootstrap::run_bootstrap_thread(
-        NULL, &dd::info_schema::initialize, SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(nullptr, nullptr,
+                                               &dd::info_schema::initialize,
+                                               SYSTEM_THREAD_DD_INITIALIZE);
 
   /*
     Creation of Dictionary Tables in old Data Directory
@@ -145,30 +156,38 @@ bool Dictionary_impl::init(enum_dd_init_type dd_init) {
   */
   else if (dd_init == enum_dd_init_type::DD_RESTART_OR_UPGRADE)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &upgrade_57::do_pre_checks_and_initialize_dd,
+        nullptr, nullptr, &upgrade_57::do_pre_checks_and_initialize_dd,
         SYSTEM_THREAD_DD_INITIALIZE);
 
   // Populate metadata in DD tables from old data directory and do cleanup.
   else if (dd_init == enum_dd_init_type::DD_POPULATE_UPGRADE)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &upgrade_57::fill_dd_and_finalize, SYSTEM_THREAD_DD_INITIALIZE);
+        nullptr, nullptr, &upgrade_57::fill_dd_and_finalize,
+        SYSTEM_THREAD_DD_INITIALIZE);
 
   // Delete DD tables and do cleanup in case of error in upgrade
   else if (dd_init == enum_dd_init_type::DD_DELETE)
-    result = ::bootstrap::run_bootstrap_thread(NULL, &upgrade_57::terminate,
-                                               SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(
+        nullptr, nullptr, &upgrade_57::terminate, SYSTEM_THREAD_DD_INITIALIZE);
 
   // Update server and plugin I_S table metadata into DD tables.
   else if (dd_init == enum_dd_init_type::DD_UPDATE_I_S_METADATA)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &dd::info_schema::update_I_S_metadata,
+        nullptr, nullptr, &dd::info_schema::update_I_S_metadata,
+        SYSTEM_THREAD_DD_INITIALIZE);
+
+  // Creation of non-dd-based INFORMATION_SCHEMA system views.
+  else if (dd_init ==
+           enum_dd_init_type::DD_INITIALIZE_NON_DD_BASED_SYSTEM_VIEWS)
+    result = ::bootstrap::run_bootstrap_thread(
+        nullptr, nullptr, &dd::info_schema::init_non_dd_based_system_view,
         SYSTEM_THREAD_DD_INITIALIZE);
 
   // Restore the table_encryption_privilege_check.
   opt_table_encryption_privilege_check = saved_table_encryption_privilege_check;
 
   /* Now that the dd is initialized, delete the cost model. */
-  delete_optimizer_cost_module();
+  if (cost_constant_inited) delete_optimizer_cost_module();
 
   return result;
 }
@@ -179,7 +198,7 @@ bool Dictionary_impl::shutdown() {
   if (!Dictionary_impl::s_instance) return true;
 
   delete Dictionary_impl::s_instance;
-  Dictionary_impl::s_instance = NULL;
+  Dictionary_impl::s_instance = nullptr;
 
   return false;
 }
@@ -195,10 +214,10 @@ uint Dictionary_impl::get_target_dd_version() { return dd::DD_VERSION; }
 uint Dictionary_impl::get_actual_dd_version(THD *thd) {
   bool exists = false;
   uint version = 0;
-  bool error MY_ATTRIBUTE((unused)) = tables::DD_properties::instance().get(
+  bool error [[maybe_unused]] = tables::DD_properties::instance().get(
       thd, "DD_VERSION", &version, &exists);
-  DBUG_ASSERT(!error);
-  DBUG_ASSERT(exists);
+  assert(!error);
+  assert(exists);
   return version;
 }
 
@@ -213,10 +232,10 @@ uint Dictionary_impl::get_target_I_S_version() {
 uint Dictionary_impl::get_actual_I_S_version(THD *thd) {
   bool exists = false;
   uint version = 0;
-  bool error MY_ATTRIBUTE((unused)) = tables::DD_properties::instance().get(
+  bool error [[maybe_unused]] = tables::DD_properties::instance().get(
       thd, "IS_VERSION", &version, &exists);
-  DBUG_ASSERT(!error);
-  DBUG_ASSERT(exists);
+  assert(!error);
+  assert(exists);
   return version;
 }
 
@@ -235,15 +254,29 @@ uint Dictionary_impl::get_target_P_S_version() { return PFS_DD_VERSION; }
 uint Dictionary_impl::get_actual_P_S_version(THD *thd) {
   bool exists = false;
   uint version = 0;
-  bool error MY_ATTRIBUTE((unused)) = tables::DD_properties::instance().get(
+  bool error [[maybe_unused]] = tables::DD_properties::instance().get(
       thd, "PS_VERSION", &version, &exists);
-  DBUG_ASSERT(!error);
-  DBUG_ASSERT(exists);
+  assert(!error);
+  assert(exists);
   return version;
 }
 
 ///////////////////////////////////////////////////////////////////////////
 
+bool Dictionary_impl::get_actual_ndbinfo_schema_version(THD *thd, uint *ver) {
+  bool exists = false;
+  tables::DD_properties::instance().get(thd, "NDBINFO_VERSION", ver, &exists);
+  return exists;
+}
+
+///////////////////////////////////////////////////////////////////////////
+
+uint Dictionary_impl::set_ndbinfo_schema_version(THD *thd, uint version) {
+  return tables::DD_properties::instance().set(thd, "NDBINFO_VERSION", version);
+}
+
+///////////////////////////////////////////////////////////////////////////
+
 uint Dictionary_impl::set_P_S_version(THD *thd, uint version) {
   return tables::DD_properties::instance().set(thd, "PS_VERSION", version);
 }
@@ -252,7 +285,7 @@ uint Dictionary_impl::set_P_S_version(THD *thd, uint version) {
 
 const Object_table *Dictionary_impl::get_dd_table(
     const String_type &schema_name, const String_type &table_name) const {
-  if (!is_dd_schema_name(schema_name)) return NULL;
+  if (!is_dd_schema_name(schema_name)) return nullptr;
 
   return System_tables::instance()->find_table(schema_name, table_name);
 }
@@ -394,17 +427,38 @@ bool Dictionary_impl::is_system_view_name(const char *schema_name,
   Following are couple of API's that InnoDB needs to acquire MDL locks.
 */
 
-static bool acquire_mdl(THD *thd, MDL_key::enum_mdl_namespace lock_namespace,
-                        const char *schema_name, const char *table_name,
-                        bool no_wait, ulong lock_wait_timeout,
-                        enum_mdl_type lock_type,
-                        enum_mdl_duration lock_duration,
-                        MDL_ticket **out_mdl_ticket) {
-  DBUG_ENTER("dd::acquire_mdl");
+static bool acquire_shared_mdl(THD *thd,
+                               MDL_key::enum_mdl_namespace lock_namespace,
+                               const char *schema_name, const char *table_name,
+                               bool no_wait, ulong lock_wait_timeout,
+                               enum_mdl_duration lock_duration,
+                               MDL_ticket **out_mdl_ticket) {
+  DBUG_TRACE;
 
   MDL_request mdl_request;
   MDL_REQUEST_INIT(&mdl_request, lock_namespace, schema_name, table_name,
-                   lock_type, lock_duration);
+                   MDL_SHARED, lock_duration);
+
+  if (no_wait) {
+    if (thd->mdl_context.try_acquire_lock(&mdl_request)) return true;
+  } else {
+    if (thd->mdl_context.acquire_lock(&mdl_request, lock_wait_timeout))
+      return true;
+  }
+
+  if (out_mdl_ticket) *out_mdl_ticket = mdl_request.ticket;
+
+  return false;
+}
+
+static bool acquire_exclusive_mdl(THD *thd,
+                                  MDL_key::enum_mdl_namespace lock_namespace,
+                                  const char *schema_name,
+                                  const char *table_name, bool no_wait,
+                                  ulong lock_wait_timeout,
+                                  enum_mdl_duration lock_duration,
+                                  MDL_ticket **out_mdl_ticket) {
+  DBUG_TRACE;
 
   /*
     If there is a request for an exclusive lock, we also need to acquire
@@ -413,24 +467,22 @@ static bool acquire_mdl(THD *thd, MDL_key::enum_mdl_namespace lock_namespace,
     setting the backup- or global read lock).
   */
   MDL_request_list mdl_requests;
+  MDL_request mdl_request, grl_request, bl_request;
+
+  // If we cannot acquire protection against GRL, err out.
+  if (thd->global_read_lock.can_acquire_protection()) return true;
+
+  MDL_REQUEST_INIT(&mdl_request, lock_namespace, schema_name, table_name,
+                   MDL_EXCLUSIVE, lock_duration);
   mdl_requests.push_front(&mdl_request);
 
-  MDL_request *grl_request = nullptr;
-  MDL_request *bl_request = nullptr;
-  if (lock_type == MDL_EXCLUSIVE) {
-    // If we cannot acquire protection against GRL, err out.
-    if (thd->global_read_lock.can_acquire_protection()) return true;
+  MDL_REQUEST_INIT(&grl_request, MDL_key::GLOBAL, "", "",
+                   MDL_INTENTION_EXCLUSIVE, MDL_TRANSACTION);
+  mdl_requests.push_front(&grl_request);
 
-    grl_request = new (thd->mem_root) MDL_request;
-    MDL_REQUEST_INIT(grl_request, MDL_key::GLOBAL, "", "",
-                     MDL_INTENTION_EXCLUSIVE, MDL_TRANSACTION);
-    mdl_requests.push_front(grl_request);
-
-    bl_request = new (thd->mem_root) MDL_request;
-    MDL_REQUEST_INIT(bl_request, MDL_key::BACKUP_LOCK, "", "",
-                     MDL_INTENTION_EXCLUSIVE, MDL_TRANSACTION);
-    mdl_requests.push_front(bl_request);
-  }
+  MDL_REQUEST_INIT(&bl_request, MDL_key::BACKUP_LOCK, "", "",
+                   MDL_INTENTION_EXCLUSIVE, MDL_TRANSACTION);
+  mdl_requests.push_front(&bl_request);
 
   /*
     With no_wait, we acquire the locks one by one. When waiting,
@@ -439,26 +491,33 @@ static bool acquire_mdl(THD *thd, MDL_key::enum_mdl_namespace lock_namespace,
   */
   if (no_wait) {
     if (thd->mdl_context.try_acquire_lock(&mdl_request) ||
-        (grl_request != nullptr &&
-         thd->mdl_context.try_acquire_lock(bl_request)) ||
-        (bl_request != nullptr &&
-         thd->mdl_context.try_acquire_lock(grl_request))) {
-      DBUG_RETURN(true);
+        thd->mdl_context.try_acquire_lock(&bl_request) ||
+        thd->mdl_context.try_acquire_lock(&grl_request)) {
+      return true;
     }
   } else if (thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout))
-    DBUG_RETURN(true);
+    return true;
+
+  /*
+    Unlike in other places where we acquire protection against global read
+    lock, the read_only state is not checked here since it is handled by
+    the caller or extra steps are taken to correctly ignore it. Also checking
+    read_only state can be problematic for background threads like drop table
+    thread and purge thread which can be initiated on behalf of statements
+    executed by replication thread where the read_only state does not apply.
+  */
 
   if (out_mdl_ticket) *out_mdl_ticket = mdl_request.ticket;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 bool acquire_shared_table_mdl(THD *thd, const char *schema_name,
                               const char *table_name, bool no_wait,
                               MDL_ticket **out_mdl_ticket) {
-  return acquire_mdl(thd, MDL_key::TABLE, schema_name, table_name, no_wait,
-                     thd->variables.lock_wait_timeout, MDL_SHARED, MDL_EXPLICIT,
-                     out_mdl_ticket);
+  return acquire_shared_mdl(thd, MDL_key::TABLE, schema_name, table_name,
+                            no_wait, thd->variables.lock_wait_timeout,
+                            MDL_EXPLICIT, out_mdl_ticket);
 }
 
 bool has_shared_table_mdl(THD *thd, const char *schema_name,
@@ -473,21 +532,13 @@ bool has_exclusive_table_mdl(THD *thd, const char *schema_name,
       MDL_key::TABLE, schema_name, table_name, MDL_EXCLUSIVE);
 }
 
-bool acquire_exclusive_tablespace_mdl(THD *thd, const char *tablespace_name,
-                                      unsigned long int lock_wait_timeout,
-                                      MDL_ticket **ticket, bool for_trx) {
-  enum_mdl_duration duration = (for_trx ? MDL_TRANSACTION : MDL_EXPLICIT);
-  return acquire_mdl(thd, MDL_key::TABLESPACE, "", tablespace_name, false,
-                     lock_wait_timeout, MDL_EXCLUSIVE, duration, ticket);
-}
-
 bool acquire_exclusive_tablespace_mdl(THD *thd, const char *tablespace_name,
                                       bool no_wait, MDL_ticket **ticket,
                                       bool for_trx) {
   enum_mdl_duration duration = (for_trx ? MDL_TRANSACTION : MDL_EXPLICIT);
-  return acquire_mdl(thd, MDL_key::TABLESPACE, "", tablespace_name, no_wait,
-                     thd->variables.lock_wait_timeout, MDL_EXCLUSIVE, duration,
-                     ticket);
+  return acquire_exclusive_mdl(thd, MDL_key::TABLESPACE, "", tablespace_name,
+                               no_wait, thd->variables.lock_wait_timeout,
+                               duration, ticket);
 }
 
 bool acquire_shared_tablespace_mdl(THD *thd, const char *tablespace_name,
@@ -495,9 +546,9 @@ bool acquire_shared_tablespace_mdl(THD *thd, const char *tablespace_name,
                                    bool for_trx) {
   // When requesting a tablespace name lock, we leave the schema name empty.
   enum_mdl_duration duration = (for_trx ? MDL_TRANSACTION : MDL_EXPLICIT);
-  return acquire_mdl(thd, MDL_key::TABLESPACE, "", tablespace_name, no_wait,
-                     thd->variables.lock_wait_timeout, MDL_SHARED, duration,
-                     ticket);
+  return acquire_shared_mdl(thd, MDL_key::TABLESPACE, "", tablespace_name,
+                            no_wait, thd->variables.lock_wait_timeout, duration,
+                            ticket);
 }
 
 bool has_shared_tablespace_mdl(THD *thd, const char *tablespace_name) {
@@ -515,33 +566,31 @@ bool has_exclusive_tablespace_mdl(THD *thd, const char *tablespace_name) {
 bool acquire_exclusive_table_mdl(THD *thd, const char *schema_name,
                                  const char *table_name, bool no_wait,
                                  MDL_ticket **out_mdl_ticket) {
-  return acquire_mdl(thd, MDL_key::TABLE, schema_name, table_name, no_wait,
-                     thd->variables.lock_wait_timeout, MDL_EXCLUSIVE,
-                     MDL_TRANSACTION, out_mdl_ticket);
+  return acquire_exclusive_mdl(thd, MDL_key::TABLE, schema_name, table_name,
+                               no_wait, thd->variables.lock_wait_timeout,
+                               MDL_TRANSACTION, out_mdl_ticket);
 }
 
 bool acquire_exclusive_table_mdl(THD *thd, const char *schema_name,
                                  const char *table_name,
                                  unsigned long int lock_wait_timeout,
                                  MDL_ticket **out_mdl_ticket) {
-  return acquire_mdl(thd, MDL_key::TABLE, schema_name, table_name, false,
-                     lock_wait_timeout, MDL_EXCLUSIVE, MDL_TRANSACTION,
-                     out_mdl_ticket);
+  return acquire_exclusive_mdl(thd, MDL_key::TABLE, schema_name, table_name,
+                               false, lock_wait_timeout, MDL_TRANSACTION,
+                               out_mdl_ticket);
 }
 
 bool acquire_exclusive_schema_mdl(THD *thd, const char *schema_name,
                                   bool no_wait, MDL_ticket **out_mdl_ticket) {
-  return acquire_mdl(thd, MDL_key::SCHEMA, schema_name, "", no_wait,
-                     thd->variables.lock_wait_timeout, MDL_EXCLUSIVE,
-                     MDL_EXPLICIT, out_mdl_ticket);
+  return acquire_exclusive_mdl(thd, MDL_key::SCHEMA, schema_name, "", no_wait,
+                               thd->variables.lock_wait_timeout, MDL_EXPLICIT,
+                               out_mdl_ticket);
 }
 
 void release_mdl(THD *thd, MDL_ticket *mdl_ticket) {
-  DBUG_ENTER("dd::release_mdl");
+  DBUG_TRACE;
 
   thd->mdl_context.release_lock(mdl_ticket);
-
-  DBUG_VOID_RETURN;
 }
 
 /* purecov: begin deadcode */
@@ -552,8 +601,8 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
   if (dd::get_dictionary()->is_dd_table_name(pt->get_schema_name(),
                                              pt->get_name())) {
     my_error(ER_NO_SYSTEM_TABLE_ACCESS, MYF(0),
-             ER_THD(thd, dd::get_dictionary()->table_type_error_code(
-                             pt->get_schema_name(), pt->get_name())),
+             ER_THD_NONCONST(thd, dd::get_dictionary()->table_type_error_code(
+                                      pt->get_schema_name(), pt->get_name())),
              pt->get_schema_name(), pt->get_name());
 
     return true;
@@ -573,7 +622,8 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
   /*
     1. Mark that we are executing a special DDL during
     plugin initialization. This will enable DDL to not be
-    committed. The called of this API would commit the transaction.
+    committed or binlogged. The called of this API would commit
+    the transaction.
 
     2. Remove metadata of native table if already exists. This could
     happen if server was crashed and restarted.
@@ -583,23 +633,26 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
     4. Undo 1.
   */
   dd::cache::Dictionary_client *client = thd->dd_client();
-  const dd::Table *table_def = NULL;
+  const dd::Table *table_def = nullptr;
   if (client->acquire(pt->get_schema_name(), pt->get_name(), &table_def))
     return true;
 
   thd->mark_plugin_fake_ddl(true);
-  ulong master_access = thd->security_context()->master_access();
-  thd->security_context()->set_master_access(~(ulong)0);
+  Access_bitmask master_access = thd->security_context()->master_access();
+  thd->security_context()->set_master_access(ALL_ACCESS);
+  {
+    Disable_binlog_guard guard(thd);
 
-  // Drop the table and related dynamic statistics too.
-  if (table_def) {
-    error =
-        client->drop(table_def) || client->remove_table_dynamic_statistics(
-                                       pt->get_schema_name(), pt->get_name());
+    // Drop the table and related dynamic statistics too.
+    if (table_def) {
+      error =
+          client->drop(table_def) || client->remove_table_dynamic_statistics(
+                                         pt->get_schema_name(), pt->get_name());
+    }
+
+    if (!error) error = dd::execute_query(thd, pt->get_ddl());
   }
 
-  if (!error) error = dd::execute_query(thd, pt->get_ddl());
-
   thd->security_context()->set_master_access(master_access);
   thd->mark_plugin_fake_ddl(false);
 
@@ -611,8 +664,8 @@ bool drop_native_table(THD *thd, const char *schema_name,
                        const char *table_name) {
   if (dd::get_dictionary()->is_dd_table_name(schema_name, table_name)) {
     my_error(ER_NO_SYSTEM_TABLE_ACCESS, MYF(0),
-             ER_THD(thd, dd::get_dictionary()->table_type_error_code(
-                             schema_name, table_name)),
+             ER_THD_NONCONST(thd, dd::get_dictionary()->table_type_error_code(
+                                      schema_name, table_name)),
              schema_name, table_name);
 
     return true;
@@ -629,7 +682,7 @@ bool drop_native_table(THD *thd, const char *schema_name,
     return true;
 
   dd::cache::Dictionary_client *client = thd->dd_client();
-  const dd::Table *table_def = NULL;
+  const dd::Table *table_def = nullptr;
   if (client->acquire(schema_name, table_name, &table_def)) {
     // Error is reported by the dictionary subsystem.
     return true;
@@ -704,4 +757,21 @@ void rename_tablespace_mdl_hook(THD *thd, MDL_ticket *src, MDL_ticket *dst) {
   thd->locked_tables_list.add_rename_tablespace_mdls(src, dst);
 }
 
+bool alter_tablespace_encryption(THD *thd, const char *tablespace_name,
+                                 bool encryption) {
+  dd::upgrade::Bootstrap_error_handler error_handler;
+  bool save_log_error = dd::upgrade::Bootstrap_error_handler::m_log_error;
+  error_handler.set_log_error(false);
+
+  thd->set_query_id(next_query_id());
+
+  dd::String_type query = dd::String_type("ALTER TABLESPACE ") +
+                          tablespace_name + dd::String_type(" ENCRYPTION = ") +
+                          dd::String_type(encryption ? "'Y'" : "'N'");
+
+  bool res = execute_query(thd, query);
+  error_handler.set_log_error(save_log_error);
+  return res;
+}
+
 }  // namespace dd
