@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +23,7 @@
 
 #include "sql/dd/impl/types/table_impl.h"
 
+#include <assert.h>
 #include <string.h>
 #include <set>
 #include <sstream>
@@ -33,29 +35,31 @@
 #include <rapidjson/prettywriter.h>
 
 #include "m_string.h"
-#include "my_dbug.h"
+
 #include "my_sys.h"
-#include "mysqld_error.h"                     // ER_*
-#include "sql/current_thd.h"                  // current_thd
-#include "sql/dd/impl/bootstrap_ctx.h"        // dd::bootstrap::DD_bootstrap_ctx
-#include "sql/dd/impl/dictionary_impl.h"      // Dictionary_impl
-#include "sql/dd/impl/properties_impl.h"      // Properties_impl
-#include "sql/dd/impl/raw/raw_record.h"       // Raw_record
-#include "sql/dd/impl/raw/raw_record_set.h"   // Raw_record_set
-#include "sql/dd/impl/raw/raw_table.h"        // Raw_table
-#include "sql/dd/impl/sdi_impl.h"             // sdi read/write functions
-#include "sql/dd/impl/tables/columns.h"       // Columns
-#include "sql/dd/impl/tables/foreign_keys.h"  // Foreign_keys
-#include "sql/dd/impl/tables/indexes.h"       // Indexes
-#include "sql/dd/impl/tables/schemata.h"      // Schemata
-#include "sql/dd/impl/tables/table_partitions.h"  // Table_partitions
-#include "sql/dd/impl/tables/tables.h"            // Tables
-#include "sql/dd/impl/tables/triggers.h"          // Triggers
-#include "sql/dd/impl/transaction_impl.h"         // Open_dictionary_tables_ctx
-#include "sql/dd/impl/types/foreign_key_impl.h"   // Foreign_key_impl
-#include "sql/dd/impl/types/index_impl.h"         // Index_impl
-#include "sql/dd/impl/types/partition_impl.h"     // Partition_impl
-#include "sql/dd/impl/types/trigger_impl.h"       // Trigger_impl
+#include "mysqld_error.h"                         // ER_*
+#include "sql/current_thd.h"                      // current_thd
+#include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // dd::bootstrap::DD_bootstrap_ctx
+#include "sql/dd/impl/dictionary_impl.h"          // Dictionary_impl
+#include "sql/dd/impl/properties_impl.h"          // Properties_impl
+#include "sql/dd/impl/raw/raw_record.h"           // Raw_record
+#include "sql/dd/impl/raw/raw_record_set.h"       // Raw_record_set
+#include "sql/dd/impl/raw/raw_table.h"            // Raw_table
+#include "sql/dd/impl/sdi_impl.h"                 // sdi read/write functions
+#include "sql/dd/impl/tables/check_constraints.h"  // Check_constraints
+#include "sql/dd/impl/tables/columns.h"            // Columns
+#include "sql/dd/impl/tables/foreign_keys.h"       // Foreign_keys
+#include "sql/dd/impl/tables/indexes.h"            // Indexes
+#include "sql/dd/impl/tables/schemata.h"           // Schemata
+#include "sql/dd/impl/tables/table_partitions.h"   // Table_partitions
+#include "sql/dd/impl/tables/tables.h"             // Tables
+#include "sql/dd/impl/tables/triggers.h"           // Triggers
+#include "sql/dd/impl/transaction_impl.h"          // Open_dictionary_tables_ctx
+#include "sql/dd/impl/types/check_constraint_impl.h"  // Check_constraint_impl
+#include "sql/dd/impl/types/foreign_key_impl.h"       // Foreign_key_impl
+#include "sql/dd/impl/types/index_impl.h"             // Index_impl
+#include "sql/dd/impl/types/partition_impl.h"         // Partition_impl
+#include "sql/dd/impl/types/trigger_impl.h"           // Trigger_impl
 #include "sql/dd/properties.h"
 #include "sql/dd/string_type.h"   // dd::String_type
 #include "sql/dd/types/column.h"  // Column
@@ -65,6 +69,7 @@
 #include "sql/dd/types/weak_object.h"
 #include "sql/sql_class.h"
 
+using dd::tables::Check_constraints;
 using dd::tables::Foreign_keys;
 using dd::tables::Indexes;
 using dd::tables::Table_partitions;
@@ -76,19 +81,13 @@ namespace dd {
 class Sdi_rcontext;
 class Sdi_wcontext;
 
-static const std::set<String_type> default_valid_se_private_data_keys = {
-    // NDB keys:
-    "object_version",
-    // InnoDB keys:
-    "autoinc", "data_directory", "discard", "instant_col", "version"};
-
 ///////////////////////////////////////////////////////////////////////////
 // Table_impl implementation.
 ///////////////////////////////////////////////////////////////////////////
 
 Table_impl::Table_impl()
     : m_se_private_id(INVALID_OBJECT_ID),
-      m_se_private_data(default_valid_se_private_data_keys),
+      m_se_private_data(),
       m_row_format(RF_FIXED),
       m_is_temporary(false),
       m_partition_type(PT_NONE),
@@ -99,6 +98,7 @@ Table_impl::Table_impl()
       m_foreign_keys(),
       m_partitions(),
       m_triggers(),
+      m_check_constraints(),
       m_collation_id(INVALID_OBJECT_ID),
       m_tablespace_id(INVALID_OBJECT_ID) {}
 
@@ -134,14 +134,14 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
 
   // 1. Read the parent's schema name based on schema_id.
   Raw_table *schema_table = otx->get_table<dd::Schema>();
-  DBUG_ASSERT(schema_table);
+  assert(schema_table);
   Primary_id_key schema_pk(schema_id());
 
   std::unique_ptr<Raw_record_set> schema_rs;
   if (schema_table->open_record_set(&schema_pk, schema_rs)) return true;
 
   Raw_record *schema_rec = schema_rs->current_record();
-  DBUG_ASSERT(schema_rec);
+  assert(schema_rec);
   if (schema_rec == nullptr) return true;
 
   // 2. Build a key for searching the FK table.
@@ -155,7 +155,7 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
 
   // 3. Get the FK record set where this table is parent.
   Raw_table *foreign_key_table = otx->get_table<dd::Foreign_key>();
-  DBUG_ASSERT(foreign_key_table);
+  assert(foreign_key_table);
 
   std::unique_ptr<Raw_record_set> child_fk_rs;
   if (foreign_key_table->open_record_set(&parent_ref_key, child_fk_rs))
@@ -167,13 +167,13 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
     Primary_id_key child_pk(
         child_fk_rec->read_int(tables::Foreign_keys::FIELD_TABLE_ID));
     Raw_table *tables_table = otx->get_table<dd::Table>();
-    DBUG_ASSERT(tables_table);
+    assert(tables_table);
 
     std::unique_ptr<Raw_record_set> child_table_rs;
     if (tables_table->open_record_set(&child_pk, child_table_rs)) return true;
 
     Raw_record *child_table = child_table_rs->current_record();
-    DBUG_ASSERT(child_table);
+    assert(child_table);
     if (child_table == nullptr) return true;
 
     /*
@@ -195,7 +195,7 @@ bool Table_impl::load_foreign_key_parents(Open_dictionary_tables_ctx *otx) {
     if (schema_table->open_record_set(&schema_pk, schema_rs)) return true;
 
     schema_rec = schema_rs->current_record();
-    DBUG_ASSERT(schema_rec);
+    assert(schema_rec);
     if (schema_rec == nullptr) return true;
 
     // 6. Collect the relevant information.
@@ -236,7 +236,7 @@ bool Table_impl::reload_foreign_key_parents(THD *thd) {
   // Register and open tables.
   trx.otx.register_tables<dd::Table>();
   if (trx.otx.open_tables()) {
-    DBUG_ASSERT(thd->is_system_thread() || thd->killed || thd->is_error());
+    assert(thd->is_system_thread() || thd->killed || thd->is_error());
     return true;
   }
 
@@ -257,6 +257,14 @@ bool Table_impl::restore_children(Open_dictionary_tables_ctx *otx) {
   //   - Partitions should be loaded at the end, as it refers to
   //     indexes.
 
+  /*
+    Do not load check constraints if upgrade is from the DD version before
+    check constraints support. Check constraint support is introduced in 80016.
+  */
+  bool skip_check_constraints =
+      (bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
+          bootstrap::DD_VERSION_80016));
+
   return (
       Abstract_table_impl::restore_children(otx) ||
       m_indexes.restore_items(this, otx, otx->get_table<Index>(),
@@ -274,7 +282,12 @@ bool Table_impl::restore_children(Open_dictionary_tables_ctx *otx) {
       m_triggers.restore_items(this, otx, otx->get_table<Trigger>(),
                                Triggers::create_key_by_table_id(this->id()),
                                Trigger_order_comparator()) ||
-      load_foreign_key_parents(otx));
+      load_foreign_key_parents(otx) ||
+      (!skip_check_constraints &&
+       m_check_constraints.restore_items(
+           this, otx, otx->get_table<Check_constraint>(),
+           Check_constraints::create_key_by_table_id(this->id()),
+           Check_constraint_order_comparator())));
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -342,11 +355,20 @@ bool Table_impl::store_triggers(Open_dictionary_tables_ctx *otx) {
 ///////////////////////////////////////////////////////////////////////////
 
 bool Table_impl::store_children(Open_dictionary_tables_ctx *otx) {
+  /*
+    Do not store check constraints if upgrade is from the DD version before
+    check constraints support. Check constraint support is introduced in 80016.
+  */
+  bool skip_check_constraints =
+      (bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
+          bootstrap::DD_VERSION_80016));
+
   return Abstract_table_impl::store_children(otx) ||
          // Note that indexes has to be stored first, as
          // partitions refer indexes.
          m_indexes.store_items(otx) || m_foreign_keys.store_items(otx) ||
-         m_partitions.store_items(otx) || store_triggers(otx);
+         m_partitions.store_items(otx) || store_triggers(otx) ||
+         (!skip_check_constraints && m_check_constraints.store_items(otx));
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -355,7 +377,10 @@ bool Table_impl::drop_children(Open_dictionary_tables_ctx *otx) const {
   // Note that partition collection has to be dropped first
   // as it has foreign key to indexes.
 
-  return m_triggers.drop_items(otx, otx->get_table<Trigger>(),
+  return m_check_constraints.drop_items(
+             otx, otx->get_table<Check_constraint>(),
+             Check_constraints::create_key_by_table_id(this->id())) ||
+         m_triggers.drop_items(otx, otx->get_table<Trigger>(),
                                Triggers::create_key_by_table_id(this->id())) ||
          m_partitions.drop_items(
              otx, otx->get_table<Partition>(),
@@ -406,6 +431,14 @@ bool Table_impl::restore_attributes(const Raw_record &r) {
 
   set_se_private_data(r.read_str(Tables::FIELD_SE_PRIVATE_DATA, ""));
 
+  // m_engine_attribute and m_secondary_engine_attribute added in 80021
+  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
+          bootstrap::DD_VERSION_80021)) {
+    m_engine_attribute = r.read_str(Tables::FIELD_ENGINE_ATTRIBUTE, "");
+    m_secondary_engine_attribute =
+        r.read_str(Tables::FIELD_SECONDARY_ENGINE_ATTRIBUTE, "");
+  }
+
   m_engine = r.read_str(Tables::FIELD_ENGINE);
 
   // m_last_checked_for_upgrade_version added in 80012
@@ -451,7 +484,7 @@ bool Table_impl::store_attributes(Raw_record *r) {
   //
 
   // Temporary table definitions are never persisted.
-  DBUG_ASSERT(!m_is_temporary);
+  assert(!m_is_temporary);
 
   // Store last_checked_for_upgrade_version_id only if we're not upgrading
   if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
@@ -461,6 +494,18 @@ bool Table_impl::store_attributes(Raw_record *r) {
     return true;
   }
 
+  // Store engine_attribute and secondary_engine_attribute only if
+  // we're not upgrading
+  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
+          bootstrap::DD_VERSION_80021) &&
+      (r->store(Tables::FIELD_ENGINE_ATTRIBUTE, m_engine_attribute,
+                m_engine_attribute.empty()) ||
+       r->store(Tables::FIELD_SECONDARY_ENGINE_ATTRIBUTE,
+                m_secondary_engine_attribute,
+                m_secondary_engine_attribute.empty()))) {
+    return true;
+  }
+
   // Store field values
   return Abstract_table_impl::store_attributes(r) ||
          r->store(Tables::FIELD_ENGINE, m_engine) ||
@@ -494,10 +539,12 @@ bool Table_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-
+static_assert(Tables::NUMBER_OF_FIELDS == 37,
+              "Tables definition has changed, check if serialize() and "
+              "deserialize() need to be updated!");
 void Table_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
   // Temporary table definitions are never persisted.
-  DBUG_ASSERT(!m_is_temporary);
+  assert(!m_is_temporary);
 
   w->StartObject();
   Abstract_table_impl::serialize(wctx, w);
@@ -507,6 +554,9 @@ void Table_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
         STRING_WITH_LEN("last_checked_for_upgrade_version_id"));
   write(w, m_comment, STRING_WITH_LEN("comment"));
   write_properties(w, m_se_private_data, STRING_WITH_LEN("se_private_data"));
+  write(w, m_engine_attribute, STRING_WITH_LEN("engine_attribute"));
+  write(w, m_secondary_engine_attribute,
+        STRING_WITH_LEN("secondary_engine_attribute"));
   write_enum(w, m_row_format, STRING_WITH_LEN("row_format"));
   write_enum(w, m_partition_type, STRING_WITH_LEN("partition_type"));
   write(w, m_partition_expression, STRING_WITH_LEN("partition_expression"));
@@ -523,6 +573,8 @@ void Table_impl::serialize(Sdi_wcontext *wctx, Sdi_writer *w) const {
              STRING_WITH_LEN("default_subpartitioning"));
   serialize_each(wctx, w, m_indexes, STRING_WITH_LEN("indexes"));
   serialize_each(wctx, w, m_foreign_keys, STRING_WITH_LEN("foreign_keys"));
+  serialize_each(wctx, w, m_check_constraints,
+                 STRING_WITH_LEN("check_constraints"));
   serialize_each(wctx, w, m_partitions, STRING_WITH_LEN("partitions"));
   write(w, m_collation_id, STRING_WITH_LEN("collation_id"));
   serialize_tablespace_ref(wctx, w, m_tablespace_id,
@@ -540,6 +592,8 @@ bool Table_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
        "last_checked_for_upgrade_version_id");
   read(&m_comment, val, "comment");
   read_properties(&m_se_private_data, val, "se_private_data");
+  read(&m_engine_attribute, val, "engine_attribute");
+  read(&m_secondary_engine_attribute, val, "secondary_engine_attribute");
   read_enum(&m_row_format, val, "row_format");
   read_enum(&m_partition_type, val, "partition_type");
   read(&m_partition_expression, val, "partition_expression");
@@ -552,7 +606,7 @@ bool Table_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
 
   // Note! Deserialization of ordinal position cross-referenced
   // objects (i.e. Index and Column) must happen before deserializing
-  // objects which refrence these objects:
+  // objects which reference these objects:
   // Foreign_key_element -> Column,
   // Foreign_key         -> Index,
   // Index_element       -> Column,
@@ -561,12 +615,16 @@ bool Table_impl::deserialize(Sdi_rcontext *rctx, const RJ_Value &val) {
   // (as we don't know the address of the referenced Column or Index
   // object).
 
-  deserialize_each(rctx, [this]() { return add_index(); }, val, "indexes");
+  deserialize_each(
+      rctx, [this]() { return add_index(); }, val, "indexes");
 
-  deserialize_each(rctx, [this]() { return add_foreign_key(); }, val,
-                   "foreign_keys");
-  deserialize_each(rctx, [this]() { return add_partition(); }, val,
-                   "partitions");
+  deserialize_each(
+      rctx, [this]() { return add_foreign_key(); }, val, "foreign_keys");
+  deserialize_each(
+      rctx, [this]() { return add_check_constraint(); }, val,
+      "check_constraints");
+  deserialize_each(
+      rctx, [this]() { return add_partition(); }, val, "partitions");
   read(&m_collation_id, val, "collation_id");
   return deserialize_tablespace_ref(rctx, &m_tablespace_id, val,
                                     "tablespace_id");
@@ -586,6 +644,8 @@ void Table_impl::debug_print(String_type &outb) const {
      << "m_comment: " << m_comment << "; "
      << "m_se_private_data " << m_se_private_data.raw_string() << "; "
      << "m_se_private_id: {OID: " << m_se_private_id << "}; "
+     << "m_engine_attribute: " << m_engine_attribute << "; "
+     << "m_secondary_engine_attribute: " << m_secondary_engine_attribute << "; "
      << "m_row_format: " << m_row_format << "; "
      << "m_is_temporary: " << m_is_temporary << "; "
      << "m_tablespace: {OID: " << m_tablespace_id << "}; "
@@ -602,9 +662,9 @@ void Table_impl::debug_print(String_type &outb) const {
 
   {
     for (const Partition *i : partitions()) {
-      String_type s;
-      i->debug_print(s);
-      ss << s << " | ";
+      String_type sp;
+      i->debug_print(sp);
+      ss << sp << " | ";
     }
   }
 
@@ -612,9 +672,9 @@ void Table_impl::debug_print(String_type &outb) const {
 
   {
     for (const Index *i : indexes()) {
-      String_type s;
-      i->debug_print(s);
-      ss << s << " | ";
+      String_type si;
+      i->debug_print(si);
+      ss << si << " | ";
     }
   }
 
@@ -622,9 +682,19 @@ void Table_impl::debug_print(String_type &outb) const {
 
   {
     for (const Foreign_key *fk : foreign_keys()) {
-      String_type s;
-      fk->debug_print(s);
-      ss << s << " | ";
+      String_type sfk;
+      fk->debug_print(sfk);
+      ss << sfk << " | ";
+    }
+  }
+
+  ss << "] m_check_constraints: " << m_check_constraints.size() << " [ ";
+
+  {
+    for (const Check_constraint *cc : check_constraints()) {
+      String_type scc;
+      cc->debug_print(scc);
+      ss << scc << " | ";
     }
   }
 
@@ -632,9 +702,9 @@ void Table_impl::debug_print(String_type &outb) const {
 
   {
     for (const Trigger *trig : triggers()) {
-      String_type s;
-      trig->debug_print(s);
-      ss << s << " | ";
+      String_type st;
+      trig->debug_print(st);
+      ss << st << " | ";
     }
   }
   ss << "] ";
@@ -669,7 +739,7 @@ Index *Table_impl::get_index(Object_id index_id) {
     if (i->id() == index_id) return i;
   }
 
-  return NULL;
+  return nullptr;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -710,7 +780,24 @@ Partition *Table_impl::get_partition(Object_id partition_id) {
     if (i->id() == partition_id) return i;
   }
 
-  return NULL;
+  return nullptr;
+}
+
+///////////////////////////////////////////////////////////////////////////
+Partition *Table_impl::get_leaf_partition(const std::string &part_name) {
+  return const_cast<Partition *>(
+      std::as_const(*this).get_leaf_partition(part_name));
+}
+
+const Partition *Table_impl::get_leaf_partition(
+    const std::string &part_name) const {
+  for (Partition *leaf : m_leaf_partitions) {
+    if (!my_strcasecmp(system_charset_info, leaf->name().c_str(),
+                       part_name.c_str())) {
+      return leaf;
+    }
+  }
+  return nullptr;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -787,8 +874,8 @@ const Trigger *Table_impl::get_trigger(const char *name) const {
 Trigger *Table_impl::add_trigger_following(const Trigger *trigger,
                                            Trigger::enum_action_timing at,
                                            Trigger::enum_event_type et) {
-  DBUG_ASSERT(trigger != nullptr && trigger->action_timing() == at &&
-              trigger->event_type() == et);
+  assert(trigger != nullptr && trigger->action_timing() == at &&
+         trigger->event_type() == et);
 
   // Allocate new Trigger object.
   Trigger_impl *new_trigger = create_trigger();
@@ -814,8 +901,8 @@ Trigger *Table_impl::add_trigger_following(const Trigger *trigger,
 Trigger *Table_impl::add_trigger_preceding(const Trigger *trigger,
                                            Trigger::enum_action_timing at,
                                            Trigger::enum_event_type et) {
-  DBUG_ASSERT(trigger != nullptr && trigger->action_timing() == at &&
-              trigger->event_type() == et);
+  assert(trigger != nullptr && trigger->action_timing() == at &&
+         trigger->event_type() == et);
 
   Trigger_impl *new_trigger = create_trigger();
   if (new_trigger == nullptr) return nullptr;
@@ -835,7 +922,7 @@ Trigger *Table_impl::add_trigger_preceding(const Trigger *trigger,
 ///////////////////////////////////////////////////////////////////////////
 
 void Table_impl::copy_triggers(const Table *tab_obj) {
-  DBUG_ASSERT(tab_obj != nullptr);
+  assert(tab_obj != nullptr);
 
   for (const Trigger *trig : tab_obj->triggers()) {
     /*
@@ -873,7 +960,7 @@ void Table_impl::copy_triggers(const Table *tab_obj) {
     */
     Trigger_impl *new_trigger =
         new Trigger_impl(*dynamic_cast<const Trigger_impl *>(trig), this);
-    DBUG_ASSERT(new_trigger != nullptr);
+    assert(new_trigger != nullptr);
 
     new_trigger->set_id(INVALID_OBJECT_ID);
 
@@ -888,7 +975,7 @@ void Table_impl::drop_all_triggers() { m_triggers.remove_all(); }
 ///////////////////////////////////////////////////////////////////////////
 
 void Table_impl::drop_trigger(const Trigger *trigger) {
-  DBUG_ASSERT(trigger != nullptr);
+  assert(trigger != nullptr);
   dd::Trigger::enum_action_timing at = trigger->action_timing();
   dd::Trigger::enum_event_type et = trigger->event_type();
 
@@ -898,6 +985,16 @@ void Table_impl::drop_trigger(const Trigger *trigger) {
   reorder_action_order(at, et);
 }
 
+///////////////////////////////////////////////////////////////////////////
+// Check constraint collection.
+///////////////////////////////////////////////////////////////////////////
+
+Check_constraint *Table_impl::add_check_constraint() {
+  Check_constraint_impl *cc = new (std::nothrow) Check_constraint_impl(this);
+  if (cc != nullptr) m_check_constraints.push_back(cc);
+  return cc;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 Partition *Table_impl::get_partition(const String_type &name) {
@@ -905,7 +1002,7 @@ Partition *Table_impl::get_partition(const String_type &name) {
     if (i->name() == name) return i;
   }
 
-  return NULL;
+  return nullptr;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -929,6 +1026,13 @@ void Table_impl::register_tables(Open_dictionary_tables_ctx *otx) {
   otx->register_tables<Foreign_key>();
   otx->register_tables<Partition>();
   otx->register_tables<Trigger>();
+  /*
+    Do not register check constraint table if upgrade is from the DD version
+    before check constraints support. Check constraint is introduced in 8.0.15.
+  */
+  if (!bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade_from_before(
+          bootstrap::DD_VERSION_80016))
+    otx->register_tables<Check_constraint>();
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -942,6 +1046,8 @@ Table_impl::Table_impl(const Table_impl &src)
       m_last_checked_for_upgrade_version_id{
           src.m_last_checked_for_upgrade_version_id},
       m_se_private_data(src.m_se_private_data),
+      m_engine_attribute(src.m_engine_attribute),
+      m_secondary_engine_attribute(src.m_secondary_engine_attribute),
       m_row_format(src.m_row_format),
       m_is_temporary(src.m_is_temporary),
       m_partition_type(src.m_partition_type),
@@ -956,6 +1062,7 @@ Table_impl::Table_impl(const Table_impl &src)
       m_foreign_keys(),
       m_partitions(),
       m_triggers(),
+      m_check_constraints(),
       m_collation_id(src.m_collation_id),
       m_tablespace_id(src.m_tablespace_id) {
   m_indexes.deep_copy(src.m_indexes, this);
@@ -965,5 +1072,6 @@ Table_impl::Table_impl(const Table_impl &src)
                                         Foreign_key_parent(*fk_parent));
   m_partitions.deep_copy(src.m_partitions, this);
   m_triggers.deep_copy(src.m_triggers, this);
+  m_check_constraints.deep_copy(src.m_check_constraints, this);
 }
 }  // namespace dd
