@@ -1,15 +1,16 @@
-/* Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -65,6 +66,10 @@
 #include "sql/trigger.h"  // Trigger
 #include "sql/trigger_def.h"
 
+#ifdef HAVE_PERCONA_TELEMETRY
+#include "sql/dd/dd_utility.h"  // check_if_server_ddse_readonly
+#endif
+
 typedef ulonglong sql_mode_t;
 extern const char *mysql_sys_schema[];
 extern const char *fill_help_tables[];
@@ -950,6 +955,116 @@ bool upgrade_system_schemas(THD *thd) {
   return dd::end_transaction(thd, err);
 }
 
+#ifdef HAVE_PERCONA_TELEMETRY
+/* 1. We INSERT INTO, because prepared statements do not support
+      INSTALL COMPONENT
+   2. We use stored procedure to be able to do conditional action.
+   3. Percona Telemetry Component creates and uses internal user
+      (locked account) percona.telemetry with the following privileges:
+        1. SELECT
+        2. REPLICATION SLAVE
+        3. REPLICATION CLIENT
+      CREATE USER and GRANT do not work yet as ACL is not initialized yet.
+      Use INSERT and UPDATES.
+      This is the same approach as for creation of mysql.user in
+      mysql_system_tables_fix.sql. First we create the user, then update grants.
+      The reasons for this approach are:
+      1. When the user is created, it is granted SHUTDOWN, SUPER, CREATE ROLE,
+         DROP ROLE privileges. percona.telemetry user do not need them, so we
+         drop
+      2. We grant SELECT, REPLICATION SLAVE, REPLICATION CLIENT explicitly.
+         It is for clarity, but also automatically fixes privileges in case
+         someone manually drops them.
+*/
+static const char *percona_telemetry_install[] = {
+    "USE mysql;\n",
+    "SET @have_percona_telemetry= (SELECT COUNT(*) FROM mysql.component WHERE "
+    "component_urn='file://component_percona_telemetry');\n",
+    "SET @cmd= 'INSERT INTO mysql.component(component_group_id, component_urn) "
+    "VALUES (1, \"file://component_percona_telemetry\");';\n",
+    "SET @str = IF(@have_percona_telemetry = 0, @cmd, 'SET @dummy = 0');\n",
+    "PREPARE stmt FROM @str;\n",
+    "EXECUTE stmt;\n",
+    "DROP PREPARE stmt;\n",
+    "SET @group_id= (SELECT LAST_INSERT_ID());\n",
+    "SET @cmd= 'UPDATE mysql.component SET component_group_id=@group_id WHERE "
+    "component_id=@group_id;';\n",
+    "SET @str = IF(@have_percona_telemetry = 0, @cmd, 'SET @dummy = 0');\n",
+    "PREPARE stmt FROM @str;\n",
+    "EXECUTE stmt;\n",
+    "DROP PREPARE stmt;\n",
+    "INSERT IGNORE INTO mysql.user VALUES "
+    "('localhost','percona.telemetry','N','N','N','N','N','N','"
+    "N','N','N','N','N','N','N','N','N','N','N','N','N','N','N','N','N','N','N'"
+    ",'N','N','N','N','','','','',0,0,0,0,'caching_sha2_password','$A$005$"
+    "THISISACOMBINATIONOFINVALIDSALTANDPASSWORDTHATMUSTNEVERBRBEUSED','N',"
+    "CURRENT_TIMESTAMP,NULL,'Y', 'N', 'N', NULL, NULL, NULL, NULL);\n",
+    "UPDATE mysql.user SET Select_priv = 'Y', Repl_slave_priv = 'Y', "
+    "Repl_client_priv = 'Y' WHERE User = 'percona.telemetry' AND Host = 'localhost';\n",
+    "UPDATE mysql.user SET Shutdown_priv = 'N', Super_priv = 'N', "
+    "Create_role_priv = 'N', Drop_role_priv = 'N' WHERE User = "
+    "'percona.telemetry' AND Host = 'localhost';\n",
+    NULL};
+
+static const char *percona_telemetry_uninstall[] = {
+    "USE mysql;\n",
+    "DELETE FROM mysql.component WHERE "
+    "component_urn=\"file://component_percona_telemetry\"\n;",
+    "DELETE FROM mysql.user WHERE user='percona.telemetry' AND Host = 'localhost';\n",
+    NULL};
+
+/**
+  Always return success, regardless of Percona Telemetry Component setup
+  status.
+*/
+bool setup_percona_telemetry(THD *thd [[maybe_unused]]) {
+  /* Do not call dd::check_if_server_ddse_readonly()
+   * to avoid issuing the warning */
+  handlerton *ddse = ha_resolve_by_legacy_type(thd, DB_TYPE_INNODB);
+  if (ddse->is_dict_readonly && ddse->is_dict_readonly()) {
+    LogErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG,
+           "Percona Telemetry Component not configured due to InnoDB "
+           "in read only mode.");
+    return false;
+  }
+
+  Disable_autocommit_guard autocommit_guard(thd);
+  Bootstrap_error_handler bootstrap_error_handler;
+
+  Server_option_guard<bool> acl_guard(&opt_noacl, true);
+  Server_option_guard<bool> general_log_guard(&opt_general_log, false);
+  Server_option_guard<bool> slow_log_guard(&opt_slow_log, false);
+  Disable_binlog_guard disable_binlog(thd);
+  Disable_sql_log_bin_guard disable_sql_log_bin(thd);
+
+  bool err = false;
+
+  bootstrap_error_handler.set_log_error(false);
+
+  const char **query_ptr = opt_percona_telemetry_disable
+                               ? &percona_telemetry_uninstall[0]
+                               : &percona_telemetry_install[0];
+  for (; *query_ptr != nullptr; query_ptr++)
+    if (ignore_error_and_execute(thd, *query_ptr)) {
+      LogErr(WARNING_LEVEL, ER_LOG_PRINTF_MSG,
+             "Failed during setup Percona Telemetry component. Ignoring.");
+      break;
+    };
+
+  bootstrap_error_handler.set_log_error(true);
+
+  close_thread_tables(thd);
+  close_cached_tables(nullptr, nullptr, false, LONG_TIMEOUT);
+
+  if (dd::end_transaction(thd, err)) {
+    LogErr(WARNING_LEVEL, ER_LOG_PRINTF_MSG,
+           "Failed to setup Percona Telemetry component. Ignoring.");
+  }
+
+  return false;
+}
+#endif /* HAVE_PERCONA_TELEMETRY */
+
 bool no_server_upgrade_required() {
   return !(
       dd::bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade() ||
