@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,21 +56,25 @@ class Index : virtual public Entity_object {
 
  public:
   enum enum_index_type  // similar to Keytype in sql_class.h but w/o FOREIGN_KEY
-  { IT_PRIMARY = 1,
+  {
+    IT_PRIMARY = 1,
     IT_UNIQUE,
     IT_MULTIPLE,
     IT_FULLTEXT,
-    IT_SPATIAL };
+    IT_SPATIAL
+  };
 
   enum enum_index_algorithm  // similar to ha_key_alg
-  { IA_SE_SPECIFIC = 1,
+  {
+    IA_SE_SPECIFIC = 1,
     IA_BTREE,
     IA_RTREE,
     IA_HASH,
-    IA_FULLTEXT };
+    IA_FULLTEXT
+  };
 
  public:
-  ~Index() override {}
+  ~Index() override = default;
 
   /**
     Dummy method to be able to use Partition_index and Index interchangeably
