@@ -1,18 +1,19 @@
 #ifndef LEX_INCLUDED
 #define LEX_INCLUDED
 
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -95,6 +96,7 @@ static const SYMBOL symbols[] = {
     {SYM("ASENSITIVE", ASENSITIVE_SYM)},
     {SYM("AT", AT_SYM)},
     {SYM("ATTRIBUTE", ATTRIBUTE_SYM)},
+    {SYM("AUTHENTICATION", AUTHENTICATION_SYM)},
     {SYM("AUTO_INCREMENT", AUTO_INC)},
     {SYM("AUTOEXTEND_SIZE", AUTOEXTEND_SIZE_SYM)},
     {SYM("AVG", AVG_SYM)},
@@ -114,6 +116,7 @@ static const SYMBOL symbols[] = {
     {SYM("BOTH", BOTH)},
     {SYM("BTREE", BTREE_SYM)},
     {SYM("BUCKETS", BUCKETS_SYM)},
+    {SYM("BULK", BULK_SYM)},
     {SYM("BY", BY)},
     {SYM("BYTE", BYTE_SYM)},
     {SYM("CACHE", CACHE_SYM)},
@@ -122,9 +125,9 @@ static const SYMBOL symbols[] = {
     {SYM("CASE", CASE_SYM)},
     {SYM("CATALOG_NAME", CATALOG_NAME_SYM)},
     {SYM("CHAIN", CHAIN_SYM)},
+    {SYM("CHALLENGE_RESPONSE", CHALLENGE_RESPONSE_SYM)},
     {SYM("CHANGE", CHANGE)},
     {SYM("CHANGED", CHANGED)},
-    {SYM_PERCONA("CHANGED_PAGE_BITMAPS", CHANGED_PAGE_BITMAPS_SYM)},
     {SYM("CHANNEL", CHANNEL_SYM)},
     {SYM("CHAR", CHAR_SYM)},
     {SYM("CHARACTER", CHAR_SYM)},
@@ -155,7 +158,6 @@ static const SYMBOL symbols[] = {
     {SYM("COMPRESSED", COMPRESSED_SYM)},
     {SYM_PERCONA("COMPRESSION_DICTIONARY", COMPRESSION_DICTIONARY_SYM)},
     {SYM("ENCRYPTION", ENCRYPTION_SYM)},
-    {SYM_PERCONA("ENCRYPTION_KEY_ID", ENCRYPTION_KEY_ID_SYM)},
     {SYM("CONCURRENT", CONCURRENT)},
     {SYM("CONDITION", CONDITION_SYM)},
     {SYM("CONNECTION", CONNECTION_SYM)},
@@ -253,6 +255,7 @@ static const SYMBOL symbols[] = {
     {SYM("EXPLAIN", DESCRIBE)},
     {SYM("EXTENDED", EXTENDED_SYM)},
     {SYM("EXTENT_SIZE", EXTENT_SIZE_SYM)},
+    {SYM("FACTOR", FACTOR_SYM)},
     {SYM("FAILED_LOGIN_ATTEMPTS", FAILED_LOGIN_ATTEMPTS_SYM)},
     {SYM("FALSE", FALSE_SYM)},
     {SYM("FAST", FAST_SYM)},
@@ -262,6 +265,7 @@ static const SYMBOL symbols[] = {
     {SYM("FILE", FILE_SYM)},
     {SYM("FILE_BLOCK_SIZE", FILE_BLOCK_SIZE_SYM)},
     {SYM("FILTER", FILTER_SYM)},
+    {SYM("FINISH", FINISH_SYM)},
     {SYM("FIRST", FIRST_SYM)},
     {SYM("FIRST_VALUE", FIRST_VALUE_SYM)},
     {SYM("FIXED", FIXED_SYM)},
@@ -289,6 +293,7 @@ static const SYMBOL symbols[] = {
     {SYM("GET_MASTER_PUBLIC_KEY", GET_MASTER_PUBLIC_KEY_SYM)},
     {SYM("GET_SOURCE_PUBLIC_KEY", GET_SOURCE_PUBLIC_KEY_SYM)},
     {SYM("GET", GET_SYM)},
+    {SYM("GENERATE", GENERATE_SYM)},
     {SYM("GENERATED", GENERATED)},
     {SYM("GLOBAL", GLOBAL_SYM)},
     {SYM("GRANT", GRANT)},
@@ -296,6 +301,7 @@ static const SYMBOL symbols[] = {
     {SYM("GROUP", GROUP_SYM)},
     {SYM("GROUPING", GROUPING_SYM)},
     {SYM("GROUPS", GROUPS_SYM)},
+    {SYM("GTID_ONLY", GTID_ONLY_SYM)},
     {SYM("HANDLER", HANDLER_SYM)},
     {SYM("HASH", HASH_SYM)},
     {SYM("HAVING", HAVING)},
@@ -319,8 +325,10 @@ static const SYMBOL symbols[] = {
     {SYM("INDEX", INDEX_SYM)},
     {SYM_PERCONA("INDEX_STATISTICS", INDEX_STATS_SYM)},
     {SYM("INDEXES", INDEXES)},
-    {SYM("INFILE", INFILE)},
+    {SYM("INFILE", INFILE_SYM)},
+    {SYM("INITIAL", INITIAL_SYM)},
     {SYM("INITIAL_SIZE", INITIAL_SIZE_SYM)},
+    {SYM("INITIATE", INITIATE_SYM)},
     {SYM("INNER", INNER_SYM)},
     {SYM("INOUT", INOUT_SYM)},
     {SYM("INSENSITIVE", INSENSITIVE_SYM)},
@@ -334,6 +342,7 @@ static const SYMBOL symbols[] = {
     {SYM("INT4", INT_SYM)},
     {SYM("INT8", BIGINT_SYM)},
     {SYM("INTEGER", INT_SYM)},
+    {SYM("INTERSECT", INTERSECT_SYM)},
     {SYM("INTERVAL", INTERVAL_SYM)},
     {SYM("INTO", INTO)},
     {SYM("IO", IO_SYM)},
@@ -508,6 +517,7 @@ static const SYMBOL symbols[] = {
     {SYM("PASSWORD", PASSWORD)},
     {SYM("PASSWORD_LOCK_TIME", PASSWORD_LOCK_TIME_SYM)},
     {SYM("PERCENT_RANK", PERCENT_RANK_SYM)},
+    {SYM_PERCONA("PERCONA_SEQUENCE_TABLE", PERCONA_SEQUENCE_TABLE_SYM)},
     {SYM("PERSIST", PERSIST_SYM)},
     {SYM("PERSIST_ONLY", PERSIST_ONLY_SYM)},
     {SYM("PHASE", PHASE_SYM)},
@@ -552,6 +562,7 @@ static const SYMBOL symbols[] = {
     {SYM("REFERENCE", REFERENCE_SYM)},
     {SYM("REFERENCES", REFERENCES)},
     {SYM("REGEXP", REGEXP)},
+    {SYM("REGISTRATION", REGISTRATION_SYM)},
     {SYM("RELAY", RELAY)},
     {SYM("RELAYLOG", RELAYLOG_SYM)},
     {SYM("RELAY_LOG_FILE", RELAY_LOG_FILE_SYM)},
@@ -767,9 +778,11 @@ static const SYMBOL symbols[] = {
     {SYM("UNKNOWN", UNKNOWN_SYM)},
     {SYM("UNLOCK", UNLOCK_SYM)},
     {SYM("UNINSTALL", UNINSTALL_SYM)},
+    {SYM("UNREGISTER", UNREGISTER_SYM)},
     {SYM("UNSIGNED", UNSIGNED_SYM)},
     {SYM("UNTIL", UNTIL_SYM)},
     {SYM("UPGRADE", UPGRADE_SYM)},
+    {SYM("URL", URL_SYM)},
     {SYM("USAGE", USAGE)},
     {SYM("USE", USE_SYM)},
     {SYM("USER", USER)},
