@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -504,12 +505,18 @@ bool File_query_log::set_file(const char *new_name) {
   if (name != nullptr) my_free(name);
 
   name = nn;
+  bool res = false;
 
-  mysql_mutex_lock(&LOCK_log);
-  cur_log_ext = 0;
-  last_removed_ext = 0;
-  bool res = set_rotated_name(false) || purge_logs();
-  mysql_mutex_unlock(&LOCK_log);
+  if (m_log_type == QUERY_LOG_SLOW) {
+    mysql_mutex_lock(&LOCK_log);
+    cur_log_ext = 0;
+    last_removed_ext = 0;
+    res = set_rotated_name(false) || purge_logs();
+    mysql_mutex_unlock(&LOCK_log);
+  } else {
+    // We can do this here since we're not actually resolving symlinks etc.
+    fn_format(log_file_name, name, mysql_data_home, "", MY_UNPACK_FILENAME);
+  }
 
   return res;
 }
@@ -2137,6 +2144,8 @@ static size_t get_last_extension(const char *const name) {
  */
 bool File_query_log::set_rotated_name(const bool need_lock) {
   DBUG_ENTER("File_query_log::set_rotated_name");
+  assert(m_log_type == QUERY_LOG_SLOW);
+
   if (need_lock) mysql_mutex_assert_owner(&LOCK_log);
 
   if (!max_slowlog_size) {
@@ -2188,6 +2197,8 @@ bool File_query_log::set_rotated_name(const bool need_lock) {
  */
 bool File_query_log::rotate(const ulong max_size) {
   DBUG_ENTER("File_query_log::rotate");
+  assert(m_log_type == QUERY_LOG_SLOW);
+
   mysql_mutex_assert_owner(&LOCK_log);
 
   if (my_b_tell(&log_file) > max_size) {
@@ -2206,6 +2217,8 @@ bool File_query_log::rotate(const ulong max_size) {
  */
 bool File_query_log::purge_logs() {
   DBUG_ENTER("File_query_log::purge_logs");
+  assert(m_log_type == QUERY_LOG_SLOW);
+
   mysql_mutex_assert_owner(&LOCK_log);
 
   if (max_slowlog_files == 0 || cur_log_ext < 1 ||
