@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -809,17 +810,21 @@ bool Persisted_variables_cache::flush_to_file() {
     /* write to file */
     if (mysql_file_fputs(dest.c_ptr(), m_fd) < 0) {
       ret = true;
+    } else {
+      DBUG_EXECUTE_IF("crash_after_write_persist_file", DBUG_SUICIDE(););
+      /* flush contents to disk immediately */
+      if ((mysql_file_fflush(m_fd) != 0) ||
+          (my_sync(my_fileno(m_fd->m_file), MYF(MY_WME)) == -1))
+        ret = true;
     }
-    DBUG_EXECUTE_IF("crash_after_write_persist_file", DBUG_SUICIDE(););
-    /* flush contents to disk immediately */
-    if (mysql_file_fflush(m_fd) != 0) ret = true;
-    if (my_sync(my_fileno(m_fd->m_file), MYF(MY_WME)) == -1) ret = true;
+    close_persist_file();
   }
-  close_persist_file();
   if (!ret) {
     DBUG_EXECUTE_IF("crash_after_close_persist_file", DBUG_SUICIDE(););
-    my_rename(m_persist_backup_filename.c_str(), m_persist_filename.c_str(),
-              MYF(MY_WME));
+    if (my_rename(m_persist_backup_filename.c_str(), m_persist_filename.c_str(),
+                  MYF(MY_WME)))
+      ret = true;
+    DBUG_EXECUTE_IF("crash_after_rename_persist_file", DBUG_SUICIDE(););
   }
   if (ret == false && do_cleanup == true) clear_sensitive_blob_and_iv();
   mysql_mutex_unlock(&m_LOCK_persist_file);
