@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -80,8 +81,8 @@ typedef Prealloced_array<Slave_worker *, 4> Slave_worker_array;
 
 typedef struct slave_job_item {
   Log_event *data;
-  uint relay_number;
   my_off_t relay_pos;
+  char event_relay_log_name[FN_REFLEN + 1];
 } Slave_job_item;
 
 /**
@@ -703,8 +704,6 @@ class Relay_log_info : public Rpl_info {
   char group_relay_log_name[FN_REFLEN];
   ulonglong group_relay_log_pos;
   char event_relay_log_name[FN_REFLEN];
-  /* The suffix number of relay log name */
-  uint event_relay_log_number;
   ulonglong event_relay_log_pos;
   ulonglong future_event_relay_log_pos;
 
@@ -904,19 +903,22 @@ class Relay_log_info : public Rpl_info {
 
   /*
     Handling of the relay_log_space_limit optional constraint.
-    ignore_log_space_limit is used to resolve a deadlock between I/O and SQL
-    threads, the SQL thread sets it to unblock the I/O thread and make it
-    temporarily forget about the constraint.
   */
-  ulonglong log_space_limit, log_space_total;
-  std::atomic<bool> ignore_log_space_limit;
+  std::atomic<ulonglong> log_space_limit, log_space_total;
 
-  /*
-    Used by the SQL thread to instructs the IO thread to rotate
-    the logs when the SQL thread needs to purge to release some
-    disk space.
-   */
-  std::atomic<bool> sql_force_rotate_relay;
+  // This flag is used by a coordinator to check if the receiver waits for
+  // a relay log space. If yes, it will enable aggressive relay log
+  // purge.
+  std::atomic_bool is_receiver_waiting_for_rl_space;
+
+  // This is file to which coordinator moved after enforced purge
+  // It is used by the receiver to check if all possible files were purged
+  // before making a decision on whether transaction may fit into the
+  // relay_log_space_limit. It is used to avoid possibly infinite waiting in
+  // case a transaction and required relay log metadata is bigger than
+  // 'relay_log_space_limit'. This filename is protected with the
+  // log_space_lock
+  std::string coordinator_log_after_purge{""};
 
   time_t last_master_timestamp;
 
@@ -1646,26 +1648,9 @@ class Relay_log_info : public Rpl_info {
   inline void set_event_relay_log_name(const char *log_file_name) {
     strmake(event_relay_log_name, log_file_name,
             sizeof(event_relay_log_name) - 1);
-    set_event_relay_log_number(relay_log_name_to_number(log_file_name));
     notify_relay_log_change();
   }
 
-  uint get_event_relay_log_number() { return event_relay_log_number; }
-  void set_event_relay_log_number(uint number) {
-    event_relay_log_number = number;
-  }
-
-  /**
-    Given the extension number of the relay log, gets the full
-    relay log path. Currently used in Slave_worker::retry_transaction()
-
-    @param [in]   number      extension number of relay log
-    @param[in, out] name      The full path of the relay log (per-channel)
-                              to be read by the slave worker.
-  */
-  void relay_log_number_to_name(uint number, char name[FN_REFLEN + 1]);
-  uint relay_log_name_to_number(const char *name);
-
   void set_event_start_pos(my_off_t pos) { event_start_pos = pos; }
   my_off_t get_event_start_pos() { return event_start_pos; }
 
@@ -2439,7 +2424,7 @@ class Applier_security_context_guard {
             false, otherwise.
    */
   bool has_access(
-      std::vector<std::tuple<ulong, TABLE const *, Rows_log_event *>>
+      std::vector<std::tuple<Access_bitmask, TABLE const *, Rows_log_event *>>
           &extra_privileges) const;
   /**
     Checks if the `PRIVILEGE_CHECKS_USER` user has access to the privilieges
@@ -2468,7 +2453,7 @@ class Applier_security_context_guard {
     @return true if the privileges are included in the security context and
             false, otherwise.
    */
-  bool has_access(std::initializer_list<ulong> extra_privileges) const;
+  bool has_access(std::initializer_list<Access_bitmask> extra_privileges) const;
 
   /**
     Returns the username for the user for which the security context was
