@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +31,6 @@
 #include "mysql/psi/mysql_mutex.h"
 #include "mysqld_error.h"
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
-#include "sql/rpl_gtid.h"            // rpl_sidno
 #include "sql/sql_class.h"           // THD
 #include "sql/transaction_info.h"
 
@@ -92,24 +92,42 @@ int set_transaction_ctx(
     Transaction_termination_ctx transaction_termination_ctx) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("thread_id=%lu, rollback_transaction=%d, "
-                       "generated_gtid=%d, sidno=%d, gno=%lld",
+                       "generated_gtid=%d, sidno=%d, gno=%" PRId64,
                        transaction_termination_ctx.m_thread_id,
                        transaction_termination_ctx.m_rollback_transaction,
                        transaction_termination_ctx.m_generated_gtid,
                        transaction_termination_ctx.m_sidno,
                        transaction_termination_ctx.m_gno));
 
-  THD *thd = nullptr;
   uint error = ER_NO_SUCH_THREAD;
   Find_thd_with_id find_thd_with_id(transaction_termination_ctx.m_thread_id,
-                                    false);
+                                    true);
 
-  thd = Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
-  if (thd) {
-    error = thd->get_transaction()
+#ifndef NDEBUG
+  static std::size_t gno = 0;
+#endif
+
+  THD_ptr thd_ptr =
+      Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  if (thd_ptr) {
+    error = thd_ptr->get_transaction()
                 ->get_rpl_transaction_ctx()
                 ->set_rpl_transaction_ctx(transaction_termination_ctx);
-    mysql_mutex_unlock(&thd->LOCK_thd_data);
+
+    if (!error && !transaction_termination_ctx.m_rollback_transaction) {
+      /*
+        Assign the session commit ticket while the transaction is
+        still under the control of the external transaction
+        arbitrator, thence matching the arbitrator's transactions
+        order.
+      */
+      DBUG_EXECUTE_IF(
+          "simulate_bgct_rpco_deadlock", ++gno; if (gno % 3 == 0) {
+            thd_ptr->rpl_thd_ctx.binlog_group_commit_ctx().push_new_ticket();
+          });
+      thd_ptr->rpl_thd_ctx.binlog_group_commit_ctx().assign_ticket();
+    }
   }
+
   return error;
 }
