@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,13 +27,15 @@
 #include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
+#include <functional>  // std::function
+#include <optional>
 
 #include "lex_string.h"
-#include "my_dbug.h"
+
+#include "field_types.h"
 #include "my_io.h"
 #include "my_sqlcommand.h"
-#include "mysql/psi/psi_base.h"
-#include "nullable.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "sql/dd/types/column.h"
 #include "sql/gis/srid.h"
 #include "sql/key.h"  // KEY
@@ -45,6 +48,7 @@
 #include "sql/sql_list.h"              // List
 #include "sql/thr_malloc.h"
 
+class Alter_info;
 class Create_field;
 class FOREIGN_KEY;
 class Value_generator;
@@ -52,32 +56,28 @@ class Item;
 class Key_spec;
 class String;
 class THD;
-struct TABLE_LIST;
-
-enum enum_field_types : int;
-
-using Mysql::Nullable;
+class Table_ref;
 
 /**
-  Class representing DROP COLUMN, DROP KEY, DROP FOREIGN KEY and DROP CHECK
-  CONSTRAINT clauses in ALTER TABLE statement.
+  Class representing DROP COLUMN, DROP KEY, DROP FOREIGN KEY, DROP CHECK
+  CONSTRAINT and DROP CONSTRAINT clauses in ALTER TABLE statement.
 */
 
 class Alter_drop {
  public:
-  enum drop_type { KEY, COLUMN, FOREIGN_KEY, CHECK_CONSTRAINT };
+  enum drop_type { KEY, COLUMN, FOREIGN_KEY, CHECK_CONSTRAINT, ANY_CONSTRAINT };
   const char *name;
   drop_type type;
 
   Alter_drop(drop_type par_type, const char *par_name)
       : name(par_name), type(par_type) {
-    DBUG_ASSERT(par_name != NULL);
+    assert(par_name != nullptr);
   }
 };
 
 /**
-  Class representing SET DEFAULT, DROP DEFAULT and RENAME
-  COLUMN clause in ALTER TABLE statement.
+  Class representing SET DEFAULT, DROP DEFAULT, RENAME COLUMN, SET VISIBLE and
+  SET INVISIBLE clause in ALTER TABLE statement.
 */
 
 class Alter_column {
@@ -88,13 +88,19 @@ class Alter_column {
   /// The default value supplied.
   Item *def;
 
-  /// The expression to be used to generated the default value.
+  /// The expression to be used to generate the default value.
   Value_generator *m_default_val_expr;
 
-  /// The new colum name.
+  /// The new column name.
   const char *m_new_name;
 
-  enum class Type { SET_DEFAULT, DROP_DEFAULT, RENAME_COLUMN };
+  enum class Type {
+    SET_DEFAULT,
+    DROP_DEFAULT,
+    RENAME_COLUMN,
+    SET_COLUMN_VISIBLE,
+    SET_COLUMN_INVISIBLE
+  };
 
  public:
   /// Type of change requested in ALTER TABLE.
@@ -128,6 +134,13 @@ class Alter_column {
         m_new_name(new_name),
         m_type(Type::RENAME_COLUMN) {}
 
+  /// Constructor used while altering column visibility.
+  Alter_column(const char *par_name, bool par_is_visible)
+      : name(par_name), def(nullptr), m_new_name(nullptr) {
+    m_type = (par_is_visible ? Type::SET_COLUMN_VISIBLE
+                             : Type::SET_COLUMN_INVISIBLE);
+  }
+
  private:
   Type m_type;
 };
@@ -137,7 +150,7 @@ class Alter_index_visibility {
  public:
   Alter_index_visibility(const char *name, bool is_visible)
       : m_name(name), m_is_visible(is_visible) {
-    assert(name != NULL);
+    assert(name != nullptr);
   }
 
   const char *name() const { return m_name; }
@@ -165,23 +178,26 @@ class Alter_rename_key {
 };
 
 /**
-  Class which instances represents state(i.e ENFORCED | NOT ENFORCED) of
-  CHECK CONSTRAINT in ALTER TABLE statement.
+  Class representing ALTER CHECK and ALTER CONSTRAINT clauses in ALTER TABLE
+  statement.
 */
 
-class Alter_state {
+class Alter_constraint_enforcement {
  public:
-  enum class Type { CHECK_CONSTRAINT };
+  enum class Type { CHECK_CONSTRAINT, ANY_CONSTRAINT };
   const char *name;
   Type type;
-  bool state;
+  bool is_enforced;
 
-  Alter_state(Type par_type, const char *par_name, bool par_state)
-      : name(par_name), type(par_type), state(par_state) {
-    DBUG_ASSERT(par_name != nullptr);
+  Alter_constraint_enforcement(Type par_type, const char *par_name,
+                               bool par_is_enforced)
+      : name(par_name), type(par_type), is_enforced(par_is_enforced) {
+    assert(par_name != nullptr);
   }
 };
 
+using CreateFieldApplier = std::function<bool(Create_field *, Alter_info *)>;
+
 /**
   Data describing the table being created by CREATE TABLE or
   altered by ALTER TABLE.
@@ -273,7 +289,7 @@ class Alter_info {
     /// Set for DROP FOREIGN KEY
     DROP_FOREIGN_KEY = 1ULL << 22,
 
-    /// Set for EXCHANGE PARITION
+    /// Set for EXCHANGE PARTITION
     ALTER_EXCHANGE_PARTITION = 1ULL << 23,
 
     /// Set by Sql_cmd_alter_table_truncate_partition::execute()
@@ -311,6 +327,24 @@ class Alter_info {
 
     /// Set for check constraint suspend.
     SUSPEND_CHECK_CONSTRAINT = 1ULL << 35,
+
+    /// Set for DROP CONSTRAINT.
+    DROP_ANY_CONSTRAINT = 1ULL << 36,
+
+    /// Set for ALTER CONSTRAINT symbol ENFORCED.
+    ENFORCE_ANY_CONSTRAINT = 1ULL << 37,
+
+    /// Set for ALTER CONSTRAINT symbol NOT ENFORCED.
+    SUSPEND_ANY_CONSTRAINT = 1ULL << 38,
+
+    /// Set if ANY engine attribute is used (also in CREATE) Note that
+    /// this is NOT to be set for SECONDARY_ENGINE_ATTRIBUTE as this flag
+    /// controls if execution should check if SE supports engine
+    /// attributes.
+    ANY_ENGINE_ATTRIBUTE = 1ULL << 39,
+
+    /// Set for column visibility attribute alter.
+    ALTER_COLUMN_VISIBILITY = 1ULL << 40
   };
 
   enum enum_enable_or_disable { LEAVE_AS_IS, ENABLE, DISABLE };
@@ -338,10 +372,10 @@ class Alter_info {
      Describes the level of concurrency during ALTER TABLE.
   */
   enum enum_alter_table_lock {
-    // Maximum supported level of concurency for the given operation.
+    // Maximum supported level of concurrency for the given operation.
     ALTER_TABLE_LOCK_DEFAULT,
 
-    // Allow concurrent reads & writes. If not supported, give erorr.
+    // Allow concurrent reads & writes. If not supported, give error.
     ALTER_TABLE_LOCK_NONE,
 
     // Allow concurrent reads only. If not supported, give error.
@@ -367,10 +401,7 @@ class Alter_info {
   };
 
   /**
-     Columns and keys to be dropped.
-     After mysql_prepare_alter_table() it contains only foreign keys and
-     virtual generated columns to be dropped. This information is necessary
-     for the storage engine to do in-place alter.
+     Columns, keys and constraints to be dropped.
   */
   Mem_root_array<const Alter_drop *> drop_list;
   // Columns for ALTER_COLUMN_CHANGE_DEFAULT.
@@ -384,8 +415,9 @@ class Alter_info {
   /// Indexes whose visibilities are to be changed.
   Mem_root_array<const Alter_index_visibility *> alter_index_visibility_list;
 
-  /// List of check constraints whose state is changed.
-  Mem_root_array<const Alter_state *> alter_state_list;
+  /// List of check constraints whose enforcement state is changed.
+  Mem_root_array<const Alter_constraint_enforcement *>
+      alter_constraint_enforcement_list;
 
   /// Check constraints specification for CREATE and ALTER TABLE operations.
   Sql_check_constraint_spec_list check_constraint_spec_list;
@@ -399,6 +431,8 @@ class Alter_info {
   // Count of keys, which creation is delayed to benefit from fast index
   // creation
   uint delayed_key_count;
+  std::vector<CreateFieldApplier> cf_appliers;
+
   // Type of ALTER TABLE operation.
   ulonglong flags;
   // Enable or disable keys.
@@ -421,7 +455,11 @@ class Alter_info {
   /// ALTER TABLE [db.]table [ RENAME [TO|AS|=] [new_db.]new_table ]
   LEX_CSTRING new_db_name;
 
-  /// New table name in the "RENAME [TO] <table_name>" clause or NULL_STR
+  /// New table name in the
+  /// \code
+  /// RENAME [TO] <table_name>
+  /// \endcode
+  /// clause or NULL_STR
   LEX_CSTRING new_table_name;
 
   explicit Alter_info(MEM_ROOT *mem_root)
@@ -430,7 +468,7 @@ class Alter_info {
         key_list(mem_root),
         alter_rename_key_list(mem_root),
         alter_index_visibility_list(mem_root),
-        alter_state_list(mem_root),
+        alter_constraint_enforcement_list(mem_root),
         check_constraint_spec_list(mem_root),
         delayed_key_list(mem_root),
         flags(0),
@@ -462,14 +500,14 @@ class Alter_info {
   bool add_field(THD *thd, const LEX_STRING *field_name,
                  enum enum_field_types type, const char *length,
                  const char *decimal, uint type_modifier, Item *default_value,
-                 Item *on_update_value, LEX_STRING *comment, const char *change,
-                 List<String> *interval_list, const CHARSET_INFO *cs,
-                 bool has_explicit_collation, uint uint_geom_type,
-                 const LEX_CSTRING *zip_dict, Value_generator *gcol_info,
-                 Value_generator *default_val_expr, const char *opt_after,
-                 Nullable<gis::srid_t> srid,
+                 Item *on_update_value, LEX_CSTRING *comment,
+                 const char *change, List<String> *interval_list,
+                 const CHARSET_INFO *cs, bool has_explicit_collation,
+                 uint uint_geom_type, const LEX_CSTRING *zip_dict,
+                 Value_generator *gcol_info, Value_generator *default_val_expr,
+                 const char *opt_after, std::optional<gis::srid_t> srid,
                  Sql_check_constraint_spec_list *check_cons_list,
-                 dd::Column::enum_hidden_type hidden);
+                 dd::Column::enum_hidden_type hidden, bool is_array = false);
 
   /**
      Checks if there are any columns with COLUMN_FORMAT COMRPESSED
@@ -490,7 +528,7 @@ class Alter_table_ctx {
  public:
   Alter_table_ctx();
 
-  Alter_table_ctx(THD *thd, TABLE_LIST *table_list, uint tables_opened_arg,
+  Alter_table_ctx(THD *thd, Table_ref *table_list, uint tables_opened_arg,
                   const char *new_db_arg, const char *new_name_arg);
 
   ~Alter_table_ctx();
@@ -517,7 +555,7 @@ class Alter_table_ctx {
      @return path to the original table.
   */
   const char *get_path() const {
-    DBUG_ASSERT(!tmp_table);
+    assert(!tmp_table);
     return path;
   }
 
@@ -566,7 +604,7 @@ class Alter_table_ctx {
   char new_path[FN_REFLEN + 1];
   char tmp_path[FN_REFLEN + 1];
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   /** Indicates that we are altering temporary table. Used only in asserts. */
   bool tmp_table;
 #endif
@@ -585,12 +623,10 @@ class Sql_cmd_common_alter_table : public Sql_cmd_ddl_table {
 
   ~Sql_cmd_common_alter_table() override = 0;  // force abstract class
 
-  enum_sql_command sql_command_code() const override final {
-    return SQLCOM_ALTER_TABLE;
-  }
+  enum_sql_command sql_command_code() const final { return SQLCOM_ALTER_TABLE; }
 };
 
-inline Sql_cmd_common_alter_table::~Sql_cmd_common_alter_table() {}
+inline Sql_cmd_common_alter_table::~Sql_cmd_common_alter_table() = default;
 
 /**
   Represents the generic ALTER TABLE statement.
@@ -601,6 +637,7 @@ class Sql_cmd_alter_table : public Sql_cmd_common_alter_table {
   using Sql_cmd_common_alter_table::Sql_cmd_common_alter_table;
 
   bool execute(THD *thd) override;
+  bool reprepare_on_execute_required() const override;
 };
 
 /**
@@ -613,7 +650,7 @@ class Sql_cmd_discard_import_tablespace : public Sql_cmd_common_alter_table {
   bool execute(THD *thd) override;
 
  private:
-  bool mysql_discard_or_import_tablespace(THD *thd, TABLE_LIST *table_list);
+  bool mysql_discard_or_import_tablespace(THD *thd, Table_ref *table_list);
 };
 
 /**
@@ -627,7 +664,7 @@ class Sql_cmd_secondary_load_unload final : public Sql_cmd_common_alter_table {
   bool execute(THD *thd) override;
 
  private:
-  bool mysql_secondary_load_or_unload(THD *thd, TABLE_LIST *table_list);
+  bool mysql_secondary_load_or_unload(THD *thd, Table_ref *table_list);
 };
 
 #endif
