@@ -853,11 +853,12 @@ extern int MYSQLdebug;
     USER_STATS_SYM = 1307,
     EFFECTIVE_SYM = 1350,
     SEQUENCE_TABLE_SYM = 1351,
-    KEYWORD_USED_AS_IDENT = 1353,
-    KEYWORD_USED_AS_KEYWORD = 1354,
-    CONDITIONLESS_JOIN = 1355,
-    SUBQUERY_AS_EXPR = 1356,
-    EMPTY_FROM_CLAUSE = 1357
+    PERCONA_SEQUENCE_TABLE_SYM = 1352,
+    KEYWORD_USED_AS_IDENT = 1354,
+    KEYWORD_USED_AS_KEYWORD = 1355,
+    CONDITIONLESS_JOIN = 1356,
+    SUBQUERY_AS_EXPR = 1357,
+    EMPTY_FROM_CLAUSE = 1358
   };
 #endif
 /* Tokens.  */
@@ -1665,11 +1666,12 @@ extern int MYSQLdebug;
 #define USER_STATS_SYM 1307
 #define EFFECTIVE_SYM 1350
 #define SEQUENCE_TABLE_SYM 1351
-#define KEYWORD_USED_AS_IDENT 1353
-#define KEYWORD_USED_AS_KEYWORD 1354
-#define CONDITIONLESS_JOIN 1355
-#define SUBQUERY_AS_EXPR 1356
-#define EMPTY_FROM_CLAUSE 1357
+#define PERCONA_SEQUENCE_TABLE_SYM 1352
+#define KEYWORD_USED_AS_IDENT 1354
+#define KEYWORD_USED_AS_KEYWORD 1355
+#define CONDITIONLESS_JOIN 1356
+#define SUBQUERY_AS_EXPR 1357
+#define EMPTY_FROM_CLAUSE 1358
 
 /* Value type.  */
 
