@@ -1,15 +1,16 @@
-/*  Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/*  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,16 +44,18 @@
 #include "my_psi_config.h"
 #include "my_thread.h"
 #include "my_thread_local.h"  // my_get_thread_local & my_set_thread_local
+#include "mysql.h"
+#include "mysql/components/my_service.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_rwlock_bits.h"
+#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/mysql_rwlock_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
-#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/plugin_audit.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql/psi/mysql_statement.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql_time.h"
 #include "mysqld_error.h"
 #include "pfs_thread_provider.h"
@@ -60,6 +63,7 @@
 #include "sql/auth/sql_security_ctx.h"
 #include "sql/conn_handler/connection_handler_manager.h"
 #include "sql/current_thd.h"
+#include "sql/debug_sync.h"          // DEBUG_SYNC
 #include "sql/derror.h"              // ER_DEFAULT
 #include "sql/log.h"                 // Query log
 #include "sql/mysqld.h"              // current_thd
@@ -153,7 +157,7 @@ class Thread_to_plugin_map {
       std::map<my_thread_t, const void *>::iterator it =
           collection.find(thread);
       if (it == collection.end()) collection[thread] = plugin;
-    } catch (const std::bad_alloc &e) {
+    } catch (const std::bad_alloc &) {
       return true;
     }
     return false;
@@ -231,8 +235,8 @@ class Thread_to_plugin_map {
       if (!plugin || (it->second == plugin)) {
         to_remove.push_back(it->first);
 
-        my_thread_handle thread;
 #ifndef _WIN32
+        my_thread_handle thread;
         /*
            On Windows we need HANDLE to cancel a thread.
            Win32 API's GetCurrentThread() returns something which seems the
@@ -270,7 +274,7 @@ class Mutexed_map_thd_srv_session {
  public:
   class Do_Impl {
    public:
-    virtual ~Do_Impl() {}
+    virtual ~Do_Impl() = default;
     /**
       Work on the session
 
@@ -344,9 +348,8 @@ class Mutexed_map_thd_srv_session {
 
     @param key Key of the element
 
-    @return
-      value of the element
-      NULL  if not found
+    @return value of the element
+    @retval NULL  if not found
   */
   Srv_session *find(const THD *key) {
     rwlock_scoped_lock lock(&LOCK_collection, false, __FILE__, __LINE__);
@@ -360,17 +363,16 @@ class Mutexed_map_thd_srv_session {
 
     @param key     key
     @param plugin  secondary key
-    @param session
+    @param session object to be added to the collection
 
-    @return
-      false  success
-      true   failure
+    @retval false  success
+    @retval true   failure
   */
   bool add(const THD *key, const void *plugin, Srv_session *session) {
     rwlock_scoped_lock lock(&LOCK_collection, true, __FILE__, __LINE__);
     try {
       collection[key] = std::make_pair(plugin, session);
-    } catch (const std::bad_alloc &e) {
+    } catch (const std::bad_alloc &) {
       return true;
     }
     return false;
@@ -454,40 +456,60 @@ class Mutexed_map_thd_srv_session {
 static Mutexed_map_thd_srv_session server_session_list;
 static Thread_to_plugin_map server_session_threads;
 
-/**
-  Constructs a session state object for Srv_session::execute_command.
-  Saves state. Uses RAII.
-
-  @param sess Session to backup
-*/
 Srv_session::Session_backup_and_attach::Session_backup_and_attach(
-    Srv_session *sess)
-    : session(sess), old_session(nullptr), backup_thd(nullptr) {
-  THD *thd = current_thd;
-  // If it is a srv_session thread and there's another session attached
-  if ((old_session = server_session_list.find(thd)))
+    Srv_session *sess, bool is_in_close_session)
+    : session(sess),
+      old_session(nullptr),
+      in_close_session(is_in_close_session) {
+  THD *c_thd = current_thd;
+  backup_thd = c_thd;
+  bool is_plugin = false;
+
+  if (session->is_associate()) {
+    attach_error = session->associate();
+    return;
+  }
+
+  /*
+    Check whether the current thread and the one we're going to switch to,
+    belong to a plugin and if the plugin is the same (so we can reuse the thd).
+  */
+  if (THR_srv_session_thread && c_thd &&
+      c_thd->get_plugin() == session->m_thd->get_plugin()) {
+    is_plugin = true;
+    backup_thd = nullptr;
+  }
+
+  if (is_plugin && c_thd != session->m_thd &&
+      (old_session = server_session_list.find(c_thd))) {
     old_session->detach();
-  else
-    backup_thd = thd;
+  } else if (is_plugin) {
+    assert(session->is_attached());
+  }
+
   attach_error = session->attach();
 }
 
-/**
-  Destructs the session state object. In other words it restores to
-  previous state.
-*/
 Srv_session::Session_backup_and_attach::~Session_backup_and_attach() {
-  if (backup_thd) {
+  if (session->is_associated()) {
+    session->disassociate();
+    if (backup_thd && backup_thd != session->m_thd) backup_thd->store_globals();
+  } else if (backup_thd) {
     session->detach();
     backup_thd->store_globals();
 #ifdef HAVE_PSI_THREAD_INTERFACE
     enum_vio_type vio_type = backup_thd->get_vio_type();
     if (vio_type != NO_VIO_TYPE) PSI_THREAD_CALL(set_connection_type)(vio_type);
 #endif /* HAVE_PSI_THREAD_INTERFACE */
-  } else {
+  } else if (in_close_session) {
+    /*
+      We should restore the old session only in case of close.
+      In case of execute we should stay attached.
+    */
     session->detach();
-    // If previously there was another session attached, then attach it back.
-    if (old_session) old_session->attach();
+    if (old_session) {
+      old_session->attach();
+    }
   }
 }
 
@@ -554,6 +576,8 @@ static void err_handle_error(void *ctx, uint err_errno, const char *err_msg,
 
 static void err_shutdown(void *, int) {}
 
+static bool err_alive(void *) { return true; }
+
 const struct st_command_service_cbs error_protocol_callbacks = {
     err_start_result_metadata,
     err_field_metadata,
@@ -573,7 +597,8 @@ const struct st_command_service_cbs error_protocol_callbacks = {
     err_get_string,
     err_handle_ok,
     err_handle_error,
-    err_shutdown};
+    err_shutdown,
+    err_alive};
 
 /**
   Modifies the PSI structures to (de)install a THD
@@ -600,6 +625,10 @@ static void set_psi(THD *thd) {
 */
 bool Srv_session::init_thread(const void *plugin) {
   char stack_start;
+
+  /* If not null, means its already initialized, so return success*/
+  if (THR_srv_session_thread != nullptr) return false;
+
   if (my_thread_init()) {
     connection_errors_internal++;
     return true;
@@ -655,7 +684,10 @@ static void close_all_sessions_of_plugin_if_any(const st_plugin_int *plugin) {
 */
 void Srv_session::deinit_thread() {
   const st_plugin_int *plugin = THR_srv_session_thread;
-  if (plugin) close_currently_attached_session_if_any(plugin);
+  if (plugin)
+    close_currently_attached_session_if_any(plugin);
+  else
+    return;
 
   if (server_session_threads.remove(my_thread_self()))
     LogErr(ERROR_LEVEL, ER_FAILED_TO_DECREMENT_NUMBER_OF_THREADS);
@@ -665,7 +697,7 @@ void Srv_session::deinit_thread() {
 
   THR_srv_session_thread = nullptr;
 
-  DBUG_ASSERT(THR_stack_start_address);
+  assert(THR_stack_start_address);
   THR_stack_start_address = nullptr;
   my_thread_end();
 }
@@ -737,76 +769,118 @@ bool Srv_session::module_deinit() {
 /**
   Checks if the session is valid.
 
-  Checked is if session is NULL, or in the list of opened sessions. If the
-  session is not in this list it was either closed or the address is invalid.
+  Checked is if session is NULL, or the state of the session is
+  SRV_SESSION_OPENED, SRV_SESSION_ATTACHED or SRV_SESSION_DETACHED.
 
   @return
     true  valid
     false not valid
 */
 bool Srv_session::is_valid(const Srv_session *session) {
-  const THD *thd = session ? &session->thd : nullptr;
-  return thd ? (bool)server_session_list.find(thd) : false;
+  assert(session != nullptr);
+  const bool is_valid_session = ((session->m_state > SRV_SESSION_CREATED) &&
+                                 (session->m_state < SRV_SESSION_CLOSED));
+
+  if (session->is_associate() || session->is_associated()) return true;
+
+  /*
+    Make sure valid session exists and invalid sessions doesn't exists in the
+    list of opened sessions.
+  */
+  assert((is_valid_session && server_session_list.find(session->m_thd)) ||
+         (!is_valid_session && !server_session_list.find(session->m_thd)));
+
+  return is_valid_session;
 }
 
 /**
   Constructs a server session
-
-  @param err_cb         Default completion callback
-  @param err_cb_ctx     Plugin's context, opaque pointer that would
-                        be provided to callbacks. Might be NULL.
 */
 Srv_session::Srv_session(srv_session_error_cb err_cb, void *err_cb_ctx)
-    : da(false),
-      err_protocol_ctx(err_cb, err_cb_ctx),
-      protocol_error(&error_protocol_callbacks, CS_TEXT_REPRESENTATION,
-                     (void *)&err_protocol_ctx),
-      state(SRV_SESSION_CREATED),
-      vio_type(NO_VIO_TYPE) {
-  thd.mark_as_srv_session();
+    : Srv_session(err_cb, err_cb_ctx, SRV_SESSION_CREATED, true, nullptr) {
+  m_thd->mark_as_srv_session();
+  m_thd->m_audited = false;
+}
+
+Srv_session::Srv_session(srv_session_error_cb err_cb, void *err_cb_ctx,
+                         THD *thd)
+    : Srv_session(err_cb, err_cb_ctx, SRV_SESSION_ASSOCIATE, false, thd) {
+  assert(thd);  // A valid thd must be passed.
+  m_thd->push_protocol(&m_protocol_error);
+  m_thd->push_diagnostics_area(&m_da);
+}
+
+/** Delegated constructor to intialize the members. */
+Srv_session::Srv_session(srv_session_error_cb err_cb, void *err_cb_ctx,
+                         srv_session_state state, bool free_resources, THD *thd)
+    : m_da(false),
+      m_err_protocol_ctx(err_cb, err_cb_ctx),
+      m_protocol_error(&error_protocol_callbacks, CS_TEXT_REPRESENTATION,
+                       (void *)&m_err_protocol_ctx),
+      m_state(state),
+      m_vio_type(NO_VIO_TYPE),
+      m_thd(thd),
+      m_free_resources(free_resources) {
+  try {
+    if (m_thd == nullptr) m_thd = new THD();
+  } catch (...) {
+    DBUG_PRINT("error", ("Can't allocate the THD object"));
+    throw;
+  }
+}
+
+Srv_session::~Srv_session() {
+  if (m_free_resources) {
+    delete m_thd;
+  } else {
+    mysql_mutex_lock(&m_thd->LOCK_thd_protocol);
+    m_thd->pop_protocol();
+    mysql_mutex_unlock(&m_thd->LOCK_thd_protocol);
+    m_thd->pop_diagnostics_area();
+  }
 }
 
 /**
-  Opens a server session
+Opens a server session
 
-  @return
-    false  on success
-    true   on failure
+@return
+false  on success
+true   on failure
 */
 bool Srv_session::open() {
   char stack_start;
-  DBUG_ENTER("Srv_session::open");
+  DBUG_TRACE;
 
-  DBUG_PRINT("info", ("Session=%p  THD=%p  DA=%p", this, &thd, &da));
-  DBUG_ASSERT(state == SRV_SESSION_CREATED || state == SRV_SESSION_CLOSED);
+  DBUG_PRINT("info", ("Session=%p  THD=%p  DA=%p", this, m_thd, &m_da));
+  assert(m_state == SRV_SESSION_CREATED || m_state == SRV_SESSION_CLOSED);
 
-  thd.push_protocol(&protocol_error);
-  thd.push_diagnostics_area(&da);
+  m_thd->push_protocol(&m_protocol_error);
+  m_thd->push_diagnostics_area(&m_da);
   /*
-    thd.stack_start will be set once we start attempt to attach.
+    m_thd.stack_start will be set once we start attempt to attach.
     store_globals() will check for it, so we will set it beforehand.
 
     No store_globals() here as the session is always created in a detached
     state. Attachment with store_globals() will happen on demand.
   */
-  if (thd_init_client_charset(&thd, my_charset_utf8_general_ci.number)) {
+  if (thd_init_client_charset(m_thd, my_charset_utf8mb3_general_ci.number)) {
     connection_errors_internal++;
-    if (err_protocol_ctx.handler)
-      err_protocol_ctx.handler(err_protocol_ctx.handler_context,
-                               ER_OUT_OF_RESOURCES,
-                               ER_DEFAULT(ER_OUT_OF_RESOURCES));
+    if (m_err_protocol_ctx.handler)
+      m_err_protocol_ctx.handler(m_err_protocol_ctx.handler_context,
+                                 ER_OUT_OF_RESOURCES,
+                                 ER_DEFAULT(ER_OUT_OF_RESOURCES));
     Connection_handler_manager::dec_connection_count();
-    DBUG_RETURN(true);
+    return true;
   }
 
-  thd.update_charset();
+  m_thd->update_charset();
 
-  thd.set_new_thread_id();
+  m_thd->set_new_thread_id();
 
-  DBUG_PRINT("info", ("thread_id=%d", thd.thread_id()));
+  DBUG_PRINT("info", ("thread_id=%d", m_thd->thread_id()));
 
-  thd.set_command(COM_SLEEP);
-  thd.init_query_mem_roots();
+  m_thd->set_command(COM_SLEEP);
+  m_thd->init_query_mem_roots();
 
   /*
     Set current_thd so that it can be used during authentication,
@@ -814,22 +888,19 @@ bool Srv_session::open() {
     separation between open() and attach() so it is likely that
     a conceptually better solution is required long-term.
   */
-  thd_set_thread_stack(&thd, &stack_start);
-  thd.store_globals();
+  thd_set_thread_stack(m_thd, &stack_start);
+  m_thd->store_globals();
 
-  Global_THD_manager::get_instance()->add_thd(&thd);
+  Global_THD_manager::get_instance()->add_thd(m_thd);
 
-  const void *plugin = THR_srv_session_thread;
+  const st_plugin_int *plugin = THR_srv_session_thread;
+  m_thd->set_plugin(plugin);
 
-  server_session_list.add(&thd, plugin, this);
+  server_session_list.add(m_thd, plugin, this);
 
-  if (mysql_audit_notify(
-          &thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE))) {
-    Connection_handler_manager::dec_connection_count();
-    DBUG_RETURN(true);
-  }
+  m_state = SRV_SESSION_OPENED;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -840,22 +911,23 @@ bool Srv_session::open() {
     true    failure
 */
 bool Srv_session::attach() {
-  const bool first_attach = (state == SRV_SESSION_CREATED);
-  DBUG_ENTER("Srv_session::attach");
+  const bool first_attach = (m_state == SRV_SESSION_OPENED);
+  DBUG_TRACE;
   DBUG_PRINT("info", ("current_thd=%p", current_thd));
+  assert(m_state > SRV_SESSION_CREATED && m_state < SRV_SESSION_CLOSED);
 
   if (is_attached()) {
-    if (!my_thread_equal(thd.real_id, my_thread_self())) {
+    if (!my_thread_equal(m_thd->real_id, my_thread_self())) {
       DBUG_PRINT("error", ("Attached to different thread. Detach in it"));
-      DBUG_RETURN(true);
+      return true;
     }
     /* As it is attached, no need to do anything */
-    DBUG_RETURN(false);
+    return false;
   }
 
   // Since we now set current_thd during open(), we need to do complete
   // attach the first time in any case.
-  if (!first_attach && &thd == current_thd) DBUG_RETURN(false);
+  if (!first_attach && m_thd == current_thd) return false;
 
   THD *old_thd = current_thd;
   DBUG_PRINT("info", ("current_thd=%p", current_thd));
@@ -866,8 +938,7 @@ bool Srv_session::attach() {
 
   const char *new_stack = THR_srv_session_thread
                               ? THR_stack_start_address
-                              : (old_thd ? old_thd->thread_stack : NULL);
-
+                              : (old_thd ? old_thd->thread_stack : nullptr);
   /*
     Attach optimistically, as this will set thread_stack,
     which needed by store_globals()
@@ -875,38 +946,35 @@ bool Srv_session::attach() {
   set_attached(new_stack);
 
   // This will install our new THD object as current_thd
-  thd.store_globals();
+  m_thd->store_globals();
 
   Srv_session *old_session = server_session_list.find(old_thd);
 
   /* Really detach only if we are sure everything went fine */
   if (old_session) old_session->set_detached();
 
-  thd_clear_errors(&thd);
+  thd_clear_errors(m_thd);
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  set_psi(&thd);
+  set_psi(m_thd);
 
   PSI_THREAD_CALL(set_connection_type)
-  (vio_type != NO_VIO_TYPE ? vio_type : thd.get_vio_type());
+  (m_vio_type != NO_VIO_TYPE ? m_vio_type : m_thd->get_vio_type());
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
   if (first_attach) {
     /*
       At first attach the security context should have been already set and
-      and this will report corect information.
+      and this will report correct information.
     */
-    if (mysql_audit_notify(&thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_CONNECT)))
-      DBUG_RETURN(true);
-
 #ifdef HAVE_PSI_THREAD_INTERFACE
-    PSI_THREAD_CALL(notify_session_connect)(thd.get_psi());
+    PSI_THREAD_CALL(notify_session_connect)(m_thd->get_psi());
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
-    query_logger.general_log_print(&thd, COM_CONNECT, NullS);
+    query_logger.general_log_print(m_thd, COM_CONNECT, NullS);
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -917,23 +985,24 @@ bool Srv_session::attach() {
     true  failure
 */
 bool Srv_session::detach() {
-  DBUG_ENTER("Srv_session::detach");
+  DBUG_TRACE;
 
-  if (!is_attached()) DBUG_RETURN(false);
+  if (!is_attached()) return false;
 
-  if (!my_thread_equal(thd.real_id, my_thread_self())) {
+  if (!my_thread_equal(m_thd->real_id, my_thread_self())) {
     DBUG_PRINT("error", ("Attached to a different thread. Detach in it"));
-    DBUG_RETURN(true);
+    return true;
   }
 
   DBUG_PRINT("info",
-             ("Session=%p THD=%p current_thd=%p", this, &thd, current_thd));
+             ("Session=%p THD=%p current_thd=%p", this, m_thd, current_thd));
 
-  DBUG_ASSERT(&thd == current_thd);
-  thd.restore_globals();
+  assert(m_thd == current_thd);
+
+  m_thd->restore_globals();
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  set_psi(NULL);
+  set_psi(nullptr);
 #endif
   /*
     We can't call PSI_THREAD_CALL(set_connection_type)(NO_VIO_TYPE) here because
@@ -951,7 +1020,7 @@ bool Srv_session::detach() {
     nulled by set_detached()
   */
   set_detached();
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -962,12 +1031,12 @@ bool Srv_session::detach() {
     true  No such session exists / Session is attached to a different thread
 */
 bool Srv_session::close() {
-  DBUG_ENTER("Srv_session::close");
+  DBUG_TRACE;
 
   DBUG_PRINT("info",
-             ("Session=%p THD=%p current_thd=%p", this, &thd, current_thd));
+             ("Session=%p THD=%p current_thd=%p", this, m_thd, current_thd));
 
-  DBUG_ASSERT(state < SRV_SESSION_CLOSED);
+  assert(m_state < SRV_SESSION_CLOSED);
 
   /*
     RAII
@@ -976,50 +1045,57 @@ bool Srv_session::close() {
     The destructor will attach the session we detached.
   */
 
-  Srv_session::Session_backup_and_attach backup(this);
+  Srv_session::Session_backup_and_attach backup(this, true);
 
-  if (backup.attach_error) DBUG_RETURN(true);
+  if (backup.attach_error) return true;
 
-  state = SRV_SESSION_CLOSED;
+  m_state = SRV_SESSION_CLOSED;
 
-  server_session_list.remove(&thd);
+  server_session_list.remove(m_thd);
 
   /*
     Log to general log must happen before release_resources() as
     current_thd will be different then.
   */
-  query_logger.general_log_print(&thd, COM_QUIT, NullS);
-  mysql_audit_notify(&thd, AUDIT_EVENT(MYSQL_AUDIT_CONNECTION_DISCONNECT), 0);
+  query_logger.general_log_print(m_thd, COM_QUIT, NullS);
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  PSI_THREAD_CALL(notify_session_disconnect)(thd.get_psi());
+  PSI_THREAD_CALL(notify_session_disconnect)(m_thd->get_psi());
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
-  thd.security_context()->logout();
-  thd.m_view_ctx_list.empty();
-  close_mysql_tables(&thd);
+  m_thd->security_context()->logout();
+  m_thd->m_view_ctx_list.clear();
+  close_mysql_tables(m_thd);
 
-  thd.pop_diagnostics_area();
+  m_thd->set_plugin(nullptr);
+  m_thd->pop_diagnostics_area();
 
-  thd.get_stmt_da()->reset_diagnostics_area();
+  m_thd->get_stmt_da()->reset_diagnostics_area();
 
-  thd.disconnect();
+  // DEBUG_SYNC control block is released under call to
+  // `m_thd->release_resource`, thus we can't put this sync
+  // point directly before `pop_protocol`.
+  // Second constrain is that `THD::disconnect` marks
+  // this connection as killed, which disables DEBUG_SYNC.
+  DEBUG_SYNC(m_thd, "srv_session_close");
+
+  m_thd->disconnect();
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  set_psi(NULL);
+  set_psi(nullptr);
 #endif
 
-  thd.release_resources();
+  m_thd->release_resources();
 
-  mysql_mutex_lock(&thd.LOCK_thd_protocol);
-  thd.pop_protocol();
-  mysql_mutex_unlock(&thd.LOCK_thd_protocol);
+  Global_THD_manager::get_instance()->remove_thd(m_thd);
 
-  Global_THD_manager::get_instance()->remove_thd(&thd);
+  mysql_mutex_lock(&m_thd->LOCK_thd_protocol);
+  m_thd->pop_protocol();
+  mysql_mutex_unlock(&m_thd->LOCK_thd_protocol);
 
   Connection_handler_manager::dec_connection_count();
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1028,16 +1104,16 @@ bool Srv_session::close() {
   @param stack  New stack address
 */
 void Srv_session::set_attached(const char *stack) {
-  state = SRV_SESSION_ATTACHED;
-  thd_set_thread_stack(&thd, stack);
+  m_state = SRV_SESSION_ATTACHED;
+  thd_set_thread_stack(m_thd, stack);
 }
 
 /**
   Changes the state of a session to detached
 */
 void Srv_session::set_detached() {
-  state = SRV_SESSION_DETACHED;
-  thd_set_thread_stack(&thd, NULL);
+  m_state = SRV_SESSION_DETACHED;
+  thd_set_thread_stack(m_thd, nullptr);
 }
 
 int Srv_session::execute_command(enum enum_server_command command,
@@ -1046,57 +1122,69 @@ int Srv_session::execute_command(enum enum_server_command command,
                                  const struct st_command_service_cbs *callbacks,
                                  enum cs_text_or_binary text_or_binary,
                                  void *callbacks_context) {
-  DBUG_ENTER("Srv_session::execute_command");
+  DBUG_TRACE;
 
   if (!srv_session_server_is_available()) {
-    if (err_protocol_ctx.handler)
-      err_protocol_ctx.handler(err_protocol_ctx.handler_context,
-                               ER_SESSION_WAS_KILLED,
-                               ER_DEFAULT(ER_SESSION_WAS_KILLED));
-    DBUG_RETURN(1);
+    if (m_err_protocol_ctx.handler)
+      m_err_protocol_ctx.handler(m_err_protocol_ctx.handler_context,
+                                 ER_SESSION_WAS_KILLED,
+                                 ER_DEFAULT(ER_SESSION_WAS_KILLED));
+    return 1;
   }
 
-  if (thd.killed) {
-    if (err_protocol_ctx.handler)
-      err_protocol_ctx.handler(err_protocol_ctx.handler_context,
-                               ER_SESSION_WAS_KILLED,
-                               ER_DEFAULT(ER_SESSION_WAS_KILLED));
-    DBUG_RETURN(1);
+  if (m_thd->killed) {
+    if (m_err_protocol_ctx.handler)
+      m_err_protocol_ctx.handler(m_err_protocol_ctx.handler_context,
+                                 ER_SESSION_WAS_KILLED,
+                                 ER_DEFAULT(ER_SESSION_WAS_KILLED));
+    return 1;
   }
 
-  DBUG_ASSERT(thd.get_protocol() == &protocol_error);
+  assert(m_thd->get_protocol() == &m_protocol_error);
 
   // RAII:the destructor restores the state
-  Srv_session::Session_backup_and_attach backup(this);
+  Srv_session::Session_backup_and_attach backup(this, false);
 
-  if (backup.attach_error) DBUG_RETURN(1);
+  if (backup.attach_error) return 1;
 
-  if (client_cs && thd.variables.character_set_results != client_cs &&
-      thd_init_client_charset(&thd, client_cs->number))
-    DBUG_RETURN(1);
+  if (client_cs && m_thd->variables.character_set_results != client_cs &&
+      thd_init_client_charset(m_thd, client_cs->number))
+    return 1;
 
   /* Switch to different callbacks */
   Protocol_callback client_proto(callbacks, text_or_binary, callbacks_context);
 
-  thd.push_protocol(&client_proto);
+  m_thd->push_protocol(&client_proto);
 
-  mysql_audit_release(&thd);
+  mysql_audit_release(m_thd);
 
   /*
-    The server does it for COM_QUERY in mysql_parse() but not for
+    The server does it for COM_QUERY in dispatch_sql_command() but not for
     COM_INIT_DB, for example
   */
-  if (command != COM_QUERY) thd.reset_for_next_command();
+  if (command != COM_QUERY) m_thd->reset_for_next_command();
 
-  DBUG_ASSERT(thd.m_statement_psi == NULL);
-  thd.m_statement_psi =
-      MYSQL_START_STATEMENT(&thd.m_statement_state, stmt_info_new_packet.m_key,
-                            thd.db().str, thd.db().length, thd.charset(), NULL);
-  int ret = dispatch_command(&thd, data, command);
+  /* For per-query performance counters with log_slow_statement */
+  struct System_status_var query_start_status;
+  m_thd->clear_copy_status_var();
+  if (opt_log_slow_extra) {
+    m_thd->copy_status_var(&query_start_status);
+  }
 
-  thd.pop_protocol();
-  DBUG_ASSERT(thd.get_protocol() == &protocol_error);
-  DBUG_RETURN(ret);
+  mysql_thread_set_secondary_engine(false);
+
+  if (m_state != SRV_SESSION_ASSOCIATED) {
+    assert(m_thd->m_statement_psi == nullptr);
+    m_thd->m_statement_psi = MYSQL_START_STATEMENT(
+        &m_thd->m_statement_state, stmt_info_new_packet.m_key, m_thd->db().str,
+        m_thd->db().length, m_thd->charset(), nullptr);
+  }
+  int ret = dispatch_command(m_thd, data, command);
+
+  DEBUG_SYNC(m_thd, "wait_before_popping_protocol");
+  m_thd->pop_protocol();
+  assert(m_thd->get_protocol() == &m_protocol_error);
+  return ret;
 }
 
 /**
@@ -1111,54 +1199,51 @@ int Srv_session::execute_command(enum enum_server_command command,
 bool Srv_session::set_connection_type(enum_vio_type v_type) {
   if (v_type < FIRST_VIO_TYPE || v_type > LAST_VIO_TYPE) return true;
 
-  vio_type = v_type;
+  m_vio_type = v_type;
 #ifdef HAVE_PSI_THREAD_INTERFACE
-  if (is_attached()) PSI_THREAD_CALL(set_connection_type)(vio_type);
+  if (is_attached()) PSI_THREAD_CALL(set_connection_type)(m_vio_type);
 #endif /* HAVE_PSI_THREAD_INTERFACE */
   return false;
 }
 
-/**
-  Template class for scanning the thd list in the Global_THD_manager and
-  modifying and element from the list. This template is for functions that
-  need to change data of a THD without getting into races.
+void Srv_session::set_associate() { m_state = SRV_SESSION_ASSOCIATED; }
 
-  If the THD is found, a callback is executed under THD::Lock_thd_data.
-  A pointer to a member variable is passed as a second parameter to the
-  callback. The callback should store its result there.
+void Srv_session::set_disassociate() { m_state = SRV_SESSION_DISASSOCIATED; }
 
-  After the search has been completed the result value can be obtained by
-  calling get_result().
-*/
-template <typename INPUT_TYPE>
-class Find_thd_by_id_with_callback_set : public Find_THD_Impl {
- public:
-  typedef void (*callback_t)(THD *thd, INPUT_TYPE *result);
+bool Srv_session::is_associate() const {
+  return m_state == SRV_SESSION_ASSOCIATE;
+}
 
-  Find_thd_by_id_with_callback_set(my_thread_id t_id, callback_t cb,
-                                   INPUT_TYPE in)
-      : thread_id(t_id), callback(cb), input(in) {}
+bool Srv_session::is_associated() const {
+  return m_state == SRV_SESSION_ASSOCIATED;
+}
 
-  /**
-    Callback called for every THD in the thd_list.
+bool Srv_session::associate() {
+  if (!is_associate()) return true;
+  const char *stack_start = m_thd->thread_stack ? m_thd->thread_stack : nullptr;
+  thd_set_thread_stack(m_thd, stack_start);
+  // This will install our new THD object as current_thd
+  m_thd->store_globals();
+  thd_clear_errors(m_thd);
+  set_associate();
+  return false;
+}
 
-    When a thread is found the callback function passed to the constructor
-    is invoked under THD::Lock_thd_data
-  */
-  virtual bool operator()(THD *thd) {
-    if (thd->thread_id() == thread_id) {
-      MUTEX_LOCK(lock, &thd->LOCK_thd_data);
-      callback(thd, &input);
-      return true;
-    }
-    return false;
+bool Srv_session::disassociate() {
+  if (!is_associated()) return true;
+
+  if (!my_thread_equal(m_thd->real_id, my_thread_self())) {
+    DBUG_PRINT("error", ("Attached to a different thread. Detach in it"));
+    return true;
   }
 
- private:
-  my_thread_id thread_id;
-  callback_t callback;
-  INPUT_TYPE input;
-};
+  DBUG_PRINT("info",
+             ("Session=%p THD=%p current_thd=%p", this, m_thd, current_thd));
+
+  m_thd->restore_globals();
+  set_disassociate();
+  return false;
+}
 
 /**
   Callback for inspecting a THD object and modifying the peer_port member
@@ -1176,9 +1261,10 @@ static void set_client_port_in_thd(THD *thd, uint16_t *input) {
   @param port  Port number
 */
 void Srv_session::set_client_port(uint16_t port) {
-  Find_thd_by_id_with_callback_set<uint16_t> find_thd_with_id(
-      thd.thread_id(), set_client_port_in_thd, port);
-  Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  Find_thd_with_id find_thd_with_id(m_thd->thread_id());
+  THD_ptr thd_ptr =
+      Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  if (thd_ptr) set_client_port_in_thd(thd_ptr.get(), &port);
 }
 
 /**
