@@ -1,15 +1,16 @@
-/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,8 +24,11 @@
 #ifndef TABLE_CACHE_INCLUDED
 #define TABLE_CACHE_INCLUDED
 
+#include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
+
+#include <atomic>
 #include <memory>
 #include <string>
 #include <unordered_map>
@@ -32,10 +36,10 @@
 
 #include "lex_string.h"
 #include "my_base.h"
-#include "my_dbug.h"
+
 #include "my_psi_config.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "sql/handler.h"
 #include "sql/sql_base.h"
@@ -43,10 +47,12 @@
 #include "sql/sql_plist.h"
 #include "sql/system_variables.h"
 #include "sql/table.h"
+#include "sql/table_trigger_dispatcher.h"
 
 class Table_cache_element;
 
-extern ulong table_cache_size_per_instance, table_cache_instances;
+extern ulong table_cache_size_per_instance, table_cache_instances,
+    table_cache_triggers, table_cache_triggers_per_instance;
 
 /**
   Cache for open TABLE objects.
@@ -115,6 +121,24 @@ class Table_cache {
   */
   uint m_table_count;
 
+  /**
+    LRU-organized list containing all TABLE instances with fully-loaded
+    triggers in this table cache which are not in use by any thread.
+  */
+  I_P_List<TABLE,
+           I_P_List_adapter<TABLE, &TABLE::triggers_lru_next,
+                            &TABLE::triggers_lru_prev>,
+           I_P_List_null_counter, I_P_List_fast_push_back<TABLE>>
+      m_unused_triggers_lru;
+
+  /**
+    Total number of TABLE instances in this table cache with fully-loaded
+    triggers (both in use and unused).
+
+    @sa notify_triggers_load() for rationale behind use of atomic here.
+  */
+  std::atomic<uint> m_table_triggers_count;
+
 #ifdef HAVE_PSI_INTERFACE
   static PSI_mutex_key m_lock_key;
   static PSI_mutex_info m_mutex_keys[];
@@ -144,7 +168,7 @@ class Table_cache {
   void assert_owner() { mysql_mutex_assert_owner(&m_lock); }
 
   inline TABLE *get_table(THD *thd, const char *key, size_t key_length,
-                          TABLE_SHARE **share);
+                          bool is_update, TABLE_SHARE **share);
 
   inline void release_table(THD *thd, TABLE *table);
 
@@ -156,7 +180,18 @@ class Table_cache {
 
   void free_all_unused_tables();
 
-#ifndef DBUG_OFF
+  /**
+    Notify the table cache that for one of its TABLE objects we have
+    finalized loading and parsing triggers.
+
+    @note We use atomic to make it MT-safe without introducing overhead
+          from lock()/unlock() pair.
+  */
+  void notify_triggers_load() { m_table_triggers_count++; }
+
+  uint loaded_triggers_tables() const { return m_table_triggers_count; }
+
+#ifndef NDEBUG
   void print_tables();
 #endif
 };
@@ -188,8 +223,11 @@ class Table_cache_manager {
 
   uint cached_tables();
 
+  uint loaded_triggers_tables() const;
+
   void lock_all_and_tdc();
   void unlock_all_and_tdc();
+  void assert_owner(THD *thd);
   void assert_owner_all();
   void assert_owner_all_and_tdc();
 
@@ -198,7 +236,7 @@ class Table_cache_manager {
 
   void free_all_unused_tables();
 
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   void print_tables();
 #endif
 
@@ -234,7 +272,14 @@ class Table_cache_element {
       TABLE_list;
 
   TABLE_list used_tables;
-  TABLE_list free_tables;
+  /**
+    List of unused TABLE objects for tables without triggers or unused TABLE
+    objects for which triggers were not fully-loaded, so they can only can be
+    used by read-only statements.
+  */
+  TABLE_list free_tables_slim;
+  /** List of unused TABLE objects with fully-loaded triggers. */
+  TABLE_list free_tables_full_triggers;
   TABLE_SHARE *share;
 
  public:
@@ -273,6 +318,8 @@ class Table_cache_iterator {
 /**
   Add table to the tail of unused tables list for table cache
   (i.e. as the most recently used table in this list).
+  If necessary, do the same thing for list of unused tables with
+  fully-loaded triggers.
 */
 
 void Table_cache::link_unused_table(TABLE *table) {
@@ -284,18 +331,28 @@ void Table_cache::link_unused_table(TABLE *table) {
   } else
     m_unused_tables = table->next = table->prev = table;
   check_unused();
+
+  if (table->triggers && table->triggers->has_load_been_finalized())
+    m_unused_triggers_lru.push_back(table);
 }
 
-/** Remove table from the unused tables list for table cache. */
+/**
+  Remove table from the unused tables list for the table cache.
+  If necessary, do the same thing for list of unused tables with
+  fully-loaded triggers for the table cache.
+*/
 
 void Table_cache::unlink_unused_table(TABLE *table) {
   table->next->prev = table->prev;
   table->prev->next = table->next;
   if (table == m_unused_tables) {
     m_unused_tables = m_unused_tables->next;
-    if (table == m_unused_tables) m_unused_tables = NULL;
+    if (table == m_unused_tables) m_unused_tables = nullptr;
   }
   check_unused();
+
+  if (table->triggers && table->triggers->has_load_been_finalized())
+    m_unused_triggers_lru.remove(table);
 }
 
 /**
@@ -314,8 +371,14 @@ void Table_cache::free_unused_tables_if_necessary(THD *thd) {
     Note that we might need to free more than one TABLE object, and thus
     need the below loop, in case when table_cache_size is changed dynamically,
     at server run time.
+
+    We also might need to get rid of TABLE instances with fully-loaded triggers
+    if there are too many of them. Unfortunately, there is no good way to
+    "unload" triggers, so we have to get rid of the whole TABLE object.
   */
-  if (m_table_count > table_cache_size_per_instance && m_unused_tables) {
+  if ((m_table_count > table_cache_size_per_instance && m_unused_tables) ||
+      (m_table_triggers_count > table_cache_triggers_per_instance &&
+       !m_unused_triggers_lru.is_empty())) {
     mysql_mutex_lock(&LOCK_open);
     while (m_table_count > table_cache_size_per_instance && m_unused_tables) {
       TABLE *table_to_free = m_unused_tables;
@@ -323,6 +386,13 @@ void Table_cache::free_unused_tables_if_necessary(THD *thd) {
       intern_close_table(table_to_free);
       thd->status_var.table_open_cache_overflows++;
     }
+    while (m_table_triggers_count > table_cache_triggers_per_instance &&
+           !m_unused_triggers_lru.is_empty()) {
+      TABLE *table_to_free = m_unused_triggers_lru.front();
+      remove_table(table_to_free);
+      intern_close_table(table_to_free);
+      thd->status_var.table_open_cache_triggers_overflows++;
+    }
     mysql_mutex_unlock(&LOCK_open);
   }
 }
@@ -344,7 +414,7 @@ bool Table_cache::add_used_table(THD *thd, TABLE *table) {
 
   assert_owner();
 
-  DBUG_ASSERT(table->in_use == thd);
+  assert(table->in_use == thd);
 
   /*
     Try to get Table_cache_element representing this table in the cache
@@ -363,7 +433,7 @@ bool Table_cache::add_used_table(THD *thd, TABLE *table) {
     */
     std::string key(table->s->table_cache_key.str,
                     table->s->table_cache_key.length);
-    DBUG_ASSERT(m_cache.count(key) == 0);
+    assert(m_cache.count(key) == 0);
 
     el = new Table_cache_element(table->s);
     m_cache.emplace(key, std::unique_ptr<Table_cache_element>(el));
@@ -398,15 +468,21 @@ void Table_cache::remove_table(TABLE *table) {
     el->used_tables.remove(table);
   } else {
     /* Remove from per-table chain of unused TABLE objects. */
-    el->free_tables.remove(table);
+    if (table->triggers && table->triggers->has_load_been_finalized())
+      el->free_tables_full_triggers.remove(table);
+    else
+      el->free_tables_slim.remove(table);
 
     /* And per-cache unused chain. */
     unlink_unused_table(table);
   }
 
   m_table_count--;
+  if (table->triggers && table->triggers->has_load_been_finalized())
+    m_table_triggers_count--;
 
-  if (el->used_tables.is_empty() && el->free_tables.is_empty()) {
+  if (el->used_tables.is_empty() && el->free_tables_full_triggers.is_empty() &&
+      el->free_tables_slim.is_empty()) {
     std::string key(table->s->table_cache_key.str,
                     table->s->table_cache_key.length);
     m_cache.erase(key);
@@ -414,7 +490,7 @@ void Table_cache::remove_table(TABLE *table) {
       Remove reference to deleted cache element from array
       in the TABLE_SHARE.
     */
-    table->s->cache_element[table_cache_manager.cache_index(this)] = NULL;
+    table->s->cache_element[table_cache_manager.cache_index(this)] = nullptr;
   }
 }
 
@@ -424,6 +500,10 @@ void Table_cache::remove_table(TABLE *table) {
   @param      thd         Thread context.
   @param      key         Key identifying table.
   @param      key_length  Length of key for the table.
+  @param      is_update   Indicates whether statement is going to use
+                          TABLE object for updating the table, so it
+                          is better to obtain TABLE instance with
+                          fully-loaded triggers.
   @param[out] share       NULL - if table cache doesn't contain any
                           information about the table (i.e. doesn't have
                           neither used nor unused TABLE objects for it).
@@ -440,28 +520,47 @@ void Table_cache::remove_table(TABLE *table) {
 */
 
 TABLE *Table_cache::get_table(THD *thd, const char *key, size_t key_length,
-                              TABLE_SHARE **share) {
+                              bool is_update, TABLE_SHARE **share) {
   TABLE *table;
 
   assert_owner();
 
-  *share = NULL;
+  *share = nullptr;
 
   std::string key_str(key, key_length);
   const auto el_it = m_cache.find(key_str);
-  if (el_it == m_cache.end()) return NULL;
+  if (el_it == m_cache.end()) return nullptr;
   Table_cache_element *el = el_it->second.get();
 
   *share = el->share;
 
-  if ((table = el->free_tables.front())) {
-    DBUG_ASSERT(!table->in_use);
-
+  /*
+    Obtain (get first and unlink) table from list of unused TABLE objects for
+    this table in this cache.
+  */
+  if (!is_update) {
     /*
-      Unlink table from list of unused TABLE objects for this
-      table in this cache.
+      For read-only statements we prefer TABLE objects which don't have
+      triggers fully-loaded. If successful this should leave unused TABLEs
+      with fully-loaded triggers for read-write statements.
+      If there are no TABLE instances sans fully-loaded triggers available
+      we will resort to using one with them. It is still better than
+      doing full-blown TABLE construction process.
     */
-    el->free_tables.remove(table);
+    table = el->free_tables_slim.pop_front();
+    if (!table) table = el->free_tables_full_triggers.pop_front();
+  } else {
+    /*
+      For read-write statements try to get TABLE object with fully-loaded
+      triggers. If there is no such object try to use sans them.
+      If necessary trigger load will be finalized later.
+    */
+    table = el->free_tables_full_triggers.pop_front();
+    if (!table) table = el->free_tables_slim.pop_front();
+  }
+
+  if (table) {
+    assert(!table->in_use);
 
     /* Unlink table from unused tables list for this cache. */
     unlink_unused_table(table);
@@ -474,9 +573,9 @@ TABLE *Table_cache::get_table(THD *thd, const char *key, size_t key_length,
 
     table->in_use = thd;
     /* The ex-unused table must be fully functional. */
-    DBUG_ASSERT(table->db_stat && table->file);
+    assert(table->db_stat && table->file);
     /* The children must be detached from the table. */
-    DBUG_ASSERT(!table->file->extra(HA_EXTRA_IS_ATTACHED_CHILDREN));
+    assert(!table->file->ha_extra(HA_EXTRA_IS_ATTACHED_CHILDREN));
   }
 
   return table;
@@ -496,18 +595,21 @@ void Table_cache::release_table(THD *thd, TABLE *table) {
 
   assert_owner();
 
-  DBUG_ASSERT(table->in_use);
-  DBUG_ASSERT(table->file);
+  assert(table->in_use);
+  assert(table->file);
 
   /* We shouldn't put the table to 'unused' list if the share is old. */
-  DBUG_ASSERT(!table->s->has_old_version());
+  assert(!table->s->has_old_version());
 
-  table->in_use = NULL;
+  table->in_use = nullptr;
 
   /* Remove TABLE from the list of used objects for the table in this cache. */
   el->used_tables.remove(table);
   /* Add TABLE to the list of unused objects for the table in this cache. */
-  el->free_tables.push_front(table);
+  if (table->triggers && table->triggers->has_load_been_finalized())
+    el->free_tables_full_triggers.push_front(table);
+  else
+    el->free_tables_slim.push_front(table);
   /* Also link it last in the list of unused TABLE objects for the cache. */
   link_unused_table(table);
 
@@ -524,7 +626,7 @@ void Table_cache::release_table(THD *thd, TABLE *table) {
   @note Assumes that caller owns locks on all table caches.
 */
 Table_cache_iterator::Table_cache_iterator(const TABLE_SHARE *share_arg)
-    : share(share_arg), current_cache_index(0), current_table(NULL) {
+    : share(share_arg), current_cache_index(0), current_table(nullptr) {
   table_cache_manager.assert_owner_all();
   move_to_next_table();
 }
@@ -569,7 +671,7 @@ TABLE *Table_cache_iterator::operator++(int) {
 
 void Table_cache_iterator::rewind() {
   current_cache_index = 0;
-  current_table = NULL;
+  current_table = nullptr;
   move_to_next_table();
 }
 
