@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -826,7 +827,7 @@ table_map Table_function_sequence::used_tables() {
 
 bool Table_function_sequence::print(const THD *thd, String *str,
                                     enum_query_type query_type) const {
-  if (str->append(STRING_WITH_LEN("sequence_table("))) return true;
+  if (str->append(STRING_WITH_LEN("percona_sequence_table("))) return true;
   m_source->print(thd, str, query_type);
   if (thd->is_error()) return true;
   return str->append(')');
@@ -850,7 +851,7 @@ bool Table_function_sequence::do_init_args() {
   assert(m_source->data_type() != MYSQL_TYPE_VAR_STRING);
   if (m_source->has_aggregation() || m_source->has_subquery() ||
       m_source != dummy) {
-    my_error(ER_WRONG_ARGUMENTS, MYF(0), "SEQUENCE_TABLE");
+    my_error(ER_WRONG_ARGUMENTS, MYF(0), "PERCONA_SEQUENCE_TABLE");
     return true;
   }
 
