@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -391,7 +392,7 @@ class Pushed_lex_guard {
 */
 class Column_privilege_tracker {
  public:
-  Column_privilege_tracker(THD *thd, ulong privilege)
+  Column_privilege_tracker(THD *thd, Access_bitmask privilege)
       : m_thd(thd), m_saved_privilege(thd->want_privilege) {
     thd->want_privilege = privilege;
   }
@@ -399,7 +400,7 @@ class Column_privilege_tracker {
 
  private:
   THD *const m_thd;
-  const ulong m_saved_privilege;
+  const Access_bitmask m_saved_privilege;
 };
 
 /**
