@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,12 +25,15 @@
 #include <sys/types.h>
 #include <time.h>
 
+#include <algorithm>
+
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "mysql/service_mysql_alloc.h"
 #include "mysqld_error.h"
 #include "storage/heap/heapdef.h"
+#include "template_utils.h"
 
 static int keys_compare(const void *a, const void *b, const void *c);
 static void init_block(HP_BLOCK *block, uint reclength, ulong min_records,
@@ -47,7 +51,7 @@ static void init_block(HP_BLOCK *block, uint reclength, ulong min_records,
 int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
                 bool *created_new_share) {
   uint i, j, key_segs, max_length, length;
-  HP_SHARE *share = 0;
+  HP_SHARE *share = nullptr;
   HA_KEYSEG *keyseg;
   HP_KEYDEF *keydef = create_info->keydef;
   uint reclength = create_info->reclength;
@@ -55,17 +59,17 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
   ulong min_records = create_info->min_records;
   ulong max_records = create_info->max_records;
   ulong max_rows_for_stated_memory;
-  DBUG_ENTER("heap_create");
+  DBUG_TRACE;
 
   if (!create_info->single_instance) {
     mysql_mutex_lock(&THR_LOCK_heap);
     share = hp_find_named_heap(name);
     if (share && share->open_count == 0) {
       hp_free(share);
-      share = 0;
+      share = nullptr;
     }
   }
-  *created_new_share = (share == NULL);
+  *created_new_share = (share == nullptr);
 
   if (!share) {
     uint chunk_dataspace_length, chunk_length;
@@ -143,7 +147,7 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
 
       /* Check whether we have any variable size records past key data */
       fixed_data_length = create_info->fixed_data_size;
-      DBUG_ASSERT(i == create_info->fixed_key_fieldnr);
+      assert(i == create_info->fixed_key_fieldnr);
       for (; i < create_info->columns; i++) {
         HP_COLUMNDEF *column = create_info->columndef + i;
         if ((column->type == MYSQL_TYPE_VARCHAR &&
@@ -189,7 +193,8 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
       We store uchar* del_link inside the data area of deleted records,
       so the data length should be at least sizeof(uchar*)
     */
-    set_if_bigger(chunk_dataspace_length, sizeof(uchar **));
+    chunk_dataspace_length =
+        std::max(chunk_dataspace_length, static_cast<uint>(sizeof(uchar **)));
 
     if (is_variable_size) {
       chunk_length = chunk_dataspace_length + VARIABLE_REC_OVERHEAD;
@@ -255,8 +260,7 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
                   key_segs * sizeof(HA_KEYSEG) + alignof(HP_KEYDEF) - 1,
               MYF(MY_ZEROFILL))))
       goto err;
-    DBUG_ASSERT(reinterpret_cast<uintptr_t>(share) % alignof(decltype(share)) ==
-                0);
+    assert(reinterpret_cast<uintptr_t>(share) % alignof(decltype(share)) == 0);
     /*
       Max_records is used for estimating block sizes and for enforcement.
       Calculate the very maximum number of rows (if everything was one chunk)
@@ -271,9 +275,9 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
                        : max_rows_for_stated_memory);
 
     share->column_defs = (HP_COLUMNDEF *)(share + 1);
-    DBUG_ASSERT(reinterpret_cast<uintptr_t>(share->column_defs) %
-                    alignof(decltype(share->column_defs)) ==
-                0);
+    assert(reinterpret_cast<uintptr_t>(share->column_defs) %
+               alignof(decltype(share->column_defs)) ==
+           0);
     memcpy(share->column_defs, create_info->columndef,
            (size_t)(sizeof(create_info->columndef[0]) * create_info->columns));
 
@@ -283,9 +287,9 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
     if (align_remainder)
       share->keydef = reinterpret_cast<HP_KEYDEF *>(
           keydef_ptr + alignof(decltype(share->keydef)) - align_remainder);
-    DBUG_ASSERT(reinterpret_cast<uintptr_t>(share->keydef) %
-                    alignof(decltype(share->keydef)) ==
-                0);
+    assert(reinterpret_cast<uintptr_t>(share->keydef) %
+               alignof(decltype(share->keydef)) ==
+           0);
     share->key_stat_version = 1;
     keyseg = (HA_KEYSEG *)(share->keydef + keys);
     init_block(&share->recordspace.block, chunk_length, min_records,
@@ -306,8 +310,8 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
         keyseg->null_bit = 0;
         keyseg++;
 
-        init_tree(&keyinfo->rb_tree, 0, 0, sizeof(uchar *), keys_compare, 1,
-                  NULL, NULL);
+        init_tree(&keyinfo->rb_tree, 0, sizeof(uchar *), keys_compare, true,
+                  nullptr, nullptr);
         keyinfo->delete_key = hp_rb_delete_key;
         keyinfo->write_key = hp_rb_write_key;
       } else {
@@ -332,7 +336,7 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
     share->auto_key = create_info->auto_key;
     share->auto_key_type = create_info->auto_key_type;
     share->auto_increment = create_info->auto_increment;
-    share->create_time = (long)time((time_t *)0);
+    share->create_time = (long)time((time_t *)nullptr);
     share->fixed_data_length = fixed_data_length;
     share->fixed_column_count = fixed_column_count;
     share->blobs = create_info->blobs;
@@ -374,18 +378,18 @@ int heap_create(const char *name, HP_CREATE_INFO *create_info, HP_SHARE **res,
   }
 
   *res = share;
-  DBUG_RETURN(0);
+  return 0;
 
 err:
   if (!create_info->single_instance) mysql_mutex_unlock(&THR_LOCK_heap);
-  DBUG_RETURN(1);
+  return 1;
 } /* heap_create */
 
 static int keys_compare(const void *a, const void *b, const void *c) {
   uint not_used[2];
-  heap_rb_param *param = (heap_rb_param *)a;
-  uchar *key1 = (uchar *)b;
-  uchar *key2 = (uchar *)c;
+  const heap_rb_param *param = pointer_cast<const heap_rb_param *>(a);
+  const uchar *key1 = pointer_cast<const uchar *>(b);
+  const uchar *key2 = pointer_cast<const uchar *>(c);
   return ha_key_cmp(param->keyseg, key1, key2, param->key_length,
                     param->search_flag, not_used);
 }
@@ -394,7 +398,7 @@ static void init_block(HP_BLOCK *block, uint chunk_length, ulong min_records,
                        ulong max_records) {
   uint i, recbuffer, records_in_block;
 
-  max_records = MY_MAX(min_records, max_records);
+  max_records = std::max(min_records, max_records);
   if (!max_records) max_records = 1000; /* As good as quess as anything */
   /*
     We want to start each chunk at 8 bytes boundary, round recbuffer to the
@@ -418,22 +422,22 @@ static void init_block(HP_BLOCK *block, uint chunk_length, ulong min_records,
   for (i = 0; i <= HP_MAX_LEVELS; i++)
     block->level_info[i].records_under_level =
         (!i ? 1
-            : i == 1 ? records_in_block
-                     : HP_PTRS_IN_NOD *
-                           block->level_info[i - 1].records_under_level);
+         : i == 1
+             ? records_in_block
+             : HP_PTRS_IN_NOD * block->level_info[i - 1].records_under_level);
 }
 
 static inline void heap_try_free(HP_SHARE *share) {
   if (share->open_count == 0)
     hp_free(share);
   else
-    share->delete_on_close = 1;
+    share->delete_on_close = true;
 }
 
 int heap_delete_table(const char *name) {
   int result;
   HP_SHARE *share;
-  DBUG_ENTER("heap_delete_table");
+  DBUG_TRACE;
 
   mysql_mutex_lock(&THR_LOCK_heap);
   if ((share = hp_find_named_heap(name))) {
@@ -444,19 +448,18 @@ int heap_delete_table(const char *name) {
     set_my_errno(result);
   }
   mysql_mutex_unlock(&THR_LOCK_heap);
-  DBUG_RETURN(result);
+  return result;
 }
 
 void heap_drop_table(HP_INFO *info) {
-  DBUG_ENTER("heap_drop_table");
+  DBUG_TRACE;
   mysql_mutex_lock(&THR_LOCK_heap);
   heap_try_free(info->s);
   mysql_mutex_unlock(&THR_LOCK_heap);
-  DBUG_VOID_RETURN;
 }
 
 void hp_free(HP_SHARE *share) {
-  bool not_internal_table = (share->open_list.data != NULL);
+  bool not_internal_table = (share->open_list.data != nullptr);
   if (not_internal_table) /* If not internal table */
     heap_share_list = list_delete(heap_share_list, &share->open_list);
   hp_clear(share); /* Remove blocks from memory */
