@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -30,6 +31,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
  *******************************************************/
 
 #include "clone0desc.h"
+#include "dict0dict.h"
 #include "mach0data.h"
 
 /** Maximum supported descriptor version. The version represents the current
@@ -102,7 +104,7 @@ static const uint CLONE_TASK_BLOCK_OFFSET = CLONE_TASK_CHUNK_OFFSET + 4;
 static const uint CLONE_TASK_META_LEN = CLONE_TASK_BLOCK_OFFSET + 4;
 
 /** Initialize header
-@param[in]	version	descriptor version */
+@param[in]      version descriptor version */
 void Clone_Desc_Task_Meta::init_header(uint version) {
   m_header.m_version = version;
 
@@ -306,10 +308,11 @@ size_t Chunk_Info::get_serialized_length(uint32_t num_tasks) {
   size_t ret_size = 4;
 
   auto num_elements = m_incomplete_chunks.size();
+  auto suggested_elements = 2 * num_tasks;
 
   /* Have bigger allocated length if requested */
-  if (num_tasks > num_elements) {
-    num_elements = num_tasks;
+  if (suggested_elements > num_elements) {
+    num_elements = suggested_elements;
   }
 
   /* Add size for incomplete chunks data. Serialized element
@@ -356,7 +359,7 @@ void Chunk_Info::serialize(byte *desc_chunk, uint &len) {
   ut_ad(len_left >= 4);
   len_left -= 4;
 
-  ulint index = 0;
+  ulint index [[maybe_unused]] = 0;
 
   for (auto &key_value : m_incomplete_chunks) {
     ut_ad(index < chunk_map_size);
@@ -392,15 +395,15 @@ void Chnunk_Bitmap::deserialize(const byte *desc_chunk, uint &len_left) {
   desc_chunk += 4;
 
   if (len_left < 4) {
-    ut_ad(false);
-    return;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   len_left -= 4;
 
   if (bitmap_size > m_size) {
-    ut_ad(false);
-    return;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   for (ulint index = 0; index < bitmap_size; index++) {
@@ -409,8 +412,8 @@ void Chnunk_Bitmap::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_ad(false);
-      return;
+      ut_d(ut_error);
+      ut_o(return);
     }
 
     len_left -= 4;
@@ -425,16 +428,19 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
   desc_chunk += 4;
 
   if (len_left < 4) {
-    ut_ad(false);
-    return;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   len_left -= 4;
 
+  auto max_map_size = static_cast<uint32_t>(2 * CLONE_MAX_TASKS);
   /* Each task can have one incomplete chunk at most */
-  if (chunk_map_size > CLONE_MAX_TASKS) {
-    ut_ad(false);
-    return;
+  if (chunk_map_size > max_map_size) {
+    ib::error(ER_IB_CLONE_RESTART)
+        << "Clone too many incomplete chunks: " << chunk_map_size;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   for (ulint index = 0; index < chunk_map_size; index++) {
@@ -443,8 +449,8 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_ad(false);
-      return;
+      ut_d(ut_error);
+      ut_o(return);
     }
     len_left -= 4;
 
@@ -452,8 +458,8 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_ad(false);
-      return;
+      ut_d(ut_error);
+      ut_o(return);
     }
     len_left -= 4;
 
@@ -465,7 +471,7 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
   ut_ad(len_left == 0);
 }
 
-void Clone_Desc_Locator::init(ib_uint64_t id, ib_uint64_t snap_id,
+void Clone_Desc_Locator::init(uint64_t id, uint64_t snap_id,
                               Snapshot_State state, uint version, uint index) {
   m_header.m_version = version;
 
@@ -538,14 +544,14 @@ bool clone_validate_locator(const byte *desc_loc, uint desc_len) {
   Clone_Desc_Header header;
 
   if (!header.deserialize(desc_loc, desc_len)) {
-    ut_ad(false);
-    return (false);
+    ut_d(ut_error);
+    ut_o(return (false));
   }
   if (desc_len < CLONE_DESC_LOC_BASE_LEN ||
       header.m_length < CLONE_DESC_LOC_BASE_LEN || header.m_length > desc_len ||
       header.m_type != CLONE_DESC_LOCATOR) {
-    ut_ad(false);
-    return (false);
+    ut_d(ut_error);
+    ut_o(return (false));
   }
   return (true);
 }
@@ -558,8 +564,8 @@ void Clone_Desc_Locator::deserialize(const byte *desc_loc, uint desc_len,
 
   if (m_header.m_length < CLONE_DESC_LOC_BASE_LEN ||
       m_header.m_length > desc_len) {
-    ut_ad(false);
-    return;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   m_clone_id = mach_read_from_8(desc_loc + CLONE_LOC_CID_OFFSET);
@@ -582,14 +588,53 @@ void Clone_Desc_Locator::deserialize(const byte *desc_loc, uint desc_len,
   }
 }
 
+/** Check a specific bit in flag.
+@param[in]      flag    bit flag
+@param[in]      bit     check bit
+@return true, iff bit is set in flag. */
+inline bool DESC_CHECK_FLAG(ulint flag, ulint bit) {
+  return ((flag & (1ULL << (bit - 1))) > 0);
+}
+
+/** Set a specific bit in flag.
+@param[in]      flag    bit flag
+@param[in]      bit     set bit */
+inline void DESC_SET_FLAG(ulint &flag, ulint bit) {
+  flag |= static_cast<ulint>(1ULL << (bit - 1));
+}
+
 /** File Metadata: Snapshot state in 4 bytes */
 static const uint CLONE_FILE_STATE_OFFSET = CLONE_DESC_HEADER_LEN;
 
 /** File Metadata: File size in 8 bytes */
 static const uint CLONE_FILE_SIZE_OFFSET = CLONE_FILE_STATE_OFFSET + 4;
 
+/** File Metadata: Sparse file allocation size on disk in 8 bytes */
+static const uint CLONE_FILE_ALLOC_SIZE_OFFSET = CLONE_FILE_SIZE_OFFSET + 8;
+
+/** File Metadata: FSP flags in 4 bytes */
+static const uint CLONE_FILE_FSP_OFFSET = CLONE_FILE_ALLOC_SIZE_OFFSET + 8;
+
+/** File Metadata: File system block size for compressed tables in 4 bytes. */
+static const uint CLONE_FILE_FSBLK_OFFSET = CLONE_FILE_FSP_OFFSET + 4;
+
+/** File Metadata: File space flags in next 2 bytes [Maximum 16 flags] */
+static const uint CLONE_FILE_FLAGS_OFFSET = CLONE_FILE_FSBLK_OFFSET + 4;
+/** Clone File Flag: Compression type ZLIB*/
+static const uint CLONE_DESC_FILE_FLAG_ZLIB = 1;
+/** Clone File Flag: Compression type LZ4*/
+static const uint CLONE_DESC_FILE_FLAG_LZ4 = 2;
+/** Clone File Flag: Encryption type AES */
+static const uint CLONE_DESC_FILE_FLAG_AES = 3;
+/** Clone File Flag: File is renamed. */
+static const uint CLONE_DESC_FILE_FLAG_RENAMED = 4;
+/** Clone File Flag: File is deleted. */
+static const uint CLONE_DESC_FILE_FLAG_DELETED = 5;
+/** Clone File Flag: File metadata has encryption key. */
+static const uint CLONE_DESC_FILE_HAS_KEY = 6;
+
 /** File Metadata: Tablespace ID in 4 bytes */
-static const uint CLONE_FILE_SPACE_ID_OFFSET = CLONE_FILE_SIZE_OFFSET + 8;
+static const uint CLONE_FILE_SPACE_ID_OFFSET = CLONE_FILE_FLAGS_OFFSET + 2;
 
 /** File Metadata: File index in 4 bytes */
 static const uint CLONE_FILE_IDX_OFFSET = CLONE_FILE_SPACE_ID_OFFSET + 4;
@@ -615,6 +660,11 @@ void Clone_Desc_File_MetaData::init_header(uint version) {
   m_header.m_length = CLONE_FILE_BASE_LEN;
   m_header.m_length += static_cast<uint>(m_file_meta.m_file_name_len);
 
+  if (m_file_meta.m_transfer_encryption_key) {
+    m_header.m_length += sizeof(m_file_meta.m_encryption_metadata.m_key);
+    m_header.m_length += sizeof(m_file_meta.m_encryption_metadata.m_iv);
+  }
+
   m_header.m_type = CLONE_DESC_FILE_METADATA;
 }
 
@@ -636,6 +686,38 @@ void Clone_Desc_File_MetaData::serialize(byte *&desc_file, uint &len,
   mach_write_to_4(desc_file + CLONE_FILE_STATE_OFFSET, m_state);
 
   mach_write_to_8(desc_file + CLONE_FILE_SIZE_OFFSET, m_file_meta.m_file_size);
+  mach_write_to_8(desc_file + CLONE_FILE_ALLOC_SIZE_OFFSET,
+                  m_file_meta.m_alloc_size);
+  mach_write_to_4(desc_file + CLONE_FILE_FSP_OFFSET, m_file_meta.m_fsp_flags);
+
+  mach_write_to_4(desc_file + CLONE_FILE_FSBLK_OFFSET,
+                  m_file_meta.m_fsblk_size);
+  /* Set file compression type for sparse file. */
+  ulint file_flags = 0;
+  if (m_file_meta.m_compress_type == Compression::ZLIB) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_ZLIB);
+  } else if (m_file_meta.m_compress_type == Compression::LZ4) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_LZ4);
+  }
+  /* Set file encryption type */
+  if (m_file_meta.m_encryption_metadata.m_type == Encryption::AES) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_AES);
+  }
+  /* Set file renamed attribute */
+  if (m_file_meta.m_renamed) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_RENAMED);
+  }
+  /* Set file deleted attribute */
+  if (m_file_meta.m_deleted) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_DELETED);
+  }
+
+  if (m_file_meta.m_transfer_encryption_key) {
+    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_HAS_KEY);
+  }
+
+  mach_write_to_2(desc_file + CLONE_FILE_FLAGS_OFFSET, file_flags);
+
   mach_write_to_4(desc_file + CLONE_FILE_SPACE_ID_OFFSET,
                   m_file_meta.m_space_id);
   mach_write_to_4(desc_file + CLONE_FILE_IDX_OFFSET, m_file_meta.m_file_index);
@@ -654,6 +736,20 @@ void Clone_Desc_File_MetaData::serialize(byte *&desc_file, uint &len,
            static_cast<const void *>(m_file_meta.m_file_name),
            m_file_meta.m_file_name_len);
   }
+
+  auto dest_key =
+      desc_file + CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len;
+
+  /* Append Encryption key information if requested. */
+  if (m_file_meta.m_transfer_encryption_key) {
+    memcpy(dest_key, m_file_meta.m_encryption_metadata.m_key,
+           sizeof(m_file_meta.m_encryption_metadata.m_key));
+
+    dest_key += sizeof(m_file_meta.m_encryption_metadata.m_key);
+
+    memcpy(dest_key, m_file_meta.m_encryption_metadata.m_iv,
+           sizeof(m_file_meta.m_encryption_metadata.m_iv));
+  }
 }
 
 bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
@@ -666,13 +762,46 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
   }
   desc_len -= CLONE_FILE_BASE_LEN;
 
-  uint int_type;
-  int_type = mach_read_from_4(desc_file + CLONE_FILE_STATE_OFFSET);
+  auto int_type = mach_read_from_4(desc_file + CLONE_FILE_STATE_OFFSET);
 
   m_state = static_cast<Snapshot_State>(int_type);
 
   m_file_meta.m_file_size =
       mach_read_from_8(desc_file + CLONE_FILE_SIZE_OFFSET);
+  m_file_meta.m_alloc_size =
+      mach_read_from_8(desc_file + CLONE_FILE_ALLOC_SIZE_OFFSET);
+
+  m_file_meta.m_fsp_flags = mach_read_from_4(desc_file + CLONE_FILE_FSP_OFFSET);
+  m_file_meta.m_fsblk_size =
+      mach_read_from_4(desc_file + CLONE_FILE_FSBLK_OFFSET);
+
+  m_file_meta.m_punch_hole = false;
+
+  m_file_meta.m_compress_type = Compression::NONE;
+  auto file_flags =
+      static_cast<ulint>(mach_read_from_2(desc_file + CLONE_FILE_FLAGS_OFFSET));
+
+  /* Get file compression type for sparse file. */
+  if (DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_ZLIB)) {
+    m_file_meta.m_compress_type = Compression::ZLIB;
+  } else if (DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_LZ4)) {
+    m_file_meta.m_compress_type = Compression::LZ4;
+  }
+
+  /* Get file encryption information */
+  m_file_meta.m_encryption_metadata.m_type = Encryption::NONE;
+  if (DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_AES)) {
+    m_file_meta.m_encryption_metadata.m_type = Encryption::AES;
+  }
+
+  /* Get file renamed attribute */
+  m_file_meta.m_renamed =
+      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_RENAMED);
+
+  /* Get file renamed attribute */
+  m_file_meta.m_deleted =
+      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_DELETED);
+
   m_file_meta.m_space_id =
       mach_read_from_4(desc_file + CLONE_FILE_SPACE_ID_OFFSET);
   m_file_meta.m_file_index =
@@ -686,14 +815,15 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
   m_file_meta.m_file_name_len =
       mach_read_from_4(desc_file + CLONE_FILE_FNAMEL_OFFSET);
 
-  ut_ad(m_header.m_length ==
-        CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len);
+  m_file_meta.m_file_name_alloc_len = 0;
 
   /* Check if we have enough length. */
   if (desc_len < m_file_meta.m_file_name_len) {
     return (false);
   }
 
+  desc_len -= m_file_meta.m_file_name_len;
+
   if (m_file_meta.m_file_name_len == 0) {
     m_file_meta.m_file_name = nullptr;
   } else {
@@ -706,6 +836,40 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
       return (false);
     }
   }
+
+  /* Check if encryption key is transferred. */
+  m_file_meta.m_transfer_encryption_key =
+      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_HAS_KEY);
+
+  /* Extract Encryption key information if transferred. */
+  if (m_file_meta.m_transfer_encryption_key) {
+    auto src_key =
+        desc_file + CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len;
+
+    /* Check if we have enough length. */
+    if (desc_len < (sizeof(m_file_meta.m_encryption_metadata.m_key) +
+                    sizeof(m_file_meta.m_encryption_metadata.m_iv))) {
+      return false; /* purecov: inspected */
+    }
+
+    memcpy(m_file_meta.m_encryption_metadata.m_key, src_key,
+           sizeof(m_file_meta.m_encryption_metadata.m_key));
+
+    src_key += sizeof(m_file_meta.m_encryption_metadata.m_key);
+
+    memcpy(m_file_meta.m_encryption_metadata.m_iv, src_key,
+           sizeof(m_file_meta.m_encryption_metadata.m_iv));
+
+    ut_ad(m_header.m_length ==
+          CLONE_FILE_FNAME_OFFSET +
+              sizeof(m_file_meta.m_encryption_metadata.m_key) +
+              sizeof(m_file_meta.m_encryption_metadata.m_iv) +
+              m_file_meta.m_file_name_len);
+  } else {
+    ut_ad(m_header.m_length ==
+          CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len);
+  }
+
   return (true);
 }
 
@@ -721,19 +885,18 @@ static const uint CLONE_DESC_STATE_NUM_CHUNKS = CLONE_DESC_TASK_OFFSET + 4;
 /** Clone State: Number of files in 4 bytes */
 static const uint CLONE_DESC_STATE_NUM_FILES = CLONE_DESC_STATE_NUM_CHUNKS + 4;
 
-/** Clone State: Estimated nuber of bytes in 8 bytes */
+/** Clone State: Estimated number of bytes in 8 bytes */
 static const uint CLONE_DESC_STATE_EST_BYTES = CLONE_DESC_STATE_NUM_FILES + 4;
 
+/** Clone State: Estimated number of bytes in 8 bytes */
+static const uint CLONE_DESC_STATE_EST_DISK = CLONE_DESC_STATE_EST_BYTES + 8;
+
 /** Clone State: flags in 2 byte [max 16 flags] */
-static const uint CLONE_DESC_STATE_FLAGS = CLONE_DESC_STATE_EST_BYTES + 8;
+static const uint CLONE_DESC_STATE_FLAGS = CLONE_DESC_STATE_EST_DISK + 8;
 
 /** Clone State: Total length */
 static const uint CLONE_DESC_STATE_LEN = CLONE_DESC_STATE_FLAGS + 2;
 
-#define DESC_CHECK_FLAG(flag, bit) (((flag) & (1UL << (bit - 1))) > 0)
-
-#define DESC_SET_FLAG(flag, bit) ((flag) |= (1UL << (bit - 1)))
-
 /** Clone State Flag: Start processing state */
 static const uint CLONE_DESC_STATE_FLAG_START = 1;
 
@@ -767,6 +930,7 @@ void Clone_Desc_State::serialize(byte *&desc_state, uint &len,
   mach_write_to_4(desc_state + CLONE_DESC_STATE_NUM_CHUNKS, m_num_chunks);
   mach_write_to_4(desc_state + CLONE_DESC_STATE_NUM_FILES, m_num_files);
   mach_write_to_8(desc_state + CLONE_DESC_STATE_EST_BYTES, m_estimate);
+  mach_write_to_8(desc_state + CLONE_DESC_STATE_EST_DISK, m_estimate_disk);
 
   ulint state_flags = 0;
 
@@ -799,6 +963,7 @@ bool Clone_Desc_State::deserialize(const byte *desc_state, uint desc_len) {
   m_num_chunks = mach_read_from_4(desc_state + CLONE_DESC_STATE_NUM_CHUNKS);
   m_num_files = mach_read_from_4(desc_state + CLONE_DESC_STATE_NUM_FILES);
   m_estimate = mach_read_from_8(desc_state + CLONE_DESC_STATE_EST_BYTES);
+  m_estimate_disk = mach_read_from_8(desc_state + CLONE_DESC_STATE_EST_DISK);
 
   auto state_flags =
       static_cast<ulint>(mach_read_from_2(desc_state + CLONE_DESC_STATE_FLAGS));
@@ -903,3 +1068,28 @@ bool Clone_Desc_Data::deserialize(const byte *desc_data, uint desc_len) {
 
   return (true);
 }
+
+void Clone_File_Meta::init() {
+  m_file_index = 0;
+  m_file_name = nullptr;
+  m_file_name_len = 0;
+  m_file_name_alloc_len = 0;
+
+  m_file_size = 0;
+  m_alloc_size = 0;
+
+  m_space_id = dict_sys_t::s_invalid_space_id;
+  m_fsp_flags = UINT32_UNDEFINED;
+
+  m_compress_type = Compression::NONE;
+  m_encryption_metadata = {};
+  m_punch_hole = false;
+  m_fsblk_size = 0;
+
+  m_transfer_encryption_key = false;
+
+  m_begin_chunk = 0;
+  m_end_chunk = 0;
+
+  reset_ddl();
+}
