@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -49,20 +50,18 @@ void Clone_persist_gtid::add(const Gtid_desc &gtid_desc) {
   if (!is_active() || gtid_table_persistor == nullptr) {
     return;
   }
-  ut_ad(trx_sys_mutex_own());
+  ut_ad(trx_sys_serialisation_mutex_own());
 
-  /* If too many GTIDs are accumulated, wait for all to get flushed. */
-  while (check_max_gtid_threshold()) {
-    trx_sys_mutex_exit();
+  /* If too many GTIDs are accumulated, wait for all to get flushed. Ignore
+  timeout and loop to avoid possible hang. The insert should already be
+  slowed down by the wait here. */
+  if (check_max_gtid_threshold() && is_thread_active()) {
+    trx_sys_serialisation_mutex_exit();
     wait_flush(false, false, nullptr);
-    trx_sys_mutex_enter();
-    /* Starvation is possible theoretically here, if the active list gets
-    filled to threshold before a transaction could get hold of the mutex
-    after being woken up. Practically not feasible as the number of waiting
-    transactions at any point in time is far less than the threshold. */
+    trx_sys_serialisation_mutex_enter();
   }
 
-  ut_ad(trx_sys_mutex_own());
+  ut_ad(trx_sys_serialisation_mutex_own());
   /* Get active GTID list */
   auto &current_gtids = get_active_list();
 
@@ -78,9 +77,9 @@ void Clone_persist_gtid::add(const Gtid_desc &gtid_desc) {
 
   DBUG_EXECUTE_IF("dont_compress_gtid_table", {
     /* For predictable outcome of mtr test we flush the GTID immediately. */
-    trx_sys_mutex_exit();
+    trx_sys_serialisation_mutex_exit();
     wait_flush(false, false, nullptr);
-    trx_sys_mutex_enter();
+    trx_sys_serialisation_mutex_enter();
   });
 }
 
@@ -215,8 +214,8 @@ bool Clone_persist_gtid::check_gtid_prepare(THD *thd, trx_t *trx,
   }
   /* Skip GTID if External XA transaction is not in IDLE state. */
   if (!xid_state->has_state(XID_STATE::XA_IDLE)) {
-    ut_ad(false);
-    return (false);
+    ut_d(ut_error);
+    ut_o(return (false));
   }
   /* Skip if SE is not set to persist GTID.*/
   if (!thd->se_persists_gtid()) {
@@ -244,8 +243,11 @@ bool Clone_persist_gtid::check_gtid_rollback(THD *thd, trx_t *trx,
   if (!found_gtid) {
     return (false);
   }
+
+  ut_ad(trx_can_be_handled_by_current_thread_or_is_hp_victim(trx));
+
   /* Skip GTID if XA transaction not in prepared state. */
-  if (trx->state != TRX_STATE_PREPARED) {
+  if (trx->state.load(std::memory_order_relaxed) != TRX_STATE_PREPARED) {
     return (false);
   }
 
@@ -276,8 +278,13 @@ bool Clone_persist_gtid::check_gtid_rollback(THD *thd, trx_t *trx,
 
 bool Clone_persist_gtid::has_gtid(trx_t *trx, THD *&thd, bool &passed_check) {
   passed_check = false;
+
+  /* Note, that the assertion does nothing useful if thd == nullptr. */
+  ut_ad(trx_can_be_handled_by_current_thread_or_is_hp_victim(trx));
+
   /* Transaction is not associated with mysql foreground session. */
-  if (trx->state == TRX_STATE_PREPARED && thd == nullptr) {
+  if (trx->state.load(std::memory_order_relaxed) == TRX_STATE_PREPARED &&
+      thd == nullptr) {
     /* For XA transaction, the current transaction THD could be NULL. Also
     check the default THD of current thread. */
     thd = thd_get_current_thd();
@@ -335,8 +342,8 @@ void Clone_persist_gtid::get_gtid_info(trx_t *trx, Gtid_desc &gtid_desc) {
   auto thd = trx->mysql_thd;
 
   if (!has_gtid(trx, thd, thd_check)) {
-    ut_ad(false);
-    return;
+    ut_d(ut_error);
+    ut_o(return);
   }
 
   gtid_desc.m_version = GTID_VERSION;
@@ -345,6 +352,7 @@ void Clone_persist_gtid::get_gtid_info(trx_t *trx, Gtid_desc &gtid_desc) {
 
   ut_ad(trx_gtid.sidno > 0);
   ut_ad(trx_gtid.gno > 0);
+  ut_ad(trx_gtid.gno < GNO_END);
 
   /* Build GTID string. */
   gtid_desc.m_info.fill(0);
@@ -474,6 +482,8 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
   Sid_map sid_map(nullptr);
   Gtid_set table_gtid_set(&sid_map, nullptr);
 
+  DBUG_EXECUTE_IF("gtid_persist_flush_disable", return;);
+
   /* During recovery, fetch existing GTIDs from gtid_executed table. */
   bool is_recovery = !m_thread_active.load();
   if (is_recovery && !opt_initialize) {
@@ -482,7 +492,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
 
   bool explicit_request = m_explicit_request.load();
 
-  trx_sys_mutex_enter();
+  trx_sys_serialisation_mutex_enter();
   /* Get oldest transaction number that is yet to be committed. Any transaction
   with lower transaction number is committed and is added to GTID list. */
   auto oldest_trx_no = trx_sys_oldest_trx_no();
@@ -493,7 +503,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
     /* Switch active list and get the previous list to write to disk table. */
     auto flush_list_number = switch_active_list();
     /* Exit trx mutex during write to table. */
-    trx_sys_mutex_exit();
+    trx_sys_serialisation_mutex_exit();
     err = write_to_table(flush_list_number, table_gtid_set, sid_map);
     m_flush_in_progress.store(false);
     /* Compress always after recovery, if GTIDs are added. */
@@ -502,7 +512,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
       ib::info(ER_IB_CLONE_GTID_PERSIST) << "GTID compression after recovery. ";
     }
   } else {
-    trx_sys_mutex_exit();
+    trx_sys_serialisation_mutex_exit();
   }
 
   if (is_recovery) {
@@ -520,7 +530,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
     my_free(gtid_buffer);
   }
 
-  /* Update trx number upto which GTID is written to table. */
+  /* Update trx number up to which GTID is written to table. */
   update_gtid_trx_no(oldest_trx_no);
 
   /* Request Compression once the counter reaches threshold. */
@@ -546,7 +556,7 @@ void Clone_persist_gtid::flush_gtids(THD *thd) {
 }
 
 bool Clone_persist_gtid::check_max_gtid_threshold() {
-  ut_ad(trx_sys_mutex_own());
+  ut_ad(trx_sys_serialisation_mutex_own());
   /* Allow only one GTID to flush at a time. */
   DBUG_EXECUTE_IF("dont_compress_gtid_table",
                   { return m_num_gtid_mem.load() > 0; });
@@ -554,7 +564,7 @@ bool Clone_persist_gtid::check_max_gtid_threshold() {
 }
 
 void Clone_persist_gtid::periodic_write() {
-  auto thd = create_thd(false, true, true, PSI_NOT_INSTRUMENTED);
+  auto thd = create_internal_thd();
 
   /* Allow GTID to be persisted on read only server. */
   thd->set_skip_readonly_check();
@@ -596,7 +606,7 @@ void Clone_persist_gtid::periodic_write() {
     }
 
     if (!flush_immediate()) {
-      os_event_wait_time(m_event, s_time_threshold_ms * 1000);
+      os_event_wait_time(m_event, s_time_threshold);
     }
     os_event_reset(m_event);
     /* Write accumulated GTIDs to disk table */
@@ -613,7 +623,7 @@ void Clone_persist_gtid::periodic_write() {
   }
 
   m_active.store(false);
-  destroy_thd(thd);
+  destroy_internal_thd(thd);
   m_thread_active.store(false);
 }
 
@@ -629,7 +639,7 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
         result = false;
         return (0);
       }
-      /* If it is flushed upto the point requested. */
+      /* If it is flushed up to the point requested. */
       if (check_flushed(flush_number)) {
         /* Check if compression is done if requested. */
         if (!compress || !m_explicit_request.load()) {
@@ -680,7 +690,7 @@ bool Clone_persist_gtid::wait_thread(bool start, bool wait_flush,
 }
 
 /** Persist GTID to on disk table from time to time.
-@param[in,out]	persist_gtid	GTID persister */
+@param[in,out]  persist_gtid    GTID persister */
 static void clone_gtid_thread(Clone_persist_gtid *persist_gtid) {
   persist_gtid->periodic_write();
 }
@@ -692,13 +702,13 @@ bool Clone_persist_gtid::start() {
   }
 
   srv_threads.m_gtid_persister =
-      os_thread_create(clone_gtid_thread_key, clone_gtid_thread, this);
+      os_thread_create(clone_gtid_thread_key, 0, clone_gtid_thread, this);
   srv_threads.m_gtid_persister.start();
 
   if (!wait_thread(true, false, 0, false, false, nullptr)) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to start timed out";
-    ut_ad(false);
-    return (false);
+    ut_d(ut_error);
+    ut_o(return (false));
   }
   m_active.store(true);
   return (true);
@@ -709,7 +719,7 @@ void Clone_persist_gtid::stop() {
   if (m_thread_active.load() &&
       !wait_thread(false, false, 0, false, false, nullptr)) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to stop timed out";
-    ut_ad(false);
+    ut_d(ut_error);
   }
 }
 
@@ -736,6 +746,6 @@ void Clone_persist_gtid::wait_flush(bool compress_gtid, bool early_timeout,
   /* No error for early timeout. */
   if (!success && !early_timeout) {
     ib::error(ER_IB_CLONE_TIMEOUT) << "Wait for GTID thread to flush timed out";
-    ut_ad(false);
+    ut_d(ut_error);
   }
 }
