@@ -1,18 +1,19 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2024, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -31,6 +32,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
  Created 1/8/1996 Heikki Tuuri
  ***********************************************************************/
 
+#include <atomic>
 #ifndef UNIV_HOTBACKUP
 #include <mysql_com.h>
 
@@ -53,13 +55,13 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "sync0sync.h"
 
-/** An interger randomly initialized at startup used to make a temporary
+/** An integer randomly initialized at startup used to make a temporary
 table name as unuique as possible. */
-static ib_uint32_t dict_temp_file_num;
+static std::atomic<uint32_t> dict_temp_file_num;
 
 /** Display an identifier.
-@param[in,out]	s	output stream
-@param[in]	id_name	SQL identifier (other than table name)
+@param[in,out]  s       output stream
+@param[in]      id_name SQL identifier (other than table name)
 @return the output stream */
 std::ostream &operator<<(std::ostream &s, const id_name_t &id_name) {
   const char q = '`';
@@ -76,36 +78,22 @@ std::ostream &operator<<(std::ostream &s, const id_name_t &id_name) {
 }
 
 /** Display a table name.
-@param[in,out]	s		output stream
-@param[in]	table_name	table name
+@param[in,out]  s               output stream
+@param[in]      table_name      table name
 @return the output stream */
 std::ostream &operator<<(std::ostream &s, const table_name_t &table_name) {
 #ifndef UNIV_HOTBACKUP
-  return (s << ut_get_name(NULL, table_name.m_name));
+  return (s << ut_get_name(nullptr, table_name.m_name));
 #else  /* !UNIV_HOTBACKUP */
   return (s << table_name.m_name);
 #endif /* !UNIV_HOTBACKUP */
 }
 
 #ifndef UNIV_HOTBACKUP
-/** Adds a virtual column definition to a table.
-@param[in,out]	table		table
-@param[in,out]	heap		temporary memory heap, or NULL. It is
-                                used to store name when we have not finished
-                                adding all columns. When all columns are
-                                added, the whole name will copy to memory from
-                                table->heap
-@param[in]	name		column name
-@param[in]	mtype		main datatype
-@param[in]	prtype		precise type
-@param[in]	len		length
-@param[in]	pos		position in a table
-@param[in]	num_base	number of base columns
-@return the virtual column definition */
 dict_v_col_t *dict_mem_table_add_v_col(dict_table_t *table, mem_heap_t *heap,
                                        const char *name, ulint mtype,
                                        ulint prtype, ulint len, ulint pos,
-                                       ulint num_base) {
+                                       ulint num_base, bool is_visible) {
   dict_v_col_t *v_col;
   ulint i;
 
@@ -119,7 +107,7 @@ dict_v_col_t *dict_mem_table_add_v_col(dict_table_t *table, mem_heap_t *heap,
 
   table->n_t_def++;
 
-  if (name != NULL) {
+  if (name != nullptr) {
     if (table->n_v_def == table->n_v_cols) {
       heap = table->heap;
     }
@@ -136,36 +124,38 @@ dict_v_col_t *dict_mem_table_add_v_col(dict_table_t *table, mem_heap_t *heap,
 
   v_col = dict_table_get_nth_v_col(table, i);
 
-  dict_mem_fill_column_struct(&v_col->m_col, pos, mtype, prtype, len);
+  dict_mem_fill_column_struct(&v_col->m_col, pos, mtype, prtype, len, true,
+                              UINT32_UNDEFINED, 0, 0);
   v_col->v_pos = i;
 
   if (num_base != 0) {
     v_col->base_col = static_cast<dict_col_t **>(
         mem_heap_zalloc(table->heap, num_base * sizeof(*v_col->base_col)));
   } else {
-    v_col->base_col = NULL;
+    v_col->base_col = nullptr;
   }
 
   v_col->num_base = num_base;
 
   /* Initialize the index list for virtual columns */
-  v_col->v_indexes = UT_NEW_NOKEY(dict_v_idx_list());
+  v_col->v_indexes = ut::new_withkey<dict_v_idx_list>(UT_NEW_THIS_FILE_PSI_KEY);
 
+  v_col->m_col.is_visible = is_visible;
   return (v_col);
 }
 
 /** Adds a stored column definition to a table.
-@param[in]	table		table
-@param[in]	num_base	number of base columns. */
+@param[in,out]  table           table
+@param[in]      num_base        number of base columns. */
 void dict_mem_table_add_s_col(dict_table_t *table, ulint num_base) {
   ulint i = table->n_def - 1;
   dict_col_t *col = table->get_col(i);
   dict_s_col_t s_col;
 
-  ut_ad(col != NULL);
+  ut_ad(col != nullptr);
 
-  if (table->s_cols == NULL) {
-    table->s_cols = UT_NEW_NOKEY(dict_s_col_list());
+  if (table->s_cols == nullptr) {
+    table->s_cols = ut::new_withkey<dict_s_col_list>(UT_NEW_THIS_FILE_PSI_KEY);
   }
 
   s_col.m_col = col;
@@ -175,7 +165,7 @@ void dict_mem_table_add_s_col(dict_table_t *table, ulint num_base) {
     s_col.base_col = static_cast<dict_col_t **>(
         mem_heap_zalloc(table->heap, num_base * sizeof(dict_col_t *)));
   } else {
-    s_col.base_col = NULL;
+    s_col.base_col = nullptr;
   }
 
   s_col.num_base = num_base;
@@ -240,7 +230,7 @@ static void dict_mem_table_col_rename_low(
             full_len - (prefix_len + from_len));
 
     /* Replace the field names in every index. */
-    for (dict_index_t *index = table->first_index(); index != NULL;
+    for (dict_index_t *index = table->first_index(); index != nullptr;
          index = index->next()) {
       ulint n_fields = dict_index_get_n_fields(index);
 
@@ -319,14 +309,15 @@ static void dict_mem_table_col_rename_low(
   }
 }
 
-/** Renames a column of a table in the data dictionary cache. */
-void dict_mem_table_col_rename(dict_table_t *table, /*!< in/out: table */
-                               ulint nth_col,       /*!< in: column index */
-                               const char *from,    /*!< in: old column name */
-                               const char *to,      /*!< in: new column name */
-                               bool is_virtual)
-/*!< in: if this is a virtual column */
-{
+/** Renames a column of a table in the data dictionary cache.
+@param[in,out] table Table
+@param[in] nth_col Column index
+@param[in] from Old column name
+@param[in] to New column name
+@param[in] is_virtual If this is a virtual column */
+void dict_mem_table_col_rename(dict_table_t *table, ulint nth_col,
+                               const char *from, const char *to,
+                               bool is_virtual) {
   const char *s = is_virtual ? table->v_col_names : table->col_names;
 
   ut_ad((!is_virtual && nth_col < table->n_def) ||
@@ -351,20 +342,20 @@ void dict_mem_table_col_rename(dict_table_t *table, /*!< in/out: table */
 dict_foreign_t *dict_mem_foreign_create(void) {
   dict_foreign_t *foreign;
   mem_heap_t *heap;
-  DBUG_ENTER("dict_mem_foreign_create");
+  DBUG_TRACE;
 
-  heap = mem_heap_create(100);
+  heap = mem_heap_create(100, UT_LOCATION_HERE);
 
   foreign = static_cast<dict_foreign_t *>(
       mem_heap_zalloc(heap, sizeof(dict_foreign_t)));
 
   foreign->heap = heap;
 
-  foreign->v_cols = NULL;
+  foreign->v_cols = nullptr;
 
   DBUG_PRINT("dict_mem_foreign_create", ("heap: %p", heap));
 
-  DBUG_RETURN(foreign);
+  return foreign;
 }
 
 /** Sets the foreign_table_name_lookup pointer based on the value of
@@ -373,7 +364,7 @@ dict_foreign_t *dict_mem_foreign_create(void) {
  allocated from foreign->heap and set to lower case. */
 void dict_mem_foreign_table_name_lookup_set(
     dict_foreign_t *foreign, /*!< in/out: foreign struct */
-    ibool do_alloc)          /*!< in: is an alloc needed */
+    bool do_alloc)           /*!< in: is an alloc needed */
 {
   if (innobase_get_lower_case_table_names() == 2) {
     if (do_alloc) {
@@ -397,7 +388,7 @@ void dict_mem_foreign_table_name_lookup_set(
  allocated from foreign->heap and set to lower case. */
 void dict_mem_referenced_table_name_lookup_set(
     dict_foreign_t *foreign, /*!< in/out: foreign struct */
-    ibool do_alloc)          /*!< in: is an alloc needed */
+    bool do_alloc)           /*!< in: is an alloc needed */
 {
   if (innobase_get_lower_case_table_names() == 2) {
     if (do_alloc) {
@@ -418,8 +409,8 @@ void dict_mem_referenced_table_name_lookup_set(
 
 /** Fill the virtual column set with virtual column information
 present in the given virtual index.
-@param[in]	index	virtual index
-@param[out]	v_cols	virtual column set. */
+@param[in]      index   virtual index
+@param[out]     v_cols  virtual column set. */
 static void dict_mem_fill_vcol_has_index(const dict_index_t *index,
                                          dict_vcol_set **v_cols) {
   for (ulint i = 0; i < index->table->n_v_cols; i++) {
@@ -433,8 +424,8 @@ static void dict_mem_fill_vcol_has_index(const dict_index_t *index,
       dict_v_idx_t v_idx = *it;
 
       if (v_idx.index == index) {
-        if (*v_cols == NULL) {
-          *v_cols = UT_NEW_NOKEY(dict_vcol_set());
+        if (*v_cols == nullptr) {
+          *v_cols = ut::new_withkey<dict_vcol_set>(UT_NEW_THIS_FILE_PSI_KEY);
         }
 
         (*v_cols)->insert(v_col);
@@ -445,15 +436,15 @@ static void dict_mem_fill_vcol_has_index(const dict_index_t *index,
 
 /** Fill the virtual column set with virtual column of the index
 if the index contains the given column name.
-@param[in]	col_name	column name
-@param[in]	table		innodb table object
-@param[out]	v_cols		set of virtual column information. */
+@param[in]      col_name        column name
+@param[in]      table           innodb table object
+@param[out]     v_cols          set of virtual column information. */
 static void dict_mem_fill_vcol_from_v_indexes(const char *col_name,
                                               const dict_table_t *table,
                                               dict_vcol_set **v_cols) {
   /* virtual column can't be Primary Key, so start with secondary index */
-  for (const dict_index_t *index = table->first_index()->next(); index != NULL;
-       index = index->next()) {
+  for (const dict_index_t *index = table->first_index()->next();
+       index != nullptr; index = index->next()) {
     /* Skip if the index have newly added
     virtual column because field name is NULL.
     Later virtual column set will be
@@ -474,9 +465,9 @@ static void dict_mem_fill_vcol_from_v_indexes(const char *col_name,
 
 /** Fill the virtual column set with virtual columns which have base columns
 as the given col_name
-@param[in]	col_name	column name
-@param[in]	table		table object
-@param[out]	v_cols		set of virtual columns. */
+@param[in]      col_name        column name
+@param[in]      table           table object
+@param[out]     v_cols          set of virtual columns. */
 static void dict_mem_fill_vcol_set_for_base_col(const char *col_name,
                                                 const dict_table_t *table,
                                                 dict_vcol_set **v_cols) {
@@ -489,8 +480,8 @@ static void dict_mem_fill_vcol_set_for_base_col(const char *col_name,
 
     for (ulint j = 0; j < v_col->num_base; j++) {
       if (strcmp(col_name, table->get_col_name(v_col->base_col[j]->ind)) == 0) {
-        if (*v_cols == NULL) {
-          *v_cols = UT_NEW_NOKEY(dict_vcol_set());
+        if (*v_cols == nullptr) {
+          *v_cols = ut::new_withkey<dict_vcol_set>(UT_NEW_THIS_FILE_PSI_KEY);
         }
 
         (*v_cols)->insert(v_col);
@@ -526,7 +517,7 @@ void dict_mem_foreign_fill_vcol_set(dict_foreign_t *foreign) {
 }
 
 /** Fill virtual columns set in each fk constraint present in the table.
-@param[in,out]	table	innodb table object. */
+@param[in,out]  table   innodb table object. */
 void dict_mem_table_fill_foreign_vcol_set(dict_table_t *table) {
   dict_foreign_set fk_set = table->foreign_set;
   dict_foreign_t *foreign;
@@ -539,7 +530,7 @@ void dict_mem_table_fill_foreign_vcol_set(dict_table_t *table) {
 }
 
 /** Free the vcol_set from all foreign key constraint on the table.
-@param[in,out]	table	innodb table object. */
+@param[in,out]  table   innodb table object. */
 void dict_mem_table_free_foreign_vcol_set(dict_table_t *table) {
   dict_foreign_set fk_set = table->foreign_set;
   dict_foreign_t *foreign;
@@ -547,17 +538,17 @@ void dict_mem_table_free_foreign_vcol_set(dict_table_t *table) {
   for (auto it = fk_set.begin(); it != fk_set.end(); ++it) {
     foreign = *it;
 
-    if (foreign->v_cols != NULL) {
-      UT_DELETE(foreign->v_cols);
-      foreign->v_cols = NULL;
+    if (foreign->v_cols != nullptr) {
+      ut::delete_(foreign->v_cols);
+      foreign->v_cols = nullptr;
     }
   }
 }
 
 /** Set default value
-@param[in]	value	Default value
-@param[in]	length	Default value length
-@param[in,out]	heap	Heap to allocate memory */
+@param[in]      value   Default value
+@param[in]      length  Default value length
+@param[in,out]  heap    Heap to allocate memory */
 void dict_col_t::set_default(const byte *value, size_t length,
                              mem_heap_t *heap) {
   ut_ad(instant_default == nullptr);
@@ -582,8 +573,34 @@ void dict_col_t::set_default(const byte *value, size_t length,
   instant_default->len = length;
 }
 
+bool dict_col_default_t::operator==(const dict_col_default_t &other) {
+  /* If the lengths are different, trivially the default values are not
+  the same, return false immediately */
+  if (len != other.len) {
+    return false;
+  }
+  /* If the lengths are null or 0, the values are empty and equal.
+  No need to check both lengths since we only reach this point
+  if len == other.len */
+  if (len == UNIV_SQL_NULL || len == 0) {
+    return true;
+  }
+
+  auto length = len;
+  while (length-- > 0) {
+    if (value[length] != other.value[length]) {
+      return false;
+    }
+  }
+  return true;
+}
+
+bool dict_col_default_t::operator!=(const dict_col_default_t &other) {
+  return !(*this == other);
+}
+
 /** Check whether index can be used by transaction
-@param[in] trx		transaction*/
+@param[in] trx          transaction*/
 bool dict_index_t::is_usable(const trx_t *trx) const {
   /* Indexes that are being created are not usable. */
   if (!is_clustered() && dict_index_is_online_ddl(this)) {
@@ -602,15 +619,99 @@ bool dict_index_t::is_usable(const trx_t *trx) const {
 }
 #endif /* !UNIV_HOTBACKUP */
 
+void dict_index_t::create_nullables(uint32_t current_row_version) {
+  ut_ad(is_clustered());
+  ut_ad(is_valid_row_version(current_row_version));
+
+  memset(nullables, 0, (MAX_ROW_VERSION + 1) * sizeof(nullables[0]));
+
+  auto update_nullable = [&](size_t start_version, bool is_increment) {
+    ut_ad(is_valid_row_version(start_version));
+    for (size_t i = start_version; i <= current_row_version; i++) {
+      ut_ad(is_increment || nullables[i] > 0);
+
+      if (is_increment) {
+        ++nullables[i];
+      } else {
+        --nullables[i];
+      }
+    }
+  };
+
+  for (uint32_t i = 0; i < n_def; i++) {
+    dict_field_t *field = get_field(i);
+
+    /* In case of redo recovery, names are not populated */
+    ut_ad(field->name != nullptr ||
+          strcmp(name, RECOVERY_INDEX_TABLE_NAME) == 0);
+
+    if (field->name != nullptr && (strcmp(field->name, "DB_ROW_ID") == 0 ||
+                                   strcmp(field->name, "DB_TRX_ID") == 0 ||
+                                   strcmp(field->name, "DB_ROLL_PTR") == 0)) {
+      continue;
+    }
+
+    if (field->col->prtype & DATA_NOT_NULL) {
+      continue;
+    }
+
+    /* For each version increment by 1 starting from field->col->v_added */
+    size_t start_from = 0;
+    if (field->col->is_instant_added()) {
+      start_from = field->col->get_version_added();
+    }
+    update_nullable(start_from, true);
+
+    /* For each version decrement by 1 starting from field->col->v_dropped */
+    if (field->col->is_instant_dropped()) {
+      update_nullable(field->col->get_version_dropped(), false);
+    }
+  }
+}
+
+bool dict_index_t::is_tuple_instant_format(
+    const uint16_t n_fields_in_tuple) const {
+  if (!has_instant_cols_or_row_versions()) {
+    return false;
+  }
+
+  ut_ad(n_fields_in_tuple <= n_total_fields);
+
+  /* In versioned rows, always materialize INSTANT cols even in from ROLLBACK */
+  if (has_row_versions()) {
+    return true;
+  }
+
+  /* For instant index, if the tuple comes from UPDATE, its fields could be less
+  than index definition. Because, we restore the table row as it was before
+  UPDATE (i.e. we get rid of INSTANT ADD columns which weren't part of row) */
+  if (n_fields_in_tuple < n_fields) {
+    /* If PK is not specified, DB_ROW_ID will be part of tuple */
+    uint16_t sys_fields_in_tuple = 0;
+    if (innobase_strcasecmp(name, innobase_index_reserve_name) == 0) {
+      sys_fields_in_tuple = table->get_n_sys_cols();
+    } else {
+      sys_fields_in_tuple = table->get_n_sys_cols() - 1;
+    }
+
+    uint16_t fields_in_tuple = n_fields_in_tuple - sys_fields_in_tuple;
+    if (fields_in_tuple == table->get_instant_cols()) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
 /** Gets the column number the nth field in an index.
-@param[in] pos	position of the field
+@param[in] pos  position of the field
 @return column number */
 ulint dict_index_t::get_col_no(ulint pos) const {
   return (dict_col_get_no(get_col(pos)));
 }
 
 /** Returns the position of a system column in an index.
-@param[in] type		DATA_ROW_ID, ...
+@param[in] type         DATA_ROW_ID, ...
 @return position, ULINT_UNDEFINED if not contained */
 ulint dict_index_t::get_sys_col_pos(ulint type) const {
   ut_ad(magic_n == DICT_INDEX_MAGIC_N);
@@ -667,12 +768,15 @@ ulint dict_index_t::get_col_pos(ulint n, bool inc_prefix, bool is_virtual,
 
   return (ULINT_UNDEFINED);
 }
+
 /** Frees an index memory object. */
 void dict_mem_index_free(dict_index_t *index) /*!< in: index */
 {
   ut_ad(index);
   ut_ad(index->magic_n == DICT_INDEX_MAGIC_N);
 
+  index->destroy_fields_array();
+
 #ifndef UNIV_HOTBACKUP
   dict_index_zip_pad_mutex_destroy(index);
 
@@ -684,12 +788,12 @@ void dict_mem_index_free(dict_index_t *index) /*!< in: index */
          it != index->rtr_track->rtr_active->end(); ++it) {
       rtr_info = *it;
 
-      rtr_info->index = NULL;
+      rtr_info->index = nullptr;
     }
 
     mutex_destroy(&index->rtr_ssn.mutex);
     mutex_destroy(&index->rtr_track->rtr_active_mutex);
-    UT_DELETE(index->rtr_track->rtr_active);
+    ut::delete_(index->rtr_track->rtr_active);
   }
   dict_index_remove_from_v_col_list(index);
 #endif /* !UNIV_HOTBACKUP */
@@ -708,9 +812,9 @@ at bootstrap. The second number is 32 bits, can use up to 10 digits, and is
 initialized at startup to a randomly distributed number. It is hoped that the
 combination of these two numbers will provide a reasonably unique temporary
 file name.
-@param[in]	heap	A memory heap
-@param[in]	dbtab	Table name in the form database/table name
-@param[in]	id	Table id
+@param[in]      heap    A memory heap
+@param[in]      dbtab   Table name in the form database/table name
+@param[in]      id      Table id
 @return A unique temporary tablename suitable for InnoDB use */
 char *dict_mem_create_temporary_tablename(mem_heap_t *heap, const char *dbtab,
                                           table_id_t id) {
@@ -721,14 +825,14 @@ char *dict_mem_create_temporary_tablename(mem_heap_t *heap, const char *dbtab,
   size_t dblen = dbend - dbtab + 1;
 
   /* Increment a randomly initialized  number for each temp file. */
-  os_atomic_increment_uint32(&dict_temp_file_num, 1);
+  auto file_num =
+      dict_temp_file_num.fetch_add(1, std::memory_order_relaxed) + 1;
 
   size = dblen + (sizeof(TEMP_FILE_PREFIX) + 3 + 20 + 1 + 10);
   name = static_cast<char *>(mem_heap_alloc(heap, size));
   memcpy(name, dbtab, dblen);
   snprintf(name + dblen, size - dblen,
-           TEMP_FILE_PREFIX_INNODB UINT64PF "-" UINT32PF, id,
-           dict_temp_file_num);
+           TEMP_FILE_PREFIX_INNODB UINT64PF "-" UINT32PF, id, file_num);
 
   return (name);
 }
@@ -736,18 +840,19 @@ char *dict_mem_create_temporary_tablename(mem_heap_t *heap, const char *dbtab,
 /** Initialize dict memory variables */
 void dict_mem_init(void) {
   /* Initialize a randomly distributed temporary file number */
-  ib_uint32_t now = static_cast<ib_uint32_t>(ut_time());
+  uint32_t now = static_cast<uint32_t>(
+      std::chrono::system_clock::to_time_t(std::chrono::system_clock::now()));
 
   const byte *buf = reinterpret_cast<const byte *>(&now);
+  auto file_num = ut_crc32(buf, sizeof(now));
+  dict_temp_file_num.store(file_num, std::memory_order_relaxed);
 
-  dict_temp_file_num = ut_crc32(buf, sizeof(now));
-
-  DBUG_PRINT("dict_mem_init", ("Starting Temporary file number is " UINT32PF,
-                               dict_temp_file_num));
+  DBUG_PRINT("dict_mem_init",
+             ("Starting Temporary file number is " UINT32PF, file_num));
 }
 
 /** Validate the search order in the foreign key set.
-@param[in]	fk_set	the foreign key set to be validated
+@param[in]      fk_set  the foreign key set to be validated
 @return true if search order is fine in the set, false otherwise. */
 bool dict_foreign_set_validate(const dict_foreign_set &fk_set) {
   dict_foreign_not_exists not_exists(fk_set);
@@ -762,24 +867,46 @@ bool dict_foreign_set_validate(const dict_foreign_set &fk_set) {
   dict_foreign_t *foreign = *it;
   std::cerr << "Foreign key lookup failed: " << *foreign;
   std::cerr << fk_set;
-  ut_ad(0);
-  return (false);
+  ut_d(ut_error);
+  ut_o(return (false));
 }
 
 /** Validate the search order in the foreign key sets of the table
 (foreign_set and referenced_set).
-@param[in]	table	table whose foreign key sets are to be validated
+@param[in]      table   table whose foreign key sets are to be validated
 @return true if foreign key sets are fine, false otherwise. */
 bool dict_foreign_set_validate(const dict_table_t &table) {
   return (dict_foreign_set_validate(table.foreign_set) &&
           dict_foreign_set_validate(table.referenced_set));
 }
+
+bool dict_foreign_t::is_fts_col_affected() const {
+  /* The check is skipped:
+     - if the table has no full text index defined.
+     - if it is a self referential foreign constaint. This is because
+       in the context of cascading DML operation, only the referenced
+       table is relevant for the validation even if the current table
+       has FTS index.*/
+  if (!foreign_table->fts || foreign_table == referenced_table) {
+    return false;
+  }
+
+  for (ulint i = 0; i < n_fields; i++) {
+    const dict_col_t *col = foreign_index->get_col(i);
+    if (dict_table_is_fts_column(foreign_table->fts->indexes,
+                                 dict_col_get_no(col),
+                                 col->is_virtual()) != ULINT_UNDEFINED) {
+      return true;
+    }
+  }
+  return false;
+}
 #endif /* !UNIV_HOTBACKUP */
 
 std::ostream &operator<<(std::ostream &out, const dict_foreign_t &foreign) {
   out << "[dict_foreign_t: id='" << foreign.id << "'";
 
-  if (foreign.foreign_table_name != NULL) {
+  if (foreign.foreign_table_name != nullptr) {
     out << ",for: '" << foreign.foreign_table_name << "'";
   }
 
@@ -793,3 +920,7 @@ std::ostream &operator<<(std::ostream &out, const dict_foreign_set &fk_set) {
   out << "]" << std::endl;
   return (out);
 }
+
+page_size_t dict_index_t::get_page_size() const {
+  return (dict_table_page_size(table));
+}
