@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -45,7 +46,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 Currently this can be the system tablespace or a temporary table tablespace */
 class Tablespace {
  public:
-  typedef std::vector<Datafile, ut_allocator<Datafile>> files_t;
+  typedef std::vector<Datafile, ut::allocator<Datafile>> files_t;
 
   /** Data file information - each Datafile can be accessed globally */
   files_t m_files;
@@ -56,6 +57,7 @@ class Tablespace {
         m_space_id(SPACE_UNKNOWN),
         m_path(),
         m_flags(),
+        m_autoextend_size(),
         m_ignore_read_only(false) {
     /* No op */
   }
@@ -64,13 +66,13 @@ class Tablespace {
     shutdown();
     ut_ad(m_files.empty());
     ut_ad(m_space_id == SPACE_UNKNOWN);
-    if (m_name != NULL) {
-      ut_free(m_name);
-      m_name = NULL;
+    if (m_name != nullptr) {
+      ut::free(m_name);
+      m_name = nullptr;
     }
-    if (m_path != NULL) {
-      ut_free(m_path);
-      m_path = NULL;
+    if (m_path != nullptr) {
+      ut::free(m_path);
+      m_path = nullptr;
     }
   }
 
@@ -79,11 +81,11 @@ class Tablespace {
   Tablespace &operator=(const Tablespace &);
 
   /** Set tablespace name
-  @param[in]	name	tablespace name */
+  @param[in]    name    tablespace name */
   void set_name(const char *name) {
-    ut_ad(m_name == NULL);
+    ut_ad(m_name == nullptr);
     m_name = mem_strdup(name);
-    ut_ad(m_name != NULL);
+    ut_ad(m_name != nullptr);
   }
 
   /** Get tablespace name
@@ -91,18 +93,18 @@ class Tablespace {
   const char *name() const { return (m_name); }
 
   /** Set tablespace path and filename members.
-  @param[in]	path	where tablespace file(s) resides
-  @param[in]	len	length of the file path */
+  @param[in]    path    where tablespace file(s) resides
+  @param[in]    len     length of the file path */
   void set_path(const char *path, size_t len) {
-    ut_ad(m_path == NULL);
+    ut_ad(m_path == nullptr);
     m_path = mem_strdupl(path, len);
-    ut_ad(m_path != NULL);
+    ut_ad(m_path != nullptr);
 
     Fil_path::normalize(m_path);
   }
 
   /** Set tablespace path and filename members.
-  @param[in]	path	where tablespace file(s) resides */
+  @param[in]    path    where tablespace file(s) resides */
   void set_path(const char *path) { set_path(path, strlen(path)); }
 
   /** Get tablespace path
@@ -110,7 +112,7 @@ class Tablespace {
   const char *path() const { return (m_path); }
 
   /** Set the space id of the tablespace
-  @param[in]	space_id	 tablespace ID to set */
+  @param[in]    space_id         tablespace ID to set */
   void set_space_id(space_id_t space_id) {
     ut_ad(m_space_id == SPACE_UNKNOWN);
     m_space_id = space_id;
@@ -121,7 +123,7 @@ class Tablespace {
   space_id_t space_id() const { return (m_space_id); }
 
   /** Set the tablespace flags
-  @param[in]	fsp_flags	tablespace flags */
+  @param[in]    fsp_flags       tablespace flags */
   void set_flags(uint32_t fsp_flags) {
     ut_ad(fsp_flags_is_valid(fsp_flags));
     m_flags = fsp_flags;
@@ -132,7 +134,7 @@ class Tablespace {
   uint32_t flags() const { return (m_flags); }
 
   /** Set Ignore Read Only Status for tablespace.
-  @param[in]	read_only_status	read only status indicator */
+  @param[in]    read_only_status        read only status indicator */
   void set_ignore_read_only(bool read_only_status) {
     m_ignore_read_only = read_only_status;
   }
@@ -152,16 +154,11 @@ class Tablespace {
     return (sum);
   }
 
-  /** Open or Create the data files if they do not exist.
-  @param[in]	is_temp	whether this is a temporary tablespace
-  @return DB_SUCCESS or error code */
-  dberr_t open_or_create(bool is_temp) MY_ATTRIBUTE((warn_unused_result));
-
   /** Delete all the data files. */
   void delete_files();
 
   /** Check if two tablespaces have common data file names.
-  @param[in]	other_space	Tablespace to check against this.
+  @param[in]    other_space     Tablespace to check against this.
   @return true if they have the same data filenames and paths */
   bool intersection(const Tablespace *other_space);
 
@@ -172,7 +169,7 @@ class Tablespace {
   extension .ibd and have a basename of at least 1 byte.
 
   Set tablespace m_path member and add a Datafile with the filename.
-  @param[in]	datafile_added	full path of the tablespace file. */
+  @param[in]    datafile_added  full path of the tablespace file. */
   dberr_t add_datafile(const char *datafile_added);
 
   /* Return a pointer to the first Datafile for this Tablespace
@@ -182,6 +179,12 @@ class Tablespace {
     return (&m_files.front());
   }
 
+  /* Set the autoextend size for the tablespace */
+  void set_autoextend_size(uint64_t size) { m_autoextend_size = size; }
+
+  /* Get the autoextend size for the tablespace */
+  uint64_t get_autoextend_size() const { return m_autoextend_size; }
+
   /** @return true if tablespace is encrypted */
   bool is_encrypted() const noexcept {
     return (FSP_FLAGS_GET_ENCRYPTION(m_flags));
@@ -189,12 +192,12 @@ class Tablespace {
 
  private:
   /**
-  @param[in]	filename	Name to lookup in the data files.
+  @param[in]    filename        Name to lookup in the data files.
   @return true if the filename exists in the data files */
   bool find(const char *filename);
 
   /** Note that the data file was found.
-  @param[in]	file	data file object */
+  @param[in,out] file   Data file object to set */
   void file_found(Datafile &file);
 
   /* DATA MEMBERS */
@@ -211,6 +214,9 @@ class Tablespace {
   /** Tablespace flags */
   uint32_t m_flags;
 
+  /** Autoextend size */
+  uint64_t m_autoextend_size;
+
  protected:
   /** Ignore server read only configuration for this tablespace. */
   bool m_ignore_read_only;
