@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -43,9 +44,6 @@ inline bool true_word_char(char c, bool extra_word_chars, char ch) {
   return result;
 }
 
-/** Check if a char is misc word */
-#define misc_word_char(X) 0
-
 /** Boolean search syntax */
 static const char *fts_boolean_syntax = DEFAULT_FTB_SYNTAX;
 
@@ -70,23 +68,21 @@ struct FT_WORD {
 
 /** Tokenizer for ngram referring to ft_get_word(ft_parser.c) in MyISAM.
 Differences: a. code format changed; b. stopword processing removed.
-@param[in]	cs	charset
-@param[in,out]	start	doc start pointer
-@param[in,out]	end	doc end pointer
-@param[in,out]	word	token
-@param[in,out]	info	token info
-@retval	0	eof
-@retval	1	word found
-@retval	2	left bracket
-@retval	3	right bracket
-@retval	4	stopword found */
+@param[in]      cs      charset
+@param[in,out]  start   doc start pointer
+@param[in,out]  end     doc end pointer
+@param[in,out]  word    token
+@param[in,out]  info    token info
+@retval 0       eof
+@retval 1       word found
+@retval 2       left bracket
+@retval 3       right bracket
+@retval 4       stopword found */
 inline uchar fts_get_word(const CHARSET_INFO *cs, bool extra_word_chars,
                           uchar **start, uchar *end, FT_WORD *word,
                           MYSQL_FTPARSER_BOOLEAN_INFO *info) {
   uchar *doc = *start;
   int ctype;
-  uint mwc;
-  uint length;
   int mbl;
 
   info->yesno = (FTB_YES == ' ') ? 1 : (info->quot != nullptr);
@@ -150,25 +146,22 @@ inline uchar fts_get_word(const CHARSET_INFO *cs, bool extra_word_chars,
       info->weight_adjust = info->wasign = 0;
     }
 
-    mwc = length = 0;
     for (word->pos = doc; doc < end;
-         length++, doc += (mbl > 0 ? mbl : (mbl < 0 ? -mbl : 1))) {
+         doc += (mbl > 0 ? mbl : (mbl < 0 ? -mbl : 1))) {
       mbl = cs->cset->ctype(cs, &ctype, doc, end);
 
       if (extra_word_chars && *doc == FTB_RQUOT) {
         break;
-      } else if (true_word_char(ctype, extra_word_chars, *doc)) {
-        mwc = 0;
-      } else if (!misc_word_char(*doc) || mwc) {
+      }
+      
+      if (!true_word_char(ctype, extra_word_chars, *doc)) {
         break;
-      } else {
-        mwc++;
       }
     }
 
     /* Be sure *prev is true_word_char. */
     info->prev = 'A';
-    word->len = (uint)(doc - word->pos) - mwc;
+    word->len = (uint)(doc - word->pos);
 
     if ((info->trunc = (doc < end && *doc == FTB_TRUNC))) {
       doc++;
