@@ -1,17 +1,18 @@
 /***********************************************************************
 
-Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+Copyright (c) 2019, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -210,7 +211,7 @@ class Encryption {
                   specified explicitly
   @param[in]      algorithm       Encryption algorithm to check
   @return true if no algorithm explicitly requested */
-  MY_NODISCARD static bool none_explicitly_specified(
+  [[nodiscard]] static bool none_explicitly_specified(
       bool explicit_encryption, const char *algorithm) noexcept;
 
   /** Generate random encryption value for key and iv.
@@ -378,7 +379,7 @@ class Encryption {
                                 byte *tmp, ulint tmp_len) const noexcept;
 
   /** Check if keyring plugin loaded. */
-  MY_NODISCARD static bool check_keyring() noexcept;
+  [[nodiscard]] static bool check_keyring() noexcept;
 
   /** Get encryption type
   @return encryption type **/
