@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2021, Oracle and/or its affiliates.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -39,6 +40,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "sync0sync.h"
 #include "univ.i"
 #include "ut0counter.h"
+#include "ut0log.h"
 #include "ut0new.h"
 
 #ifdef UNIV_DEBUG
@@ -54,24 +56,6 @@ typedef CRITICAL_SECTION sys_mutex_t;
 typedef pthread_mutex_t sys_mutex_t;
 #endif /* _WIN32 */
 
-/** The new (C++11) syntax allows the following and we should use it when it
-is available on platforms that we support.
-
-        enum class mutex_state_t : lock_word_t { ... };
-*/
-
-/** Mutex states. */
-enum mutex_state_t {
-  /** Mutex is free */
-  MUTEX_STATE_UNLOCKED = 0,
-
-  /** Mutex is acquired by some thread. */
-  MUTEX_STATE_LOCKED = 1,
-
-  /** Mutex is contended and there are threads waiting on the lock. */
-  MUTEX_STATE_WAITERS = 2
-};
-
 /*
                 LATCHING ORDER WITHIN THE DATABASE
                 ==================================
@@ -82,40 +66,40 @@ the corresponding file data structure. In the latching order below, these
 file page object latches are placed immediately below the corresponding
 central memory object latch or mutex.
 
-Synchronization object			Notes
-----------------------			-----
+Synchronization object                  Notes
+----------------------                  -----
 
-Dictionary mutex			If we have a pointer to a dictionary
-|					object, e.g., a table, it can be
-|					accessed without reserving the
-|					dictionary mutex. We must have a
-|					reservation, a memoryfix, to the
-|					appropriate table object in this case,
-|					and the table must be explicitly
-|					released later.
+Dictionary mutex                        If we have a pointer to a dictionary
+|                                       object, e.g., a table, it can be
+|                                       accessed without reserving the
+|                                       dictionary mutex. We must have a
+|                                       reservation, a memoryfix, to the
+|                                       appropriate table object in this case,
+|                                       and the table must be explicitly
+|                                       released later.
 V
 Dictionary header
 |
 V
-Secondary index tree latch		The tree latch protects also all
-|					the B-tree non-leaf pages. These
-V					can be read with the page only
-Secondary index non-leaf		bufferfixed to save CPU time,
-|					no s-latch is needed on the page.
-|					Modification of a page requires an
-|					x-latch on the page, however. If a
-|					thread owns an x-latch to the tree,
-|					it is allowed to latch non-leaf pages
-|					even after it has acquired the fsp
-|					latch.
+Secondary index tree latch              The tree latch protects also all
+|                                       the B-tree non-leaf pages. These
+V                                       can be read with the page only
+Secondary index non-leaf                bufferfixed to save CPU time,
+|                                       no s-latch is needed on the page.
+|                                       Modification of a page requires an
+|                                       x-latch on the page, however. If a
+|                                       thread owns an x-latch to the tree,
+|                                       it is allowed to latch non-leaf pages
+|                                       even after it has acquired the fsp
+|                                       latch.
 V
-Secondary index leaf			The latch on the secondary index leaf
-|					can be kept while accessing the
-|					clustered index, to save CPU time.
+Secondary index leaf                    The latch on the secondary index leaf
+|                                       can be kept while accessing the
+|                                       clustered index, to save CPU time.
 V
-Clustered index tree latch		To increase concurrency, the tree
-|					latch is usually released when the
-|					leaf page latch has been acquired.
+Clustered index tree latch              To increase concurrency, the tree
+|                                       latch is usually released when the
+|                                       leaf page latch has been acquired.
 V
 Clustered index non-leaf
 |
@@ -126,27 +110,27 @@ V
 Transaction system header
 |
 V
-Transaction undo mutex			The undo log entry must be written
-|					before any index page is modified.
-|					Transaction undo mutex is for the undo
-|					logs the analogue of the tree latch
-|					for a B-tree. If a thread has the
-|					trx undo mutex reserved, it is allowed
-|					to latch the undo log pages in any
-|					order, and also after it has acquired
-|					the fsp latch.
+Transaction undo mutex                  The undo log entry must be written
+|                                       before any index page is modified.
+|                                       Transaction undo mutex is for the undo
+|                                       logs the analogue of the tree latch
+|                                       for a B-tree. If a thread has the
+|                                       trx undo mutex reserved, it is allowed
+|                                       to latch the undo log pages in any
+|                                       order, and also after it has acquired
+|                                       the fsp latch.
 V
-Rollback segment mutex			The rollback segment mutex must be
-|					reserved, if, e.g., a new page must
-|					be added to an undo log. The rollback
-|					segment and the undo logs in its
-|					history list can be seen as an
-|					analogue of a B-tree, and the latches
-|					reserved similarly, using a version of
-|					lock-coupling. If an undo log must be
-|					extended by a page when inserting an
-|					undo log record, this corresponds to
-|					a pessimistic insert in a B-tree.
+Rollback segment mutex                  The rollback segment mutex must be
+|                                       reserved, if, e.g., a new page must
+|                                       be added to an undo log. The rollback
+|                                       segment and the undo logs in its
+|                                       history list can be seen as an
+|                                       analogue of a B-tree, and the latches
+|                                       reserved similarly, using a version of
+|                                       lock-coupling. If an undo log must be
+|                                       extended by a page when inserting an
+|                                       undo log record, this corresponds to
+|                                       a pessimistic insert in a B-tree.
 V
 Rollback segment header
 |
@@ -154,48 +138,48 @@ V
 Purge system latch
 |
 V
-Undo log pages				If a thread owns the trx undo mutex,
-|					or for a log in the history list, the
-|					rseg mutex, it is allowed to latch
-|					undo log pages in any order, and even
-|					after it has acquired the fsp latch.
-|					If a thread does not have the
-|					appropriate mutex, it is allowed to
-|					latch only a single undo log page in
-|					a mini-transaction.
+Undo log pages                          If a thread owns the trx undo mutex,
+|                                       or for a log in the history list, the
+|                                       rseg mutex, it is allowed to latch
+|                                       undo log pages in any order, and even
+|                                       after it has acquired the fsp latch.
+|                                       If a thread does not have the
+|                                       appropriate mutex, it is allowed to
+|                                       latch only a single undo log page in
+|                                       a mini-transaction.
 V
-File space management latch		If a mini-transaction must allocate
-|					several file pages, it can do that,
-|					because it keeps the x-latch to the
-|					file space management in its memo.
+File space management latch             If a mini-transaction must allocate
+|                                       several file pages, it can do that,
+|                                       because it keeps the x-latch to the
+|                                       file space management in its memo.
 V
 File system pages
 |
 V
-lock_sys_wait_mutex			Mutex protecting lock timeout data
+lock_sys_wait_mutex                     Mutex protecting lock timeout data
 |
 V
-lock_sys->global_sharded_latch		Sharded rw-latch protecting lock_sys_t
+lock_sys->global_sharded_latch          Sharded rw-latch protecting lock_sys_t
 |
 V
-lock_sys->table_mutexes			Mutexes protecting lock_sys_t table
-|					lock queues
+lock_sys->table_mutexes                 Mutexes protecting lock_sys_t table
+|                                       lock queues
 |
 V
-lock_sys->page_mutexes			Mutexes protecting lock_sys_t page
-|					lock queues
+lock_sys->page_mutexes                  Mutexes protecting lock_sys_t page
+|                                       lock queues
 |
 V
-trx_sys->mutex				Mutex protecting trx_sys_t
+trx_sys->mutex                          Mutex protecting trx_sys_t
 |
 V
-Threads mutex				Background thread scheduling mutex
+Threads mutex                           Background thread scheduling mutex
 |
 V
-query_thr_mutex				Mutex protecting query threads
+query_thr_mutex                         Mutex protecting query threads
 |
 V
-trx_mutex				Mutex protecting trx_t fields
+trx_mutex                               Mutex protecting trx_t fields
 |
 V
 Search system mutex
@@ -231,6 +215,8 @@ enum latch_level_t {
 
   SYNC_ANY_LATCH,
 
+  SYNC_ALTER_STAGE,
+
   SYNC_FIL_SHARD,
 
   SYNC_PAGE_ARCH_OPER,
@@ -258,10 +244,11 @@ enum latch_level_t {
   SYNC_RECV,
 
   SYNC_LOG_LIMITS,
+  SYNC_LOG_FLUSHER,
+  SYNC_LOG_FILES,
   SYNC_LOG_WRITER,
   SYNC_LOG_WRITE_NOTIFIER,
   SYNC_LOG_FLUSH_NOTIFIER,
-  SYNC_LOG_FLUSHER,
   SYNC_LOG_CLOSER,
   SYNC_LOG_CHECKPOINTER,
   SYNC_LOG_SN,
@@ -273,12 +260,14 @@ enum latch_level_t {
   SYNC_LOG_ONLINE,
 
   SYNC_PAGE_CLEANER,
-  SYNC_PURGE_QUEUE,
   SYNC_TRX_SYS_HEADER,
+  SYNC_TRX_SYS_SERIALISATION,
+  SYNC_PURGE_QUEUE,
   SYNC_THREADS,
   SYNC_TRX,
   SYNC_POOL,
   SYNC_POOL_MANAGER,
+  SYNC_TRX_SYS_SHARD,
   SYNC_TRX_SYS,
   SYNC_LOCK_SYS_SHARDED,
   SYNC_LOCK_SYS_GLOBAL,
@@ -330,6 +319,8 @@ enum latch_level_t {
 
   SYNC_DICT_OPERATION,
 
+  SYNC_AHI_ENABLED,
+
   SYNC_TRX_I_S_LAST_READ,
 
   SYNC_TRX_I_S_RWLOCK,
@@ -352,6 +343,8 @@ up its meta-data. See sync0debug.c. The order does not matter here, but
 alphabetical ordering seems useful */
 enum latch_id_t {
   LATCH_ID_NONE = 0,
+  LATCH_ID_AHI_ENABLED,
+  LATCH_ID_ALTER_STAGE,
   LATCH_ID_AUTOINC,
   LATCH_ID_BUF_BLOCK_MUTEX,
   LATCH_ID_BUF_POOL_CHUNKS,
@@ -363,6 +356,7 @@ enum latch_id_t {
   LATCH_ID_BUF_POOL_FLUSH_STATE,
   LATCH_ID_DBLWR,
   LATCH_ID_DBLWR_SPACE_CACHE,
+  LATCH_ID_DDL_AUTOINC,
   LATCH_ID_CACHE_LAST_READ,
   LATCH_ID_DICT_FOREIGN_ERR,
   LATCH_ID_DICT_SYS,
@@ -393,6 +387,7 @@ enum latch_id_t {
   LATCH_ID_LOG_FLUSH_NOTIFIER,
   LATCH_ID_LOG_LIMITS,
   LATCH_ID_LOG_ONLINE,
+  LATCH_ID_LOG_FILES,
   LATCH_ID_PARSER,
   LATCH_ID_LOG_ARCH,
   LATCH_ID_PAGE_ARCH,
@@ -427,6 +422,8 @@ enum latch_id_t {
   LATCH_ID_TEMP_POOL_TBLSP,
   LATCH_ID_TRX,
   LATCH_ID_TRX_SYS,
+  LATCH_ID_TRX_SYS_SHARD,
+  LATCH_ID_TRX_SYS_SERIALISATION,
   LATCH_ID_SRV_SYS,
   LATCH_ID_SRV_SYS_TASKS,
   LATCH_ID_PAGE_ZIP_STAT_PER_INDEX,
@@ -436,9 +433,7 @@ enum latch_id_t {
   LATCH_ID_ZIP_PAD_MUTEX,
   LATCH_ID_OS_AIO_READ_MUTEX,
   LATCH_ID_OS_AIO_WRITE_MUTEX,
-  LATCH_ID_OS_AIO_LOG_MUTEX,
   LATCH_ID_OS_AIO_IBUF_MUTEX,
-  LATCH_ID_OS_AIO_SYNC_MUTEX,
   LATCH_ID_ROW_DROP_LIST,
   LATCH_ID_INDEX_ONLINE_LOG,
   LATCH_ID_WORK_QUEUE,
@@ -461,15 +456,7 @@ enum latch_id_t {
   LATCH_ID_HASH_TABLE_RW_LOCK,
   LATCH_ID_BUF_CHUNK_MAP_LATCH,
   LATCH_ID_SYNC_DEBUG_MUTEX,
-  LATCH_ID_SCRUB_STAT_MUTEX,
   LATCH_ID_MASTER_KEY_ID_MUTEX,
-  LATCH_ID_FIL_CRYPT_MUTEX,
-  LATCH_ID_FIL_CRYPT_STAT_MUTEX,
-  LATCH_ID_FIL_CRYPT_DATA_MUTEX,
-  LATCH_ID_FIL_CRYPT_THREADS_MUTEX,
-  LATCH_ID_FIL_CRYPT_THREADS_SET_CNT_MUTEX,
-  LATCH_ID_FIL_CRYPT_LIST_MUTEX,
-  LATCH_ID_FIL_CRYPT_START_ROTATE_MUTEX,
   LATCH_ID_FILE_OPEN,
   LATCH_ID_CLONE_SYS,
   LATCH_ID_CLONE_TASK,
@@ -509,7 +496,7 @@ struct OSMutex {
   OSMutex &operator=(const OSMutex &) = default;
 
   /** Destructor */
-  ~OSMutex() {}
+  ~OSMutex() = default;
 
   /** Destroy the mutex */
   void destroy() UNIV_NOTHROW {
@@ -555,6 +542,9 @@ struct OSMutex {
 #endif /* _WIN32 */
   }
 
+  void lock() { enter(); }
+  void unlock() { exit(); }
+
   /** @return true if locking succeeded */
   bool try_lock() UNIV_NOTHROW {
     ut_ad(innodb_calling_exit || !m_freed);
@@ -592,35 +582,36 @@ struct OSMutex {
 #ifdef UNIV_PFS_MUTEX
 /** Latch element
 Used for mutexes which have PFS keys defined under UNIV_PFS_MUTEX.
-@param[in]	id		Latch id
-@param[in]	level		Latch level
-@param[in]	key		PFS key */
-#define LATCH_ADD_MUTEX(id, level, key) \
-  latch_meta[LATCH_ID_##id] =           \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level, key))
+@param[in]      id              Latch id
+@param[in]      level           Latch level
+@param[in]      key             PFS key */
+#define LATCH_ADD_MUTEX(id, level, key)                      \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level, key)
 
 #ifdef UNIV_PFS_RWLOCK
 /** Latch element.
 Used for rwlocks which have PFS keys defined under UNIV_PFS_RWLOCK.
-@param[in]	id		Latch id
-@param[in]	level		Latch level
-@param[in]	key		PFS key */
-#define LATCH_ADD_RWLOCK(id, level, key) \
-  latch_meta[LATCH_ID_##id] =            \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level, key))
+@param[in]      id              Latch id
+@param[in]      level           Latch level
+@param[in]      key             PFS key */
+#define LATCH_ADD_RWLOCK(id, level, key)                     \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level, key)
 #else
-#define LATCH_ADD_RWLOCK(id, level, key)    \
-  latch_meta[LATCH_ID_##id] = UT_NEW_NOKEY( \
-      latch_meta_t(LATCH_ID_##id, #id, level, #level, PSI_NOT_INSTRUMENTED))
+#define LATCH_ADD_RWLOCK(id, level, key)                                     \
+  latch_meta[LATCH_ID_##id] =                                                \
+      ut::new_withkey<latch_meta_t>(UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, \
+                                    #id, level, #level, PSI_NOT_INSTRUMENTED)
 #endif /* UNIV_PFS_RWLOCK */
 
 #else
-#define LATCH_ADD_MUTEX(id, level, key) \
-  latch_meta[LATCH_ID_##id] =           \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level))
-#define LATCH_ADD_RWLOCK(id, level, key) \
-  latch_meta[LATCH_ID_##id] =            \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level))
+#define LATCH_ADD_MUTEX(id, level, key)                      \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level)
+#define LATCH_ADD_RWLOCK(id, level, key)                     \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level)
 #endif /* UNIV_PFS_MUTEX */
 
 /** Default latch counter */
@@ -661,7 +652,7 @@ class LatchCounter {
     m_mutex.destroy();
 
     for (Count *count : m_counters) {
-      UT_DELETE(count);
+      ut::delete_(count);
     }
   }
 
@@ -686,7 +677,7 @@ class LatchCounter {
     Count *count;
 
     if (m_counters.empty()) {
-      count = UT_NEW_NOKEY(Count());
+      count = ut::new_withkey<Count>(UT_NEW_THIS_FILE_PSI_KEY);
       m_counters.push_back(count);
     } else {
       ut_a(m_counters.size() == 1);
@@ -699,8 +690,9 @@ class LatchCounter {
   }
 
   /** Deregister the count. We don't do anything
-  @param[in]	count		The count instance to deregister */
-  void sum_deregister(Count *count) const UNIV_NOTHROW { /* Do nothing */
+  @param[in]    count           The count instance to deregister */
+  void sum_deregister(Count *count
+                      [[maybe_unused]]) const UNIV_NOTHROW { /* Do nothing */
   }
 
   /** Register a single instance counter */
@@ -713,7 +705,7 @@ class LatchCounter {
   }
 
   /** Deregister a single instance counter
-  @param[in]	count		The count instance to deregister */
+  @param[in]    count           The count instance to deregister */
   void single_deregister(Count *count) UNIV_NOTHROW {
     m_mutex.enter();
 
@@ -805,14 +797,14 @@ class LatchMeta {
   }
 
   /** Destructor */
-  ~LatchMeta() {}
+  ~LatchMeta() = default;
 
   /** Constructor
-  @param[in]	id		Latch id
-  @param[in]	name		Latch name
-  @param[in]	level		Latch level
-  @param[in]	level_name	Latch level text representation
-  @param[in]	key		PFS key */
+  @param[in]    id              Latch id
+  @param[in]    name            Latch name
+  @param[in]    level           Latch level
+  @param[in]    level_name      Latch level text representation
+  @param[in]    key             PFS key */
   LatchMeta(latch_id_t id, const char *name, latch_level_t level,
             const char *level_name
 #ifdef UNIV_PFS_MUTEX
@@ -833,7 +825,7 @@ class LatchMeta {
   }
 
   /* Less than operator.
-  @param[in]	rhs		Instance to compare against
+  @param[in]    rhs             Instance to compare against
   @return true if this.get_id() < rhs.get_id() */
   bool operator<(const LatchMeta &rhs) const {
     return (get_id() < rhs.get_id());
@@ -882,7 +874,8 @@ class LatchMeta {
 };
 
 typedef LatchMeta<LatchCounter> latch_meta_t;
-typedef std::vector<latch_meta_t *, ut_allocator<latch_meta_t *>> LatchMetaData;
+typedef std::vector<latch_meta_t *, ut::allocator<latch_meta_t *>>
+    LatchMetaData;
 
 /** Note: This is accessed without any mutex protection. It is initialised
 at startup and elements should not be added to or removed from it after
@@ -890,7 +883,7 @@ that.  See sync_latch_meta_init() */
 extern LatchMetaData latch_meta;
 
 /** Get the latch meta-data from the latch ID
-@param[in]	id		Latch ID
+@param[in]      id              Latch ID
 @return the latch meta data */
 inline latch_meta_t &sync_latch_get_meta(latch_id_t id) {
   ut_ad(static_cast<size_t>(id) < latch_meta.size());
@@ -900,7 +893,7 @@ inline latch_meta_t &sync_latch_get_meta(latch_id_t id) {
 }
 
 /** Fetch the counter for the latch
-@param[in]	id		Latch ID
+@param[in]      id              Latch ID
 @return the latch counter */
 inline latch_meta_t::CounterType *sync_latch_get_counter(latch_id_t id) {
   latch_meta_t &meta = sync_latch_get_meta(id);
@@ -909,7 +902,7 @@ inline latch_meta_t::CounterType *sync_latch_get_counter(latch_id_t id) {
 }
 
 /** Get the latch name from the latch ID
-@param[in]	id		Latch ID
+@param[in]      id              Latch ID
 @return the name, will assert if not found */
 inline const char *sync_latch_get_name(latch_id_t id) {
   const latch_meta_t &meta = sync_latch_get_meta(id);
@@ -918,7 +911,7 @@ inline const char *sync_latch_get_name(latch_id_t id) {
 }
 
 /** Get the latch ordering level
-@param[in]	id		Latch id to lookup
+@param[in]      id              Latch id to lookup
 @return the latch level */
 inline latch_level_t sync_latch_get_level(latch_id_t id) {
   const latch_meta_t &meta = sync_latch_get_meta(id);
@@ -928,7 +921,7 @@ inline latch_level_t sync_latch_get_level(latch_id_t id) {
 
 #ifdef UNIV_PFS_MUTEX
 /** Get the latch PFS key from the latch ID
-@param[in]	id		Latch ID
+@param[in]      id              Latch ID
 @return the PFS key */
 inline mysql_pfs_key_t sync_latch_get_pfs_key(latch_id_t id) {
   const latch_meta_t &meta = sync_latch_get_meta(id);
@@ -940,34 +933,29 @@ inline mysql_pfs_key_t sync_latch_get_pfs_key(latch_id_t id) {
 #ifndef UNIV_HOTBACKUP
 /** String representation of the filename and line number where the
 latch was created
-@param[in]	id		Latch ID
-@param[in]	created		Filename and line number where it was crated
+@param[in]      id              Latch ID
+@param[in]      created         Filename and line number where it was created
 @return the string representation */
 std::string sync_mutex_to_string(latch_id_t id, const std::string &created);
 
-/** Get the latch name from a sync level
-@param[in]	level		Latch level to lookup
-@return nullptr if not found. */
-const char *sync_latch_get_name(latch_level_t level);
-
 /** Print the filename "basename"
 @return the basename */
 const char *sync_basename(const char *filename);
 #endif /* !UNIV_HOTBACKUP */
 
 /** Register a latch, called when it is created
-@param[in]	ptr		Latch instance that was created
-@param[in]	filename	Filename where it was created
-@param[in]	line		Line number in filename */
+@param[in]      ptr             Latch instance that was created
+@param[in]      filename        Filename where it was created
+@param[in]      line            Line number in filename */
 void sync_file_created_register(const void *ptr, const char *filename,
                                 uint16_t line);
 
 /** Deregister a latch, called when it is destroyed
-@param[in]	ptr		Latch to be destroyed */
+@param[in]      ptr             Latch to be destroyed */
 void sync_file_created_deregister(const void *ptr);
 
 /** Get the string where the file was created. Its format is "name:line"
-@param[in]	ptr		Latch instance
+@param[in]      ptr             Latch instance
 @return created information or "" if can't be found */
 std::string sync_file_created_get(const void *ptr);
 
@@ -978,7 +966,7 @@ std::string sync_file_created_get(const void *ptr);
 /** All (ordered) latches, used in debugging, must derive from this class. */
 struct latch_t {
   /** Constructor
-  @param[in]	id	The latch ID */
+  @param[in]    id      The latch ID */
   explicit latch_t(latch_id_t id = LATCH_ID_NONE) UNIV_NOTHROW : m_id(id),
                                                                  m_rw_lock(),
                                                                  m_temp_fsp() {}
@@ -986,7 +974,7 @@ struct latch_t {
   latch_t &operator=(const latch_t &) = default;
 
   /** Destructor */
-  virtual ~latch_t() UNIV_NOTHROW {}
+  virtual ~latch_t() UNIV_NOTHROW = default;
 
   /** @return the latch ID */
   latch_id_t get_id() const { return (m_id); }
@@ -1058,7 +1046,7 @@ struct latch_t {
 
 /** Subclass this to iterate over a thread's acquired latch levels. */
 struct sync_check_functor_t {
-  virtual ~sync_check_functor_t() {}
+  virtual ~sync_check_functor_t() = default;
   virtual bool operator()(const latch_level_t) = 0;
   virtual bool result() const = 0;
 };
@@ -1066,15 +1054,15 @@ struct sync_check_functor_t {
 /** Functor to check whether the calling thread owns the btr search mutex. */
 struct btrsea_sync_check : public sync_check_functor_t {
   /** Constructor
-  @param[in]	has_search_latch	true if owns the latch */
+  @param[in]    has_search_latch        true if owns the latch */
   explicit btrsea_sync_check(bool has_search_latch)
       : m_result(), m_has_search_latch(has_search_latch) {}
 
   /** Destructor */
-  ~btrsea_sync_check() override {}
+  ~btrsea_sync_check() override = default;
 
   /** Called for every latch owned by the calling thread.
-  @param[in]	level		Level of the existing latch
+  @param[in]    level           Level of the existing latch
   @return true if the predicate check fails */
   bool operator()(const latch_level_t level) override {
     /* If calling thread doesn't hold search latch then
@@ -1089,7 +1077,7 @@ struct btrsea_sync_check : public sync_check_functor_t {
     Plugin in this case is I_S which is sharing the latch vector
     of InnoDB and so there could be lock conflicts. Ideally
     the Plugin should use a difference namespace latch vector
-    as it doesn't have any depedency with SE latching protocol.
+    as it doesn't have any dependency with SE latching protocol.
 
     Added check that will allow thread to hold I_S latches */
 
@@ -1128,16 +1116,16 @@ struct btrsea_sync_check : public sync_check_functor_t {
 /** Functor to check for dictionary latching constraints. */
 struct dict_sync_check : public sync_check_functor_t {
   /** Constructor
-  @param[in]	dict_mutex_allowed	true if the dict mutex
+  @param[in]    dict_mutex_allowed      true if the dict mutex
                                           is allowed */
   explicit dict_sync_check(bool dict_mutex_allowed)
       : m_result(), m_dict_mutex_allowed(dict_mutex_allowed) {}
 
   /** Destructor */
-  ~dict_sync_check() override {}
+  ~dict_sync_check() override = default;
 
   /** Check the latching constraints
-  @param[in]	level		The level held by the thread */
+  @param[in]    level           The level held by the thread */
   bool operator()(const latch_level_t level) override {
     if (!m_dict_mutex_allowed ||
         (level != SYNC_DICT && level != SYNC_UNDO_SPACES &&
@@ -1172,17 +1160,20 @@ struct dict_sync_check : public sync_check_functor_t {
 /** Functor to check for given latching constraints. */
 struct sync_allowed_latches : public sync_check_functor_t {
   /** Constructor
-  @param[in]	from	first element in an array of latch_level_t
-  @param[in]	to	last element in an array of latch_level_t */
+  @param[in]    from    first element in an array of latch_level_t
+  @param[in]    to      last element in an array of latch_level_t */
   sync_allowed_latches(const latch_level_t *from, const latch_level_t *to)
       : m_result(), m_latches(from, to) {}
 
+  /** Default constructor.  The list of allowed latches is empty. */
+  sync_allowed_latches() : m_result(), m_latches() {}
+
   /** Check whether the given latch_t violates the latch constraint.
   This object maintains a list of allowed latch levels, and if the given
   latch belongs to a latch level that is not there in the allowed list,
   then it is a violation.
 
-  @param[in]	level	The latch level to check
+  @param[in]    level   The latch level to check
   @return true if there is a latch ordering violation */
   virtual bool operator()(const latch_level_t level) override {
     for (latch_level_t allowed_level : m_latches) {
@@ -1212,17 +1203,12 @@ struct sync_allowed_latches : public sync_check_functor_t {
   True if all OK */
   bool m_result;
 
-  typedef std::vector<latch_level_t, ut_allocator<latch_level_t>> latches_t;
+  typedef std::vector<latch_level_t, ut::allocator<latch_level_t>> latches_t;
 
   /** List of latch levels that are allowed to be held */
   latches_t m_latches;
 };
 
-/** Get the latch id from a latch name.
-@param[in]	name	Latch name
-@return latch id if found else LATCH_ID_NONE. */
-latch_id_t sync_latch_get_id(const char *name);
-
 typedef ulint rw_lock_flags_t;
 
 /* Flags to specify lock types for rw_lock_own_flagged() */
