@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2021, Oracle and/or its affiliates.
+Copyright (c) 1996, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -203,7 +204,7 @@ Validates a read view list. */
 bool MVCC::validate() const {
   ViewCheck check;
 
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   ut_list_map(m_views, check);
 
@@ -226,7 +227,8 @@ void ReadView::ids_t::reserve(ulint n) {
 
   value_type *p = m_ptr;
 
-  m_ptr = UT_NEW_ARRAY_NOKEY(value_type, n);
+  m_ptr =
+      ut::new_arr_withkey<value_type>(UT_NEW_THIS_FILE_PSI_KEY, ut::Count{n});
 
   m_reserved = n;
 
@@ -235,14 +237,14 @@ void ReadView::ids_t::reserve(ulint n) {
   if (p != nullptr) {
     ::memmove(m_ptr, p, size() * sizeof(value_type));
 
-    UT_DELETE_ARRAY(p);
+    ut::delete_arr(p);
   }
 }
 
 /**
 Copy and overwrite this array contents
-@param start		Source array
-@param end		Pointer to end of array */
+@param start            Source array
+@param end              Pointer to end of array */
 
 void ReadView::ids_t::assign(const value_type *start, const value_type *end) {
   ut_ad(end >= start);
@@ -264,7 +266,7 @@ void ReadView::ids_t::assign(const value_type *start, const value_type *end) {
 
 /**
 Append a value to the array.
-@param value		the value to append */
+@param value            the value to append */
 
 void ReadView::ids_t::push_back(value_type value) {
   if (capacity() <= size()) {
@@ -329,24 +331,20 @@ ReadView::~ReadView() {
 }
 
 /** Constructor
-@param size		Number of views to pre-allocate */
-MVCC::MVCC(ulint size) {
-  UT_LIST_INIT(m_free, &ReadView::m_view_list);
-  UT_LIST_INIT(m_views, &ReadView::m_view_list);
-
+@param size             Number of views to pre-allocate */
+MVCC::MVCC(ulint size) : m_free(), m_views() {
   for (ulint i = 0; i < size; ++i) {
-    ReadView *view = UT_NEW_NOKEY(ReadView());
+    ReadView *view = ut::new_withkey<ReadView>(UT_NEW_THIS_FILE_PSI_KEY);
 
     UT_LIST_ADD_FIRST(m_free, view);
   }
 }
 
 MVCC::~MVCC() {
-  for (ReadView *view = UT_LIST_GET_FIRST(m_free); view != nullptr;
-       view = UT_LIST_GET_FIRST(m_free)) {
+  while (ReadView *view = UT_LIST_GET_FIRST(m_free)) {
     UT_LIST_REMOVE(m_free, view);
 
-    UT_DELETE(view);
+    ut::delete_(view);
   }
 
   ut_a(UT_LIST_GET_LEN(m_views) == 0);
@@ -369,6 +367,8 @@ Copy the transaction ids from the source vector */
 
 void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
   ut_ad(!m_cloned);
+  ut_ad(trx_sys_mutex_own());
+
   ulint size = trx_ids.size();
 
   if (m_creator_trx_id > 0) {
@@ -423,12 +423,33 @@ void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
   m_up_limit_id = m_ids.front();
 
 #ifdef UNIV_DEBUG
-  /* Assert that all transaction ids in list are active. */
-  for (trx_ids_t::const_iterator it = trx_ids.begin(); it != trx_ids.end();
-       ++it) {
-    trx_t *trx = trx_get_rw_trx_by_id(*it);
-    ut_ad(trx != nullptr);
-    ut_ad(trx->state == TRX_STATE_ACTIVE || trx->state == TRX_STATE_PREPARED);
+  /* The check is done randomly from time to time, because the check adds
+  a kind of extra synchronization which itself could hide existing bugs. */
+  if (ut::random_from_interval_fast(0, 99) == 0) {
+    /* Assert that all transaction ids in list are active. */
+    for (auto trx_id : trx_ids) {
+      while (trx_sys->latch_and_execute_with_active_trx(
+          trx_id,
+          [](trx_t *trx) {
+            if (trx != nullptr) {
+              const auto trx_state = trx->state.load(std::memory_order_relaxed);
+              /* Transaction in active_rw_trxs might only be ACTIVE or
+              PREPARED, before it becomes COMMITTED it is removed from
+              active_rw_trxs. */
+              ut_ad(trx_state == TRX_STATE_ACTIVE ||
+                    trx_state == TRX_STATE_PREPARED);
+              return false;
+            }
+            return true;
+          },
+          UT_LOCATION_HERE)) {
+        /* It might happen that transaction became added to rw_trx_ids,
+        then trx_sys mutex has been released and thread become scheduled
+        out before the call to trx_sys_rw_trx_add(trx). We need to wait,
+        it will come (if we hang forever in this loop - it's a bug). */
+        std::this_thread::sleep_for(std::chrono::microseconds(10));
+      }
+    }
   }
 #endif /* UNIV_DEBUG */
 }
@@ -436,15 +457,19 @@ void ReadView::copy_trx_ids(const trx_ids_t &trx_ids) {
 /**
 Opens a read view where exactly the transactions serialized before this
 point in time are seen in the view.
-@param id		Creator transaction id */
+@param id               Creator transaction id */
 
 void ReadView::prepare(trx_id_t id) {
   ut_ad(!m_cloned);
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   m_creator_trx_id = id;
 
-  m_low_limit_no = m_low_limit_id = m_up_limit_id = trx_sys->max_trx_id;
+  m_low_limit_no = trx_get_serialisation_min_trx_no();
+
+  m_low_limit_id = trx_sys_get_next_trx_id_or_no();
+
+  ut_a(m_low_limit_no <= m_low_limit_id);
 
   if (!trx_sys->rw_trx_ids.empty()) {
     copy_trx_ids(trx_sys->rw_trx_ids);
@@ -452,17 +477,10 @@ void ReadView::prepare(trx_id_t id) {
     m_ids.clear();
   }
 
-  ut_ad(m_up_limit_id <= m_low_limit_id);
+  /* The first active transaction has the smallest id. */
+  m_up_limit_id = !m_ids.empty() ? m_ids.front() : m_low_limit_id;
 
-  if (UT_LIST_GET_LEN(trx_sys->serialisation_list) > 0) {
-    const trx_t *trx;
-
-    trx = UT_LIST_GET_FIRST(trx_sys->serialisation_list);
-
-    if (trx->no < m_low_limit_no) {
-      m_low_limit_no = trx->no;
-    }
-  }
+  ut_a(m_up_limit_id <= m_low_limit_id);
 
   ut_d(m_view_low_limit_no = m_low_limit_no);
   m_closed = false;
@@ -474,7 +492,7 @@ a new view.
 @return a view to use */
 
 ReadView *MVCC::get_view() {
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   ReadView *view;
 
@@ -482,7 +500,7 @@ ReadView *MVCC::get_view() {
     view = UT_LIST_GET_FIRST(m_free);
     UT_LIST_REMOVE(m_free, view);
   } else {
-    view = UT_NEW_NOKEY(ReadView());
+    view = ut::new_withkey<ReadView>(UT_NEW_THIS_FILE_PSI_KEY);
 
     if (view == nullptr) {
       ib::error(ER_IB_MSG_918) << "Failed to allocate MVCC view";
@@ -495,7 +513,7 @@ ReadView *MVCC::get_view() {
 /**
 Release a view that is inactive but not closed. Caller must own
 the trx_sys_t::mutex.
-@param view		View to release */
+@param view             View to release */
 void MVCC::view_release(ReadView *&view) {
   ut_ad(!srv_read_only_mode);
   ut_ad(trx_sys_mutex_own());
@@ -522,9 +540,9 @@ void MVCC::view_release(ReadView *&view) {
 }
 
 /** Allocate and create a view.
-@param view	View owned by this class created for the caller. Must be
+@param view     View owned by this class created for the caller. Must be
 freed by calling view_close()
-@param trx	Transaction instance of caller */
+@param trx      Transaction instance of caller */
 void MVCC::view_open(ReadView *&view, trx_t *trx) {
   ut_ad(!srv_read_only_mode);
 
@@ -538,7 +556,7 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
     ut_ad(view->m_closed);
 
     /* NOTE: This can be optimised further, for now we only
-    resuse the view iff there are no active RW transactions.
+    reuse the view if there are no active RW transactions.
 
     There is an inherent race here between purge and this
     thread. Purge will skip views that are marked as closed.
@@ -548,20 +566,20 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
     if (trx_is_autocommit_non_locking(trx) && view->empty()) {
       view->m_closed = false;
 
-      if (view->m_low_limit_id == trx_sys_get_max_trx_id()) {
+      if (view->m_low_limit_id == trx_sys_get_next_trx_id_or_no()) {
         return;
       } else {
         view->m_closed = true;
       }
     }
+  }
 
-    mutex_enter(&trx_sys->mutex);
+  trx_sys_mutex_enter();
 
+  if (view != nullptr) {
     UT_LIST_REMOVE(m_views, view);
 
   } else {
-    mutex_enter(&trx_sys->mutex);
-
     view = get_view();
   }
 
@@ -577,7 +595,7 @@ void MVCC::view_open(ReadView *&view, trx_t *trx) {
 ReadView *MVCC::get_view_created_by_trx_id(trx_id_t trx_id) const {
   ReadView *view;
 
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   for (view = UT_LIST_GET_LAST(m_views); view != nullptr;
        view = UT_LIST_GET_PREV(m_view_list, view)) {
@@ -600,7 +618,7 @@ Get the oldest (active) view in the system.
 ReadView *MVCC::get_oldest_view() const {
   ReadView *view;
 
-  ut_ad(mutex_own(&trx_sys->mutex));
+  ut_ad(trx_sys_mutex_own());
 
   for (view = UT_LIST_GET_LAST(m_views); view != nullptr;
        view = UT_LIST_GET_PREV(m_view_list, view)) {
@@ -614,7 +632,7 @@ ReadView *MVCC::get_oldest_view() const {
 
 /**
 Copy state from another view. Must call copy_complete() to finish.
-@param other		view to copy from */
+@param other            view to copy from */
 
 void ReadView::copy_prepare(const ReadView &other) {
   ut_ad(&other != this);
@@ -689,7 +707,7 @@ void ReadView::clone(ReadView *&result, trx_t *from_trx) const {
     // yet which the cloned view must see, if it assigned later
     if (!from_trx->preallocated_id) {
       // Preallocate a transaction id for the donor
-      from_trx_id = from_trx->preallocated_id = trx_sys_get_new_trx_id();
+      from_trx_id = from_trx->preallocated_id = trx_sys_allocate_trx_id();
     } else {
       // This transaction has already been cloned
       from_trx_id = from_trx->preallocated_id;
@@ -714,9 +732,9 @@ call view_close(). The caller owns the view that is passed in.
 It will also move the closed views from the m_views list to the
 m_free list. This function is called by Purge to determine whether it should
 purge the delete marked record or not.
-@param view		Preallocated view, owned by the caller */
+@param view             Preallocated view, owned by the caller */
 void MVCC::clone_oldest_view(ReadView *view) {
-  mutex_enter(&trx_sys->mutex);
+  trx_sys_mutex_enter();
 
   ReadView *oldest_view = get_oldest_view();
 
@@ -746,8 +764,7 @@ ulint MVCC::size() const {
 
   ulint size = 0;
 
-  for (const ReadView *view = UT_LIST_GET_FIRST(m_views); view != nullptr;
-       view = UT_LIST_GET_NEXT(m_view_list, view)) {
+  for (const ReadView *view : m_views) {
     if (!view->is_closed()) {
       ++size;
     }
@@ -760,8 +777,8 @@ ulint MVCC::size() const {
 
 /**
 Close a view created by the above function.
-@param view		view allocated by trx_open.
-@param own_mutex	true if caller owns trx_sys_t::mutex */
+@param view             view allocated by trx_open.
+@param own_mutex        true if caller owns trx_sys_t::mutex */
 
 void MVCC::view_close(ReadView *&view, bool own_mutex) {
   uintptr_t p = reinterpret_cast<uintptr_t>(view);
@@ -793,20 +810,6 @@ void MVCC::view_close(ReadView *&view, bool own_mutex) {
   }
 }
 
-/**
-Set the view creator transaction id. Note: This shouldbe set only
-for views created by RW transactions.
-@param view		Set the creator trx id for this view
-@param id		Transaction id to set */
-
-void MVCC::set_view_creator_trx_id(ReadView *view, trx_id_t id) {
-  ut_ad(!view->is_cloned());
-  ut_ad(id > 0);
-  ut_ad(mutex_own(&trx_sys->mutex));
-
-  view->creator_trx_id(id);
-}
-
 i_s_xtradb_read_view_t *read_fill_i_s_xtradb_read_view(
     i_s_xtradb_read_view_t *rv) {
   ReadView *view;
