@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -86,31 +87,24 @@ Parallel_reader::Scan_ctx::Iter::~Iter() {
   m_heap = nullptr;
 }
 
-Parallel_reader::Ctx::~Ctx() {}
-
-Parallel_reader::Scan_ctx::~Scan_ctx() {}
-
 Parallel_reader::~Parallel_reader() {
   mutex_destroy(&m_mutex);
   os_event_destroy(m_event);
-
-  if (!m_single_threaded_mode) {
+  if (!m_sync) {
     release_unused_threads(m_n_threads);
   }
-
   for (auto thread_ctx : m_thread_ctxs) {
     if (thread_ctx != nullptr) {
-      UT_DELETE(thread_ctx);
+      ut::delete_(thread_ctx);
     }
   }
 }
 
 size_t Parallel_reader::available_threads(size_t n_required,
                                           bool use_reserved) {
-  const auto RELAXED = std::memory_order_relaxed;
-  auto active = s_active_threads.fetch_add(n_required, RELAXED);
-
-  size_t max_threads = MAX_THREADS;
+  auto max_threads = MAX_THREADS;
+  constexpr auto SEQ_CST = std::memory_order_seq_cst;
+  auto active = s_active_threads.fetch_add(n_required, SEQ_CST);
 
   if (use_reserved) {
     max_threads += MAX_RESERVED_THREADS;
@@ -120,23 +114,26 @@ size_t Parallel_reader::available_threads(size_t n_required,
     const auto available = max_threads - active;
 
     if (n_required <= available) {
-      return (n_required);
+      return n_required;
     } else {
-      s_active_threads.fetch_sub(n_required - available, RELAXED);
-      return (available);
+      const auto release = n_required - available;
+      const auto o = s_active_threads.fetch_sub(release, SEQ_CST);
+      ut_a(o >= release);
+      return available;
     }
   }
 
-  s_active_threads.fetch_sub(n_required, RELAXED);
+  const auto o = s_active_threads.fetch_sub(n_required, SEQ_CST);
+  ut_a(o >= n_required);
 
-  return (0);
+  return 0;
 }
 
 void Parallel_reader::Scan_ctx::index_s_lock() {
   if (m_s_locks.fetch_add(1, std::memory_order_acquire) == 0) {
     auto index = m_config.m_index;
     /* The latch can be unlocked by a thread that didn't originally lock it. */
-    rw_lock_s_lock_gen(dict_index_get_lock(index), true);
+    rw_lock_s_lock_gen(dict_index_get_lock(index), true, UT_LOCATION_HERE);
   }
 }
 
@@ -188,11 +185,11 @@ dberr_t Parallel_reader::Ctx::split() {
   return err;
 }
 
-Parallel_reader::Parallel_reader(size_t max_threads, bool sync)
+Parallel_reader::Parallel_reader(size_t max_threads)
     : m_max_threads(max_threads),
-      m_n_threads{max_threads},
+      m_n_threads(max_threads),
       m_ctxs(),
-      m_sync(sync),
+      m_sync(max_threads == 0),
       m_trx_for_slow_log(innobase_get_trx_for_slow_log()) {
   m_n_completed = 0;
 
@@ -212,39 +209,137 @@ Parallel_reader::Scan_ctx::Scan_ctx(Parallel_reader *reader, size_t id,
 class PCursor {
  public:
   /** Constructor.
-  @param[in,out]  pcur  Persistent cursor in use.
-  @param[in]      mtr   Mini-transaction used by the persistent cursor.
-  @param[in]      read_level  read level where the block should be present. */
+  @param[in,out] pcur           Persistent cursor in use.
+  @param[in] mtr                Mini-transaction used by the persistent cursor.
+  @param[in] read_level         Read level where the block should be present. */
   PCursor(btr_pcur_t *pcur, mtr_t *mtr, size_t read_level)
       : m_mtr(mtr), m_pcur(pcur), m_read_level(read_level) {}
 
-  /** Check if there are threads waiting on the index latch. Yield the latch
-  so that other threads can progress. */
-  void yield();
+ private:
+  /** Create a savepoint and commit the mini-transaction.*/
+  void savepoint() noexcept;
+
+  /** Resume from savepoint. */
+  void resume() noexcept;
+
+ public:
+  /** Store the position of the user record that is added to the key buffer and,
+  commit the mini-transaction. The cursor must not be positioned at infimum
+  or supremum.
+  @note This method must be paired with
+  restore_to_last_processed_user_record() to restore the record position */
+  void save_current_user_record_as_last_processed() noexcept;
+
+  /** Restore the cursor to the saved record. If the saved record was physically
+  removed then restore to the largest record which is smaller than the saved
+  record. If all records were removed then restore the cursor to the infimum;
+  this must be fine if the next step of the caller is to advance the cursor.
+  @note This method must be paired with
+  save_current_user_record_as_last_processed() to save the record position */
+  void restore_to_last_processed_user_record() noexcept;
+
+  /** This method must be called after all records on a page are processed and
+  cursor is positioned at supremum. Under this assumption, it stores the
+  position BTR_PCUR_AFTER the last user record on the page.
+  This method must be paired with restore_to_first_unprocessed() to restore to
+  a record which comes right after the value of the stored last processed
+  record @see restore_to_first_unprocessed for details. */
+  void save_previous_user_record_as_last_processed() noexcept;
+
+  /** Restore the cursor to the record which comes next after the saved
+  position by the paired method save_previous_user_record_as_last_processed().
+  If there is no such record, it will restore to supremum on the last leaf
+  page.
+  @note This method must be paired with
+  save_previous_user_record_as_last_processed() to save the record position */
+  void restore_to_first_unprocessed() noexcept;
 
   /** Move to the next block.
-  @param[in]  index  Index being traversed.
+  @param[in]  index             Index being traversed.
   @return DB_SUCCESS or error code. */
-  dberr_t move_to_next_block(dict_index_t *index)
-      MY_ATTRIBUTE((warn_unused_result));
+  [[nodiscard]] dberr_t move_to_next_block(dict_index_t *index);
 
   /** Restore the cursor position. */
-  void restore_position() {
-    auto relative = m_pcur->m_rel_pos;
+  void restore_position() noexcept {
+    constexpr auto MODE = BTR_SEARCH_LEAF;
+    const auto relative = m_pcur->m_rel_pos;
+    auto equal = m_pcur->restore_position(MODE, m_mtr, UT_LOCATION_HERE);
 
-    auto equal =
-        m_pcur->restore_position(BTR_SEARCH_LEAF, m_mtr, __FILE__, __LINE__);
-
-    if (relative == BTR_PCUR_ON) {
-      if (!equal) {
-        page_cur_move_to_next(m_pcur->get_page_cur());
-      }
+#ifdef UNIV_DEBUG
+    if (m_pcur->m_pos_state == BTR_PCUR_IS_POSITIONED_OPTIMISTIC) {
+      ut_ad(m_pcur->m_rel_pos == BTR_PCUR_BEFORE ||
+            m_pcur->m_rel_pos == BTR_PCUR_AFTER);
     } else {
-      ut_ad(relative == BTR_PCUR_AFTER ||
-            relative == BTR_PCUR_AFTER_LAST_IN_TREE);
+      ut_ad(m_pcur->m_pos_state == BTR_PCUR_IS_POSITIONED);
+      ut_ad((m_pcur->m_rel_pos == BTR_PCUR_ON) == m_pcur->is_on_user_rec());
+    }
+#endif /* UNIV_DEBUG */
+
+    switch (relative) {
+      case BTR_PCUR_ON:
+        if (!equal) {
+          page_cur_move_to_next(m_pcur->get_page_cur());
+        }
+        break;
+
+      case BTR_PCUR_UNSET:
+      case BTR_PCUR_BEFORE_FIRST_IN_TREE:
+        ut_error;
+        break;
+
+      case BTR_PCUR_AFTER:
+      case BTR_PCUR_AFTER_LAST_IN_TREE:
+        break;
+
+      case BTR_PCUR_BEFORE:
+        /* For non-optimistic restoration:
+        The position is now set to the record before pcur->old_rec.
+
+        For optimistic restoration:
+        The position also needs to take the previous search_mode into
+        consideration. */
+        switch (m_pcur->m_pos_state) {
+          case BTR_PCUR_IS_POSITIONED_OPTIMISTIC:
+            m_pcur->m_pos_state = BTR_PCUR_IS_POSITIONED;
+            /* The cursor always moves "up" ie. in ascending order. */
+            break;
+
+          case BTR_PCUR_IS_POSITIONED:
+            if (m_pcur->is_on_user_rec()) {
+              m_pcur->move_to_next(m_mtr);
+            }
+            break;
+
+          case BTR_PCUR_NOT_POSITIONED:
+          case BTR_PCUR_WAS_POSITIONED:
+            ut_error;
+        }
+        break;
     }
   }
 
+  /** @return the current page cursor. */
+  [[nodiscard]] page_cur_t *get_page_cursor() noexcept {
+    return m_pcur->get_page_cur();
+  }
+
+ private:
+  /** Restore from a saved position.
+  @return DB_SUCCESS or error code. */
+  [[nodiscard]] dberr_t restore_from_savepoint() noexcept;
+
+  /** Move to the first user rec on the restored page. */
+  [[nodiscard]] dberr_t move_to_user_rec() noexcept;
+
+ public:
+  /** @return true if cursor is after last on page. */
+  [[nodiscard]] bool is_after_last_on_page() const noexcept {
+    return m_pcur->is_after_last_on_page();
+  }
+
+  /** @return Level where the cursor is intended. */
+  size_t read_level() const noexcept { return m_read_level; }
+
  private:
   /** Mini-transaction. */
   mtr_t *m_mtr{};
@@ -258,29 +353,31 @@ class PCursor {
 };
 
 buf_block_t *Parallel_reader::Scan_ctx::block_get_s_latched(
-    const page_id_t &page_id, mtr_t *mtr, int line) const {
-  auto block = buf_page_get_gen(page_id, m_config.m_page_size, RW_S_LATCH,
-                                nullptr, Page_fetch::SCAN, __FILE__, line, mtr);
+    const page_id_t &page_id, mtr_t *mtr, size_t line) const {
+  /* We never scan undo tablespaces. */
+  ut_a(!fsp_is_undo_tablespace(page_id.space()));
+
+  auto block =
+      buf_page_get_gen(page_id, m_config.m_page_size, RW_S_LATCH, nullptr,
+                       Page_fetch::SCAN, {__FILE__, line}, mtr);
 
   buf_block_dbg_add_level(block, SYNC_TREE_NODE);
 
   return (block);
 }
 
-void PCursor::yield() {
-  /* We should always yield on a block boundary. */
-  ut_ad(m_pcur->is_after_last_on_page());
-
-  /* Store the cursor position on the last user record on the page. */
+void PCursor::savepoint() noexcept {
+  ut_ad(m_read_level != 0);
+  /* Store the cursor position on the previous user record on the page. */
   m_pcur->move_to_prev_on_page();
 
   m_pcur->store_position(m_mtr);
 
   m_mtr->commit();
+}
 
-  /* Yield so that another thread can proceed. */
-  std::this_thread::yield();
-
+void PCursor::resume() noexcept {
+  ut_ad(m_read_level != 0);
   m_mtr->start();
 
   m_mtr->set_log_mode(MTR_LOG_NO_REDO);
@@ -296,44 +393,101 @@ void PCursor::yield() {
   }
 }
 
-dberr_t PCursor::move_to_next_block(dict_index_t *index) {
-  ut_ad(m_pcur->is_after_last_on_page());
+void PCursor::save_current_user_record_as_last_processed() noexcept {
+  ut_a(m_pcur->is_on_user_rec());
+  m_pcur->store_position(m_mtr);
+  m_mtr->commit();
+}
 
-  if (rw_lock_get_waiters(dict_index_get_lock(index))) {
-    /* There are waiters on the index tree lock. Store and restore
-    the cursor position, and yield so that scanning a large table
-    will not starve other threads. */
+void PCursor::restore_to_last_processed_user_record() noexcept {
+  ut_a(m_pcur->m_rel_pos == BTR_PCUR_ON);
+  m_mtr->start();
+  m_mtr->set_log_mode(MTR_LOG_NO_REDO);
 
-    yield();
+  [[maybe_unused]] const auto same_row =
+      m_pcur->restore_position(BTR_SEARCH_LEAF, m_mtr, UT_LOCATION_HERE);
 
-    /* It's possible that the restore places the cursor in the middle of
-    the block. We need to account for that too. */
+  /* Since m_rel_pos was BTR_PCUR_ON, the state can't be
+  BTR_PCUR_IS_POSITIONED_OPTIMISTIC */
+  ut_a(BTR_PCUR_IS_POSITIONED_OPTIMISTIC != m_pcur->m_pos_state);
 
-    if (m_pcur->is_on_user_rec()) {
-      return (DB_SUCCESS);
-    }
+  /* If the cursor was pointing to the user record then 'PAGE_CUR_LE' mode
+  used by the btr_pcur_t::restore_position() restores the cursor to a record
+  lower than the original one if it was removed, and in particular, if all
+  smaller ones were removed, cur could be on infimum now. This would be fine,
+  as our next step will be to advance the cursor, anyway.
+  However, right now the only usage of this method is in the bulk inserter
+  for a B-Tree. which only calls it when the current row was visible to the
+  scan, and thus can't be purged. So, we know it can't happen, unless someone
+  reuses this function in a new context, in which case they should probably
+  repeat the proof of correctness. The purpose of following debug assert is
+  to show the limited the scope of this method, and thus how risky it
+  is to use it in new context: */
+  ut_ad(same_row);
+}
+
+void PCursor::save_previous_user_record_as_last_processed() noexcept {
+  ut_a(m_pcur->is_after_last_on_page());
+  m_pcur->store_position(m_mtr);
+  ut_a(m_pcur->m_rel_pos == BTR_PCUR_AFTER);
+  m_mtr->commit();
+}
+
+void PCursor::restore_to_first_unprocessed() noexcept {
+  ut_a(m_pcur->m_rel_pos == BTR_PCUR_AFTER);
+  m_mtr->start();
+  m_mtr->set_log_mode(MTR_LOG_NO_REDO);
+  m_pcur->restore_position(BTR_SEARCH_LEAF, m_mtr, UT_LOCATION_HERE);
+  if (m_pcur->m_pos_state == BTR_PCUR_IS_POSITIONED_OPTIMISTIC) {
+    /* The BTR_PCUR_IS_POSITIONED_OPTIMISTIC means the implementation of
+    btr_pcur_t::restore_position() successfully re-acquired a block stored in
+    hint and it wasn't modified meanwhile, and in such case it does not advance
+    the cursor to the next position even though BTR_PCUR_AFTER was used, so we
+    have to do it ourselves. */
+    ut_a(!m_pcur->is_after_last_on_page());
+    m_pcur->move_to_next_on_page();
   }
+}
 
+dberr_t PCursor::move_to_user_rec() noexcept {
   auto cur = m_pcur->get_page_cur();
-  auto next_page_no = btr_page_get_next(page_cur_get_page(cur), m_mtr);
+  const auto next_page_no = btr_page_get_next(page_cur_get_page(cur), m_mtr);
 
   if (next_page_no == FIL_NULL) {
     m_mtr->commit();
-    return (DB_END_OF_INDEX);
+    return DB_END_OF_INDEX;
   }
 
   auto block = page_cur_get_block(cur);
   const auto &page_id = block->page.id;
 
-  block = buf_page_get_gen(page_id_t(page_id.space(), next_page_no),
-                           block->page.size, RW_S_LATCH, nullptr,
-                           Page_fetch::SCAN, __FILE__, __LINE__, m_mtr);
+  DEBUG_SYNC_C("parallel_reader_next_block");
+
+  /* We never scan undo tablespaces. */
+  ut_a(!fsp_is_undo_tablespace(page_id.space()));
+
+  if (m_read_level == 0) {
+    block = buf_page_get_gen(page_id_t(page_id.space(), next_page_no),
+                             block->page.size, RW_S_LATCH, nullptr,
+                             Page_fetch::SCAN, UT_LOCATION_HERE, m_mtr);
+  } else {
+    /* Read IO should be waited for. But s-latch should be nowait,
+    to avoid deadlock opportunity completely. */
+    block = buf_page_get_gen(page_id_t(page_id.space(), next_page_no),
+                             block->page.size, RW_NO_LATCH, nullptr,
+                             Page_fetch::SCAN, UT_LOCATION_HERE, m_mtr);
+    bool success = buf_page_get_known_nowait(
+        RW_S_LATCH, block, Cache_hint::KEEP_OLD, __FILE__, __LINE__, m_mtr);
+    btr_leaf_page_release(block, RW_NO_LATCH, m_mtr);
+
+    if (!success) {
+      return DB_LOCK_NOWAIT;
+    }
+  }
 
   buf_block_dbg_add_level(block, SYNC_TREE_NODE);
 
-  if (page_is_leaf(buf_block_get_frame(block))) {
-    btr_leaf_page_release(page_cur_get_block(cur), RW_S_LATCH, m_mtr);
-  }
+  btr_leaf_page_release(page_cur_get_block(cur), RW_S_LATCH, m_mtr);
 
   page_cur_set_before_first(block, cur);
 
@@ -343,7 +497,96 @@ dberr_t PCursor::move_to_next_block(dict_index_t *index) {
   /* Page can't be empty unless it is a root page. */
   ut_ad(!page_cur_is_after_last(cur));
 
-  return (DB_SUCCESS);
+  return DB_SUCCESS;
+}
+
+dberr_t PCursor::restore_from_savepoint() noexcept {
+  ut_ad(m_read_level != 0);
+  resume();
+  return m_pcur->is_on_user_rec() ? DB_SUCCESS : move_to_user_rec();
+}
+
+void Parallel_reader::Thread_ctx::
+    save_current_user_record_as_last_processed() noexcept {
+  m_pcursor->save_current_user_record_as_last_processed();
+}
+
+void Parallel_reader::Thread_ctx::
+    restore_to_last_processed_user_record() noexcept {
+  m_pcursor->restore_to_last_processed_user_record();
+}
+
+void Parallel_reader::Thread_ctx::
+    save_previous_user_record_as_last_processed() noexcept {
+  m_pcursor->save_previous_user_record_as_last_processed();
+}
+
+void Parallel_reader::Thread_ctx::restore_to_first_unprocessed() noexcept {
+  m_pcursor->restore_to_first_unprocessed();
+}
+
+dberr_t PCursor::move_to_next_block(dict_index_t *index) {
+  ut_ad(m_pcur->is_after_last_on_page());
+  dberr_t err = DB_SUCCESS;
+
+  if (DBUG_EVALUATE_IF("pcursor_move_to_next_block_release_latches", true,
+                       rw_lock_get_waiters(dict_index_get_lock(index)))) {
+    /* There are waiters on the index tree lock. Store and restore
+    the cursor position, and yield so that scanning a large table
+    will not starve other threads. */
+
+    /* We should always yield on a block boundary. */
+    ut_ad(m_pcur->is_after_last_on_page());
+
+    if (m_read_level == 0) {
+      save_previous_user_record_as_last_processed();
+      DEBUG_SYNC_C("pcursor_move_to_next_block_latches_released");
+      std::this_thread::yield();
+      restore_to_first_unprocessed();
+      if (m_pcur->is_after_last_on_page()) {
+        /* There's nothing wrong here, as restore_to_first_unprocessed() is not
+        obliged to position the cursor on a *user* record - it can be a
+        supremum. This can happen if
+        a) the block in BP wasn't modified and thus we've restored the cursor
+        exactly where it was, i.e. on supremum
+        b) there is no more records in the table
+        These options aren't mutually exclusive. Both are well handled by: */
+        return move_to_user_rec();
+      } else {
+        /* This happens if we were descending the B-tree, in which case we try
+        to navigate in such a way to arrive at a user record. */
+        ut_a(m_pcur->is_on_user_rec());
+        return DB_SUCCESS;
+      }
+    }
+    savepoint();
+
+    /* Yield so that another thread can proceed. */
+    std::this_thread::yield();
+
+    err = restore_from_savepoint();
+  } else {
+    err = move_to_user_rec();
+  }
+
+  int n_retries [[maybe_unused]] = 0;
+  while (err == DB_LOCK_NOWAIT) {
+    /* We should restore the cursor from index root page,
+    to avoid deadlock opportunity. */
+    ut_ad(m_read_level != 0);
+
+    savepoint();
+
+    /* Forces to restore from index root. */
+    m_pcur->m_block_when_stored.clear();
+
+    err = restore_from_savepoint();
+
+    n_retries++;
+    ut_ad(n_retries < 10);
+  }
+
+  return err;
 }
 
 bool Parallel_reader::Scan_ctx::check_visibility(const rec_t *&rec,
@@ -386,16 +629,6 @@ bool Parallel_reader::Scan_ctx::check_visibility(const rec_t *&rec,
     } else {
       /* Secondary index scan not supported yet. */
       ut_error;
-
-      auto max_trx_id = page_get_max_trx_id(page_align(rec));
-
-      ut_ad(max_trx_id > 0);
-
-      if (!view->sees(max_trx_id)) {
-        /* FIXME: This is not sufficient. We may need to read in the cluster
-        index record to be 100% sure. */
-        return (false);
-      }
     }
   }
 
@@ -406,14 +639,15 @@ bool Parallel_reader::Scan_ctx::check_visibility(const rec_t *&rec,
   }
 
   ut_ad(!m_trx || m_trx->isolation_level == TRX_ISO_READ_UNCOMMITTED ||
-        !rec_offs_any_null_extern(rec, offsets));
+        !rec_offs_any_null_extern(m_config.m_index, rec, offsets));
 
   return (true);
 }
 
 void Parallel_reader::Scan_ctx::copy_row(const rec_t *rec, Iter *iter) const {
-  iter->m_offsets = rec_get_offsets(rec, m_config.m_index, nullptr,
-                                    ULINT_UNDEFINED, &iter->m_heap);
+  iter->m_offsets =
+      rec_get_offsets(rec, m_config.m_index, nullptr, ULINT_UNDEFINED,
+                      UT_LOCATION_HERE, &iter->m_heap);
 
   /* Copy the row from the page to the scan iterator. The copy should use
   memory from the iterator heap because the scan iterator owns the copy. */
@@ -446,7 +680,8 @@ Parallel_reader::Scan_ctx::create_persistent_cursor(
 
   std::shared_ptr<Iter> iter = std::make_shared<Iter>();
 
-  iter->m_heap = mem_heap_create(sizeof(btr_pcur_t) + (srv_page_size / 16));
+  iter->m_heap = mem_heap_create(sizeof(btr_pcur_t) + (srv_page_size / 16),
+                                 UT_LOCATION_HERE);
 
   auto rec = page_cursor.rec;
 
@@ -477,8 +712,8 @@ Parallel_reader::Scan_ctx::create_persistent_cursor(
   iter->m_pcur->open_on_user_rec(page_cursor, PAGE_CUR_GE,
                                  BTR_ALREADY_S_LATCHED | BTR_SEARCH_LEAF);
 
-  ut_ad(btr_page_get_level(buf_block_get_frame(iter->m_pcur->get_block()),
-                           mtr) == m_config.m_read_level);
+  ut_ad(btr_page_get_level(buf_block_get_frame(iter->m_pcur->get_block())) ==
+        m_config.m_read_level);
 
   iter->m_pcur->store_position(mtr);
   iter->m_pcur->set_fetch_type(Page_fetch::SCAN);
@@ -486,22 +721,20 @@ Parallel_reader::Scan_ctx::create_persistent_cursor(
   return (iter);
 }
 
-bool Parallel_reader::Ctx::move_to_next_node(PCursor *pcursor, mtr_t *mtr) {
-  page_cur_t *cur MY_ATTRIBUTE((unused)) =
-      m_range.first->m_pcur->get_page_cur();
+bool Parallel_reader::Ctx::move_to_next_node(PCursor *pcursor) {
+  IF_DEBUG(auto cur = m_range.first->m_pcur->get_page_cur();)
 
   auto err = pcursor->move_to_next_block(const_cast<dict_index_t *>(index()));
 
   if (err != DB_SUCCESS) {
     ut_a(err == DB_END_OF_INDEX);
-    return (false);
+    return false;
+  } else {
+    /* Page can't be empty unless it is a root page. */
+    ut_ad(!page_cur_is_after_last(cur));
+    ut_ad(!page_cur_is_before_first(cur));
+    return true;
   }
-
-  /* Page can't be empty unless it is a root page. */
-  ut_ad(!page_cur_is_after_last(cur));
-  ut_ad(!page_cur_is_before_first(cur));
-
-  return (true);
 }
 
 dberr_t Parallel_reader::Ctx::traverse() {
@@ -522,12 +755,16 @@ dberr_t Parallel_reader::Ctx::traverse() {
 
   dberr_t err{DB_SUCCESS};
 
+  m_thread_ctx->m_pcursor = &pcursor;
+
   err = traverse_recs(&pcursor, &mtr);
 
   if (mtr.is_active()) {
     mtr.commit();
   }
 
+  m_thread_ctx->m_pcursor = nullptr;
+
   if (m_scan_ctx->m_config.m_read_level != 0) {
     m_scan_ctx->index_s_unlock();
   }
@@ -537,16 +774,35 @@ dberr_t Parallel_reader::Ctx::traverse() {
 
 dberr_t Parallel_reader::Ctx::traverse_recs(PCursor *pcursor, mtr_t *mtr) {
   const auto &end_tuple = m_range.second->m_tuple;
-  auto heap = mem_heap_create(srv_page_size / 4);
+  auto heap = mem_heap_create(srv_page_size / 4, UT_LOCATION_HERE);
   auto index = m_scan_ctx->m_config.m_index;
-  auto cur = m_range.first->m_pcur->get_page_cur();
 
   m_start = true;
 
   dberr_t err{DB_SUCCESS};
 
+  if (m_scan_ctx->m_reader->m_start_callback) {
+    /* Page start. */
+    m_thread_ctx->m_state = State::PAGE;
+    err = m_scan_ctx->m_reader->m_start_callback(m_thread_ctx);
+  }
+
+  bool call_end_page{true};
+  auto cur = pcursor->get_page_cursor();
+
   while (err == DB_SUCCESS) {
     if (page_cur_is_after_last(cur)) {
+      call_end_page = false;
+
+      if (m_scan_ctx->m_reader->m_finish_callback) {
+        /* End of page. */
+        m_thread_ctx->m_state = State::PAGE;
+        err = m_scan_ctx->m_reader->m_finish_callback(m_thread_ctx);
+        if (err != DB_SUCCESS) {
+          break;
+        }
+      }
+
       mem_heap_empty(heap);
 
       if (!(m_n_pages % TRX_IS_INTERRUPTED_PROBE) &&
@@ -555,12 +811,29 @@ dberr_t Parallel_reader::Ctx::traverse_recs(PCursor *pcursor, mtr_t *mtr) {
         break;
       }
 
-      if (is_error_set() || !move_to_next_node(pcursor, mtr)) {
+      if (is_error_set()) {
+        break;
+      }
+
+      /* Note: The page end callback (above) can save and restore the cursor.
+      The restore can end up in the middle of a page. */
+      if (pcursor->is_after_last_on_page() && !move_to_next_node(pcursor)) {
         break;
       }
 
       ++m_n_pages;
       m_first_rec = true;
+
+      call_end_page = true;
+
+      if (m_scan_ctx->m_reader->m_start_callback) {
+        /* Page start. */
+        m_thread_ctx->m_state = State::PAGE;
+        err = m_scan_ctx->m_reader->m_start_callback(m_thread_ctx);
+        if (err != DB_SUCCESS) {
+          break;
+        }
+      }
     }
 
     ulint offsets_[REC_OFFS_NORMAL_SIZE];
@@ -569,18 +842,19 @@ dberr_t Parallel_reader::Ctx::traverse_recs(PCursor *pcursor, mtr_t *mtr) {
     rec_offs_init(offsets_);
 
     const rec_t *rec = page_cur_get_rec(cur);
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
+                              UT_LOCATION_HERE, &heap);
 
     if (end_tuple != nullptr) {
       ut_ad(rec != nullptr);
 
       /* Key value of a record can change only if the record is deleted or if
-      its updated. And an update is essentially a delete + insert. So in both
+      it's updated. An update is essentially a delete + insert. So in both
       the cases we just delete mark the record and the original key value is
       preserved on the page.
 
       Since the range creation is based on the key values and the key value do
-      not ever change the latest (non-MVCC) version of the record should always
+      not ever change the, latest (non-MVCC) version of the record should always
       tell us correctly whether we're within the range or outside of it. */
       auto ret = end_tuple->compare(rec, index, offsets);
 
@@ -599,11 +873,15 @@ dberr_t Parallel_reader::Ctx::traverse_recs(PCursor *pcursor, mtr_t *mtr) {
 
     if (!skip) {
       m_rec = rec;
-      m_block = cur->block;
       m_offsets = offsets;
+      m_block = cur->block;
 
       err = m_scan_ctx->m_f(this);
 
+      if (err != DB_SUCCESS) {
+        break;
+      }
+
       m_start = false;
     }
 
@@ -616,10 +894,18 @@ dberr_t Parallel_reader::Ctx::traverse_recs(PCursor *pcursor, mtr_t *mtr) {
     m_scan_ctx->set_error_state(err);
   }
 
-  m_thread_ctx->m_prev_partition_id = partition_id();
-
   mem_heap_free(heap);
 
+  if (call_end_page && m_scan_ctx->m_reader->m_finish_callback) {
+    /* Page finished. */
+    m_thread_ctx->m_state = State::PAGE;
+    auto cb_err = m_scan_ctx->m_reader->m_finish_callback(m_thread_ctx);
+
+    if (cb_err != DB_SUCCESS && !m_scan_ctx->is_error_set()) {
+      err = cb_err;
+    }
+  }
+
   return (err);
 }
 
@@ -654,22 +940,32 @@ bool Parallel_reader::is_queue_empty() const {
 
 void Parallel_reader::worker(Parallel_reader::Thread_ctx *thread_ctx) {
   dberr_t err{DB_SUCCESS};
+  dberr_t cb_err{DB_SUCCESS};
 
   if (m_start_callback) {
-    err = m_start_callback(thread_ctx);
+    /* Thread start. */
+    thread_ctx->m_state = State::THREAD;
+    cb_err = m_start_callback(thread_ctx);
+
+    if (cb_err != DB_SUCCESS) {
+      err = cb_err;
+      set_error_state(cb_err);
+    }
   }
 
   /* Wait for all the threads to be spawned as it's possible that we could
   abort the operation if there are not enough resources to spawn all the
   threads. */
-  constexpr auto FOREVER = OS_SYNC_INFINITE_TIME;
-  os_event_wait_time_low(m_event, FOREVER, m_sig_count);
+  if (!m_sync) {
+    os_event_wait_time_low(m_event, std::chrono::microseconds::max(),
+                           m_sig_count);
+  }
 
   for (;;) {
-    size_t n_completed = 0;
+    size_t n_completed{};
     int64_t sig_count = os_event_reset(m_event);
 
-    while (err == DB_SUCCESS && !is_error_set()) {
+    while (err == DB_SUCCESS && cb_err == DB_SUCCESS && !is_error_set()) {
       auto ctx = dequeue();
 
       if (ctx == nullptr) {
@@ -689,7 +985,21 @@ void Parallel_reader::worker(Parallel_reader::Thread_ctx *thread_ctx) {
         /* Tell the other threads that there is work to do. */
         os_event_set(m_event);
       } else {
-        err = ctx->traverse();
+        if (m_start_callback) {
+          /* Context start. */
+          thread_ctx->m_state = State::CTX;
+          cb_err = m_start_callback(thread_ctx);
+        }
+
+        if (cb_err == DB_SUCCESS && err == DB_SUCCESS) {
+          err = ctx->traverse();
+        }
+
+        if (m_finish_callback) {
+          /* Context finished. */
+          thread_ctx->m_state = State::CTX;
+          cb_err = m_finish_callback(thread_ctx);
+        }
       }
 
       /* Check for trx interrupted (useful in the case of small tables). */
@@ -704,7 +1014,7 @@ void Parallel_reader::worker(Parallel_reader::Thread_ctx *thread_ctx) {
       ++n_completed;
     }
 
-    if (err != DB_SUCCESS || is_error_set()) {
+    if (cb_err != DB_SUCCESS || err != DB_SUCCESS || is_error_set()) {
       break;
     }
 
@@ -716,16 +1026,15 @@ void Parallel_reader::worker(Parallel_reader::Thread_ctx *thread_ctx) {
       break;
     }
 
-    constexpr auto FOREVER = OS_SYNC_INFINITE_TIME;
-
-    os_event_wait_time_low(m_event, FOREVER, sig_count);
+    if (!m_sync) {
+      os_event_wait_time_low(m_event, std::chrono::microseconds::max(),
+                             sig_count);
+    }
   }
 
-  if (err != DB_SUCCESS) {
+  if (err != DB_SUCCESS && !is_error_set()) {
     /* Set the "global" error state. */
-    if (!is_error_set()) {
-      set_error_state(err);
-    }
+    set_error_state(err);
   }
 
   if (is_error_set()) {
@@ -734,10 +1043,14 @@ void Parallel_reader::worker(Parallel_reader::Thread_ctx *thread_ctx) {
   }
 
   if (m_finish_callback) {
-    dberr_t finish_err = m_finish_callback(thread_ctx);
+    /* Thread finished. */
+    thread_ctx->m_state = State::THREAD;
+    cb_err = m_finish_callback(thread_ctx);
 
-    if (finish_err != DB_SUCCESS) {
-      set_error_state(finish_err);
+    /* Keep the err status from previous failed operations */
+    if (cb_err != DB_SUCCESS) {
+      err = cb_err;
+      set_error_state(cb_err);
     }
   }
 
@@ -770,7 +1083,8 @@ page_no_t Parallel_reader::Scan_ctx::search(const buf_block_t *block,
 
   rec_offs_init(offsets_);
 
-  offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
+  offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
+                            UT_LOCATION_HERE, &heap);
 
   auto page_no = btr_node_ptr_get_child_page_no(rec, offsets);
 
@@ -796,7 +1110,7 @@ page_cur_t Parallel_reader::Scan_ctx::start_range(
 
     auto block = block_get_s_latched(page_id, mtr, __LINE__);
 
-    height = btr_page_get_level(buf_block_get_frame(block), mtr);
+    height = btr_page_get_level(buf_block_get_frame(block));
 
     savepoints.push_back({savepoint, block});
 
@@ -819,10 +1133,6 @@ page_cur_t Parallel_reader::Scan_ctx::start_range(
 
     return (page_cursor);
   }
-
-  ut_error;
-
-  return (page_cur_t{});
 }
 
 void Parallel_reader::Scan_ctx::create_range(Ranges &ranges,
@@ -847,7 +1157,6 @@ dberr_t Parallel_reader::Scan_ctx::create_ranges(const Scan_range &scan_range,
                                                  const size_t split_level,
                                                  Ranges &ranges, mtr_t *mtr) {
   ut_ad(index_s_own());
-  ut_a(max_threads() > 0);
   ut_a(page_no != FIL_NULL);
 
   /* Do a breadth first traversal of the B+Tree using recursion. We want to
@@ -867,7 +1176,7 @@ dberr_t Parallel_reader::Scan_ctx::create_ranges(const Scan_range &scan_range,
 
   /* read_level requested should be less than the tree height. */
   ut_ad(m_config.m_read_level <
-        btr_page_get_level(buf_block_get_frame(block), mtr) + 1);
+        btr_page_get_level(buf_block_get_frame(block)) + 1);
 
   savepoint.second = block;
 
@@ -899,7 +1208,7 @@ dberr_t Parallel_reader::Scan_ctx::create_ranges(const Scan_range &scan_range,
   mem_heap_t *heap{};
 
   const auto at_leaf = page_is_leaf(buf_block_get_frame(block));
-  const auto at_level = btr_page_get_level(buf_block_get_frame(block), mtr);
+  const auto at_level = btr_page_get_level(buf_block_get_frame(block));
 
   Savepoints savepoints{};
 
@@ -910,10 +1219,11 @@ dberr_t Parallel_reader::Scan_ctx::create_ranges(const Scan_range &scan_range,
          !dict_table_is_comp(index->table));
 
     if (heap == nullptr) {
-      heap = mem_heap_create(srv_page_size / 4);
+      heap = mem_heap_create(srv_page_size / 4, UT_LOCATION_HERE);
     }
 
-    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED, &heap);
+    offsets = rec_get_offsets(rec, index, offsets, ULINT_UNDEFINED,
+                              UT_LOCATION_HERE, &heap);
 
     const auto end = scan_range.m_end;
 
@@ -1016,7 +1326,8 @@ dberr_t Parallel_reader::Scan_ctx::partition(
 
     ut_a(iter->m_heap == nullptr);
 
-    iter->m_heap = mem_heap_create(sizeof(btr_pcur_t) + (srv_page_size / 16));
+    iter->m_heap = mem_heap_create(sizeof(btr_pcur_t) + (srv_page_size / 16),
+                                   UT_LOCATION_HERE);
 
     iter->m_tuple = dtuple_copy(scan_range.m_end, iter->m_heap);
 
@@ -1038,8 +1349,8 @@ dberr_t Parallel_reader::Scan_ctx::create_context(const Range &range,
   // clang-format off
 
   auto ctx = std::shared_ptr<Ctx>(
-      UT_NEW_NOKEY(Ctx(ctx_id, this, range)),
-      [](Ctx *ctx) { UT_DELETE(ctx); });
+      ut::new_withkey<Ctx>(UT_NEW_THIS_FILE_PSI_KEY, ctx_id, this, range),
+      [](Ctx *ctx) { ut::delete_(ctx); });
 
   // clang-format on
 
@@ -1059,15 +1370,18 @@ dberr_t Parallel_reader::Scan_ctx::create_context(const Range &range,
 dberr_t Parallel_reader::Scan_ctx::create_contexts(const Ranges &ranges) {
   size_t split_point{};
 
-  ut_a(max_threads() > 0 &&
-       max_threads() <= Parallel_reader::MAX_TOTAL_THREADS);
+  {
+    const auto n = std::max(max_threads(), size_t{1});
 
-  if (ranges.size() > max_threads()) {
-    split_point = (ranges.size() / max_threads()) * max_threads();
-  } else if (m_depth < SPLIT_THRESHOLD) {
-    /* If the tree is not very deep then don't split. For smaller tables
-    it is more expensive to split because we end up traversing more blocks*/
-    split_point = max_threads();
+    ut_a(n <= Parallel_reader::MAX_TOTAL_THREADS);
+
+    if (ranges.size() > n) {
+      split_point = (ranges.size() / n) * n;
+    } else if (m_depth < SPLIT_THRESHOLD) {
+      /* If the tree is not very deep then don't split. For smaller tables
+      it is more expensive to split because we end up traversing more blocks*/
+      split_point = n;
+    }
   }
 
   size_t i{};
@@ -1082,7 +1396,7 @@ dberr_t Parallel_reader::Scan_ctx::create_contexts(const Ranges &ranges) {
     ++i;
   }
 
-  return (DB_SUCCESS);
+  return DB_SUCCESS;
 }
 
 void Parallel_reader::parallel_read() {
@@ -1090,8 +1404,8 @@ void Parallel_reader::parallel_read() {
     return;
   }
 
-  if (m_n_threads == 0) {
-    auto ptr = UT_NEW_NOKEY(Thread_ctx{0});
+  if (m_sync) {
+    auto ptr = ut::new_withkey<Thread_ctx>(UT_NEW_THIS_FILE_PSI_KEY, 0);
 
     if (ptr == nullptr) {
       set_error_state(DB_OUT_OF_MEMORY);
@@ -1108,21 +1422,23 @@ void Parallel_reader::parallel_read() {
     return;
   }
 
+  ut_a(m_n_threads > 0);
+
   m_thread_ctxs.reserve(m_n_threads);
 
   dberr_t err{DB_SUCCESS};
 
   for (size_t i = 0; i < m_n_threads; ++i) {
     try {
-      auto ptr = UT_NEW_NOKEY(Thread_ctx{i});
+      auto ptr = ut::new_withkey<Thread_ctx>(UT_NEW_THIS_FILE_PSI_KEY, i);
       if (ptr == nullptr) {
         set_error_state(DB_OUT_OF_MEMORY);
         return;
       }
       m_thread_ctxs.emplace_back(ptr);
       m_parallel_read_threads.emplace_back(
-          os_thread_create(parallel_read_thread_key, &Parallel_reader::worker,
-                           this, m_thread_ctxs[i]));
+          os_thread_create(parallel_read_thread_key, i + 1,
+                           &Parallel_reader::worker, this, m_thread_ctxs[i]));
       m_parallel_read_threads.back().start();
     } catch (...) {
       err = DB_OUT_OF_RESOURCES;
@@ -1134,56 +1450,67 @@ void Parallel_reader::parallel_read() {
 
   DEBUG_SYNC_C("parallel_read_wait_for_kill_query");
 
-  DBUG_EXECUTE_IF("innodb_pread_thread_OOR", err = DB_OUT_OF_RESOURCES;
-                  set_error_state(err););
+  DBUG_EXECUTE_IF(
+      "innodb_pread_thread_OOR", if (!m_sync) {
+        err = DB_OUT_OF_RESOURCES;
+        set_error_state(err);
+      });
 
   os_event_set(m_event);
 
   DBUG_EXECUTE_IF("bug28079850", set_error_state(DB_INTERRUPTED););
+}
 
-  /* Don't wait for the threads to finish if the read is not synchronous. */
-  if (!m_sync) {
-    return;
+dberr_t Parallel_reader::spawn(size_t n_threads) noexcept {
+  /* In case this is a retry after a DB_OUT_OF_RESOURCES error. */
+  m_err = DB_SUCCESS;
+
+  m_n_threads = n_threads;
+
+  if (max_threads() > m_n_threads) {
+    release_unused_threads(max_threads() - m_n_threads);
   }
 
-  join();
+  parallel_read();
+
+  return DB_SUCCESS;
 }
 
 dberr_t Parallel_reader::run(size_t n_threads) {
-  if (!m_scan_ctxs.empty()) {
-    if (n_threads == 0) {
-      n_threads = std::min(m_max_threads, m_ctxs.size());
+  /* In case this is a retry after a DB_OUT_OF_RESOURCES error. */
+  m_err = DB_SUCCESS;
 
-      /* No need to spawn any threads if only one thread is required. */
-      if (n_threads == 1 || m_single_threaded_mode) {
-        n_threads = 0;
+  ut_a(max_threads() >= n_threads);
+
+  if (n_threads == 0) {
+    m_sync = true;
+  }
+
+  const auto err = spawn(n_threads);
+
+  /* Don't wait for the threads to finish if the read is not synchronous or
+  if there's no parallel read. */
+  if (m_sync) {
+    if (err != DB_SUCCESS) {
+      return err;
+    }
+    ut_a(m_n_threads == 0);
+    return is_error_set() ? m_err.load() : DB_SUCCESS;
+  } else {
+    join();
+
+    if (err != DB_SUCCESS) {
+      return err;
+    } else if (is_error_set()) {
+      return m_err;
+    }
+
+    for (auto &scan_ctx : m_scan_ctxs) {
+      if (scan_ctx->is_error_set()) {
+        /* Return the state of the first Scan context that is in state ERROR. */
+        return scan_ctx->m_err;
       }
     }
-
-    m_n_threads = n_threads;
-
-    if (!m_single_threaded_mode) {
-      release_unused_threads(m_max_threads - m_n_threads);
-    }
-
-    parallel_read();
-  }
-
-  /* Don't wait for the threads to finish if the read is not synchronous or if
-  there's no parallel read. */
-  if (!m_sync || m_n_threads == 0) {
-    return DB_SUCCESS;
-  }
-
-  if (is_error_set()) {
-    return m_err;
-  }
-
-  for (auto &scan_ctx : m_scan_ctxs) {
-    if (scan_ctx->is_error_set()) {
-      /* Return the state of the first Scan context that is in state ERROR. */
-      return scan_ctx->m_err;
-    }
   }
 
   return DB_SUCCESS;
@@ -1195,8 +1522,8 @@ dberr_t Parallel_reader::add_scan(trx_t *trx,
   // clang-format off
 
   auto scan_ctx = std::shared_ptr<Scan_ctx>(
-      UT_NEW_NOKEY(Scan_ctx(this, m_scan_ctx_id, trx, config, std::move(f))),
-      [](Scan_ctx *scan_ctx) { UT_DELETE(scan_ctx); });
+      ut::new_withkey<Scan_ctx>(UT_NEW_THIS_FILE_PSI_KEY, this, m_scan_ctx_id, trx, config, std::move(f)),
+      [](Scan_ctx *scan_ctx) { ut::delete_(scan_ctx); });
 
   // clang-format on
 
