@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -64,7 +65,7 @@ Tablespace_pool *tbsp_pool = nullptr;
 /* Directory to store session temporary tablespaces, provided by user */
 char *srv_temp_dir = nullptr;
 
-/** Sesssion Temporary tablespace */
+/** Session Temporary tablespace */
 Tablespace::Tablespace()
     : m_space_id(++m_last_used_space_id), m_inited(), m_thread_id() {
   ut_ad(m_space_id <= dict_sys_t::s_max_temp_space_id);
@@ -90,7 +91,7 @@ Tablespace::~Tablespace() {
     ib::error(ER_IB_FAILED_TO_DELETE_TABLESPACE_FILE)
         << "Failed to delete file " << path();
     os_file_get_last_error(true);
-    ut_ad(0);
+    ut_d(ut_error);
   }
 }
 
@@ -119,8 +120,7 @@ dberr_t Tablespace::create() {
 
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
 
-  bool ret =
-      fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr, false);
+  bool ret = fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr);
   mtr_commit(&mtr);
 
   if (!ret) {
@@ -156,7 +156,7 @@ bool Tablespace::truncate() {
 
   mtr_start(&mtr);
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
-  fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr, false);
+  fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr);
   mtr_commit(&mtr);
 
   release();
@@ -191,7 +191,7 @@ void Tablespace::decrypt() {
   (zero-filled) to initial size. Just make sure that in-memory
   tablespace structure (fil_space_t) doesn't have encryption info */
 
-  rw_lock_x_lock(&space->latch);
+  rw_lock_x_lock(&space->latch, UT_LOCATION_HERE);
   /* Reset In-mem encryption for tablespace */
 
   /* fil_space_t of session temp tablespace will be always found and
@@ -203,20 +203,6 @@ void Tablespace::decrypt() {
 }
 
 void Tablespace::rotate_encryption_key() {
-  if (!is_encrypted()) {
-    return;
-  }
-
-  acquire();
-
-  fil_space_t *space = fil_space_get(m_space_id);
-
-  bool success = encryption_rotate_low(space);
-  if (!success) {
-    ib::warn(ER_XB_MSG_6, space->name);
-  }
-
-  release();
 }
 
 uint32_t Tablespace::file_id() const {
@@ -252,14 +238,14 @@ Tablespace_pool::Tablespace_pool(size_t init_size)
 Tablespace_pool::~Tablespace_pool() {
   mutex_destroy(&m_mutex);
   for (Tablespace *ts : *m_active) {
-    UT_DELETE(ts);
+    ut::delete_(ts);
   }
 
   for (Tablespace *ts : *m_free) {
-    UT_DELETE(ts);
+    ut::delete_(ts);
   }
-  UT_DELETE(m_active);
-  UT_DELETE(m_free);
+  ut::delete_(m_active);
+  ut::delete_(m_free);
 }
 
 Tablespace *Tablespace_pool::get(my_thread_id id, enum tbsp_purpose purpose) {
@@ -312,7 +298,7 @@ void Tablespace_pool::free_ts(Tablespace *ts) {
   if (it != m_active->end()) {
     m_active->erase(it);
   } else {
-    ut_ad(0);
+    ut_d(ut_error);
   }
 
   ts->reset_thread_id_and_purpose();
@@ -328,12 +314,12 @@ dberr_t Tablespace_pool::initialize(bool create_new_db) {
 
   ut_ad(m_active == nullptr && m_free == nullptr);
 
-  m_active = UT_NEW_NOKEY(Pool());
+  m_active = ut::new_withkey<Pool>(UT_NEW_THIS_FILE_PSI_KEY);
   if (m_active == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
 
-  m_free = UT_NEW_NOKEY(Pool());
+  m_free = ut::new_withkey<Pool>(UT_NEW_THIS_FILE_PSI_KEY);
   if (m_free == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
@@ -351,10 +337,10 @@ dberr_t Tablespace_pool::initialize(bool create_new_db) {
 dberr_t Tablespace_pool::expand(size_t size) {
   ut_ad(!m_pool_initialized || mutex_own(&m_mutex));
   for (size_t i = 0; i < size; i++) {
-    Tablespace *ts = UT_NEW_NOKEY(Tablespace());
+    Tablespace *ts = ut::new_withkey<Tablespace>(UT_NEW_THIS_FILE_PSI_KEY);
 
     if (ts == nullptr) {
-      return (DB_OUT_OF_MEMORY);
+      return DB_OUT_OF_MEMORY;
     }
 
     dberr_t err = ts->create();
@@ -362,11 +348,11 @@ dberr_t Tablespace_pool::expand(size_t size) {
     if (err == DB_SUCCESS) {
       m_free->push_back(ts);
     } else {
-      UT_DELETE(ts);
-      return (err);
+      ut::delete_(ts);
+      return err;
     }
   }
-  return (DB_SUCCESS);
+  return DB_SUCCESS;
 }
 
 void Tablespace_pool::delete_old_pool(bool create_new_db) {
@@ -442,7 +428,8 @@ dberr_t open_or_create(bool create_new_db) {
     return (err);
   }
 
-  tbsp_pool = UT_NEW_NOKEY(Tablespace_pool(INIT_SIZE));
+  tbsp_pool =
+      ut::new_withkey<Tablespace_pool>(UT_NEW_THIS_FILE_PSI_KEY, INIT_SIZE);
   if (tbsp_pool == nullptr) {
     return (DB_OUT_OF_MEMORY);
   }
@@ -454,11 +441,13 @@ dberr_t open_or_create(bool create_new_db) {
 
 void free_tmp(Tablespace *ts) { tbsp_pool->free_ts(ts); }
 
-void delete_pool_manager() { UT_DELETE(tbsp_pool); }
+void delete_pool_manager() { ut::delete_(tbsp_pool); }
 
 void close_files() {
   auto close = [&](const ibt::Tablespace *ts) { ts->close(); };
 
+  if (!ibt::tbsp_pool) return;
+
   ibt::tbsp_pool->iterate_tbsp(close);
 }
 
