@@ -1,18 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2021, Oracle and/or its affiliates.
-Copyright (c) 2016, Percona Inc. All Rights Reserved.
+Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -36,6 +36,9 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "my_compiler.h"
 #include "ut0new.h"
 
+/** System-default huge (large) page setting. */
+const size_t large_page_default_size = ut::detail::large_page_size();
+
 /** Maximum number of retries to allocate memory. */
 const size_t alloc_max_retries = 60;
 
@@ -51,10 +54,11 @@ PSI_memory_key mem_key_dict_stats_bg_recalc_pool_t;
 PSI_memory_key mem_key_dict_stats_index_map_t;
 PSI_memory_key mem_key_dict_stats_n_diff_on_level;
 PSI_memory_key mem_key_fil_space_t;
+PSI_memory_key mem_key_lock_sys;
 PSI_memory_key mem_key_other;
 PSI_memory_key mem_key_partitioning;
 PSI_memory_key mem_key_row_log_buf;
-PSI_memory_key mem_key_row_merge_sort;
+PSI_memory_key mem_key_ddl;
 PSI_memory_key mem_key_std;
 PSI_memory_key mem_key_trx_sys_t_rw_trx_ids;
 PSI_memory_key mem_key_undo_spaces;
@@ -98,6 +102,7 @@ static PSI_memory_info pfs_info[] = {
     {&mem_key_dict_stats_n_diff_on_level, "dict_stats_n_diff_on_level", 0, 0,
      PSI_DOCUMENT_ME},
     {&mem_key_fil_space_t, "fil_space_t", 0, 0, PSI_DOCUMENT_ME},
+    {&mem_key_lock_sys, "lock_sys", 0, 0, PSI_DOCUMENT_ME},
     {&mem_key_log_online_iterator_files, "log_online_iterator_files", 0, 0,
      PSI_DOCUMENT_ME},
     {&mem_key_log_online_iterator_page, "log_online_iterator_page", 0, 0,
@@ -110,7 +115,7 @@ static PSI_memory_info pfs_info[] = {
     {&mem_key_other, "other", 0, 0, PSI_DOCUMENT_ME},
     {&mem_key_partitioning, "partitioning", 0, 0, PSI_DOCUMENT_ME},
     {&mem_key_row_log_buf, "row_log_buf", 0, 0, PSI_DOCUMENT_ME},
-    {&mem_key_row_merge_sort, "row_merge_sort", 0, 0, PSI_DOCUMENT_ME},
+    {&mem_key_ddl, "ddl", 0, 0, PSI_DOCUMENT_ME},
     {&mem_key_std, "std", 0, 0, PSI_DOCUMENT_ME},
     {&mem_key_trx_sys_t_rw_trx_ids, "trx_sys_t::rw_trx_ids", 0, 0,
      PSI_DOCUMENT_ME},
@@ -125,8 +130,8 @@ PSI_memory_info pfs_info_auto[n_auto];
 
 #endif /* UNIV_PFS_MEMORY */
 
-/** Setup the internal objects needed for UT_NEW() to operate.
-This must be called before the first call to UT_NEW(). */
+/** Setup the internal objects needed for ut::new_withkey() to operate.
+This must be called before the first call to ut::new_withkey(). */
 void ut_new_boot() {
 #ifdef UNIV_PFS_MEMORY
   for (size_t i = 0; i < n_auto; i++) {
