@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -636,7 +637,7 @@ class NdbScanOperation : public NdbOperation {
   int send_next_scan(Uint32 cnt, bool close);
   void receiver_delivered(NdbReceiver *);
   void receiver_completed(NdbReceiver *);
-  void execCLOSE_SCAN_REP();
+  void execCLOSE_SCAN_REP(Uint32 errorCode, bool needClose);
 
   int getKeyFromKEYINFO20(Uint32 *data, Uint32 &size);
   NdbOperation *takeOverScanOp(OperationType opType, NdbTransaction *);
@@ -718,6 +719,13 @@ class NdbScanOperation : public NdbOperation {
    */
   NdbBlob *getBlobHandle(const char *anAttrName) const override;
   NdbBlob *getBlobHandle(Uint32 anAttrId) const override;
+
+ protected:
+  /*
+    Owned by the receiver thread
+    Read by API thread under mutex protection
+  */
+  Uint32 m_kernel_error_code;
 };
 
 inline NdbOperation *NdbScanOperation::lockCurrentTuple() {
