@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1137,9 +1138,10 @@ class NdbTransaction {
                                      // Current meanings :
                                      //   0x00FE11DC : NdbTransaction not in use
                                      //   0x37412619 : NdbTransaction in use
-                          //   0x00FE11DF : NdbTransaction for scan operation
-                          //                scan definition not yet complete
-  Uint32 thePriority;  // Transaction Priority
+                                     //   0x00FE11DF : NdbTransaction for scan
+                                     //                operation scan definition
+                                     //                not yet complete
+  Uint32 thePriority;                // Transaction Priority
 
   enum ReturnType { ReturnSuccess, ReturnFailure };
   ReturnType theReturnStatus;  // Did we have any read/update/delete failing
@@ -1155,9 +1157,10 @@ class NdbTransaction {
     InCompletedList
   } theListState;
 
-  Uint32 theDBnode;        // The database node we are connected to
-  Uint32 theNodeSequence;  // The sequence no of the db node
-  bool theReleaseOnClose;
+  Uint32 theDBnode;             // The database node we are connected to
+  Uint32 theNodeSequence;       // The sequence no of the db node
+  bool theReleaseOnClose;       // Data node gone, recycle API object
+  bool theForceReleaseOnClose;  // Data node out of sync, force TC release
 
   /**
    * handle transaction spanning
