@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -104,6 +105,44 @@ DECLARE_NDBINFO_TABLE(POOLS, 14) = {
      {"resource_id", Ndbinfo::Number, ""},
      {"type_id", Ndbinfo::Number, "Record type id within resource"}}};
 
+DECLARE_NDBINFO_TABLE(TRANSPORTER_DETAILS, 19) = {
+    {"transporter_details", 19, 0,
+     [](const Ndbinfo::Counts &counts) {
+       return (counts.data_nodes) * (counts.all_nodes - 1);
+     },
+     "detailed transporter status"},
+    {{"node_id", Ndbinfo::Number, "Node id reporting"},
+     {"block_instance", Ndbinfo::Number, "Block instance reporting"},
+     {"trp_id", Ndbinfo::Number, "Transporter id"},
+     {"remote_node_id", Ndbinfo::Number, "Node id at other end of link"},
+
+     {"connection_status", Ndbinfo::Number, "State of inter-node link"},
+
+     {"remote_address", Ndbinfo::String, "Address of remote node"},
+     {"bytes_sent", Ndbinfo::Number64, "Bytes sent to remote node"},
+     {"bytes_received", Ndbinfo::Number64, "Bytes received from remote node"},
+
+     {"connect_count", Ndbinfo::Number, "Number of times connected"},
+
+     {"overloaded", Ndbinfo::Number, "Is link reporting overload"},
+     {"overload_count", Ndbinfo::Number,
+      "Number of overload onsets since connect"},
+
+     {"slowdown", Ndbinfo::Number, "Is link requesting slowdown"},
+     {"slowdown_count", Ndbinfo::Number,
+      "Number of slowdown onsets since connect"},
+     {"encrypted", Ndbinfo::Number, "Is link using TLS encryption"},
+
+     {"sendbuffer_used_bytes", Ndbinfo::Number64, "SendBuffer bytes in use"},
+     {"sendbuffer_max_used_bytes", Ndbinfo::Number64,
+      "SendBuffer historical max bytes in use"},
+     {"sendbuffer_alloc_bytes", Ndbinfo::Number64,
+      "SendBuffer bytes allocated"},
+     {"sendbuffer_max_alloc_bytes", Ndbinfo::Number64,
+      "SendBuffer historical max bytes allocated"},
+
+     {"type", Ndbinfo::Number, "Transporter type"}}};
+
 DECLARE_NDBINFO_TABLE(TRANSPORTERS, 11) = {
     {"transporters", 11, 0,
      [](const Ndbinfo::Counts &counts) {
@@ -1163,6 +1202,33 @@ DECLARE_NDBINFO_TABLE(CPUDATA_20SEC, 10) = {
          "Elapsed time in microseconds for measurement"},
     }};
 
+/* Transactions_full == Transactions schema */
+DECLARE_NDBINFO_TABLE(TRANSACTIONS_FULL, 11) = {
+    {"transactions_full", 11, 0,
+     [](const Ndbinfo::Counts &) {
+       /* It is difficult to estimate row counts for transactions, operations,
+          and acc_operations because they depend on current load. By guessing
+          5 transactions, 10 operations, and 15 acc_operations, we can keep the
+          three tables in correct relative order and allow the optimizer to
+          correctly rank them from largest to smallest most of the time.
+       */
+       return 5;
+     },
+     "transactions_full"},
+    {
+        {"node_id", Ndbinfo::Number, "node id"},
+        {"block_instance", Ndbinfo::Number, "TC instance no"},
+        {"objid", Ndbinfo::Number, "Object id of transaction object"},
+        {"apiref", Ndbinfo::Number, "API reference"},
+        {"transid0", Ndbinfo::Number, "Transaction id"},
+        {"transid1", Ndbinfo::Number, "Transaction id"},
+        {"state", Ndbinfo::Number, "Transaction state"},
+        {"flags", Ndbinfo::Number, "Transaction flags"},
+        {"c_ops", Ndbinfo::Number, "No of operations in transaction"},
+        {"outstanding", Ndbinfo::Number, "Currently outstanding request"},
+        {"timer", Ndbinfo::Number, "Timer (seconds)"},
+    }};
+
 #define DBINFOTBL(x) \
   { Ndbinfo::x##_TABLEID, (const Ndbinfo::Table *)&ndbinfo_##x }
 
@@ -1226,7 +1292,8 @@ static struct ndbinfo_table_list_entry {
     DBINFOTBL(CPUDATA_20SEC),
     DBINFOTBL_UNSUPPORTED(CERTIFICATES),
     DBINFOTBL(THREADBLOCK_DETAILS),
-};
+    DBINFOTBL(TRANSPORTER_DETAILS),
+    DBINFOTBL(TRANSACTIONS_FULL)};
 
 static int no_ndbinfo_tables =
     sizeof(ndbinfo_tables) / sizeof(ndbinfo_tables[0]);
