@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2022, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2022, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -773,9 +774,7 @@ class IovList {
   IovList(const IovList &) = default;
   void set_count(int n) { nbuf = n; }
   int count() const { return nbuf; }
-  struct iovec &iov(int n) {
-    return m_iovec[n];
-  }
+  struct iovec &iov(int n) { return m_iovec[n]; }
   void free_all() {
     for (int n = 0; n < nbuf; n++) free(iov(n).iov_base);
   }
