@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +38,7 @@
 #define JAM_FILE_ID 430
 
 #if (defined(VM_TRACE) || defined(ERROR_INSERT))
-//#define DEBUG_MULTI_TRP 1
+// #define DEBUG_MULTI_TRP 1
 #endif
 
 #ifdef DEBUG_MULTI_TRP
@@ -65,7 +66,9 @@ Trpman::Trpman(Block_context &ctx, Uint32 instanceno)
   addRecSignal(GSN_SYNC_THREAD_VIA_REQ, &Trpman::execSYNC_THREAD_VIA_REQ);
   addRecSignal(GSN_ACTIVATE_TRP_REQ, &Trpman::execACTIVATE_TRP_REQ);
   addRecSignal(GSN_UPD_QUERY_DIST_ORD, &Trpman::execUPD_QUERY_DIST_ORD);
-
+  addRecSignal(GSN_NODE_START_REP, &Trpman::execNODE_START_REP, true);
+  addRecSignal(GSN_READ_CONFIG_REQ, &Trpman::execREAD_CONFIG_REQ);
+  addRecSignal(GSN_STTOR, &Trpman::execSTTOR);
   addRecSignal(GSN_NDB_TAMPER, &Trpman::execNDB_TAMPER, true);
   addRecSignal(GSN_DUMP_STATE_ORD, &Trpman::execDUMP_STATE_ORD);
   addRecSignal(GSN_DBINFO_SCANREQ, &Trpman::execDBINFO_SCANREQ);
@@ -79,61 +82,51 @@ static NodeBitmask c_error_9000_nodes_mask;
 extern Uint32 MAX_RECEIVED_SIGNALS;
 #endif
 
-bool Trpman::handles_this_node(Uint32 nodeId, bool all) {
+bool Trpman::handles_this_trp(TrpId trpId) {
   /* If there's only one receiver then no question */
   if (globalData.ndbMtReceiveThreads <= (Uint32)1) return true;
 
-  /**
-   * Multiple receive threads can handle the node, but only one of the receive
-   * threads will act to change state and so forth, we define this to always
-   * be the first transporter for this node. Often this method is called in
-   * the setup and close phase where only one transporter is existing.
-   * Thus we only look for first transporter below.
-   */
-  Uint32 num_ids;
-  Uint32 max_ids = 1;
-  TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
-  if (all) {
-    max_ids = MAX_NODE_GROUP_TRANSPORTERS;
-  }
-  globalTransporterRegistry.lockMultiTransporters();
-  globalTransporterRegistry.get_trps_for_node(nodeId, &trp_ids[0], num_ids,
-                                              max_ids);
   /* There's a global receiver->thread index - look it up */
-  bool ret_val = false;
-  for (Uint32 i = 0; i < num_ids; i++) {
-    if (instance() == (get_recv_thread_idx(trp_ids[i]) + /* proxy */ 1)) {
-      ret_val = true;
-      break;
-    }
-  }
-  globalTransporterRegistry.unlockMultiTransporters();
-  return ret_val;
+  return (instance() == (get_recv_thread_idx(trpId) + /* proxy */ 1));
+}
+
+/**
+ * During the setup phase the node will connect with only the base
+ * transporter. Get the TrpId of this transporter - Crash if
+ * multiple transporters are found as that would be a breakage
+ * of the transporter protocol.
+ */
+TrpId Trpman::get_the_only_base_trp(NodeId nodeId) const {
+  return globalTransporterRegistry.get_the_only_base_trp(nodeId);
 }
 
 void Trpman::execOPEN_COMORD(Signal *signal) {
-  // Connect to the specified NDB node, only QMGR allowed communication
-  // so far with the node
-
-  const BlockReference userRef = signal->theData[0];
+  /**
+   * Connect to the specified NDB node, only QMGR allowed communication
+   * so far with the node. Even if multi-transporters will be used to
+   * communicate with node, we initially open only the single base transporter.
+   */
+  const BlockReference userRef [[maybe_unused]] = signal->theData[0];
   jamEntry();
 
   const Uint32 len = signal->getLength();
   if (len == 2) {
-    Uint32 tStartingNode = signal->theData[1];
+    const NodeId tStartingNode = signal->theData[1];
     ndbrequire(tStartingNode > 0 && tStartingNode < MAX_NODES);
 #ifdef ERROR_INSERT
     if (!((ERROR_INSERTED(9000) || ERROR_INSERTED(9002)) &&
           c_error_9000_nodes_mask.get(tStartingNode)))
 #endif
     {
-      if (!handles_this_node(tStartingNode)) {
+      // Connection is initially opened using a non-multi_transporter
+      const TrpId trpId = get_the_only_base_trp(tStartingNode);
+      if (!handles_this_trp(trpId)) {
         jam();
         goto done;
       }
 
-      globalTransporterRegistry.start_connecting(tStartingNode);
-      globalTransporterRegistry.setIOState(tStartingNode, HaltIO);
+      globalTransporterRegistry.start_connecting(trpId);
+      globalTransporterRegistry.setIOState(trpId, HaltIO);
 
       //-----------------------------------------------------
       // Report that the connection to the node is opened
@@ -147,18 +140,20 @@ void Trpman::execOPEN_COMORD(Signal *signal) {
     Uint32 tData2 = signal->theData[2];
     for (unsigned int i = 1; i < MAX_NODES; i++) {
       jam();
-      if (i != getOwnNodeId() && getNodeInfo(i).m_type == tData2 &&
-          handles_this_node(i)) {
+      if (i != getOwnNodeId() && getNodeInfo(i).m_type == tData2) {
         jam();
 
+        const TrpId trpId = get_the_only_base_trp(i);
+        if (!handles_this_trp(trpId)) continue;
+
 #ifdef ERROR_INSERT
         if ((ERROR_INSERTED(9000) || ERROR_INSERTED(9002)) &&
             c_error_9000_nodes_mask.get(i))
           continue;
 #endif
 
-        globalTransporterRegistry.start_connecting(i);
-        globalTransporterRegistry.setIOState(i, HaltIO);
+        globalTransporterRegistry.start_connecting(trpId);
+        globalTransporterRegistry.setIOState(trpId, HaltIO);
 
         signal->theData[0] = NDB_LE_CommunicationOpened;
         signal->theData[1] = i;
@@ -166,12 +161,11 @@ void Trpman::execOPEN_COMORD(Signal *signal) {
       }
     }
   }
-
 done:
   /**
    * NO REPLY for now
    */
-  (void)userRef;
+  return;
 }
 
 void Trpman::execCONNECT_REP(Signal *signal) {
@@ -196,7 +190,10 @@ void Trpman::execCONNECT_REP(Signal *signal) {
    */
   if (type == NodeInfo::MGM) {
     jam();
-    globalTransporterRegistry.setIOState(hostId, NoHalt);
+    const TrpId trpId = globalTransporterRegistry.get_the_only_base_trp(hostId);
+    if (trpId != 0) {
+      globalTransporterRegistry.setIOState(trpId, NoHalt);
+    }
   }
 
   //------------------------------------------
@@ -207,19 +204,32 @@ void Trpman::execCONNECT_REP(Signal *signal) {
   sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 2, JBB);
 }
 
-void Trpman::close_com_failed_node(Signal *signal, Uint32 nodeId) {
-  if (handles_this_node(nodeId)) {
-    jam();
+void Trpman::close_com_failed_node(Signal *signal, NodeId nodeId) {
+  Uint32 num_ids;
+  TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
+  globalTransporterRegistry.lockMultiTransporters();
+  globalTransporterRegistry.get_trps_for_node(nodeId, &trp_ids[0], num_ids,
+                                              MAX_NODE_GROUP_TRANSPORTERS);
 
+  // This TRPMAN should only close the transporters handled by its own
+  // recv-thread
+  for (unsigned i = 0; i < num_ids; i++) {
+    const TrpId trpId = trp_ids[i];
+    if (!handles_this_trp(trpId)) continue;
+    globalTransporterRegistry.setIOState(trpId, HaltIO);
+    globalTransporterRegistry.start_disconnecting(trpId);
+  }
+  globalTransporterRegistry.unlockMultiTransporters();
+
+  // Only the TRPMAN responsible for the first transporter
+  // sends the EVENT_REP for the *NodeId* disconnect.
+  if (num_ids > 0 && handles_this_trp(trp_ids[0])) {
     //-----------------------------------------------------
     // Report that the connection to the node is closed
     //-----------------------------------------------------
     signal->theData[0] = NDB_LE_CommunicationClosed;
     signal->theData[1] = nodeId;
     sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 2, JBB);
-
-    globalTransporterRegistry.setIOState(nodeId, HaltIO);
-    globalTransporterRegistry.start_disconnecting(nodeId);
   }
 }
 
@@ -299,18 +309,20 @@ void Trpman::execCLOSE_COMCONF(Signal *signal) {
              JBA);
 }
 
-void Trpman::enable_com_node(Signal *signal, Uint32 node) {
-  if (!handles_this_node(node)) return;
-  globalTransporterRegistry.setIOState(node, NoHalt);
-  setNodeInfo(node).m_connected = true;
+void Trpman::enable_com_node(Signal *signal, NodeId nodeId) {
+  const TrpId trpId = get_the_only_base_trp(nodeId);
+  if (!handles_this_trp(trpId)) return;
+
+  globalTransporterRegistry.setIOState(trpId, NoHalt);
+  setNodeInfo(nodeId).m_connected = true;
 
   //-----------------------------------------------------
   // Report that the version of the node
   //-----------------------------------------------------
   signal->theData[0] = NDB_LE_ConnectedApiVersion;
-  signal->theData[1] = node;
-  signal->theData[2] = getNodeInfo(node).m_version;
-  signal->theData[3] = getNodeInfo(node).m_mysql_version;
+  signal->theData[1] = nodeId;
+  signal->theData[2] = getNodeInfo(nodeId).m_version;
+  signal->theData[3] = getNodeInfo(nodeId).m_mysql_version;
 
   sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 4, JBB);
 }
@@ -450,17 +462,105 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
   jamEntry();
 
   switch (req.tableId) {
+    case Ndbinfo::TRANSPORTER_DETAILS_TABLEID: {
+      jam();
+      TrpId trpId = cursor->data[0];
+
+      while (trpId <= globalTransporterRegistry.get_transporter_count()) {
+        if (globalTransporterRegistry.get_transporter(trpId) == nullptr ||
+            globalTransporterRegistry.is_inactive_trp(trpId) ||
+            !handles_this_trp(trpId)) {
+          trpId++;
+          continue;
+        }
+
+        const NodeId nodeId =
+            globalTransporterRegistry.get_transporter_node_id(trpId);
+        Ndbinfo::Row row(signal, req);
+        row.write_uint32(getOwnNodeId());  // Node id
+        row.write_uint32(instance());      // Block instance reporting
+        row.write_uint32(trpId);           // Transporter id
+        row.write_uint32(nodeId);          // Remote node id
+
+        row.write_uint32(globalTransporterRegistry.getPerformState(trpId));
+
+        const ndb_sockaddr conn_addr =
+            globalTransporterRegistry.get_connect_address(trpId);
+        /* Connect address */
+        if (!conn_addr.is_unspecified()) {
+          jam();
+          char *addr_str =
+              Ndb_inet_ntop(&conn_addr, addr_buf, sizeof(addr_buf));
+          row.write_string(addr_str);
+        } else {
+          jam();
+          row.write_null();
+        }
+
+        /* Bytes sent/received */
+        row.write_uint64(globalTransporterRegistry.get_bytes_sent(trpId));
+        row.write_uint64(globalTransporterRegistry.get_bytes_received(trpId));
+
+        /* Connect count, overload and slowdown states */
+        row.write_uint32(globalTransporterRegistry.get_connect_count(trpId));
+        /* FIXME: overload & slowdown is still pr NodeId */
+        row.write_uint32(
+            globalTransporterRegistry.get_status_overloaded().get(nodeId));
+        row.write_uint32(globalTransporterRegistry.get_overload_count(nodeId));
+        row.write_uint32(
+            globalTransporterRegistry.get_status_slowdown().get(nodeId));
+        row.write_uint32(globalTransporterRegistry.get_slowdown_count(nodeId));
+
+        /* TLS, not available until 8.3 -> is_encrypted=false*/
+        row.write_uint32(false);
+
+        /* Send buffer bytes/pages usage */
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_used_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_max_used_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_alloc_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_max_alloc_bytes(trpId));
+
+        /* Transporter type */
+        row.write_uint32(globalTransporterRegistry.get_transporter_type(trpId));
+
+        ndbinfo_send_row(signal, req, row, rl);
+        trpId++;
+        if (rl.need_break(req)) {
+          jam();
+          ndbinfo_send_scan_break(signal, req, rl, trpId);
+          return;
+        }
+      }
+      break;
+    }
     case Ndbinfo::TRANSPORTERS_TABLEID: {
       jam();
       Uint32 rnode = cursor->data[0];
       if (rnode == 0) rnode++;  // Skip node 0
 
       while (rnode < MAX_NODES) {
-        if (globalTransporterRegistry.get_node_transporter(rnode) == NULL) {
+        if (globalTransporterRegistry.get_node_transporter(rnode) == nullptr) {
           rnode++;
           continue;
         }
-        if (!handles_this_node(rnode)) {
+        // Find all active transporters for each node
+        Uint32 num_ids;
+        TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
+        globalTransporterRegistry.lockMultiTransporters();
+        globalTransporterRegistry.get_trps_for_node(
+            rnode, trp_ids, num_ids, MAX_NODE_GROUP_TRANSPORTERS);
+        globalTransporterRegistry.unlockMultiTransporters();
+
+        /**
+         * The TRPMAN having the first transporter is responsible for reporting
+         * the ndbinfo. It might dirty-read the ndbinfo added from other
+         * (multi-)transporters.
+         */
+        if (num_ids == 0 || !handles_this_trp(trp_ids[0])) {
           rnode++;
           continue;
         }
@@ -468,25 +568,32 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
         switch (getNodeInfo(rnode).m_type) {
           default: {
             jam();
+            Uint64 bytes_sent = 0;
+            Uint64 bytes_received = 0;
+            Uint32 connect_count = 0;
+            Uint32 perform_state = 0;
+            ndb_sockaddr conn_addr;
+
+            // Aggregate information over all (multi?) transporters
+            for (unsigned i = 0; i < num_ids; i++) {
+              const TrpId trpId = trp_ids[i];
+              bytes_sent += globalTransporterRegistry.get_bytes_sent(trpId);
+              bytes_received +=
+                  globalTransporterRegistry.get_bytes_received(trpId);
+              connect_count =
+                  std::max(connect_count,
+                           globalTransporterRegistry.get_connect_count(trpId));
+
+              // Not aggregated, will be the same for all trps;
+              conn_addr = globalTransporterRegistry.get_connect_address(trpId);
+              perform_state = globalTransporterRegistry.getPerformState(trpId);
+            }
+
             Ndbinfo::Row row(signal, req);
             row.write_uint32(getOwnNodeId());  // Node id
             row.write_uint32(rnode);           // Remote node id
-            /**
-             * FIXME: We should include all the seperate (multi-)Transporters
-             * in the ndbinfo.transporters table. For now we just pick the
-             * first active transporter and report the state for that one
-             * as representative for the 'node-connect-state':
-             */
-            TrpId trpId;
-            Uint32 num_ids;
-            globalTransporterRegistry.lockMultiTransporters();
-            globalTransporterRegistry.get_trps_for_node(rnode, &trpId, num_ids,
-                                                        1);
-            globalTransporterRegistry.unlockMultiTransporters();
-            row.write_uint32(globalTransporterRegistry.getPerformState(trpId));
+            row.write_uint32(perform_state);
 
-            ndb_sockaddr conn_addr =
-                globalTransporterRegistry.get_connect_address(rnode);
             /* Connect address */
             if (!conn_addr.is_unspecified()) {
               jam();
@@ -499,13 +606,11 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
             }
 
             /* Bytes sent/received */
-            row.write_uint64(globalTransporterRegistry.get_bytes_sent(rnode));
-            row.write_uint64(
-                globalTransporterRegistry.get_bytes_received(rnode));
+            row.write_uint64(bytes_sent);
+            row.write_uint64(bytes_received);
 
-            /* Connect count, overload and Slowdown states */
-            row.write_uint32(
-                globalTransporterRegistry.get_connect_count(rnode));
+            /* Connect count, overload and slowdown states */
+            row.write_uint32(connect_count);
             row.write_uint32(
                 globalTransporterRegistry.get_status_overloaded().get(rnode));
             row.write_uint32(
@@ -540,14 +645,77 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
   ndbinfo_send_scan_conf(signal, req, rl);
 }
 
-void Trpman::execNDB_TAMPER(Signal *signal) {
+void Trpman::execNODE_START_REP(Signal *signal) {
   jamEntry();
 #ifdef ERROR_INSERT
-  if (signal->getLength() == 1) {
-    SET_ERROR_INSERT_VALUE(signal->theData[0]);
-  } else {
-    SET_ERROR_INSERT_VALUE2(signal->theData[0], signal->theData[1]);
+  if (ERROR_INSERTED(9002) && signal->theData[0] == getOwnNodeId()) {
+    CLEAR_ERROR_INSERT_VALUE;
+    signal->theData[0] = 9001;
+    execDUMP_STATE_ORD(signal);
   }
+#endif
+}
+
+void Trpman::execREAD_CONFIG_REQ(Signal *signal) {
+  jamEntry();
+  const ReadConfigReq *req = (ReadConfigReq *)signal->getDataPtr();
+  Uint32 ref = req->senderRef;
+  Uint32 senderData = req->senderData;
+
+  ReadConfigConf *conf = (ReadConfigConf *)signal->getDataPtrSend();
+  conf->senderRef = reference();
+  conf->senderData = senderData;
+  sendSignal(ref, GSN_READ_CONFIG_CONF, signal, ReadConfigConf::SignalLength,
+             JBB);
+}
+
+void Trpman::execSTTOR(Signal *signal) {
+  jamEntry();
+  Uint32 theStartPhase = signal->theData[1];
+
+  jamEntry();
+  if (theStartPhase == 8) {
+#ifdef ERROR_INSERT
+    if (ERROR_INSERTED(9004)) {
+      CLEAR_ERROR_INSERT_VALUE;
+      Uint32 tmp[25];
+      Uint32 len = signal->getLength();
+      memcpy(tmp, signal->theData, sizeof(tmp));
+
+      Uint32 db;
+      for (db = 1; db < MAX_NDB_NODES; db++) {
+        if (db == getOwnNodeId()) continue;
+        if (getNodeInfo(db).getType() == NodeInfo::DB) break;
+      }
+      ndbrequire(db < MAX_NDB_NODES);
+
+      Uint32 i = c_error_9000_nodes_mask.find(1);
+      const TrpId trpId = get_the_only_base_trp(i);
+      if (handles_this_trp(trpId)) {
+        signal->theData[0] = i;
+        sendSignal(calcQmgrBlockRef(db), GSN_API_FAILREQ, signal, 1, JBA);
+        g_eventLogger->info("stopping %u using %u", i, db);
+
+        memcpy(signal->theData, tmp, sizeof(tmp));
+        sendSignalWithDelay(reference(), GSN_STTOR, signal, 100, len);
+        return;
+      }
+    }
+#endif
+  }
+
+  signal->theData[3] = 1;
+  signal->theData[4] = 8;
+  signal->theData[5] = 255;
+  BlockReference ref = !isNdbMtLqh() ? NDBCNTR_REF : TRPMAN_REF;
+  sendSignal(ref, GSN_STTORRY, signal, 6, JBB);
+  return;
+}
+
+void Trpman::execNDB_TAMPER(Signal *signal) {
+  jamEntry();
+  SimulatedBlock::execNDB_TAMPER(signal);
+#ifdef ERROR_INSERT
 
   if (signal->theData[0] == 9003) {
     if (MAX_RECEIVED_SIGNALS < 1024) {
@@ -558,17 +726,32 @@ void Trpman::execNDB_TAMPER(Signal *signal) {
     g_eventLogger->info("MAX_RECEIVED_SIGNALS: %d", MAX_RECEIVED_SIGNALS);
     CLEAR_ERROR_INSERT_VALUE;
   }
+
+  if (ERROR_INSERTED(9006)) {
+    g_eventLogger->info("Activating error 9006 for SEGV of all nodes");
+    /*
+     * Disable this injected crash to generate core files. We can not use the
+     * CRASH_INSERTION macro here since it modifies the node start type in an
+     * unwanted way when testing fix for Bug #24945638 STOPONERROR = 0 WITH
+     * UNCONTROLLED EXIT RESTARTS IN SAME WAY AS PREVIOUS RESTART.
+     * Instead, we explicitly turn off core file generation by directly
+     * modifying the opt_core variable of main.cpp.
+     */
+    extern bool opt_core;
+    opt_core = false;
+    raise(SIGSEGV);
+  }
 #endif
 }  // execNDB_TAMPER()
 
 void Trpman::execDUMP_STATE_ORD(Signal *signal) {
   DumpStateOrd *const &dumpState = (DumpStateOrd *)&signal->theData[0];
-  Uint32 arg = dumpState->args[0];
-  (void)arg;
+  Uint32 arg [[maybe_unused]] = dumpState->args[0];
 
 #ifdef ERROR_INSERT
   if (arg == 9000 || arg == 9002) {
     SET_ERROR_INSERT_VALUE(arg);
+    c_error_9000_nodes_mask.clear();
     for (Uint32 i = 1; i < signal->getLength(); i++)
       c_error_9000_nodes_mask.set(signal->theData[i]);
   }
@@ -578,7 +761,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     if (signal->getLength() == 1 || signal->theData[1]) {
       signal->header.theLength = 2;
       for (Uint32 i = 1; i < MAX_NODES; i++) {
-        if (c_error_9000_nodes_mask.get(i) && handles_this_node(i)) {
+        if (c_error_9000_nodes_mask.get(i)) {
           signal->theData[0] = 0;
           signal->theData[1] = i;
           execOPEN_COMORD(signal);
@@ -594,15 +777,29 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     c_error_9000_nodes_mask.set(signal->theData[1]);
   }
 
-  if (arg == 9005 && signal->getLength() == 2 && ERROR_INSERTED(9004)) {
-    Uint32 db = signal->theData[1];
-    Uint32 i = c_error_9000_nodes_mask.find(1);
-    if (handles_this_node(i)) {
-      signal->theData[0] = i;
-      sendSignal(calcQmgrBlockRef(db), GSN_API_FAILREQ, signal, 1, JBA);
-      g_eventLogger->info("stopping %u using %u", i, db);
+  if (arg == 9006) {
+    Uint32 delay = 1000;
+    switch (signal->getLength()) {
+      case 1:
+        break;
+      case 2:
+        delay = signal->theData[1];
+        break;
+      default: {
+        Uint32 dmin = signal->theData[1];
+        Uint32 dmax = signal->theData[2];
+        delay = dmin + (rand() % (dmax - dmin));
+        break;
+      }
+    }
+    signal->theData[0] = arg;
+    if (delay == 0) {
+      execNDB_TAMPER(signal);
+    } else if (delay < 10) {
+      sendSignal(reference(), GSN_NDB_TAMPER, signal, 1, JBB);
+    } else {
+      sendSignalWithDelay(reference(), GSN_NDB_TAMPER, signal, delay, 1);
     }
-    CLEAR_ERROR_INSERT_VALUE;
   }
 #endif
 
@@ -628,9 +825,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     TransporterReceiveHandle *recvdata = mt_get_trp_receive_handle(instance());
     assert(recvdata != 0);
     for (Uint32 n = 1; n < signal->getLength(); n++) {
-      Uint32 nodeId = signal->theData[n];
-      if (!handles_this_node(nodeId, true)) continue;
-
+      const NodeId nodeId = signal->theData[n];
       if ((nodeId > 0) && (nodeId < MAX_NODES)) {
         TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
         Uint32 num_ids;
@@ -640,6 +835,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
         globalTransporterRegistry.unlockMultiTransporters();
 
         for (unsigned i = 0; i < num_ids; i++) {
+          if (!handles_this_trp(trp_ids[i])) continue;
           if (block) {
             if (!globalTransporterRegistry.isBlocked(trp_ids[i])) {
               g_eventLogger->info(
@@ -689,7 +885,6 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     assert(recvdata != 0);
     for (Uint32 node = 1; node < MAX_NDB_NODES; node++) {
       if (node == getOwnNodeId()) continue;
-      if (!handles_this_node(node, true)) continue;
 
       // Get all node (multi-)Transporters, block all/some
       TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
@@ -700,6 +895,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
       globalTransporterRegistry.unlockMultiTransporters();
 
       for (unsigned i = 0; i < num_ids; i++) {
+        if (!handles_this_trp(trp_ids[i])) continue;
         if (globalTransporterRegistry.is_connected(trp_ids[i])) {
           if (getNodeInfo(node).m_type == NodeInfo::DB) {
             if (!globalTransporterRegistry.isBlocked(trp_ids[i])) {
@@ -735,7 +931,6 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     assert(recvdata != 0);
     for (Uint32 node = 1; node < MAX_NODES; node++) {
       if (node == getOwnNodeId()) continue;
-      if (!handles_this_node(node, true)) continue;
 
       // Get all node (multi-)Transporters, unblock all
       TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
@@ -746,6 +941,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
       globalTransporterRegistry.unlockMultiTransporters();
 
       for (unsigned i = 0; i < num_ids; i++) {
+        if (!handles_this_trp(trp_ids[i])) continue;
         if (globalTransporterRegistry.isBlocked(trp_ids[i])) {
           g_eventLogger->info(
               "(%u)TRPMAN : Unblocking receive on transporter %u"
@@ -763,8 +959,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
     TransporterReceiveHandle *recvdata = mt_get_trp_receive_handle(instance());
     assert(recvdata != 0);
     for (Uint32 n = 1; n < signal->getLength(); n++) {
-      Uint32 nodeId = signal->theData[n];
-      if (!handles_this_node(nodeId)) continue;
+      const NodeId nodeId = signal->theData[n];
 
       if ((nodeId > 0) && (nodeId < MAX_NODES)) {
         TrpId trp_ids[MAX_NODE_GROUP_TRANSPORTERS];
@@ -775,6 +970,7 @@ void Trpman::execDUMP_STATE_ORD(Signal *signal) {
         globalTransporterRegistry.unlockMultiTransporters();
 
         for (unsigned i = 0; i < num_ids; i++) {
+          if (!handles_this_trp(trp_ids[i])) continue;
           g_eventLogger->info(
               "TRPMAN : Send on transporter %u to node %u"
               " is %sblocked",
@@ -854,7 +1050,7 @@ void Trpman::execACTIVATE_TRP_REQ(Signal *signal) {
    * to an already enabled node.
    */
   if (is_recv_thread_for_new_trp(trp_id)) {
-    globalTransporterRegistry.setIOState_trp(trp_id, NoHalt);
+    globalTransporterRegistry.setIOState(trp_id, NoHalt);
     DEB_MULTI_TRP(("(%u)ACTIVATE_TRP_REQ is receiver (%u,%u)", instance(),
                    node_id, trp_id));
     ActivateTrpConf *conf = CAST_PTR(ActivateTrpConf, signal->getDataPtrSend());
@@ -918,6 +1114,7 @@ TrpmanProxy::TrpmanProxy(Block_context &ctx) : LocalProxy(TRPMAN, ctx) {
   addRecSignal(GSN_SYNC_THREAD_VIA_CONF,
                &TrpmanProxy::execSYNC_THREAD_VIA_CONF);
   addRecSignal(GSN_ACTIVATE_TRP_REQ, &TrpmanProxy::execACTIVATE_TRP_REQ);
+  addRecSignal(GSN_NODE_START_REP, &TrpmanProxy::execNODE_START_REP, true);
 }
 
 TrpmanProxy::~TrpmanProxy() {}
@@ -1128,3 +1325,13 @@ void TrpmanProxy::execACTIVATE_TRP_REQ(Signal *signal) {
     sendSignal(ref, GSN_ACTIVATE_TRP_REQ, signal, signal->getLength(), JBB);
   }
 }
+
+void TrpmanProxy::execNODE_START_REP(Signal *signal) {
+  jamEntry();
+  // Resend to workers
+  for (Uint32 i = 0; i < c_workers; i++) {
+    jam();
+    Uint32 ref = numberToRef(number(), workerInstance(i), getOwnNodeId());
+    sendSignal(ref, GSN_NODE_START_REP, signal, signal->getLength(), JBB);
+  }
+}
