@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -439,7 +440,7 @@ int Ndb::computeHash(Uint32 *retval, const NdbDictionary::Table *table,
   while (true) {
     if (buf == nullptr) {
       bufLen = sumlen;
-      buf = malloc(bufLen);
+      buf = calloc(bufLen, 1);
       if (unlikely(buf == nullptr)) return 4000;
       malloced_buf = buf; /* Remember to free */
     }
@@ -967,7 +968,7 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
 // application behaviour.
 //-----------------------------------------------------
 #ifdef VM_TRACE
-    printf("NULL into closeTransaction\n");
+    g_eventLogger->info("Ndb::closeTransaction passed nullptr");
 #endif
     DBUG_VOID_RETURN;
   }  // if
@@ -995,23 +996,11 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
         //-----------------------------------------------------
         // closeTransaction called on non-existing transaction
         //-----------------------------------------------------
-
-        if (aConnection->theError.code == 4008) {
-          /**
-           * When a SCAN timed-out, returning the NdbTransaction leads
-           * to reuse. And TC crashes when the API tries to reuse it to
-           * something else...
-           */
 #ifdef VM_TRACE
-          printf(
-              "Scan timeout:ed NdbTransaction-> "
-              "not returning it-> memory leak\n");
-#endif
-          DBUG_VOID_RETURN;
-        }
-
-#ifdef VM_TRACE
-        printf("Non-existing transaction into closeTransaction\n");
+        fprintf(stderr,
+                "%s NDBAPI FATAL ERROR : Non-existing transaction %p "
+                "passed into closeTransaction on Ndb %p\n",
+                Logger::Timestamp().c_str(), aConnection, this);
         abort();
 #endif
         DBUG_VOID_RETURN;
@@ -1026,18 +1015,6 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
 
   theImpl->incClientStat(TransCloseCount, 1);
 
-  if (aConnection->theError.code == 4008) {
-    /**
-     * Something timed-out, returning the NdbTransaction leads
-     * to reuse. And TC crashes when the API tries to reuse it to
-     * something else...
-     */
-#ifdef VM_TRACE
-    printf("Con timeout:ed NdbTransaction-> not returning it-> memory leak\n");
-#endif
-    DBUG_VOID_RETURN;
-  }
-
   /**
    * NOTE: It's ok to call getNodeSequence() here wo/ having mutex,
    */
@@ -1045,6 +1022,19 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
   Uint32 seq = theImpl->getNodeSequence(nodeId);
   if (aConnection->theNodeSequence != seq) {
     aConnection->theReleaseOnClose = true;
+    aConnection->theForceReleaseOnClose = false;
+  }
+
+  if (aConnection->theForceReleaseOnClose) {
+#ifdef VM_TRACE
+    g_eventLogger->info(
+        "closeTransaction() forcing release of "
+        "kernel state");
+#endif
+    aConnection->theReleaseOnClose = false;
+    aConnection->theForceReleaseOnClose = false;
+    releaseConnectToNdb(aConnection);
+    DBUG_VOID_RETURN;
   }
 
   if (aConnection->theReleaseOnClose == false) {
@@ -1055,6 +1045,10 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
 
     DBUG_VOID_RETURN;
   } else {
+    /**
+     * The kernel side of this transaction has gone
+     * recycle the API object
+     */
     aConnection->theReleaseOnClose = false;
     releaseNdbCon(aConnection);
   }  // if
@@ -1861,9 +1855,6 @@ NdbEventOperation *Ndb::getEventOperation(NdbEventOperation *tOp) {
 
 int Ndb::pollEvents2(int aMillisecondNumber, Uint64 *highestQueuedEpoch) {
   if (unlikely(aMillisecondNumber < 0)) {
-    g_eventLogger->error(
-        "Ndb::pollEvents2: negative aMillisecondNumber %d 0x%x %s",
-        aMillisecondNumber, getReference(), getNdbObjectName());
     return -1;
   }
 
@@ -1893,19 +1884,21 @@ bool Ndb::isExpectingHigherQueuedEpochs() {
 }
 
 void Ndb::printOverflowErrorAndExit() {
-  g_eventLogger->error("Ndb Event Buffer : 0x%x %s", getReference(),
-                       getNdbObjectName());
-  g_eventLogger->error("Ndb Event Buffer : Event buffer out of memory.");
-  g_eventLogger->error("Ndb Event Buffer : Fatal error.");
+  fprintf(stderr,
+          "%s NDBAPI FATAL ERROR : Ndb Event Buffer : 0x%x %s\n"
+          "Event buffer out of memory.\n",
+          Logger::Timestamp().c_str(), getReference(), getNdbObjectName());
   Uint64 maxalloc = get_eventbuf_max_alloc();
   if (maxalloc != 0) {
     // limited memory is allocated for event buffer, give recommendation
-    g_eventLogger->error(
-        "Ndb Event Buffer : Change eventbuf_max_alloc (Current max_alloc is "
-        "%llu).",
-        maxalloc);
+    fprintf(stderr,
+            "Consider changing eventbuf_max_alloc (Current max_alloc is "
+            "%llu).\n",
+            maxalloc);
   }
-  g_eventLogger->error("Ndb Event Buffer : Consider using the new API.");
+  fprintf(stderr,
+          "Out of Memory events can be handled non fatally "
+          "using nextEvent2().\n");
   exit(-1);
 }
 
@@ -1962,11 +1955,14 @@ NdbEventOperation *Ndb::nextEvent() {
   }
 
   if (unlikely(op->isEmptyEpoch())) {
-    g_eventLogger->error(
-        "Ndb::nextEvent: Found exceptional event type "
-        "TE_EMPTY when using old event API. "
-        "Turn off empty epoch queuing by "
-        "setEventBufferQueueEmptyEpoch(false).");
+    fprintf(stderr,
+            "%s NDBAPI FATAL ERROR : Ndb::nextEvent() : Found "
+            "exceptional event type TE_EMPTY when using old "
+            "event API nextEvent() call. "
+            "Turn off empty epoch queuing by "
+            "setEventBufferQueueEmptyEpoch(false) "
+            "or use new API nextEvent2().\n",
+            Logger::Timestamp().c_str());
     exit(-1);
   }
   return op;
@@ -2328,3 +2324,7 @@ const char *Ndb::getClientStatName(Uint32 id) const {
 
   return nullptr;
 }
+
+void Ndb::set_optimized_node_selection(int val) {
+  theImpl->m_optimized_node_selection = val;
+}
