@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1548,10 +1549,10 @@ int NdbIndexStatImpl::cache_init(Con &con) {
   c.m_keyBytes = head.m_keyBytes;
   c.m_valueLen = 4 + c.m_keyAttrs * 4;
   c.m_valueBytes = c.m_sampleCount * c.m_valueLen;
-  c.m_addrLen =
-      c.m_keyBytes < (1 << 8)
-          ? 1
-          : c.m_keyBytes < (1 << 16) ? 2 : c.m_keyBytes < (1 << 24) ? 3 : 4;
+  c.m_addrLen = c.m_keyBytes < (1 << 8)    ? 1
+                : c.m_keyBytes < (1 << 16) ? 2
+                : c.m_keyBytes < (1 << 24) ? 3
+                                           : 4;
   c.m_addrBytes = c.m_sampleCount * c.m_addrLen;
 
   // wl4124_todo omit addrArray if keys have fixed size
