@@ -1,17 +1,18 @@
 /*
-   Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -79,7 +80,7 @@ static BaseString path(const char *first, ...) {
 
 class Mgmd {
  protected:
-  NdbProcess *m_proc;
+  std::unique_ptr<NdbProcess> m_proc;
   int m_nodeid;
   BaseString m_name;
   BaseString m_exe;
@@ -89,13 +90,13 @@ class Mgmd {
   Mgmd(const Mgmd &other) = delete;
 
  public:
-  Mgmd(int nodeid) : m_proc(NULL), m_nodeid(nodeid) {
+  Mgmd(int nodeid) : m_nodeid(nodeid) {
     m_name.assfmt("ndb_mgmd_%d", nodeid);
 
     NDBT_find_ndb_mgmd(m_exe);
   }
 
-  Mgmd() : m_proc(NULL) {
+  Mgmd() {
     no_node_config = no_node_config + 1;
     m_name.assfmt("ndb_mgmd_autonode_%d", no_node_config);
     NDBT_find_ndb_mgmd(m_exe);
@@ -132,7 +133,7 @@ class Mgmd {
       m_proc = NdbProcess::create(name(), BaseString(exe_valgrind), working_dir,
                                   copy);
     }
-    return (m_proc != NULL);
+    return (bool)m_proc;
   }
 
   bool start_from_config_ini(const char *working_dir,
@@ -191,7 +192,7 @@ class Mgmd {
     // Diconnect and close our "builtin" client
     m_mgmd_client.close();
 
-    if (m_proc == 0 || !m_proc->stop()) {
+    if (!m_proc || !m_proc->stop()) {
       fprintf(stderr, "Failed to stop process %s\n", name());
       return false;  // Can't kill with -9 -> fatal error
     }
@@ -210,9 +211,7 @@ class Mgmd {
       fprintf(stderr, "Process %s stopped with ret: %u\n", name(), ret);
     }
 
-    delete m_proc;
-    m_proc = 0;
-
+    m_proc.reset();
     return true;
   }
 
@@ -223,9 +222,8 @@ class Mgmd {
       fprintf(stderr, "Failed to wait for process %s\n", name());
       return false;
     }
-    delete m_proc;
-    m_proc = 0;
 
+    m_proc.reset();
     return true;
   }
 
@@ -1503,6 +1501,37 @@ int runTestMultiMGMDDisconnection(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
+int runTestMyCnf(NDBT_Context *ctx, NDBT_Step *step) {
+  /* Construct path name to my.cnf */
+  NDBT_Workingdir wd("test_mgmd");
+  BaseString my_cnf_file = path(wd.path(), "my.cnf", NULL);
+
+  /* Create args to start mgmd using a my.cnf file */
+  NdbProcess::Args args;
+  args.add("--defaults-file=", my_cnf_file.c_str());
+  args.add("--configdir=", wd.path());
+  args.add("--ndb-nodeid=", 1);
+  args.add("--mycnf");
+
+  /* Write a my.cnf file containing an illegal config parameter */
+  FILE *fp = fopen(my_cnf_file.c_str(), "w");
+  CHECK(fp != nullptr);
+  fprintf(fp, "[cluster_config]\n");
+  fprintf(fp, "ndbcluster=on\n");
+  fclose(fp);
+
+  /* Start mgmd */
+  Mgmd mgmd;
+  CHECK(mgmd.start(wd.path(), args));
+
+  /* Mgmd exits with status 1 due to syntax error in my.cnf */
+  int ret;
+  mgmd.wait(ret);
+  CHECK(ret == 1);
+
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testMgmd);
 DRIVER(DummyDriver); /* turn off use of NdbApi */
 
@@ -1575,6 +1604,9 @@ TESTCASE("MultiMGMDDisconnection",
          "Test multi mgmd robustness against other mgmd disconnections") {
   INITIALIZER(runTestMultiMGMDDisconnection);
 }
+TESTCASE("MyCnf", "Test reading config from my.cnf") {
+  INITIALIZER(runTestMyCnf);
+}
 
 NDBT_TESTSUITE_END(testMgmd)
 
@@ -1585,6 +1617,7 @@ int main(int argc, const char **argv) {
   testMgmd.setRunAllTables(true);
   testMgmd.setConnectCluster(false);
   testMgmd.setEnsureIndexStatTables(false);
+  testMgmd.setCheckErrorInsert(false);
 
 #ifdef NDB_USE_GET_ENV
   char buf1[255], buf2[255];
