@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1331,7 +1332,7 @@ void NdbImportUtil::free_rows(RowList &src) {
 NdbImportUtil::Blob::Blob() {
   m_blobsize = 0;
   m_allocsize = 0;
-  m_data = new uchar[0];
+  m_data = nullptr;
 }
 
 NdbImportUtil::Blob::~Blob() { delete[] m_data; }
@@ -1346,7 +1347,9 @@ NdbImportUtil::BlobList::~BlobList() {
 }
 
 void NdbImportUtil::Blob::resize(uint size) {
-  if (m_allocsize < size) {
+  if (m_data == nullptr ||  // Make sure to always allocate data buffer as it's
+                            // used to distinguish between empty and NULL data
+      m_allocsize < size) {
     delete[] m_data;
     m_data = new uchar[size];
     m_allocsize = size;
