@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -907,7 +908,8 @@ class PFS_internal_schema_access : public ACL_internal_schema_access {
 
   ~PFS_internal_schema_access() override = default;
 
-  ACL_internal_access_result check(ulong want_access, ulong *save_priv,
+  ACL_internal_access_result check(Access_bitmask want_access,
+                                   Access_bitmask *save_priv,
                                    bool any_combination_will_do) const override;
 
   const ACL_internal_table_access *lookup(const char *name) const override;
@@ -947,10 +949,9 @@ static bool allow_drop_schema_privilege() {
           (thd->lex->sql_command == SQLCOM_DROP_TABLE));
 }
 
-ACL_internal_access_result PFS_internal_schema_access::check(ulong want_access,
-                                                             ulong *,
-                                                             bool) const {
-  const ulong always_forbidden =
+ACL_internal_access_result PFS_internal_schema_access::check(
+    Access_bitmask want_access, Access_bitmask *, bool) const {
+  constexpr Access_bitmask always_forbidden =
       CREATE_ACL | REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_TMP_ACL |
       EXECUTE_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | CREATE_PROC_ACL |
       ALTER_PROC_ACL | EVENT_ACL | TRIGGER_ACL;
@@ -1031,18 +1032,18 @@ static bool allow_drop_table_privilege() {
 PFS_readonly_acl pfs_readonly_acl;
 
 ACL_internal_access_result PFS_readonly_acl::check(
-    ulong want_access, ulong *granted_access,
+    Access_bitmask want_access, Access_bitmask *granted_access,
     bool any_combination_will_do) const {
-  const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
-                                 CREATE_ACL | DROP_ACL | REFERENCES_ACL |
-                                 INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL |
-                                 SHOW_VIEW_ACL | TRIGGER_ACL | LOCK_TABLES_ACL;
+  constexpr Access_bitmask always_forbidden =
+      INSERT_ACL | UPDATE_ACL | DELETE_ACL | CREATE_ACL | DROP_ACL |
+      REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
+      TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const ulong want_forbidden = want_access & always_forbidden;
+  const Access_bitmask want_forbidden = want_access & always_forbidden;
 
-  const ulong want_allowable = want_access & can_be_allowed;
+  const Access_bitmask want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
@@ -1063,7 +1064,8 @@ ACL_internal_access_result PFS_readonly_acl::check(
 PFS_readonly_world_acl pfs_readonly_world_acl;
 
 ACL_internal_access_result PFS_readonly_world_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
+    Access_bitmask want_access, Access_bitmask *save_priv,
+    bool any_combination_will_do) const {
   ACL_internal_access_result res =
       PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
@@ -1075,7 +1077,8 @@ ACL_internal_access_result PFS_readonly_world_acl::check(
 PFS_readonly_processlist_acl pfs_readonly_processlist_acl;
 
 ACL_internal_access_result PFS_readonly_processlist_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
+    Access_bitmask want_access, Access_bitmask *save_priv,
+    bool any_combination_will_do) const {
   const ACL_internal_access_result res =
       PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
 
@@ -1088,18 +1091,18 @@ ACL_internal_access_result PFS_readonly_processlist_acl::check(
 PFS_truncatable_acl pfs_truncatable_acl;
 
 ACL_internal_access_result PFS_truncatable_acl::check(
-    ulong want_access, ulong *granted_access,
+    Access_bitmask want_access, Access_bitmask *granted_access,
     bool any_combination_will_do) const {
-  const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
-                                 CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
-                                 ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
-                                 TRIGGER_ACL | LOCK_TABLES_ACL;
+  constexpr Access_bitmask always_forbidden =
+      INSERT_ACL | UPDATE_ACL | DELETE_ACL | CREATE_ACL | REFERENCES_ACL |
+      INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL |
+      LOCK_TABLES_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  ulong want_allowable = want_access & can_be_allowed;
+  Access_bitmask want_allowable = want_access & can_be_allowed;
 
-  ulong want_forbidden = want_access & always_forbidden;
+  Access_bitmask want_forbidden = want_access & always_forbidden;
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
@@ -1127,7 +1130,8 @@ ACL_internal_access_result PFS_truncatable_acl::check(
 PFS_truncatable_world_acl pfs_truncatable_world_acl;
 
 ACL_internal_access_result PFS_truncatable_world_acl::check(
-    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
+    Access_bitmask want_access, Access_bitmask *save_priv,
+    bool any_combination_will_do) const {
   ACL_internal_access_result res = PFS_truncatable_acl::check(
       want_access, save_priv, any_combination_will_do);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
@@ -1139,17 +1143,17 @@ ACL_internal_access_result PFS_truncatable_world_acl::check(
 PFS_updatable_acl pfs_updatable_acl;
 
 ACL_internal_access_result PFS_updatable_acl::check(
-    ulong want_access, ulong *granted_access,
+    Access_bitmask want_access, Access_bitmask *granted_access,
     bool any_combination_will_do) const {
-  const ulong always_forbidden =
+  constexpr Access_bitmask always_forbidden =
       INSERT_ACL | DELETE_ACL | CREATE_ACL | DROP_ACL | REFERENCES_ACL |
       INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const ulong want_forbidden = want_access & always_forbidden;
+  const Access_bitmask want_forbidden = want_access & always_forbidden;
 
-  const ulong want_allowable = want_access & can_be_allowed;
+  const Access_bitmask want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
@@ -1170,17 +1174,17 @@ ACL_internal_access_result PFS_updatable_acl::check(
 PFS_editable_acl pfs_editable_acl;
 
 ACL_internal_access_result PFS_editable_acl::check(
-    ulong want_access, ulong *granted_access,
+    Access_bitmask want_access, Access_bitmask *granted_access,
     bool any_combination_will_do) const {
-  const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
-                                 ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
-                                 TRIGGER_ACL;
+  constexpr Access_bitmask always_forbidden =
+      CREATE_ACL | REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL |
+      SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  ulong want_forbidden = want_access & always_forbidden;
+  Access_bitmask want_forbidden = want_access & always_forbidden;
 
-  ulong want_allowable = want_access & can_be_allowed;
+  Access_bitmask want_allowable = want_access & can_be_allowed;
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
@@ -1208,7 +1212,7 @@ ACL_internal_access_result PFS_editable_acl::check(
 PFS_unknown_acl pfs_unknown_acl;
 
 ACL_internal_access_result PFS_unknown_acl::check(
-    ulong want_access, ulong *granted_access,
+    Access_bitmask want_access, Access_bitmask *granted_access,
     bool any_combination_will_do) const {
   /*
     Only enforce ACL_INTERNAL_ACCESS_DENIED
@@ -1216,14 +1220,15 @@ ACL_internal_access_result PFS_unknown_acl::check(
     in the performance schema,
     relax error messages otherwise.
   */
-  const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
-                                 ALTER_ACL | CREATE_VIEW_ACL | TRIGGER_ACL;
+  constexpr Access_bitmask always_forbidden = CREATE_ACL | REFERENCES_ACL |
+                                              INDEX_ACL | ALTER_ACL |
+                                              CREATE_VIEW_ACL | TRIGGER_ACL;
 
-  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const ulong want_forbidden = want_access & always_forbidden;
+  const Access_bitmask want_forbidden = want_access & always_forbidden;
 
-  const ulong want_allowable = want_access & can_be_allowed;
+  const Access_bitmask want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
@@ -1508,6 +1513,7 @@ void PFS_engine_index::read_key(const uchar *key, uint key_len,
   }
 
   m_fields = reader.m_parts_found;
+  m_key_fetch_count = 0;
 }
 
 /** @} */
