@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,11 +28,12 @@
 
 #include "storage/perfschema/table_host_cache.h"
 
-#include "my_dbug.h"
+#include <assert.h>
+
 #include "my_thread.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
-#include "sql/hostname.h"
+#include "sql/hostname_cache.h"
 #include "sql/plugin_table.h"
 #include "sql/sql_class.h"
 #include "sql/table.h"
@@ -45,7 +47,7 @@ Plugin_table table_host_cache::m_table_def(
     "host_cache",
     /* Definition */
     "  IP VARCHAR(64) not null,\n"
-    "  HOST VARCHAR(255) collate utf8mb4_bin,\n"
+    "  HOST VARCHAR(255) CHARACTER SET ASCII default null,\n"
     "  HOST_VALIDATED ENUM ('YES', 'NO') not null,\n"
     "  SUM_CONNECT_ERRORS BIGINT not null,\n"
     "  COUNT_HOST_BLOCKED_ERRORS BIGINT not null,\n"
@@ -69,10 +71,10 @@ Plugin_table table_host_cache::m_table_def(
     "  COUNT_INIT_CONNECT_ERRORS BIGINT not null,\n"
     "  COUNT_LOCAL_ERRORS BIGINT not null,\n"
     "  COUNT_UNKNOWN_ERRORS BIGINT not null,\n"
-    "  FIRST_SEEN TIMESTAMP(0) NOT NULL default 0,\n"
-    "  LAST_SEEN TIMESTAMP(0) NOT NULL default 0,\n"
-    "  FIRST_ERROR_SEEN TIMESTAMP(0) null default 0,\n"
-    "  LAST_ERROR_SEEN TIMESTAMP(0) null default 0,\n"
+    "  FIRST_SEEN TIMESTAMP(0) not null,\n"
+    "  LAST_SEEN TIMESTAMP(0) not null,\n"
+    "  FIRST_ERROR_SEEN TIMESTAMP(0) null,\n"
+    "  LAST_ERROR_SEEN TIMESTAMP(0) null,\n"
     "  PRIMARY KEY (IP) USING HASH,\n"
     "  KEY (HOST) USING HASH\n",
     /* Options */
@@ -83,13 +85,13 @@ Plugin_table table_host_cache::m_table_def(
 PFS_engine_table_share table_host_cache::m_share = {
     &pfs_truncatable_acl,
     table_host_cache::create,
-    NULL, /* write_row */
+    nullptr, /* write_row */
     table_host_cache::delete_all_rows,
     table_host_cache::get_row_count,
     sizeof(PFS_simple_index), /* ref length */
     &m_table_lock,
     &m_table_def,
-    false, /* perpetual */
+    true, /* perpetual */
     PFS_engine_table_proxy(),
     {0},
     false /* m_in_purgatory */
@@ -114,16 +116,16 @@ bool PFS_index_host_cache_by_host::match(const row_host_cache *row) {
 }
 
 PFS_engine_table *table_host_cache::create(PFS_engine_table_share *) {
-  table_host_cache *t = new table_host_cache();
-  if (t != NULL) {
+  auto *t = new table_host_cache();
+  if (t != nullptr) {
     THD *thd = current_thd;
-    DBUG_ASSERT(thd != NULL);
+    assert(thd != nullptr);
     t->materialize(thd);
   }
   return t;
 }
 
-int table_host_cache::delete_all_rows(void) {
+int table_host_cache::delete_all_rows() {
   /*
     TRUNCATE TABLE performance_schema.host_cache
     is an alternate syntax for
@@ -133,7 +135,7 @@ int table_host_cache::delete_all_rows(void) {
   return 0;
 }
 
-ha_rows table_host_cache::get_row_count(void) {
+ha_rows table_host_cache::get_row_count() {
   ha_rows count;
   hostname_cache_lock();
   count = hostname_cache_size();
@@ -143,9 +145,9 @@ ha_rows table_host_cache::get_row_count(void) {
 
 table_host_cache::table_host_cache()
     : PFS_engine_table(&m_share, &m_pos),
-      m_all_rows(NULL),
+      m_all_rows(nullptr),
       m_row_count(0),
-      m_row(NULL),
+      m_row(nullptr),
       m_pos(0),
       m_next_pos(0) {}
 
@@ -155,8 +157,8 @@ void table_host_cache::materialize(THD *thd) {
   row_host_cache *rows;
   row_host_cache *row;
 
-  DBUG_ASSERT(m_all_rows == NULL);
-  DBUG_ASSERT(m_row_count == 0);
+  assert(m_all_rows == nullptr);
+  assert(m_row_count == 0);
 
   hostname_cache_lock();
 
@@ -167,7 +169,7 @@ void table_host_cache::materialize(THD *thd) {
   }
 
   rows = (row_host_cache *)thd->alloc(size * sizeof(row_host_cache));
-  if (rows == NULL) {
+  if (rows == nullptr) {
     /* Out of memory, this thread will error out. */
     goto end;
   }
@@ -240,12 +242,12 @@ int table_host_cache::make_row(Host_entry *entry, row_host_cache *row) {
   return 0;
 }
 
-void table_host_cache::reset_position(void) {
+void table_host_cache::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
-int table_host_cache::rnd_next(void) {
+int table_host_cache::rnd_next() {
   int result;
 
   m_pos.set_at(&m_next_pos);
@@ -255,7 +257,7 @@ int table_host_cache::rnd_next(void) {
     m_next_pos.set_after(&m_pos);
     result = 0;
   } else {
-    m_row = NULL;
+    m_row = nullptr;
     result = HA_ERR_END_OF_FILE;
   }
 
@@ -264,13 +266,13 @@ int table_host_cache::rnd_next(void) {
 
 int table_host_cache::rnd_pos(const void *pos) {
   set_position(pos);
-  DBUG_ASSERT(m_pos.m_index < m_row_count);
+  assert(m_pos.m_index < m_row_count);
   m_row = &m_all_rows[m_pos.m_index];
   return 0;
 }
 
 int table_host_cache::index_init(uint idx, bool) {
-  PFS_index_host_cache *result = NULL;
+  PFS_index_host_cache *result = nullptr;
 
   switch (idx) {
     case 0:
@@ -280,7 +282,7 @@ int table_host_cache::index_init(uint idx, bool) {
       result = PFS_NEW(PFS_index_host_cache_by_host);
       break;
     default:
-      DBUG_ASSERT(false);
+      assert(false);
       break;
   }
 
@@ -289,7 +291,7 @@ int table_host_cache::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_host_cache::index_next(void) {
+int table_host_cache::index_next() {
   int result;
 
   for (m_pos.set_at(&m_next_pos); m_pos.m_index < m_row_count; m_pos.next()) {
@@ -302,7 +304,7 @@ int table_host_cache::index_next(void) {
     }
   }
 
-  m_row = NULL;
+  m_row = nullptr;
   result = HA_ERR_END_OF_FILE;
 
   return result;
@@ -312,22 +314,22 @@ int table_host_cache::read_row_values(TABLE *table, unsigned char *buf,
                                       Field **fields, bool read_all) {
   Field *f;
 
-  DBUG_ASSERT(m_row);
+  assert(m_row);
 
   /* Set the null bits */
-  DBUG_ASSERT(table->s->null_bytes == 1);
+  assert(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
-      switch (f->field_index) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
+      switch (f->field_index()) {
         case 0: /* IP */
-          set_field_varchar_utf8(f, m_row->m_ip, m_row->m_ip_length);
+          set_field_varchar_utf8mb4(f, m_row->m_ip, m_row->m_ip_length);
           break;
         case 1: /* HOST */
           if (m_row->m_hostname_length > 0)
-            set_field_varchar_utf8(f, m_row->m_hostname,
-                                   m_row->m_hostname_length);
+            set_field_varchar_utf8mb4(f, m_row->m_hostname,
+                                      m_row->m_hostname_length);
           else {
             f->set_null();
           }
@@ -423,7 +425,7 @@ int table_host_cache::read_row_values(TABLE *table, unsigned char *buf,
           }
           break;
         default:
-          DBUG_ASSERT(false);
+          assert(false);
       }
     }
   }
