@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -271,9 +272,9 @@ int table_log_status::make_row() {
         DBUG_EVALUATE_IF("log_status_oom_collecting", 1, (*it)->collect_info());
     if (error) {
       my_error(ER_UNABLE_TO_COLLECT_LOG_STATUS, MYF(0),
-               (*it)->get_json() == &json_storage_engines
-                   ? "STORAGE_ENGINES"
-                   : (*it)->get_json() == &json_local ? "LOCAL" : "REPLICATION",
+               (*it)->get_json() == &json_storage_engines ? "STORAGE_ENGINES"
+               : (*it)->get_json() == &json_local         ? "LOCAL"
+                                                          : "REPLICATION",
                "failed to allocate memory to collect information");
       goto err_unlock;
     }
