@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +25,6 @@
   Minimal code to be able to link a unit test.
 */
 
-#include <components/mysql_server/server_component.h>
 #include <mysql/service_plugin_registry.h>
 #include <stddef.h>
 #include <sys/types.h>
@@ -34,29 +34,43 @@
 #include "sql/sql_class.h"
 #include "sql/sql_show.h"
 
+ulong max_connections;
+
 struct System_status_var global_status_var;
 
 struct sql_digest_storage;
 volatile bool ready_to_exit = false;
 
 uint lower_case_table_names = 0;
-CHARSET_INFO *files_charset_info = NULL;
-CHARSET_INFO *system_charset_info = NULL;
+CHARSET_INFO *files_charset_info = nullptr;
+CHARSET_INFO *system_charset_info = nullptr;
+
+extern "C" void sql_alloc_error_handler() {}
+
+extern "C" unsigned int thd_get_current_thd_terminology_use_previous() {
+  return 0;
+}
 
 void compute_digest_hash(const sql_digest_storage *, unsigned char *) {}
 
 void reset_status_vars() {}
 
-std::pair<struct System_status_var *, bool> get_thd_status_var(THD *) {
-  return std::make_pair(nullptr, false);
-}
+struct System_status_var *get_thd_status_var(THD *, bool *) { return nullptr; }
+
+#ifndef NDEBUG
+void thd_mem_cnt_alloc(THD *, size_t, const char *) {}
+#else
+void thd_mem_cnt_alloc(THD *, size_t) {}
+#endif
+
+void thd_mem_cnt_free(THD *, size_t) {}
 
 unsigned int mysql_errno_to_sqlstate_index(unsigned int) { return 0; }
 
-SERVICE_TYPE(registry) * mysql_plugin_registry_acquire() { return NULL; }
+SERVICE_TYPE(registry) * mysql_plugin_registry_acquire() { return nullptr; }
 
-int mysql_plugin_registry_release(SERVICE_TYPE(registry) *
-                                  reg MY_ATTRIBUTE((unused))) {
+int mysql_plugin_registry_release(SERVICE_TYPE(registry) * reg
+                                  [[maybe_unused]]) {
   return 0;
 }
 
@@ -64,3 +78,9 @@ int log_message(int, ...) {
   /* Do not pollute the unit test output with annoying messages. */
   return 0;
 }
+
+bool acl_is_utility_user(const char *, const char *, const char *) {
+  return false;
+}
+
+void reset_status_by_thd() {}
