@@ -56,7 +56,7 @@ struct generic_indexer_t {
 };
 
 #ifdef HAVE_SCHED_GETCPU
-//#include <utmpx.h>  // Including this causes problems with EMPTY symbol
+// #include <utmpx.h>  // Including this causes problems with EMPTY symbol
 #include <sched.h>  // Include this instead
 /** Use the cpu id to index into the counter array. If it fails then
 use the thread id. */
@@ -84,7 +84,9 @@ struct thread_id_indexer_t : public generic_indexer_t<Type, N> {
   /* @return a random number, currently we use the thread id. Where
   thread id is represented as a pointer, it may not work as
   effectively. */
-  size_t get_rnd_index() const { return get_curr_thread_id(); }
+  size_t get_rnd_index() const {
+    return reinterpret_cast<std::uintptr_t>(get_curr_thread_id());
+  }
 };
 
 /** For counters wher N=1 */
