@@ -20,12 +20,15 @@
 #include <string>
 #include <unordered_map>
 
+/* MySQL header files */
+#include "m_ctype.h"
+
 /* RocksDB header files */
 #include "rocksdb/table.h"
 #include "rocksdb/utilities/options_util.h"
 
 /* MyRocks header files */
-#include "./rdb_comparator.h"
+#include "./rdb_global.h"
 
 namespace myrocks {
 
@@ -47,7 +50,7 @@ class Rdb_cf_options {
   Rdb_cf_options() = default;
 
   /* bool true return indicates cf_name was found */
-  MY_NODISCARD bool get(const std::string &cf_name,
+  [[nodiscard]] bool get(const std::string &cf_name,
                         rocksdb::ColumnFamilyOptions *const opts);
 
   void update(const std::string &cf_name, const std::string &cf_options);
@@ -62,18 +65,20 @@ class Rdb_cf_options {
     return m_default_cf_opts;
   }
 
-  static const rocksdb::Comparator *
-  get_cf_comparator(const std::string &cf_name);
+  static const rocksdb::Comparator *get_cf_comparator(
+      const std::string &cf_name);
 
-  std::shared_ptr<rocksdb::MergeOperator>
-  get_cf_merge_operator(const std::string &cf_name);
+  std::shared_ptr<rocksdb::MergeOperator> get_cf_merge_operator(
+      const std::string &cf_name);
 
-  /* bool true return indicates cf_name was found, otherwise default */
+  /* return true when success */
   bool get_cf_options(const std::string &cf_name,
-                      rocksdb::ColumnFamilyOptions *const opts);
+                      rocksdb::ColumnFamilyOptions *const opts, bool &cf_exists)
+      MY_ATTRIBUTE((__nonnull__));
 
   static bool parse_cf_options(const std::string &cf_options,
-                               Name_to_config_t *option_map);
+                               Name_to_config_t *option_map,
+                               std::stringstream *output = nullptr);
 
  private:
   bool set_default(const std::string &default_config);
@@ -89,10 +94,6 @@ class Rdb_cf_options {
                                    std::string *const cf,
                                    std::string *const opt_str);
 
- private:
-  static Rdb_pk_comparator s_pk_comparator;
-  static Rdb_rev_comparator s_rev_pk_comparator;
-
   /* CF name -> value map */
   Name_to_config_t m_name_map;
 
