@@ -24,9 +24,9 @@
 #include <vector>
 
 /* MySQL header files */
-#include "log.h"
 #include "my_stacktrace.h"
-#include "sql_array.h"
+#include "sql/debug_sync.h"
+#include "sql/sql_array.h"
 
 /* MyRocks header files */
 #include "./ha_rocksdb.h"
@@ -34,6 +34,7 @@
 #include "./rdb_datadic.h"
 #include "./rdb_psi.h"
 #include "./rdb_utils.h"
+#include "./sql_dd.h"
 
 namespace myrocks {
 
@@ -48,6 +49,54 @@ void dbug_modify_key_varchar8(String *on_disk_rec) {
   on_disk_rec->append(res.data(), res.size());
 }
 
+int Rdb_convert_to_record_value_decoder::decode_instant(
+    uchar *const buf, Rdb_field_encoder *field_dec) {
+  // m_instant_default_value store default value for instant cols
+  if (field_dec->m_instant_default_value) {  // default value isn't null
+    // BLOB/JSON doesn't support default value
+    assert(field_dec->m_field_type != MYSQL_TYPE_BLOB ||
+           field_dec->m_field_type != MYSQL_TYPE_JSON);
+
+    if (field_dec->maybe_null()) {
+      // This sets the NULL-bit of this record
+      buf[field_dec->m_field_null_offset] &= ~(field_dec->m_field_null_mask);
+    }
+
+    if (field_dec->m_field_type == MYSQL_TYPE_VARCHAR) {
+      if (field_dec->m_instant_default_value_len > field_dec->m_field_length) {
+        assert(false);
+        // The data on disk is longer than table DDL allows?
+        return HA_ERR_ROCKSDB_CORRUPT_DATA;
+      }
+
+      // store length first
+      if (field_dec->m_field_length_bytes == 1) {
+        buf[field_dec->m_field_offset] = field_dec->m_instant_default_value_len;
+      } else {
+        assert(field_dec->m_field_length_bytes == 2);
+        int2store(buf + field_dec->m_field_offset,
+                  field_dec->m_instant_default_value_len);
+      }
+
+      // copy data
+      memcpy(buf + field_dec->m_field_offset + field_dec->m_field_length_bytes,
+             field_dec->m_instant_default_value,
+             field_dec->m_instant_default_value_len);
+
+    } else {  // fixed-length type
+      memcpy(buf + field_dec->m_field_offset,
+             field_dec->m_instant_default_value,
+             field_dec->m_instant_default_value_len);
+    }
+  } else {  // default value is null
+    if (field_dec->maybe_null()) {
+      // This sets the NULL-bit of this record
+      buf[field_dec->m_field_null_offset] |= field_dec->m_field_null_mask;
+    }
+  }
+
+  return HA_EXIT_SUCCESS;
+}
 /*
   Convert field from rocksdb storage format into Mysql Record format
   @param    buf         OUT          start memory to fill converted data
@@ -60,53 +109,43 @@ void dbug_modify_key_varchar8(String *on_disk_rec) {
     0      OK
     other  HA_ERR error code (can be SE-specific)
 */
-int Rdb_convert_to_record_value_decoder::decode(uchar *const buf, uint *offset,
-                                                TABLE *table,
-                                                my_core::Field *field,
+int Rdb_convert_to_record_value_decoder::decode(uchar *const buf, TABLE *table,
                                                 Rdb_field_encoder *field_dec,
                                                 Rdb_string_reader *reader,
                                                 bool decode, bool is_null) {
+  DBUG_TRACE;
   int err = HA_EXIT_SUCCESS;
-
-  uint field_offset = field->ptr - table->record[0];
-  *offset = field_offset;
-  uint null_offset = field->null_offset();
-  bool maybe_null = field->real_maybe_null();
-  field->move_field(buf + field_offset,
-                    maybe_null ? buf + null_offset : nullptr, field->null_bit);
-
+  auto ptr = buf + field_dec->m_field_offset;
   if (is_null) {
-    if (decode) {
+    if (decode && field_dec->maybe_null()) {
       // This sets the NULL-bit of this record
-      field->set_null();
+      buf[field_dec->m_field_null_offset] |= field_dec->m_field_null_mask;
+
       /*
         Besides that, set the field value to default value. CHECKSUM TABLE
         depends on this.
       */
-      memcpy(field->ptr, table->s->default_values + field_offset,
-             field->pack_length());
+      memcpy(ptr, table->s->default_values + field_dec->m_field_offset,
+             field_dec->m_field_pack_length);
     }
   } else {
-    if (decode) {
+    if (decode && field_dec->maybe_null()) {
       // sets non-null bits for this record
-      field->set_notnull();
+      buf[field_dec->m_field_null_offset] &= ~(field_dec->m_field_null_mask);
     }
 
-    if (field_dec->m_field_type == MYSQL_TYPE_BLOB ||
-        field_dec->m_field_type == MYSQL_TYPE_JSON) {
-      err = decode_blob(table, field, reader, decode);
-    } else if (field_dec->m_field_type == MYSQL_TYPE_VARCHAR) {
-      err = decode_varchar(field, reader, decode);
-    } else {
-      err = decode_fixed_length_field(field, field_dec, reader, decode);
+    if (!field_dec->m_is_virtual_gcol) {
+      if (field_dec->m_field_type == MYSQL_TYPE_BLOB ||
+          field_dec->m_field_type == MYSQL_TYPE_JSON) {
+        err = decode_blob(table, ptr, field_dec, reader, decode);
+      } else if (field_dec->m_field_type == MYSQL_TYPE_VARCHAR) {
+        err = decode_varchar(ptr, field_dec, reader, decode);
+      } else {
+        err = decode_fixed_length_field(ptr, field_dec, reader, decode);
+      }
     }
   }
 
-  // Restore field->ptr and field->null_ptr
-  field->move_field(table->record[0] + field_offset,
-                    maybe_null ? table->record[0] + null_offset : nullptr,
-                    field->null_bit);
-
   return err;
 }
 
@@ -120,23 +159,21 @@ int Rdb_convert_to_record_value_decoder::decode(uchar *const buf, uint *offset,
     0      OK
     other  HA_ERR error code (can be SE-specific)
 */
-int Rdb_convert_to_record_value_decoder::decode_blob(TABLE *table, Field *field,
-                                                     Rdb_string_reader *reader,
-                                                     bool decode) {
-  my_core::Field_blob *blob = (my_core::Field_blob *)field;
-
+int Rdb_convert_to_record_value_decoder::decode_blob(
+    TABLE *table, uchar *const buf, Rdb_field_encoder *field_dec,
+    Rdb_string_reader *reader, bool decode) {
   // Get the number of bytes needed to store length
-  const uint length_bytes = blob->pack_length() - portable_sizeof_char_ptr;
+  const uint length_bytes =
+      field_dec->m_field_pack_length - portable_sizeof_char_ptr;
 
   const char *data_len_str;
   if (!(data_len_str = reader->read(length_bytes))) {
     return HA_ERR_ROCKSDB_CORRUPT_DATA;
   }
 
-  memcpy(blob->ptr, data_len_str, length_bytes);
-  uint32 data_len =
-      blob->get_length(reinterpret_cast<const uchar *>(data_len_str),
-                       length_bytes, table->s->db_low_byte_first);
+  memcpy(buf, data_len_str, length_bytes);
+  uint32 data_len = Field_blob::get_length(
+      reinterpret_cast<const uchar *>(data_len_str), length_bytes);
   const char *blob_ptr;
   if (!(blob_ptr = reader->read(data_len))) {
     return HA_ERR_ROCKSDB_CORRUPT_DATA;
@@ -145,8 +182,8 @@ int Rdb_convert_to_record_value_decoder::decode_blob(TABLE *table, Field *field,
   if (decode) {
     // set 8-byte pointer to 0, like innodb does (relevant for 32-bit
     // platforms)
-    memset(blob->ptr + length_bytes, 0, 8);
-    memcpy(blob->ptr + length_bytes, &blob_ptr, sizeof(uchar **));
+    memset(buf + length_bytes, 0, 8);
+    memcpy(buf + length_bytes, &blob_ptr, sizeof(uchar **));
   }
 
   return HA_EXIT_SUCCESS;
@@ -164,9 +201,9 @@ int Rdb_convert_to_record_value_decoder::decode_blob(TABLE *table, Field *field,
     other  HA_ERR error code (can be SE-specific)
 */
 int Rdb_convert_to_record_value_decoder::decode_fixed_length_field(
-    my_core::Field *const field, Rdb_field_encoder *field_dec,
+    uchar *const buf, Rdb_field_encoder *field_dec,
     Rdb_string_reader *const reader, bool decode) {
-  uint len = field_dec->m_pack_length_in_rec;
+  uint len = field_dec->m_field_pack_length;
   if (len > 0) {
     const char *data_bytes;
     if ((data_bytes = reader->read(len)) == nullptr) {
@@ -174,7 +211,7 @@ int Rdb_convert_to_record_value_decoder::decode_fixed_length_field(
     }
 
     if (decode) {
-      memcpy(field->ptr, data_bytes, len);
+      memcpy(buf, data_bytes, len);
     }
   }
 
@@ -192,24 +229,23 @@ int Rdb_convert_to_record_value_decoder::decode_fixed_length_field(
     other  HA_ERR error code (can be SE-specific)
 */
 int Rdb_convert_to_record_value_decoder::decode_varchar(
-    Field *field, Rdb_string_reader *const reader, bool decode) {
-  my_core::Field_varstring *const field_var = (my_core::Field_varstring *)field;
-
+    uchar *const buf, Rdb_field_encoder *field_dec,
+    Rdb_string_reader *const reader, bool decode) {
   const char *data_len_str;
-  if (!(data_len_str = reader->read(field_var->length_bytes))) {
+  if (!(data_len_str = reader->read(field_dec->m_field_length_bytes))) {
     return HA_ERR_ROCKSDB_CORRUPT_DATA;
   }
 
   uint data_len;
-  // field_var->length_bytes is 1 or 2
-  if (field_var->length_bytes == 1) {
+  // field_dec->length_bytes is 1 or 2
+  if (field_dec->m_field_length_bytes == 1) {
     data_len = (uchar)data_len_str[0];
   } else {
-    assert(field_var->length_bytes == 2);
+    assert(field_dec->m_field_length_bytes == 2);
     data_len = uint2korr(data_len_str);
   }
 
-  if (data_len > field_var->field_length) {
+  if (data_len > field_dec->m_field_length) {
     // The data on disk is longer than table DDL allows?
     return HA_ERR_ROCKSDB_CORRUPT_DATA;
   }
@@ -219,7 +255,7 @@ int Rdb_convert_to_record_value_decoder::decode_varchar(
   }
 
   if (decode) {
-    memcpy(field_var->ptr, data_len_str, field_var->length_bytes + data_len);
+    memcpy(buf, data_len_str, field_dec->m_field_length_bytes + data_len);
   }
 
   return HA_EXIT_SUCCESS;
@@ -239,7 +275,6 @@ Rdb_value_field_iterator<value_field_decoder>::Rdb_value_field_iterator(
   m_field_iter = fields->begin();
   m_field_end = fields->end();
   m_null_bytes = rdb_converter->get_null_bytes();
-  m_offset = 0;
 }
 
 // Iterate each requested field and decode one by one
@@ -250,21 +285,34 @@ int Rdb_value_field_iterator<value_field_decoder>::next() {
     m_field_dec = m_field_iter->m_field_enc;
     bool decode = m_field_iter->m_decode;
     bool maybe_null = m_field_dec->maybe_null();
-    // This is_null value is bind to how stroage format store its value
-    m_is_null = maybe_null && ((m_null_bytes[m_field_dec->m_null_offset] &
-                                m_field_dec->m_null_mask) != 0);
 
-    // Skip the bytes we need to skip
+    // For non-instant cols, Skip the bytes we need to skip
     int skip = m_field_iter->m_skip;
-    if (skip && !m_value_slice_reader->read(skip)) {
+    if (skip && !m_field_dec->m_is_instant_field &&
+        !m_value_slice_reader->read(skip)) {
+      assert(false);
       return HA_ERR_ROCKSDB_CORRUPT_DATA;
+    } else if (skip && m_field_dec->m_is_instant_field) {
+      // For instant cols, skip the bytes if record contains that cols data
+      m_value_slice_reader->read(
+          std::min<int32_t>(skip, m_value_slice_reader->remaining_bytes()));
+    }
+
+    // If a row is inserted after instant ddl, value_slice will contain instant
+    // column data
+    if (!m_field_dec->m_is_instant_field ||
+        m_value_slice_reader->remaining_bytes() > 0) {
+      // This is_null value is bind to how stroage format store its value
+      m_is_null = maybe_null && ((m_null_bytes[m_field_dec->m_null_offset] &
+                                  m_field_dec->m_null_mask) != 0);
+
+      // Decode each field
+      err = value_field_decoder::decode(
+          m_buf, m_table, m_field_dec, m_value_slice_reader, decode, m_is_null);
+    } else if (m_field_dec->m_is_instant_field && decode) {
+      err = value_field_decoder::decode_instant(m_buf, m_field_dec);
     }
 
-    m_field = m_table->field[m_field_dec->m_field_index];
-    // Decode each field
-    err = value_field_decoder::decode(m_buf, &m_offset, m_table, m_field,
-                                      m_field_dec, m_value_slice_reader, decode,
-                                      m_is_null);
     if (err != HA_EXIT_SUCCESS) {
       return err;
     }
@@ -282,18 +330,6 @@ bool Rdb_value_field_iterator<value_field_decoder>::end_of_fields() const {
   return m_field_iter == m_field_end;
 }
 
-template <typename value_field_decoder>
-Field *Rdb_value_field_iterator<value_field_decoder>::get_field() const {
-  assert(m_field != nullptr);
-  return m_field;
-}
-
-template <typename value_field_decoder>
-void *Rdb_value_field_iterator<value_field_decoder>::get_dst() const {
-  assert(m_buf != nullptr);
-  return m_buf + m_offset;
-}
-
 template <typename value_field_decoder>
 int Rdb_value_field_iterator<value_field_decoder>::get_field_index() const {
   assert(m_field_dec != nullptr);
@@ -309,7 +345,6 @@ enum_field_types Rdb_value_field_iterator<value_field_decoder>::get_field_type()
 
 template <typename value_field_decoder>
 bool Rdb_value_field_iterator<value_field_decoder>::is_null() const {
-  assert(m_field != nullptr);
   return m_is_null;
 }
 
@@ -320,7 +355,7 @@ bool Rdb_value_field_iterator<value_field_decoder>::is_null() const {
   @param    table      IN      Current open table
 */
 Rdb_converter::Rdb_converter(const THD *thd, const Rdb_tbl_def *tbl_def,
-                             TABLE *table)
+                             TABLE *table, const dd::Table *dd_table)
     : m_thd(thd), m_tbl_def(tbl_def), m_table(table) {
   assert(thd != nullptr);
   assert(tbl_def != nullptr);
@@ -331,14 +366,19 @@ Rdb_converter::Rdb_converter(const THD *thd, const Rdb_tbl_def *tbl_def,
   m_maybe_unpack_info = false;
   m_row_checksums_checked = 0;
   m_null_bytes = nullptr;
-  setup_field_encoders();
+  setup_field_encoders(dd_table);
+  m_lookup_bitmap = {nullptr, 0, 0, nullptr, nullptr};
 }
 
 Rdb_converter::~Rdb_converter() {
+  for (uint i = 0; i < m_table->s->fields; i++) {
+    my_free(m_encoder_arr[i].m_instant_default_value);
+  }
   my_free(m_encoder_arr);
   m_encoder_arr = nullptr;
   // These are needed to suppress valgrind errors in rocksdb.partition
   m_storage_record.mem_free();
+  bitmap_free(&m_lookup_bitmap);
 }
 
 /*
@@ -347,7 +387,8 @@ Rdb_converter::~Rdb_converter() {
 void Rdb_converter::get_storage_type(Rdb_field_encoder *const encoder,
                                      const uint kp) {
   auto pk_descr =
-      m_tbl_def->m_key_descr_arr[ha_rocksdb::pk_index(m_table, m_tbl_def)];
+      m_tbl_def->m_key_descr_arr[ha_rocksdb::pk_index(*m_table, *m_tbl_def)];
+
   // STORE_SOME uses unpack_info.
   if (pk_descr->has_unpack_info(kp)) {
     assert(pk_descr->can_unpack(kp));
@@ -363,31 +404,52 @@ void Rdb_converter::get_storage_type(Rdb_field_encoder *const encoder,
     Setup which fields will be unpacked when reading rows
 
   @detail
-    Three special cases when we still unpack all fields:
+    Two special cases when we still unpack all fields:
     - When client requires decode_all_fields, such as this table is being
   updated (m_lock_rows==RDB_LOCK_WRITE).
-    - When @@rocksdb_verify_row_debug_checksums is ON (In this mode, we need to
-  read all fields to find whether there is a row checksum at the end. We could
-  skip the fields instead of decoding them, but currently we do decoding.)
-    - On index merge as bitmap is cleared during that operation
+    - When @@rocksdb_verify_row_debug_checksums is ON (In this mode, we need
+  to read all fields to find whether there is a row checksum at the end. We
+  could skip the fields instead of decoding them, but currently we do
+  decoding.)
 
   @seealso
     Rdb_converter::setup_field_encoders()
     Rdb_converter::convert_record_from_storage_format()
 */
 void Rdb_converter::setup_field_decoders(const MY_BITMAP *field_map,
+                                         uint active_index, bool keyread_only,
                                          bool decode_all_fields) {
+  DBUG_TRACE;
   m_key_requested = false;
   m_decoders_vect.clear();
+  bitmap_free(&m_lookup_bitmap);
   int last_useful = 0;
   int skip_size = 0;
 
+  // We also have to load the fields that are required to decode the requested
+  // virtual fields.
+
+  std::vector<bool> bases(m_table->s->fields);
+
+  for (uint i = 0; i < m_table->s->fields; i++) {
+    const bool field_requested =
+        decode_all_fields || m_verify_row_debug_checksums ||
+        bitmap_is_set(field_map, m_table->field[i]->field_index());
+
+    if (field_requested && m_table->field[i]->is_virtual_gcol()) {
+      for (uint j = 0; j < m_table->s->fields; j++) {
+        if (bitmap_is_set(&m_table->field[i]->gcol_info->base_columns_map, j)) {
+          bases[j] = true;
+        }
+      }
+    }
+  }
+
   for (uint i = 0; i < m_table->s->fields; i++) {
-    // bitmap is cleared on index merge, but it still needs to decode columns
     bool field_requested =
         decode_all_fields || m_verify_row_debug_checksums ||
-        bitmap_is_clear_all(field_map) ||
-        bitmap_is_set(field_map, m_table->field[i]->field_index);
+        (bitmap_is_set(field_map, m_table->field[i]->field_index())) ||
+        bases[i];
 
     // We only need the decoder if the whole record is stored.
     if (m_encoder_arr[i].m_storage_type != Rdb_field_encoder::STORE_ALL) {
@@ -413,7 +475,7 @@ void Rdb_converter::setup_field_decoders(const MY_BITMAP *field_map,
       } else {
         // Fixed-width field can be skipped without looking at it.
         // Add appropriate skip_size to the next field.
-        skip_size += m_encoder_arr[i].m_pack_length_in_rec;
+        skip_size += m_encoder_arr[i].m_field_pack_length;
       }
     }
   }
@@ -422,9 +484,14 @@ void Rdb_converter::setup_field_decoders(const MY_BITMAP *field_map,
   // skipping. Remove them.
   m_decoders_vect.erase(m_decoders_vect.begin() + last_useful,
                         m_decoders_vect.end());
+
+  if (!keyread_only && active_index != m_table->s->primary_key) {
+    m_tbl_def->m_key_descr_arr[active_index]->get_lookup_bitmap(
+        m_table, &m_lookup_bitmap);
+  }
 }
 
-void Rdb_converter::setup_field_encoders() {
+void Rdb_converter::setup_field_encoders(const dd::Table *dd_table) {
   uint null_bytes_length = 0;
   uchar cur_null_mask = 0x1;
 
@@ -440,6 +507,19 @@ void Rdb_converter::setup_field_encoders() {
     return;
   }
 
+  // whether table contains instant cols
+  bool is_instant_table = false;
+  // num of cols before first instant cols append
+  uint instant_cols = 0;
+  if (!rocksdb_disable_instant_ddl && dd_table != nullptr) {
+    // check whether table contain instant col
+    is_instant_table = dd_table_has_instant_cols(*dd_table);
+    if (is_instant_table) {
+      dd_table->se_private_data().get(
+          dd_table_key_strings[DD_TABLE_INSTANT_COLS], &instant_cols);
+    }
+  }
+
   for (uint i = 0; i < m_table->s->fields; i++) {
     Field *const field = m_table->field[i];
     m_encoder_arr[i].m_storage_type = Rdb_field_encoder::STORE_ALL;
@@ -454,24 +534,64 @@ void Rdb_converter::setup_field_encoders() {
       If hidden pk exists, we skip this check since the field will never be
       part of the hidden pk.
     */
-    if (!Rdb_key_def::table_has_hidden_pk(m_table)) {
+    if (!Rdb_key_def::table_has_hidden_pk(*m_table)) {
       KEY *const pk_info = &m_table->key_info[m_table->s->primary_key];
       for (uint kp = 0; kp < pk_info->user_defined_key_parts; kp++) {
         // key_part->fieldnr is counted from 1
-        if (field->field_index + 1 == pk_info->key_part[kp].fieldnr) {
+        if (field->field_index() + 1 == pk_info->key_part[kp].fieldnr) {
           get_storage_type(&m_encoder_arr[i], kp);
           break;
         }
       }
     }
 
-    m_encoder_arr[i].m_field_type = field->real_type();
-    m_encoder_arr[i].m_field_index = i;
-    m_encoder_arr[i].m_pack_length_in_rec = field->pack_length_in_rec();
+    /*
+      The difference between pack_length and pack_length_in_rec is fairly
+      subtle. The only difference is in Field_bit case where it borrows some
+      bits in null bytes in memory to store the 'uneven' high bits, therefore
+      the pack_length is the length of remaining bits while the
+      pack_length_in_rec is the full length of all bits when you store it on
+      disk. Only MyIsam and archive supports it, indicating by
+      HA_CAN_BIT_FIELD. We don't handle this case today at all (nor do we need
+      to), and we use pack_length everywhere, so just assert it and move on.
+    */
+    assert(field->pack_length() == field->pack_length_in_rec());
 
-    if (field->real_maybe_null()) {
+    auto field_type = field->real_type();
+    m_encoder_arr[i].m_field_type = field_type;
+    m_encoder_arr[i].m_field_index = i;
+    m_encoder_arr[i].m_field_pack_length = field->pack_length();
+    m_encoder_arr[i].m_field_offset = field->field_ptr() - m_table->record[0];
+    m_encoder_arr[i].m_field_type = field_type;
+    m_encoder_arr[i].m_is_virtual_gcol = field->is_virtual_gcol();
+
+    if (field_type == MYSQL_TYPE_VARCHAR) {
+      auto varchar = reinterpret_cast<const Field_varstring *>(field);
+      m_encoder_arr[i].m_field_length = varchar->field_length;
+      m_encoder_arr[i].m_field_length_bytes = varchar->get_length_bytes();
+    } else {
+      m_encoder_arr[i].m_field_length = -1;
+      m_encoder_arr[i].m_field_length_bytes = -1;
+    }
+
+    m_encoder_arr[i].m_is_instant_field = false;
+    m_encoder_arr[i].m_instant_default_value = nullptr;
+    // currently instant ddl only support append column
+    if (is_instant_table && i >= instant_cols) {
+      m_encoder_arr[i].m_is_instant_field = true;
+      dd::Column *column =
+          const_cast<dd::Column *>(dd_find_column(dd_table, field->field_name));
+      dd_table_get_instant_default(
+          *column, &m_encoder_arr[i].m_instant_default_value,
+          &m_encoder_arr[i].m_instant_default_value_len);
+    }
+
+    auto maybe_null = field->is_nullable();
+    if (maybe_null) {
       m_encoder_arr[i].m_null_mask = cur_null_mask;
       m_encoder_arr[i].m_null_offset = null_bytes_length;
+      m_encoder_arr[i].m_field_null_offset = field->null_offset();
+      m_encoder_arr[i].m_field_null_mask = field->null_bit;
       if (cur_null_mask == 0x80) {
         cur_null_mask = 0x1;
         null_bytes_length++;
@@ -479,6 +599,7 @@ void Rdb_converter::setup_field_encoders() {
         cur_null_mask = cur_null_mask << 1;
       }
     } else {
+      m_encoder_arr[i].m_null_offset = 0;
       m_encoder_arr[i].m_null_mask = 0;
     }
   }
@@ -489,6 +610,14 @@ void Rdb_converter::setup_field_encoders() {
   }
 
   m_null_bytes_length_in_record = null_bytes_length;
+  // Current myrocks store 1 bit per nullable field in null flags
+  // if later myrocks changes to use different length null bytes, please also
+  // update ha_rocksdb::rocksdb_support_instant()
+  // For temporary table, its null_fields value maybe different than normal
+  // table. see create_tmp_table()
+  assert(m_table->s->table_category == TABLE_CATEGORY_TEMPORARY ||
+         ceil((double)m_table->s->null_fields / 8) ==
+             (uint)m_null_bytes_length_in_record);
 }
 
 /*
@@ -506,10 +635,11 @@ void Rdb_converter::setup_field_encoders() {
 int Rdb_converter::decode(const std::shared_ptr<Rdb_key_def> &key_def,
                           uchar *dst,  // address to fill data
                           const rocksdb::Slice *key_slice,
-                          const rocksdb::Slice *value_slice) {
+                          const rocksdb::Slice *value_slice,
+                          bool decode_value) {
   // Currently only support decode primary key, Will add decode secondary later
   assert(key_def->m_index_type == Rdb_key_def::INDEX_TYPE_PRIMARY ||
-              key_def->m_index_type == Rdb_key_def::INDEX_TYPE_HIDDEN_PRIMARY);
+         key_def->m_index_type == Rdb_key_def::INDEX_TYPE_HIDDEN_PRIMARY);
 
   const rocksdb::Slice *updated_key_slice = key_slice;
 #ifndef NDEBUG
@@ -521,11 +651,11 @@ int Rdb_converter::decode(const std::shared_ptr<Rdb_key_def> &key_def,
   updated_key_slice = &rowkey_slice;
 #endif
   return convert_record_from_storage_format(key_def, updated_key_slice,
-                                            value_slice, dst);
+                                            value_slice, dst, decode_value);
 }
 
 /*
-  Decode value slice header
+  Decode value slice header for pk
   @param    reader         IN          value slice reader
   @param    pk_def         IN          key definition to decode
   @param    unpack_slice   OUT         unpack info slice
@@ -533,7 +663,7 @@ int Rdb_converter::decode(const std::shared_ptr<Rdb_key_def> &key_def,
     0      OK
     other  HA_ERR error code (can be SE-specific)
 */
-int Rdb_converter::decode_value_header(
+int Rdb_converter::decode_value_header_for_pk(
     Rdb_string_reader *reader, const std::shared_ptr<Rdb_key_def> &pk_def,
     rocksdb::Slice *unpack_slice) {
   /* If it's a TTL record, skip the 8 byte TTL value */
@@ -583,12 +713,18 @@ int Rdb_converter::decode_value_header(
 int Rdb_converter::convert_record_from_storage_format(
     const std::shared_ptr<Rdb_key_def> &pk_def,
     const rocksdb::Slice *const key_slice,
-    const rocksdb::Slice *const value_slice, uchar *const dst) {
+    const rocksdb::Slice *const value_slice, uchar *const dst,
+    bool decode_value = true) {
+  bool skip_value = !decode_value || get_decode_fields()->size() == 0;
+  if (!m_key_requested && skip_value) {
+    return HA_EXIT_SUCCESS;
+  }
+
   int err = HA_EXIT_SUCCESS;
 
   Rdb_string_reader value_slice_reader(value_slice);
   rocksdb::Slice unpack_slice;
-  err = decode_value_header(&value_slice_reader, pk_def, &unpack_slice);
+  err = decode_value_header_for_pk(&value_slice_reader, pk_def, &unpack_slice);
   if (err != HA_EXIT_SUCCESS) {
     return err;
   }
@@ -600,9 +736,14 @@ int Rdb_converter::convert_record_from_storage_format(
     err = pk_def->unpack_record(m_table, dst, key_slice,
                                 !unpack_slice.empty() ? &unpack_slice : nullptr,
                                 false /* verify_checksum */);
+    if (err != HA_EXIT_SUCCESS) {
+      return err;
+    }
   }
-  if (err != HA_EXIT_SUCCESS) {
-    return err;
+
+  if (skip_value) {
+    // We are done
+    return HA_EXIT_SUCCESS;
   }
 
   Rdb_value_field_iterator<Rdb_convert_to_record_value_decoder>
@@ -650,6 +791,7 @@ int Rdb_converter::verify_row_debug_checksum(
         my_core::my_checksum(0, rdb_slice_to_uchar_ptr(value),
                              value->size() - RDB_CHECKSUM_CHUNK_SIZE);
 
+    DEBUG_SYNC(const_cast<THD *>(m_thd), "myrocks_verify_row_debug_checksum");
     DBUG_EXECUTE_IF("myrocks_simulate_bad_pk_checksum1", stored_key_chksum++;);
 
     if (stored_key_chksum != computed_key_chksum) {
@@ -692,7 +834,7 @@ int Rdb_converter::encode_value_slice(
   assert(pk_def != nullptr);
   // Currently only primary key will store value slice
   assert(pk_def->m_index_type == Rdb_key_def::INDEX_TYPE_PRIMARY ||
-              pk_def->m_index_type == Rdb_key_def::INDEX_TYPE_HIDDEN_PRIMARY);
+         pk_def->m_index_type == Rdb_key_def::INDEX_TYPE_HIDDEN_PRIMARY);
   assert_IMP(m_maybe_unpack_info, pk_unpack_info);
 
   bool has_ttl = pk_def->has_ttl();
@@ -713,10 +855,30 @@ int Rdb_converter::encode_value_slice(
     if (has_ttl_column) {
       assert(pk_def->get_ttl_field_index() != UINT_MAX);
       Field *const field = m_table->field[pk_def->get_ttl_field_index()];
-      assert(field->pack_length_in_rec() == ROCKSDB_SIZEOF_TTL_RECORD);
-      assert(field->real_type() == MYSQL_TYPE_LONGLONG);
+      assert(field->real_type() == MYSQL_TYPE_LONGLONG ||
+             field->type() == MYSQL_TYPE_TIMESTAMP);
+
+      uint64 ts;
+
+      if (field->type() == MYSQL_TYPE_TIMESTAMP) {
+        /*
+          Timestamp packed length can be 4 to 7 bytes depending on precision.
+          The first four bytes is the unix time network byte order and is used
+          for MYSQL_TYPE_TIMESTAMP and MYSQL_TYPE_TIMESTAMP2. The optional
+          remaining bytes are for fractional seconds. Only the first four bytes
+          are needed.
+         */
+        assert(field->real_type() == MYSQL_TYPE_TIMESTAMP ||
+               field->real_type() == MYSQL_TYPE_TIMESTAMP2);
+        assert(field->pack_length_in_rec() >= 4);
+        assert(field->key_type() == HA_KEYTYPE_BINARY);
+        ts = (uint64)rdb_netbuf_to_uint32(field->field_ptr());
+      } else {
+        assert(field->key_type() == HA_KEYTYPE_ULONGLONG);
+        assert(field->pack_length_in_rec() == ROCKSDB_SIZEOF_TTL_RECORD);
+        ts = uint8korr(field->field_ptr());
+      }
 
-      uint64 ts = uint8korr(field->ptr);
 #ifndef NDEBUG
       ts += rdb_dbug_set_ttl_rec_ts();
 #endif
@@ -767,6 +929,10 @@ int Rdb_converter::encode_value_slice(
     }
 
     Field *const field = m_table->field[i];
+
+    if (field->is_virtual_gcol()) {
+      continue;
+    }
     if (encoder.maybe_null()) {
       char *data = const_cast<char *>(m_storage_record.ptr());
       if (has_ttl) {
@@ -788,34 +954,37 @@ int Rdb_converter::encode_value_slice(
       const uint length_bytes = blob->pack_length() - portable_sizeof_char_ptr;
 
       /* Store the length of the value */
-      m_storage_record.append(reinterpret_cast<char *>(blob->ptr),
+      m_storage_record.append(reinterpret_cast<char *>(blob->field_ptr()),
                               length_bytes);
 
       /* Store the blob value itself */
       char *data_ptr;
-      memcpy(&data_ptr, blob->ptr + length_bytes, sizeof(uchar **));
+      memcpy(&data_ptr, blob->field_ptr() + length_bytes, sizeof(uchar **));
       m_storage_record.append(data_ptr, blob->get_length());
     } else if (encoder.m_field_type == MYSQL_TYPE_VARCHAR) {
       Field_varstring *const field_var =
           reinterpret_cast<Field_varstring *>(field);
       uint data_len;
       /* field_var->length_bytes is 1 or 2 */
-      if (field_var->length_bytes == 1) {
-        data_len = field_var->ptr[0];
+      if (field_var->get_length_bytes() == 1) {
+        data_len = field_var->field_ptr()[0];
       } else {
-        assert(field_var->length_bytes == 2);
-        data_len = uint2korr(field_var->ptr);
+        assert(field_var->get_length_bytes() == 2);
+        data_len = uint2korr(field_var->field_ptr());
       }
-      m_storage_record.append(reinterpret_cast<char *>(field_var->ptr),
-                              field_var->length_bytes + data_len);
+      m_storage_record.append(reinterpret_cast<char *>(field_var->field_ptr()),
+                              field_var->get_length_bytes() + data_len);
     } else {
       /* Copy the field data */
-      const uint len = field->pack_length_in_rec();
-      m_storage_record.append(reinterpret_cast<char *>(field->ptr), len);
+      const uint len = field->pack_length();
+      m_storage_record.append(reinterpret_cast<char *>(field->field_ptr()),
+                              len);
     }
   }
 
   if (store_row_debug_checksums) {
+    DEBUG_SYNC(const_cast<THD *>(m_thd), "rocksdb.encode_value_slice");
+
     const ha_checksum key_crc32 = my_core::my_checksum(
         0, rdb_slice_to_uchar_ptr(&pk_packed_slice), pk_packed_slice.size());
     const ha_checksum val_crc32 =
