@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All Rights Reserved.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -23,39 +24,42 @@ this program; if not, write to the Free Software Foundation, Inc.,
 /** @file storage/temptable/src/handler.cc
 TempTable public handler API implementation. */
 
-#include <limits>
-#include <new>
-#include <unordered_map>
+#include "storage/temptable/include/temptable/handler.h"
 
-#ifndef DBUG_OFF
-#include <thread>
-#endif
+#include <string.h>
 
 #include "my_base.h"
 #include "my_dbug.h"
-#include "sql/handler.h"
+#include "mysql/plugin.h"
 #include "sql/mysqld.h"
+#include "sql/sql_thd_internal_api.h"
 #include "sql/system_variables.h"
-#include "sql/table.h"
-#include "storage/temptable/include/temptable/handler.h"
 #include "storage/temptable/include/temptable/row.h"
-#include "storage/temptable/include/temptable/storage.h"
-#include "storage/temptable/include/temptable/table.h"
-#include "storage/temptable/include/temptable/test.h"
+#include "storage/temptable/include/temptable/sharded_kv_store.h"
+#include "storage/temptable/include/temptable/shared_block_pool.h"
 
 namespace temptable {
 
-/** A container for the list of the tables. Don't allocate memory for it from
- * the Allocator because the Allocator keeps one block for reuse and it is
- * only marked for reuse after all elements from it have been removed. This
- * container, being a global variable may allocate some memory and never free
- * it before its destructor is called at thread termination time. */
-using Tables = std::unordered_map<std::string, Table>;
+/** Key-value store containing all tables for all existing connections.
+ * See `Sharded_key_value_store` documentation for more details.
+ * */
+static Sharded_key_value_store<KV_STORE_SHARDS_COUNT> kv_store_shard;
 
-/** A list of the tables that currently exist for this OS thread. */
-static Tables &TlsTables() {
-  static thread_local Tables tls_tables;
-  return tls_tables;
+/* Pool of shared-blocks, an external state to the custom `TempTable` memory
+ * allocator. See `Lock_free_shared_block_pool` documentation for more details.
+ * */
+static Lock_free_shared_block_pool<SHARED_BLOCK_POOL_SIZE> shared_block_pool;
+
+/** Small helper function which debug-prints the miscellaneous statistics which
+ * key-value store has collected.
+ * */
+void kv_store_shards_debug_dump() { kv_store_shard.dbug_print(); }
+
+/** Small helper function which releases the slot (and memory occupied by the
+ * Block) in shared-block pool.
+ * */
+void shared_block_pool_release(THD *thd) {
+  shared_block_pool.try_release(thd_thread_id(thd));
 }
 
 #if defined(HAVE_WINNUMA)
@@ -63,11 +67,12 @@ static Tables &TlsTables() {
 DWORD win_page_size;
 #endif /* HAVE_WINNUMA */
 
-#define DBUG_RET(result) DBUG_RETURN(static_cast<int>(result))
+#define DBUG_RET(result) return static_cast<int>(result)
 
-Handler::Handler(handlerton *hton, TABLE_SHARE *table_share)
-    : ::handler(hton, table_share),
+Handler::Handler(handlerton *hton, TABLE_SHARE *table_share_arg)
+    : ::handler(hton, table_share_arg),
       m_opened_table(),
+      m_shared_block(shared_block_pool.try_acquire(thd_thread_id(ha_thd()))),
       m_rnd_iterator(),
       m_rnd_iterator_is_positioned(),
       m_index_cursor(),
@@ -75,95 +80,98 @@ Handler::Handler(handlerton *hton, TABLE_SHARE *table_share)
       m_deleted_rows() {
   handler::ref_length = sizeof(Storage::Element *);
 
+  // Overriding `handlerton::` with a function-pointer in `TempTable`, or
+  // any other plugin, is not always sufficient for server to actually invoke
+  // that hook. This is the behavior which is in contrast to what one would
+  // expect and this is not how most handlerton interfaces work. However, there
+  // is a subset of handlerton interfaces which require special care and in
+  // addition to overriding the function-pointer, they require initializing
+  // `handlerton::ha_data` with some data which is not a `nullptr`.
+  //
+  // `handlerton::close_connection()` is one of such interfaces, and being
+  // relied upon by the `TempTable` implementation, we have to fill in the
+  // `ha_data` with some existing data. In this particular case,
+  // `&shared_block_pool` is selected but from `TempTable` PoV this does not
+  // have any semantic value, e.g. it could very well have been something else.
+  //
+  // This is a bit confusing and unexpected, and hence this comment to make life
+  // easier for readers of this part of the code.
+  thd_set_ha_data(ha_thd(), hton, &shared_block_pool);
+
 #if defined(HAVE_WINNUMA)
   SYSTEM_INFO systemInfo;
   GetSystemInfo(&systemInfo);
 
   win_page_size = systemInfo.dwPageSize;
 #endif /* HAVE_WINNUMA */
-
-#ifndef DBUG_OFF
-  m_owner = std::this_thread::get_id();
-#endif /* DBUG_OFF */
 }
 
-Handler::~Handler() {}
-
 int Handler::create(const char *table_name, TABLE *mysql_table,
                     HA_CREATE_INFO *, dd::Table *) {
-  DBUG_ENTER("temptable::Handler::create");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
-  DBUG_ASSERT(mysql_table != nullptr);
-  DBUG_ASSERT(mysql_table->s != nullptr);
-  DBUG_ASSERT(mysql_table->field != nullptr);
-  DBUG_ASSERT(table_name != nullptr);
-
-#ifdef TEMPTABLE_CPP_HOOKED_TESTS
-  /* To run this test:
-   * - CREATE TABLE t (__temptable_embedded_unit_tests CHAR(120) NOT NULL);
-   *   (the SELECT below will create a temporary table with t's structure
-   *    plus one unique hash index on the column)
-   * - SELECT DISTINCT * FROM t; */
-  if (mysql_table->s->fields == 1 &&
-      strcmp(mysql_table->field[0]->field_name,
-             "__temptable_embedded_unit_tests") == 0) {
-    test(mysql_table);
-  }
-#endif /* TEMPTABLE_CPP_HOOKED_TESTS */
+  assert(mysql_table != nullptr);
+  assert(mysql_table->s != nullptr);
+  assert(mysql_table->field != nullptr);
+  assert(table_name != nullptr);
 
   bool all_columns_are_fixed_size = true;
   for (uint i = 0; i < mysql_table->s->fields; ++i) {
     Field *mysql_field = mysql_table->field[i];
-    DBUG_ASSERT(mysql_field != nullptr);
+    assert(mysql_field != nullptr);
     if (!is_field_type_fixed_size(*mysql_field)) {
       all_columns_are_fixed_size = false;
       break;
     }
   }
 
-  Result ret = Result::OUT_OF_MEM;
+  Result ret;
 
   try {
-    // clang-format off
-    DBUG_EXECUTE_IF(
-        "temptable_create_return_full",
-        ret = Result::RECORD_FILE_FULL;
-        throw std::bad_alloc();
-    );
-    // clang-format on
+    DBUG_EXECUTE_IF("temptable_create_return_full",
+                    throw Result::RECORD_FILE_FULL;);
+    DBUG_EXECUTE_IF("temptable_create_return_non_result_type_exception",
+                    throw 42;);
 
-    const auto insert_result = TlsTables().emplace(
-        std::piecewise_construct, std::make_tuple(table_name),
-        std::forward_as_tuple(mysql_table, all_columns_are_fixed_size));
+    // Calculate m_number_of_elements_per_page, see Table::Table():
+    if (all_columns_are_fixed_size) {
+      Storage rows_of_the_table = Storage(nullptr);
+      rows_of_the_table.element_size(mysql_table->s->rec_buff_length);
+      if (rows_of_the_table.number_of_elements_per_page() == 0)
+        DBUG_RET(Result::TOO_BIG_ROW);
+    }
+
+    size_t per_table_limit = thd_get_tmp_table_size(ha_thd());
+    auto &kv_store = kv_store_shard[thd_thread_id(ha_thd())];
+    const auto insert_result = kv_store.emplace(
+        std::piecewise_construct, std::forward_as_tuple(table_name),
+        std::forward_as_tuple(mysql_table, m_shared_block,
+                              all_columns_are_fixed_size, per_table_limit));
 
     ret = insert_result.second ? Result::OK : Result::TABLE_EXIST;
-  } catch (...) {
-    /* ret is already set above. */
-  }
 
-  DBUG_PRINT("temptable_api",
-             ("this=%p %s; return=%s", this,
-              table_definition(table_name, mysql_table).c_str(),
-              result_to_string(ret)));
+  } catch (Result ex) {
+    ret = ex;
+  } catch (...) {
+    ret = Result::OUT_OF_MEM;
+  }
 
   DBUG_RET(ret);
 }
 
 int Handler::delete_table(const char *table_name, const dd::Table *) {
-  DBUG_ENTER("temptable::Handler::delete_table");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
-  DBUG_ASSERT(table_name != nullptr);
+  assert(table_name != nullptr);
 
   Result ret;
 
   try {
-    const auto pos = TlsTables().find(table_name);
-
-    if (pos != TlsTables().end()) {
-      if (&pos->second != m_opened_table) {
-        TlsTables().erase(pos);
+    auto &kv_store = kv_store_shard[thd_thread_id(ha_thd())];
+    auto table = kv_store.find(table_name);
+    if (table) {
+      if (m_opened_table != table) {
+        kv_store.erase(table_name);
         ret = Result::OK;
       } else {
         /* Attempt to delete the currently opened table. */
@@ -185,25 +193,25 @@ int Handler::delete_table(const char *table_name, const dd::Table *) {
 }
 
 int Handler::open(const char *table_name, int, uint, const dd::Table *) {
-  DBUG_ENTER("temptable::Handler::open");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
-  DBUG_ASSERT(m_opened_table == nullptr);
-  DBUG_ASSERT(table_name != nullptr);
-  DBUG_ASSERT(!m_rnd_iterator_is_positioned);
-  DBUG_ASSERT(!m_index_cursor.is_positioned());
-  DBUG_ASSERT(handler::active_index == MAX_KEY);
+  assert(m_opened_table == nullptr);
+  assert(table_name != nullptr);
+  assert(!m_rnd_iterator_is_positioned);
+  assert(!m_index_cursor.is_positioned());
+  assert(handler::active_index == MAX_KEY);
 
   Result ret;
 
   try {
-    Tables::iterator iter = TlsTables().find(table_name);
-    if (iter == TlsTables().end()) {
-      ret = Result::NO_SUCH_TABLE;
-    } else {
-      m_opened_table = &iter->second;
-      opened_table_validate();
+    auto &kv_store = kv_store_shard[thd_thread_id(ha_thd())];
+    m_opened_table = kv_store.find(table_name);
+    if (m_opened_table) {
       ret = Result::OK;
+      opened_table_validate();
+      info(HA_STATUS_VARIABLE);
+    } else {
+      ret = Result::NO_SUCH_TABLE;
     }
   } catch (std::bad_alloc &) {
     ret = Result::OUT_OF_MEM;
@@ -216,10 +224,9 @@ int Handler::open(const char *table_name, int, uint, const dd::Table *) {
 }
 
 int Handler::close() {
-  DBUG_ENTER("temptable::Handler::close");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
-  DBUG_ASSERT(m_opened_table != nullptr);
+  assert(m_opened_table != nullptr);
 
   m_opened_table = nullptr;
 
@@ -237,9 +244,7 @@ int Handler::close() {
 }
 
 int Handler::rnd_init(bool) {
-  DBUG_ENTER("temptable::Handler::rnd_init");
-
-  DBUG_ASSERT(current_thread_is_creator());
+  DBUG_TRACE;
 
   m_rnd_iterator_is_positioned = false;
 
@@ -253,9 +258,8 @@ int Handler::rnd_init(bool) {
 }
 
 int Handler::rnd_next(uchar *mysql_row) {
-  DBUG_ENTER("temptable::Handler::rnd_next");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_read_rnd_next_count);
@@ -275,7 +279,7 @@ int Handler::rnd_next(uchar *mysql_row) {
       ret = Result::END_OF_FILE;
     }
   } else {
-    DBUG_ASSERT(m_rnd_iterator != rows.end());
+    assert(m_rnd_iterator != rows.end());
     Storage::Element *previous = *m_rnd_iterator;
     ++m_rnd_iterator;
     if (m_rnd_iterator != rows.end()) {
@@ -305,25 +309,18 @@ int Handler::rnd_next(uchar *mysql_row) {
     }
   }
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p out=(%s); return=%s", this,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::rnd_pos(uchar *mysql_row, uchar *position) {
-  DBUG_ENTER("temptable::Handler::rnd_pos");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_read_rnd_count);
 
-  Storage::Element *row = *reinterpret_cast<Storage::Element **>(position);
+  Storage::Element *row;
+  memcpy(&row, position, sizeof(row));
 
   m_rnd_iterator = Storage::Iterator(&m_opened_table->rows(), row);
 
@@ -334,18 +331,11 @@ int Handler::rnd_pos(uchar *mysql_row, uchar *position) {
   /* Marked unused - temporary fix for GCC 8 bug 82728. */
   const Result ret TEMPTABLE_UNUSED = Result::OK;
 
-  DBUG_PRINT("temptable_api",
-             ("this=%p position=%p out=(%s); return=%s", this, position,
-              row_to_string(mysql_row, handler::table).c_str(),
-              result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::rnd_end() {
-  DBUG_ENTER("temptable::Handler::rnd_end");
-
-  DBUG_ASSERT(current_thread_is_creator());
+  DBUG_TRACE;
 
   m_rnd_iterator_is_positioned = false;
 
@@ -359,9 +349,8 @@ int Handler::rnd_end() {
 }
 
 int Handler::index_init(uint index_no, bool) {
-  DBUG_ENTER("temptable::Handler::index_init");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   Result ret;
@@ -382,14 +371,13 @@ int Handler::index_init(uint index_no, bool) {
 int Handler::index_read(uchar *mysql_row, const uchar *mysql_search_cells,
                         uint mysql_search_cells_len_bytes,
                         ha_rkey_function find_flag) {
-  DBUG_ENTER("temptable::Handler::index_read");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_read_key_count);
 
-  DBUG_ASSERT(handler::active_index < m_opened_table->number_of_indexes());
+  assert(handler::active_index < m_opened_table->number_of_indexes());
 
   Result ret = Result::UNSUPPORTED;
 
@@ -487,44 +475,24 @@ int Handler::index_read(uchar *mysql_row, const uchar *mysql_search_cells,
     ret = Result::OUT_OF_MEM;
   }
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p cells=(%s) cells_len=%u find_flag=%s out=(%s); return=%s", this,
-       indexed_cells_to_string(mysql_search_cells, mysql_search_cells_len_bytes,
-                               handler::table->key_info[handler::active_index])
-           .c_str(),
-       mysql_search_cells_len_bytes, ha_rkey_function_to_str(find_flag),
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::index_next(uchar *mysql_row) {
-  DBUG_ENTER("temptable::Handler::index_next");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_read_next_count);
 
   const Result ret = index_next_conditional(mysql_row, NextCondition::NO);
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p out=(%s); return=%s", this,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::index_next_same(uchar *mysql_row, const uchar *, uint) {
-  DBUG_ENTER("temptable::Handler::index_next_same");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_read_next_count);
@@ -532,23 +500,16 @@ int Handler::index_next_same(uchar *mysql_row, const uchar *, uint) {
   const Result ret =
       index_next_conditional(mysql_row, NextCondition::ONLY_IF_SAME);
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p out=(%s); return=%s", this,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 Result Handler::index_next_conditional(uchar *mysql_row,
                                        NextCondition condition) {
-  DBUG_ENTER("temptable::Handler::index_next_conditional");
+  DBUG_TRACE;
 
   opened_table_validate();
 
-  DBUG_ASSERT(m_index_cursor.is_positioned());
+  assert(m_index_cursor.is_positioned());
 
   Result ret;
 
@@ -562,8 +523,8 @@ Result Handler::index_next_conditional(uchar *mysql_row,
     } else {
       Indexed_cells indexed_cells_previous = m_index_cursor.indexed_cells();
       /* Lower the number of cells to what was given to `index_read()`. */
-      DBUG_ASSERT(m_index_read_number_of_cells <=
-                  indexed_cells_previous.number_of_cells());
+      assert(m_index_read_number_of_cells <=
+             indexed_cells_previous.number_of_cells());
       indexed_cells_previous.number_of_cells(m_index_read_number_of_cells);
 
       ++m_index_cursor;
@@ -609,48 +570,28 @@ Result Handler::index_next_conditional(uchar *mysql_row,
     ret = Result::OUT_OF_MEM;
   }
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p out=(%s); return=%s", this,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 int Handler::index_read_last(uchar *mysql_row, const uchar *mysql_search_cells,
                              uint mysql_search_cells_len_bytes) {
-  DBUG_ENTER("temptable::Handler::index_read_last");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   const Result ret = static_cast<Result>(
       index_read(mysql_row, mysql_search_cells, mysql_search_cells_len_bytes,
                  HA_READ_PREFIX_LAST));
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p cells=(%s) cells_len=%u out=(%s); return=%s", this,
-       indexed_cells_to_string(mysql_search_cells, mysql_search_cells_len_bytes,
-                               handler::table->key_info[handler::active_index])
-           .c_str(),
-       mysql_search_cells_len_bytes,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::index_prev(uchar *mysql_row) {
-  DBUG_ENTER("temptable::Handler::index_prev");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
-  DBUG_ASSERT(m_index_cursor.is_positioned());
+  assert(m_index_cursor.is_positioned());
 
   handler::ha_statistic_increment(&System_status_var::ha_read_prev_count);
 
@@ -676,20 +617,11 @@ int Handler::index_prev(uchar *mysql_row) {
     ret = Result::OUT_OF_MEM;
   }
 
-  DBUG_PRINT(
-      "temptable_api",
-      ("this=%p out=(%s); return=%s", this,
-       (ret == Result::OK ? row_to_string(mysql_row, handler::table).c_str()
-                          : ""),
-       result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::index_end() {
-  DBUG_ENTER("temptable::Handler::index_end");
-
-  DBUG_ASSERT(current_thread_is_creator());
+  DBUG_TRACE;
 
   handler::active_index = MAX_KEY;
 
@@ -705,48 +637,40 @@ int Handler::index_end() {
 }
 
 void Handler::position(const uchar *) {
-  DBUG_ENTER("temptable::Handler::position");
-
-  DBUG_ASSERT(current_thread_is_creator());
+  DBUG_TRACE;
 
   Storage::Element *row;
 
   if (m_rnd_iterator_is_positioned) {
-    DBUG_ASSERT(!m_index_cursor.is_positioned());
+    assert(!m_index_cursor.is_positioned());
     row = *m_rnd_iterator;
   } else {
-    DBUG_ASSERT(m_index_cursor.is_positioned());
+    assert(m_index_cursor.is_positioned());
     row = m_index_cursor.row();
   }
 
   *reinterpret_cast<Storage::Element **>(handler::ref) = row;
 
   DBUG_PRINT("temptable_api", ("this=%p; saved position=%p", this, row));
-
-  DBUG_VOID_RETURN;
 }
 
 int Handler::write_row(uchar *mysql_row) {
-  DBUG_ENTER("temptable::Handler::write_row");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_write_count);
 
   const Result ret = m_opened_table->insert(mysql_row);
 
-  DBUG_PRINT("temptable_api", ("this=%p row=(%s); return=%s", this,
-                               row_to_string(mysql_row, handler::table).c_str(),
-                               result_to_string(ret)));
+  info(HA_STATUS_VARIABLE);
 
   DBUG_RET(ret);
 }
 
 int Handler::update_row(const uchar *mysql_row_old, uchar *mysql_row_new) {
-  DBUG_ENTER("temptable::Handler::update_row");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   handler::ha_statistic_increment(&System_status_var::ha_update_count);
@@ -754,32 +678,27 @@ int Handler::update_row(const uchar *mysql_row_old, uchar *mysql_row_new) {
   Storage::Element *target_row;
 
   if (m_rnd_iterator_is_positioned) {
-    DBUG_ASSERT(!m_index_cursor.is_positioned());
+    assert(!m_index_cursor.is_positioned());
     target_row = *m_rnd_iterator;
   } else {
-    DBUG_ASSERT(m_index_cursor.is_positioned());
+    assert(m_index_cursor.is_positioned());
     target_row = m_index_cursor.row();
   }
 
   const Result ret =
       m_opened_table->update(mysql_row_old, mysql_row_new, target_row);
 
-  DBUG_PRINT("temptable_api",
-             ("this=%p old=(%s), new=(%s); return=%s", this,
-              row_to_string(mysql_row_old, handler::table).c_str(),
-              row_to_string(mysql_row_new, handler::table).c_str(),
-              result_to_string(ret)));
+  info(HA_STATUS_VARIABLE);
 
   DBUG_RET(ret);
 }
 
 int Handler::delete_row(const uchar *mysql_row) {
-  DBUG_ENTER("temptable::Handler::delete_row");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
-  DBUG_ASSERT(m_rnd_iterator_is_positioned);
+  assert(m_rnd_iterator_is_positioned);
 
   ha_statistic_increment(&System_status_var::ha_delete_count);
 
@@ -799,17 +718,12 @@ int Handler::delete_row(const uchar *mysql_row) {
     ++m_deleted_rows;
   }
 
-  DBUG_PRINT("temptable_api", ("this=%p row=(%s); return=%s", this,
-                               row_to_string(mysql_row, handler::table).c_str(),
-                               result_to_string(ret)));
-
   DBUG_RET(ret);
 }
 
 int Handler::truncate(dd::Table *) {
-  DBUG_ENTER("temptable::Handler::truncate");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   m_opened_table->truncate();
@@ -827,19 +741,19 @@ int Handler::truncate(dd::Table *) {
 }
 
 int Handler::delete_all_rows() {
-  DBUG_ENTER("temptable::Handler::delete_all_rows");
-  DBUG_RETURN(truncate(nullptr));
+  DBUG_TRACE;
+  return truncate(nullptr);
 }
 
 int Handler::info(uint) {
-  DBUG_ENTER("temptable::Handler::info");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
   opened_table_validate();
 
   stats.deleted = m_deleted_rows;
   stats.records = m_opened_table->number_of_rows();
   stats.table_in_mem_estimate = 1.0;
+  stats.data_file_length = m_opened_table->get_mem_counter();
 
   for (uint i = 0; i < table->s->keys; ++i) {
     KEY *key = &table->key_info[i];
@@ -857,21 +771,21 @@ int Handler::info(uint) {
 }
 
 longlong Handler::get_memory_buffer_size() const {
-  DBUG_ENTER("temptable::Handler::get_memory_buffer_size");
+  DBUG_TRACE;
 
   DBUG_PRINT("temptable_api",
              ("this=%p; return=%lld", this, temptable_max_ram));
 
-  DBUG_RETURN(temptable_max_ram);
+  return temptable_max_ram;
 }
 
 const char *Handler::table_type() const {
-  DBUG_ENTER("temptable::Handler::table_type");
-  DBUG_RETURN("TempTable");
+  DBUG_TRACE;
+  return "TempTable";
 }
 
 ::handler::Table_flags Handler::table_flags() const {
-  DBUG_ENTER("temptable::Handler::table_flags");
+  DBUG_TRACE;
 
   // clang-format off
   const Table_flags flags =
@@ -887,11 +801,11 @@ const char *Handler::table_type() const {
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%lld", this, flags));
 
-  DBUG_RETURN(flags);
+  return flags;
 }
 
 ulong Handler::index_flags(uint index_no, uint, bool) const {
-  DBUG_ENTER("temptable::Handler::index_flags");
+  DBUG_TRACE;
 
   ulong flags = 0;
 
@@ -923,88 +837,87 @@ ulong Handler::index_flags(uint index_no, uint, bool) const {
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%lu", this, flags));
 
-  DBUG_RETURN(flags);
+  return flags;
 }
 
 ha_key_alg Handler::get_default_index_algorithm() const {
-  DBUG_ENTER("temptable::Handler::get_default_index_algorithm");
-  DBUG_RETURN(HA_KEY_ALG_HASH);
+  DBUG_TRACE;
+  return HA_KEY_ALG_HASH;
 }
 
 bool Handler::is_index_algorithm_supported(ha_key_alg algorithm) const {
-  DBUG_ENTER("temptable::Handler::is_index_algorithm_supported");
-  DBUG_RETURN(algorithm == HA_KEY_ALG_BTREE || algorithm == HA_KEY_ALG_HASH);
+  DBUG_TRACE;
+  return algorithm == HA_KEY_ALG_BTREE || algorithm == HA_KEY_ALG_HASH;
 }
 
 uint Handler::max_supported_key_length() const {
-  DBUG_ENTER("temptable::Handler::max_supported_key_length");
+  DBUG_TRACE;
 
   const uint length = std::numeric_limits<uint>::max();
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%u", this, length));
 
-  DBUG_RETURN(length);
+  return length;
 }
 
-uint Handler::max_supported_key_part_length(
-    HA_CREATE_INFO *create_info MY_ATTRIBUTE((unused))) const {
-  DBUG_ENTER("temptable::Handler::max_supported_key_part_length");
+uint Handler::max_supported_key_part_length(HA_CREATE_INFO *create_info
+                                            [[maybe_unused]]) const {
+  DBUG_TRACE;
 
   const uint length = std::numeric_limits<uint>::max();
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%u", this, length));
 
-  DBUG_RETURN(length);
+  return length;
 }
 
 #if 0
 /* This is disabled in order to mimic ha_heap's implementation which relies on
  * the method from the parent class which adds a magic +10. */
 ha_rows Handler::estimate_rows_upper_bound() {
-  DBUG_ENTER("temptable::Handler::estimate_rows_upper_bound");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(m_opened_table != nullptr);
+  assert(m_opened_table != nullptr);
 
   const ha_rows n = m_opened_table->number_of_rows();
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%llu", this, n));
 
-  DBUG_RETURN(n);
+  return n;
 }
 #endif
 
 THR_LOCK_DATA **Handler::store_lock(THD *, THR_LOCK_DATA **, thr_lock_type) {
-  DBUG_ENTER("temptable::Handler::store_lock");
-  DBUG_RETURN(nullptr);
+  DBUG_TRACE;
+  return nullptr;
 }
 
 double Handler::scan_time() {
-  DBUG_ENTER("temptable::Handler::scan_time");
+  DBUG_TRACE;
 
   /* Mimic ha_heap::scan_time() to avoid a storm of execution plan changes. */
   const double t = (stats.records + stats.deleted) / 20.0 + 10;
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%.4lf", this, t));
 
-  DBUG_RETURN(t);
+  return t;
 }
 
 double Handler::read_time(uint, uint, ha_rows rows) {
-  DBUG_ENTER("temptable::Handler::read_time");
+  DBUG_TRACE;
 
   /* Mimic ha_heap::read_time() to avoid a storm of execution plan changes. */
   const double t = rows / 20.0 + 1;
 
   DBUG_PRINT("temptable_api", ("this=%p; return=%.4lf", this, t));
 
-  DBUG_RETURN(t);
+  return t;
 }
 
 int Handler::disable_indexes(uint mode) {
-  DBUG_ENTER("temptable::Handler::disable_indexes");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(current_thread_is_creator());
-  DBUG_ASSERT(m_opened_table != nullptr);
+  assert(m_opened_table != nullptr);
 
   Result ret;
 
@@ -1021,9 +934,7 @@ int Handler::disable_indexes(uint mode) {
 }
 
 int Handler::enable_indexes(uint mode) {
-  DBUG_ENTER("temptable::Handler::enable_indexes");
-
-  DBUG_ASSERT(current_thread_is_creator());
+  DBUG_TRACE;
 
   Result ret;
 
@@ -1041,170 +952,106 @@ int Handler::enable_indexes(uint mode) {
 
 /* Not implemented methods. */
 
-char *Handler::get_foreign_key_create_info() {
-  DBUG_ENTER("temptable::Handler::get_foreign_key_create_info");
-  DBUG_ABORT();
-  DBUG_RETURN(nullptr);
-}
-
-void Handler::free_foreign_key_create_info(char *) {
-  DBUG_ENTER("temptable::Handler::free_foreign_key_create_info");
-  DBUG_ABORT();
-  DBUG_VOID_RETURN;
-}
-
 int Handler::external_lock(THD *, int) {
-  DBUG_ENTER("temptable::Handler::external_lock");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
-void Handler::unlock_row() {
-  DBUG_ENTER("temptable::Handler::unlock_row");
-  DBUG_VOID_RETURN;
-}
+void Handler::unlock_row() { DBUG_TRACE; }
 
 handler *Handler::clone(const char *, MEM_ROOT *) {
-  DBUG_ENTER("temptable::Handler::clone");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(nullptr);
-}
-
-bool Handler::was_semi_consistent_read() {
-  DBUG_ENTER("temptable::Handler::was_semi_consistent_read");
-  DBUG_ABORT();
-  DBUG_RETURN(true);
-}
-
-void Handler::try_semi_consistent_read(bool) {
-  DBUG_ENTER("temptable::Handler::try_semi_consistent_read");
-  DBUG_ABORT();
-  DBUG_VOID_RETURN;
+  return nullptr;
 }
 
 int Handler::index_first(uchar *) {
-  DBUG_ENTER("temptable::Handler::index_first");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::index_last(uchar *) {
-  DBUG_ENTER("temptable::Handler::index_last");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::analyze(THD *, HA_CHECK_OPT *) {
-  DBUG_ENTER("temptable::Handler::analyze");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::optimize(THD *, HA_CHECK_OPT *) {
-  DBUG_ENTER("temptable::Handler::optimize");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::check(THD *, HA_CHECK_OPT *) {
-  DBUG_ENTER("temptable::Handler::check");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::start_stmt(THD *, thr_lock_type) {
-  DBUG_ENTER("temptable::Handler::start_stmt");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::reset() {
-  DBUG_ENTER("temptable::Handler::reset");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Handler::records(ha_rows *) {
-  DBUG_ENTER("temptable::Handler::records");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 void Handler::update_create_info(HA_CREATE_INFO *) {
-  DBUG_ENTER("temptable::Handler::update_create_info");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_VOID_RETURN;
 }
 
 int Handler::rename_table(const char *, const char *, const dd::Table *,
                           dd::Table *) {
-  DBUG_ENTER("temptable::Handler::rename_table");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 void Handler::init_table_handle_for_HANDLER() {
-  DBUG_ENTER("temptable::Handler::init_table_handle_for_HANDLER");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_VOID_RETURN;
 }
 
 bool Handler::get_error_message(int, String *) {
-  DBUG_ENTER("temptable::Handler::get_error_message");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(false);
+  return false;
 }
 
 bool Handler::primary_key_is_clustered() const {
-  DBUG_ENTER("temptable::Handler::primary_key_is_clustered");
-  // This is actually getting called in Percona Server TokuDB due to covering
-  // key query optimizer patch.
-  DBUG_RETURN(false);
+  DBUG_TRACE;
+  return false;
 }
 
 int Handler::cmp_ref(const uchar *, const uchar *) const {
-  DBUG_ENTER("temptable::Handler::cmp_ref");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(0);
+  return 0;
 }
 
 bool Handler::check_if_incompatible_data(HA_CREATE_INFO *, uint) {
-  DBUG_ENTER("temptable::Handler::check_if_incompatible_data");
+  DBUG_TRACE;
   DBUG_ABORT();
-  DBUG_RETURN(false);
+  return false;
 }
 
-ha_rows Handler::records_in_range(uint, key_range *, key_range *) {
-  DBUG_ENTER("temptable::Handler::records_in_range");
-  DBUG_ABORT();
-  DBUG_RETURN(0);
-}
-
-#ifdef TEMPTABLE_CPP_HOOKED_TESTS
-void Handler::test(TABLE *mysql_table) {
-  /* The test will call Handler::create() itself, avoid infinite recursion. */
-  static bool should_run = true;
-  if (should_run) {
-    should_run = false;
-
-    handler::table = mysql_table;
-    init_alloc_root(0, &handler::table->mem_root, TABLE_ALLOC_BLOCK_SIZE, 0);
-
-    Test t(handler::ht, handler::table_share, mysql_table);
-    t.correctness();
-    t.performance();
-
-    free_root(&handler::table->mem_root, 0);
-
-    should_run = true;
-  }
-}
-#endif /* TEMPTABLE_CPP_HOOKED_TESTS */
-
-#ifndef DBUG_OFF
-bool Handler::current_thread_is_creator() const {
-  return m_owner == std::this_thread::get_id();
-}
-#endif /* DBUG_OFF */
-
 } /* namespace temptable */
