@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,14 +37,14 @@ using namespace keyring;
 
 class Keyring_api_test : public ::testing::Test {
  public:
-  Keyring_api_test() {}
-  ~Keyring_api_test() {
+  Keyring_api_test() = default;
+  ~Keyring_api_test() override {
     delete[] plugin_name;
     delete[] keyring_filename;
   }
 
  protected:
-  virtual void SetUp() {
+  void SetUp() override {
     plugin_name = new char[strlen("FakeKeyring") + 1];
     strcpy(plugin_name, "FakeKeyring");
     keyring_filename = new char[strlen("./keyring") + 1];
@@ -62,7 +63,7 @@ class Keyring_api_test : public ::testing::Test {
     key_LOCK_keyring = PSI_NOT_INSTRUMENTED;
     sample_key_data = "Robi";
   }
-  virtual void TearDown() {
+  void TearDown() override {
     keyring_deinit_with_mock_logger();
     remove(keyring_file_data_value);
     remove("./keyring.backup");
@@ -85,7 +86,7 @@ void Keyring_api_test::keyring_init_with_mock_logger() {
 }
 
 void Keyring_api_test::keyring_deinit_with_mock_logger() {
-  keyring_deinit(NULL);
+  keyring_deinit(nullptr);
 }
 
 TEST_F(Keyring_api_test, StoreFetchRemove) {
@@ -102,14 +103,14 @@ TEST_F(Keyring_api_test, StoreFetchRemove) {
   EXPECT_EQ(key_len, sample_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
   EXPECT_EQ(mysql_key_remove("Robert_key", "Robert"), 0);
   // make sure the key was removed - fetch it
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
 }
 
 TEST_F(Keyring_api_test, CheckIfInmemoryKeyIsXORed) {
@@ -118,9 +119,9 @@ TEST_F(Keyring_api_test, CheckIfInmemoryKeyIsXORed) {
                       sample_key_data.length() + 1),
       0);
 
-  Key key_id("Robert_key", NULL, "Robert", NULL, 0);
+  Key key_id("Robert_key", nullptr, "Robert", nullptr, 0);
   IKey *fetched_key = keys->fetch_key(&key_id);
-  ASSERT_TRUE(fetched_key != NULL);
+  ASSERT_TRUE(fetched_key != nullptr);
   std::string expected_key_signature = "Robert_keyRobert";
   EXPECT_STREQ(fetched_key->get_key_signature()->c_str(),
                expected_key_signature.c_str());
@@ -128,7 +129,7 @@ TEST_F(Keyring_api_test, CheckIfInmemoryKeyIsXORed) {
             expected_key_signature.length());
   uchar *key_data_fetched = fetched_key->get_key_data();
   size_t key_data_fetched_size = fetched_key->get_key_data_size();
-  EXPECT_STREQ("AES", fetched_key->get_key_type()->c_str());
+  EXPECT_STREQ("AES", fetched_key->get_key_type_as_string()->c_str());
 
   // make sure that the key was xored before it was put into keys_container,
   // i.e.  the fetched key data is not equal to the key data that was stored
@@ -145,12 +146,12 @@ TEST_F(Keyring_api_test, CheckIfInmemoryKeyIsXORed) {
 }
 
 TEST_F(Keyring_api_test, FetchNotExisting) {
-  char *key_type = NULL;
-  void *key = NULL;
+  char *key_type = nullptr;
+  void *key = nullptr;
   size_t key_len = 0;
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
 }
 
 TEST_F(Keyring_api_test, RemoveNotExisting) {
@@ -167,7 +168,7 @@ TEST_F(Keyring_api_test, StoreFetchNotExisting) {
   void *key;
   EXPECT_EQ(mysql_key_fetch("NotExisting", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
 }
 
 TEST_F(Keyring_api_test, StoreStoreStoreFetchRemove) {
@@ -193,14 +194,14 @@ TEST_F(Keyring_api_test, StoreStoreStoreFetchRemove) {
   EXPECT_EQ(key_len, key_data1.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, key_data1.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
   EXPECT_EQ(mysql_key_remove("Robert_key2", "Robert"), 0);
   // make sure the key was removed - fetch it
   EXPECT_EQ(mysql_key_fetch("Robert_key2", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
 }
 
 TEST_F(Keyring_api_test, StoreValidTypes) {
@@ -231,7 +232,7 @@ TEST_F(Keyring_api_test, StoreInvalidType) {
   void *key;
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
 }
 
 TEST_F(Keyring_api_test, StoreTwiceTheSameDifferentTypes) {
@@ -245,275 +246,6 @@ TEST_F(Keyring_api_test, StoreTwiceTheSameDifferentTypes) {
       1);
 }
 
-TEST_F(Keyring_api_test, StorePBStoreSKFetchPBRemovePB) {
-  EXPECT_EQ(
-      mysql_key_store("percona_binlog", "AES", NULL, sample_key_data.c_str(),
-                      sample_key_data.length() + 1),
-      0);
-  EXPECT_EQ(mysql_key_store("percona_RGRGRG_1", "AES", NULL, "1234_",
-                            strlen("1234_") + 1),
-            0);
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key = NULL;
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  std::string key_data_with_version = "1:" + sample_key_data;
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-  EXPECT_EQ(mysql_key_remove("percona_binlog", NULL), 1);
-  // make sure the key was not removed - fetch it
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  my_free(key);
-}
-
-TEST_F(Keyring_api_test, GeneratePBFetchPBRotatePBFetchPB) {
-  EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
-
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key_ver1 = NULL;
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver1, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, static_cast<size_t>(18));
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1), "0:", 2) == 0);
-  my_free(key_type);
-  key_type = NULL;
-
-  void *key_ver2 = NULL;
-  EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
-
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, static_cast<size_t>(18));
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1), "1:", 2) == 0);
-  my_free(key_type);
-  key_type = NULL;
-
-  // make sure that rotated key is different than the original one
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1) + 2,
-                     reinterpret_cast<char *>(key_ver2) + 2, 16) != 0);
-
-  my_free(key_ver1);
-  my_free(key_ver2);
-}
-
-TEST_F(Keyring_api_test, GeneratePBRotatePBFetchFirstVersionFetchLatestPB) {
-  EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
-
-  EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
-
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key_ver1 = NULL;
-
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog:1", &key_type, NULL, &key_ver1, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, static_cast<size_t>(16));
-  my_free(key_type);
-  key_type = NULL;
-
-  void *key_ver2 = NULL;
-
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, static_cast<size_t>(18));
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1), "2:", 2) == 0);
-  my_free(key_type);
-  key_type = NULL;
-
-  // make sure that rotated key is different than the original one
-  // + 2 to skip key version of retrieved latest percona_binlog key
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1),
-                     reinterpret_cast<char *>(key_ver2) + 2, 16) != 0);
-
-  my_free(key_ver1);
-  my_free(key_ver2);
-}
-
-TEST_F(Keyring_api_test, StorePBRotatePBFetchFirstVersionFetchLatestPB) {
-  std::string percona_binlog_key_ver1_data("key_ver1");
-
-  EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL,
-                            percona_binlog_key_ver1_data.c_str(),
-                            percona_binlog_key_ver1_data.length() + 1),
-            0);
-
-  std::string percona_binlog_key_ver2_data("key_ver2");
-
-  EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL,
-                            percona_binlog_key_ver2_data.c_str(),
-                            percona_binlog_key_ver2_data.length() + 1),
-            0);
-
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key_ver1 = NULL;
-
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog:1", &key_type, NULL, &key_ver1, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, percona_binlog_key_ver1_data.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver1),
-                     percona_binlog_key_ver1_data.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-
-  void *key_ver2 = NULL;
-
-  EXPECT_EQ(
-      mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2, &key_len),
-      0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, percona_binlog_key_ver2_data.length() + 3);
-  std::string expected_percona_binlog_key_ver2_data_with_verion =
-      "2:" + percona_binlog_key_ver1_data;
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key_ver2),
-                     expected_percona_binlog_key_ver2_data_with_verion.c_str(),
-                     key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-
-  my_free(key_ver1);
-  my_free(key_ver2);
-}
-
-TEST_F(Keyring_api_test,
-       RotatePBStoreSKFetchPBRotatePBFetchPBRotatePBRotateSKFetchPBFetchSK) {
-  std::string percona_binlog_key_data_1("key1");
-
-  EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL,
-                            percona_binlog_key_data_1.c_str(),
-                            percona_binlog_key_data_1.length() + 1),
-            0);
-
-  std::string percona_sk_data_1("system_key1");
-
-  EXPECT_EQ(
-      mysql_key_store("percona_sk", "AES", NULL, percona_sk_data_1.c_str(),
-                      percona_sk_data_1.length() + 1),
-      0);
-
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key = NULL;
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  std::string key_data_with_version = "1:" + percona_binlog_key_data_1;
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-
-  std::string percona_binlog_key_data_2("key2");
-
-  EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL,
-                            percona_binlog_key_data_2.c_str(),
-                            percona_binlog_key_data_2.length() + 1),
-            0);
-
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  key_data_with_version = "2:" + percona_binlog_key_data_2;
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-
-  std::string percona_binlog_key_data_3("key3___");
-
-  EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL,
-                            percona_binlog_key_data_3.c_str(),
-                            percona_binlog_key_data_3.length() + 1),
-            0);
-
-  std::string percona_sk_data_2("percona_sk_data2");
-
-  EXPECT_EQ(
-      mysql_key_store("percona_sk", "AES", NULL, percona_sk_data_2.c_str(),
-                      percona_sk_data_2.length() + 1),
-      0);
-
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  key_data_with_version = "3:" + percona_binlog_key_data_3;
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-
-  EXPECT_EQ(mysql_key_fetch("percona_sk", &key_type, NULL, &key, &key_len), 0);
-  EXPECT_STREQ("AES", key_type);
-  key_data_with_version = "2:" + percona_sk_data_2;
-  EXPECT_EQ(key_len, key_data_with_version.length() + 1);
-  ASSERT_TRUE(memcmp(reinterpret_cast<char *>(key),
-                     key_data_with_version.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-}
-
-TEST_F(Keyring_api_test, FetchSystemKeyOnJustInitializedContainer) {
-  char *key_type = NULL;
-  size_t key_len = 0;
-  void *key = NULL;
-
-  EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key, &key_len),
-            0);
-  ASSERT_TRUE(key == NULL);
-}
-
-TEST_F(Keyring_api_test, KeyGenerate) {
-  EXPECT_EQ(mysql_key_generate("Robert_key", "AES", "Robert", 128), 0);
-  char *key_type;
-  size_t key_len;
-  void *key;
-  EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
-            0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, (size_t)128);
-  // Try accessing the last byte of key
-  char ch = ((char *)key)[key_len - 1];
-  // Just to get rid of unused variable compiler error
-  (void)ch;
-  my_free(key);
-  my_free(key_type);
-}
-
 TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   EXPECT_EQ(
       mysql_key_store("Robert_key", "AES", "Robert", sample_key_data.c_str(),
@@ -528,9 +260,9 @@ TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   EXPECT_EQ(key_len, sample_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
   delete[] keyring_filename;
   keyring_filename = new char[strlen("./new_keyring") + 1];
   strcpy(keyring_filename, "./new_keyring");
@@ -540,7 +272,7 @@ TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   keyring_init_with_mock_logger();
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
   EXPECT_EQ(
       mysql_key_store("Robert_key_new", "AES", "Robert",
                       sample_key_data.c_str(), sample_key_data.length() + 1),
@@ -554,16 +286,16 @@ TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   EXPECT_EQ(
       mysql_key_fetch("Robert_key_new", &key_type, "Robert", &key, &key_len),
       0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
   EXPECT_STREQ("AES", key_type);
   EXPECT_EQ(key_len, sample_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
   delete[] keyring_filename;
   keyring_filename = new char[strlen("./new_keyring") + 1];
   strcpy(keyring_filename, "./new_keyring");
@@ -577,9 +309,9 @@ TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
   EXPECT_EQ(key_len, sample_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
   remove("./new_keyring");
   // backup will stay as adding percona_binlog key will be unsuccessful - we
   // have already added it in keyring
@@ -587,34 +319,39 @@ TEST_F(Keyring_api_test, InitWithDifferentKeyringFile) {
 }
 
 TEST_F(Keyring_api_test, NullUser) {
-  EXPECT_EQ(mysql_key_store("Robert_key", "AES", NULL, sample_key_data.c_str(),
-                            sample_key_data.length() + 1),
-            0);
+  EXPECT_EQ(
+      mysql_key_store("Robert_key", "AES", nullptr, sample_key_data.c_str(),
+                      sample_key_data.length() + 1),
+      0);
   char *key_type;
   size_t key_len;
   void *key;
-  EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, NULL, &key, &key_len), 0);
-  EXPECT_STREQ("AES", key_type);
-  EXPECT_EQ(key_len, sample_key_data.length() + 1);
-  ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
-  my_free(key_type);
-  key_type = NULL;
-  my_free(key);
-  key = NULL;
-  EXPECT_EQ(mysql_key_store("Robert_key", "RSA", NULL, sample_key_data.c_str(),
-                            sample_key_data.length() + 1),
-            1);
-  EXPECT_EQ(mysql_key_store("Kamil_key", "AES", NULL, sample_key_data.c_str(),
-                            sample_key_data.length() + 1),
+  EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, nullptr, &key, &key_len),
             0);
-  EXPECT_EQ(mysql_key_fetch("Kamil_key", &key_type, NULL, &key, &key_len), 0);
   EXPECT_STREQ("AES", key_type);
   EXPECT_EQ(key_len, sample_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
+  EXPECT_EQ(
+      mysql_key_store("Robert_key", "RSA", nullptr, sample_key_data.c_str(),
+                      sample_key_data.length() + 1),
+      1);
+  EXPECT_EQ(
+      mysql_key_store("Kamil_key", "AES", nullptr, sample_key_data.c_str(),
+                      sample_key_data.length() + 1),
+      0);
+  EXPECT_EQ(mysql_key_fetch("Kamil_key", &key_type, nullptr, &key, &key_len),
+            0);
+  EXPECT_STREQ("AES", key_type);
+  EXPECT_EQ(key_len, sample_key_data.length() + 1);
+  ASSERT_TRUE(memcmp((char *)key, sample_key_data.c_str(), key_len) == 0);
+  my_free(key_type);
+  key_type = nullptr;
+  my_free(key);
+  key = nullptr;
   std::string arturs_key_data = "Artur";
   EXPECT_EQ(
       mysql_key_store("Artur_key", "AES", "Artur", arturs_key_data.c_str(),
@@ -626,35 +363,35 @@ TEST_F(Keyring_api_test, NullUser) {
   EXPECT_EQ(key_len, arturs_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, arturs_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
-  EXPECT_EQ(mysql_key_remove("Robert_key", NULL), 0);
+  key = nullptr;
+  EXPECT_EQ(mysql_key_remove("Robert_key", nullptr), 0);
   EXPECT_EQ(mysql_key_fetch("Robert_key", &key_type, "Robert", &key, &key_len),
             0);
-  ASSERT_TRUE(key == NULL);
+  ASSERT_TRUE(key == nullptr);
   EXPECT_EQ(mysql_key_fetch("Artur_key", &key_type, "Artur", &key, &key_len),
             0);
   EXPECT_STREQ("AES", key_type);
   EXPECT_EQ(key_len, arturs_key_data.length() + 1);
   ASSERT_TRUE(memcmp((char *)key, arturs_key_data.c_str(), key_len) == 0);
   my_free(key_type);
-  key_type = NULL;
+  key_type = nullptr;
   my_free(key);
-  key = NULL;
+  key = nullptr;
 }
 
 TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while storing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_store(NULL, "AES", "Robert", sample_key_data.c_str(),
+  EXPECT_EQ(mysql_key_store(nullptr, "AES", "Robert", sample_key_data.c_str(),
                             sample_key_data.length() + 1),
             1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while storing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_store(NULL, "AES", NULL, sample_key_data.c_str(),
+  EXPECT_EQ(mysql_key_store(nullptr, "AES", nullptr, sample_key_data.c_str(),
                             sample_key_data.length() + 1),
             1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
@@ -666,7 +403,7 @@ TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while storing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_store("", "AES", NULL, sample_key_data.c_str(),
+  EXPECT_EQ(mysql_key_store("", "AES", nullptr, sample_key_data.c_str(),
                             sample_key_data.length() + 1),
             1);
   char *key_type;
@@ -675,11 +412,11 @@ TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while fetching key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_fetch(NULL, &key_type, "Robert", &key, &key_len), 1);
+  EXPECT_EQ(mysql_key_fetch(nullptr, &key_type, "Robert", &key, &key_len), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while fetching key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_fetch(NULL, &key_type, NULL, &key, &key_len), 1);
+  EXPECT_EQ(mysql_key_fetch(nullptr, &key_type, nullptr, &key, &key_len), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while fetching key: key_id cannot be empty")));
@@ -687,16 +424,16 @@ TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while fetching key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_fetch("", &key_type, NULL, &key, &key_len), 1);
+  EXPECT_EQ(mysql_key_fetch("", &key_type, nullptr, &key, &key_len), 1);
 
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while removing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_remove(NULL, "Robert"), 1);
+  EXPECT_EQ(mysql_key_remove(nullptr, "Robert"), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while removing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_remove(NULL, NULL), 1);
+  EXPECT_EQ(mysql_key_remove(nullptr, nullptr), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while removing key: key_id cannot be empty")));
@@ -704,16 +441,16 @@ TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while removing key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_remove("", NULL), 1);
+  EXPECT_EQ(mysql_key_remove("", nullptr), 1);
 
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while generating key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_generate(NULL, "AES", "Robert", 128), 1);
+  EXPECT_EQ(mysql_key_generate(nullptr, "AES", "Robert", 128), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while generating key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_generate(NULL, "AES", NULL, 128), 1);
+  EXPECT_EQ(mysql_key_generate(nullptr, "AES", nullptr, 128), 1);
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while generating key: key_id cannot be empty")));
@@ -721,7 +458,7 @@ TEST_F(Keyring_api_test, NullKeyId) {
   EXPECT_CALL(*((Mock_logger *)logger.get()),
               log(ERROR_LEVEL,
                   StrEq("Error while generating key: key_id cannot be empty")));
-  EXPECT_EQ(mysql_key_generate("", "AES", NULL, 128), 1);
+  EXPECT_EQ(mysql_key_generate("", "AES", nullptr, 128), 1);
 }
 
 }  // namespace keyring__api_unittest
