@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +21,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-// First include (the generated) my_config.h, to get correct platform defines.
-#include "my_config.h"
-
 #include <gtest/gtest.h>
 
 #include "my_sys.h"
@@ -33,7 +31,7 @@ void test1(const char *res, const char *fmt, ...) {
   IO_CACHE info;
   va_list args;
   size_t len;
-  int init_res = init_io_cache(&info, -1, 0, WRITE_CACHE, 0, 1, MYF(0));
+  int init_res = init_io_cache(&info, -1, 0, WRITE_CACHE, 0, true, MYF(0));
   EXPECT_EQ(init_res, 0);
   memset(info.write_buffer, 0, 64); /* RECORD_CACHE_SIZE is 64K */
   va_start(args, fmt);
