@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1393,7 +1394,7 @@ Extract the zip size from tablespace flags.
 @param[in] flags        tablespace flags
 @return	compressed page size of the file-per-table tablespace in bytes, or zero
 if the table is not compressed. */
-MY_NODISCARD
+[[nodiscard]]
 static ulint fsp_flags_get_zip_size(ulint flags) noexcept {
   ulint zip_size = 0;
   const ulint ssize = FSP_FLAGS_GET_ZIP_SSIZE(flags);
